/****************************************************************************
    Copyright (C) 1991-2000    Canada-France-Hawaii Telescope Corp.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact information:
    CFHT, PO Box 1597, Kamuela, HI 96743, USA
    daprog@cfht.hawaii.edu

****************************************************************************/
/*!****************************************************************************
 * FILE log.c
 * $Id: cfht_log.c,v 1.8 2002/01/23 16:18:28 isani Exp $
 * $Locker:  $
 *    This file contains the code for cfht_log program
 *
 * ROUTINES
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          01 Jun 1989     Original coding
 *
 * $Log:	cfht_log.c,v $
 * Revision 1.8  2002/01/23  16:18:28  isani
 * Eliminated the 100 character length limit (cfht_log will
 * impose its own limit.)
 * 
 * Revision 1.7  2000/11/16  00:19:24  thomas
 * Added WARN type
 * 
 * Revision 1.6  1998/06/08 09:08:08  thomas
 * Changed usage message to show what args were provided
 *
 * Revision 1.5  98/02/25  16:34:26  16:34:26  thomas (Jim Thomas)
 * Changed log message to cfht_logpv to print PID of caller
 * Added message on bad type
 * 
 * Revision 1.4  94/12/13  17:56:36  17:56:36  thomas (Jim Thomas)
 * The msg definition was missing two commas between "lskdjf"
 * 
 * Revision 1.3  94/11/02  09:54:41  09:54:41  thomas (Jim Thomas)
 * added CFHT_LOGONLY and abbreviations
 * 
 * Revision 1.2  92/11/30  08:38:11  08:38:11  steve (Steven Smith)
 * if no environment vars CFHT_LOGS or CFHT_LOGU, then default to something.
 * 
 * Revision 1.1  91/11/01  19:55:10  19:55:10  steve (Steven S Smith)
 * Initial revision
 * 
 * Revision 1.1  91/09/25  15:34:56  15:34:56  jwright (Jim Wright)
 * Original checkin
 * 
 **************************************************************************!*/

#include <stdio.h>
#include <stdlib.h>

#include "cfht/cfht.h"
#include "cfht/exits.h"

RCSID("$Id: cfht_log.c,v 1.8 2002/01/23 16:18:28 isani Exp $");

/* print out the invalid command and the above */
static void
logusage(int argc, const char* argv[])
{
  static const char *msg[] =
    {
    "usage: cfht_log who type message",
    "    who     - name of program/script to charge log against",
    "    type    - one of CFHT_START, CFHT_STATUS, CFHT_LOGONLY,",
    "                     CFHT_ERROR, CFHT_WARN, CFHT_DONE, or CFHT_DEBUG,",
    "                     (without the CFHT_ OK too)",
    "    message - a single argument for the message",
    "              (quote the string, if neccessary)",
    (char *) NULL
    };

    int Count;
    const char** Pointer;
    (void) fprintf(stderr, "Bad cfht_log call -");
    for (Count = 0; Count < argc; Count++) {
	(void) fprintf(stderr, "%s ", argv[Count]);
    }
    (void) fprintf(stderr, "-\n");
    Pointer = msg;
    while (*Pointer != (char *) NULL) {
	(void) fprintf(stderr, "%s\n", *Pointer);
	Pointer++;
    }
    exit(EXIT_FAIL);
    return;			  /* keep compilers quiet */
}

int
main(int argc, const char* argv[])
{
    static char env_str[100];
    int type;

    if (argc != 4)
      logusage(argc, argv);

    /* since this program may be executed from remote machines, */
    /* and not have access to the logu and logs environment variables */
    /* we need to kludge them up */

    if ((char *) getenv(CFHT_LOGU) == (char *)NULL) {
	sprintf(env_str, "%s=%s/%s", CFHT_LOGU, getenv("HOME"), ".,session.np");
	putenv(env_str);
    }
    if ((char *) getenv(CFHT_LOGS) == (char *)NULL) {
	sprintf(env_str, "%s=%s", CFHT_LOGS, "/tmp/pipes/syslog.np");
	putenv(env_str);
    }
    cfht_log(CFHT_MAIN, CFHT_LOG_ID, (char*)argv[1]);

    if ((strcmp(argv[2], "CFHT_START") == 0)
	|| (strcmp(argv[2], "START") == 0))
      type = CFHT_START;
    else if ((strcmp(argv[2], "CFHT_STATUS") == 0)
	     || (strcmp(argv[2], "STATUS") == 0))
      type = CFHT_STATUS;
    else if ((strcmp(argv[2], "CFHT_LOGONLY") == 0)
	     || (strcmp(argv[2], "LOGONLY") == 0))
      type = CFHT_LOGONLY;
    else if ((strcmp(argv[2], "CFHT_ERROR") == 0)
	     || (strcmp(argv[2], "ERROR") == 0))
      type = CFHT_ERROR;
    else if ((strcmp(argv[2], "CFHT_WARN") == 0)
	     || (strcmp(argv[2], "WARN") == 0))
      type = CFHT_WARN;
    else if ((strcmp(argv[2], "CFHT_FATAL") == 0) /* obsolete */
	     || (strcmp(argv[2], "FATAL") == 0))
      type = CFHT_ERROR;
    else if ((strcmp(argv[2], "CFHT_DONE") == 0)
	     || (strcmp(argv[2], "DONE") == 0))
      type = CFHT_DONE;
    else if ((strcmp(argv[2], "CFHT_DEBUG") == 0)
	     || (strcmp(argv[2], "DEBUG") == 0))
      type = CFHT_DEBUG;
    else {
	cfht_logpv(CFHT_MAIN, CFHT_ERROR,
		  "Bad type code '%.20s', message '%.100s'",
		  argv[2], argv[3]);
	exit(EXIT_FAIL);
    }

    cfht_log_proxy(CFHT_MAIN, type, (char*)argv[3], /*id=*/ NULL, getppid());

    exit(EXIT_SUCCESS);
}
