/*                                             -*- c-file-style: "Ellemtel" -*-

`agentsmith.c' - Agent that returns OK for any arbitrary command

This file is part of version 3 of the Fake substitute agent for director.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "cli/cli.h"

RCSID("$Id$");

static const char* basename = "";
static BOOLEAN done = FALSE;

static PASSFAIL
com_exit( const char* args UNUSED ) { done = TRUE; return PASS; }

static Command comlist[] = {
   { "cd",   cli_cd, "Change working directories" },
   { "exit", com_exit, "Exit agentsmith" },
   { "help", cli_help, "Displays this help for agentsmith" },
   { "?",    cli_help, "Synonym for help" },
   { 0, 0, 0 } /* This terminates the command list */
};

static void
cleanup(void)
{
   printf("\ragentsmith (%s) exiting on signal.\n", basename);
   exit(EXIT_SUCCESS);
}

int
main(int argc, const char* argv[])
{
   basename = strrchr(argv[0], '/');
   if (basename) basename++; else basename = argv[0];

   if (cli_init(basename, comlist, 2)==FAIL) exit(EXIT_FAILURE);

   /*
    * Make sure the user is aware that we are faking some subsystem.
    */
   printf("\rwarning: Fake agentsmith running instead of %s!\n", basename);

   cli_signal(SIGTERM, cleanup);
   cli_signal(SIGINT, CLI_SIG_IGN);  /* usually ^C if on a tty */
   cli_signal(SIGTSTP, CLI_SIG_IGN); /* usually ^Z if on a tty */
   cli_signal(SIGQUIT, CLI_SIG_IGN); /* usually ^\ if on a tty */

   while (!done)
   {
      static char* line = 0;

      if (line != 0) free(line);	/* free memory used by last command */
      line = cli_getline();		/* get the next command */

      if (!line)
      {
	 if (errno == 0) done = TRUE;
      }
      else
      {
	 /*
	  * Normally we'd just cli_execute the line, but in this case
	  * we want to just return a PASS for every unrecognized command.
	  * It's only a special function of the "fake" agent.
	  */
	 if (!cli_search(line, comlist))
	 {
	    printf("warning: Faking command %s.%s\n", basename, line);
	    cli_setprompt(PASS);
	 }
	 else
	 {
	    cli_execute(line);
	 }
      }
   }
   printf("\ragentsmith (%s) exiting normally.\n", basename);
   exit(EXIT_SUCCESS);
}

/* End of agentsmith.c */
