/*                                             -*- c-file-style: "Ellemtel" -*-

`clicmd.c' - Small program that calls libcli's cli_cmd/cap/dup()

This file is part of version 3 of the Utility to pass commands to director.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "cli/cli.h"
#include "cli/clidefs.h"

#define CMDBUF_SIZE 2000

#ifdef CLICAP
#define CLI_NAME "clicap"
#define CLI_FUNC cli_cap
#else
#ifdef CLIDUP
#define CLI_NAME "clidup"
#define CLI_FUNC cli_dup
#else
#define CLI_NAME "clicmd"
#define CLI_FUNC cli_cmd
#endif
#endif

static void
send_abort(void)
{
   static int max = 4;
   if (max)
   {
      max--;
#ifndef NO_CFHTLOG
      cfht_log(CFHT_MAIN, CFHT_LOGONLY, CLI_NAME" sending abort");
#endif
      cli_cmd("abort");
   }
}

static void
send_stop(void)
{
   static int max = 4;
   if (max)
   {
      max--;
#ifndef NO_CFHTLOG
      cfht_log(CFHT_MAIN, CFHT_LOGONLY, CLI_NAME" sending stop");
#endif
      cli_cmd("stop");
   }
}

static void
send_quit(void)
{
   static int max = 4;
   if (max)
   {
      max--;
#ifndef NO_CFHTLOG
      cfht_log(CFHT_MAIN, CFHT_LOGONLY, CLI_NAME" sending quit");
#endif
      cli_cmd("quit");
   }
}

int
main(int argc, const char* argv[])
{
   const char* basename = argv[0];
   const char* p;
   char cmdbuf[CMDBUF_SIZE] = "";
   int length = 0;
   int i = 0;
   int retries = 0;

   {
      char sessionhostfile[1000];
      char sessionhost[256];
      char hostname[256];
      int len;
      
      if (gethostname(hostname, sizeof(hostname)-1)!=-1)
      {
	 sprintf(sessionhostfile, "%.800s/.director/sessionhost", cli_gethome());
	 /* %%% Use define from SESSIONHOSTFILE */
	 len = readlink(sessionhostfile, sessionhost, sizeof(sessionhost)-1);
	 if (len != -1)
	 {
	    sessionhost[len] = '\0';
	    if (strcmp(sessionhost, hostname)!=0)
	    {
	       cli_runon(sessionhost, 0, argc, argv);
	       exit(EXIT_FAILURE);
	    }
	 }
      }
   }

   putenv("CFHTLOGU="); /* Prevent potential feedback problems.
			 * If clicmd has anything to report, it
			 * will go into the system log.  If called
			 * with DIRECTOR set, however, cfht_log
			 * will still attempt to write to stderr.
			 */
#ifndef NO_CFHTLOG
   cfht_log(CFHT_MAIN, CFHT_LOG_ID, CLI_NAME);
#endif

   /*
    * Trap signals and send them to director.
    */
   cli_signal(SIGINT, send_abort);
   cli_signal(SIGTSTP, send_stop);
   cli_signal(SIGQUIT, send_quit);

   while ((p=strchr(basename, '/'))!=0) basename = p + 1;

   if (!strncmp(basename, CLI_NAME, 6)) i++;

   {
      const char* s;

      if (i == 0) s = basename;
      else if (i < argc) s = argv[i];
      else s = "";

      while (strchr(CHARS_SPECIAL, *s)) s++;

      if (!strcmp(s, "running"))
      {
	 if (i+1 < argc)
	    retries = atoi(argv[i+1]);
      }
   }

   while (i < argc && length < CMDBUF_SIZE - 2)
   {
#ifndef SUNOS
      length +=
#endif
	 sprintf(cmdbuf + length, "%s%.*s",
		 (length>0)?" ":"",
		 CMDBUF_SIZE - length - 2,
		 i?argv[i]:basename);
#ifdef SUNOS
      length += strlen(cmdbuf + length);
#endif
      i++;
   }

#ifndef NO_CFHTLOG
   if (length >= CMDBUF_SIZE - 2)
      cfht_log(CFHT_MAIN, CFHT_WARN, "command line buffer is full");
#endif

   do
   {
#ifndef NO_CFHTLOG
      cfht_logv(CFHT_MAIN, CFHT_DEBUG, "now starting "CLI_NAME" %s", cmdbuf);
#endif
      i=CLI_FUNC(cmdbuf);
      if (i!=PASS && retries)
      {
#ifndef NO_CFHTLOG
	 cfht_log(CFHT_MAIN, CFHT_DEBUG, CLI_NAME" waiting for director");
#endif
	 sleep(1);
      }
   } while (i!=PASS && retries--);

#ifndef NO_CFHTLOG
   if ((i)!=PASS)
      cfht_logv(CFHT_MAIN, CFHT_LOGONLY, CLI_NAME" %s failed", cmdbuf);
   else
      cfht_logv(CFHT_MAIN, CFHT_DEBUG, CLI_NAME" %s passed", cmdbuf);
#endif
   
   exit((i==PASS)?EXIT_SUCCESS:EXIT_FAILURE);
}
