/*                                             -*- c-file-style: "Ellemtel" -*-

`Curs.hh' - manages screen output and keyboard input

This file is part of version 2.12 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2000, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef _INCLUDED_CURS
#define _INCLUDED_CURS 1

#include <ncurses/curses.h>
#include <ncurses/form.h>

#include "Roll.hh"

class Curs
{
   private:
      WINDOW* feedback;
      WINDOW* statbar;
      WINDOW* prompt;
      WINDOW* info;
      FORM* form;
      FIELD* field[2];
      char* instant_;
      int statbar_showing_;
      int prompt_showing_;
      int feedback_size_;
      int infolines_;
      int pendsize_;
      int titleclock_;
      int verbose_;
      int debug_;
      int clone_;
      int jumpscroll_;
      int keep_bgcolor_;
      int always_refresh_;
      int titlebar_ok_;
      int input_buffer_empty_;

      /*
       * (Form field list.  First one is the entry blank for
       * the command line, and the second should always contain
       * a NULL to indicate the end of the list.)
       */

      void cleanup(void);
      void init(int clear_feedback = 1);

   public:
      Curs(int infolines, int keep_bgcolor);
      ~Curs() { cleanup(); if (instant_) free(instant_); }

      void infosize(int infolines);
      void resize();
      void repaint();
      void update(char prompt_char='\0');

      int confirm(const char* s);
      int password(const char* prompt_string, const char* passwd);

      void print(const char* s, const char* prefix = 0);

      void message(const char* s, int logtype=-1);

      const char* titlename(const char* u = 0);
      void titleclock(int onoff = 1);
      void titlebar(const char* s = 0);
      int  statusbar(const char *s); /* === non-zero return means a redraw
				      * of the info panel may be needed
				      */
      void infoline(const char* s, int n);
      void pending(const char* s);
      void instant(const char* s);

      void debug(int onoff=1) { debug_ = onoff; }
      void verbose(int onoff=1) { verbose_ = onoff; }
      void clone(int onoff=1) { clone_ = onoff; }
      void jumpscroll(int onoff=1) { jumpscroll_ = onoff; }
      void jumpscroll_check(void) { if (jumpscroll_==-1) jumpscroll_=0; }

      const char* check_line();
      /*
       * this routine should be called anytime that there
       * might be input ready on stdin.  If there isn't any,
       * it will not block.  Instead it will return 0
       * immediately.  In all other cases, it processes all
       * pending input on stdin and returns a pointer to the
       * line that the user has typed in (when they hit enter).
       * During the time that they are still editing their
       * line, it keeps returning 0 until they hit enter.
       */
};

#endif
