/*                                             -*- c-file-style: "Ellemtel" -*-

`Pipes.cc' - See Pipes.hh

This file is part of version 2.12 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2000, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <pwd.h>

extern "C" {
#include "setserial.h"
#include <cfht/cfht.h>
#include "cli/cli.h"
}

/*
 * Timeout in milliseconds after which failure to write
 * to a pipe should cause a failure.
 */
#define WRITE_MSEC 10000

void set_interval_timer(int onoff);
void do_sleep(int seconds); /* temporarily disable the interval timer and do
			     * a regular sleep 
			     */


#include "Pipes.hh"
#include "Roll.hh" /* needed for just a few little things */
extern Roll* roll;

void
log_error(const char* msg, int errno_)
{
   char junkbuf[5000];
   sprintf(junkbuf, "%s%.4500s (%.400s)",
	   roll?"error: ":"", msg, strerror(errno_));

   if (roll)
      roll->add_line(junkbuf);
   else
      cfht_log(CFHT_MAIN, CFHT_ERROR, (char*)msg);
}

void
log_verbose(const char* msg)
{
   char junkbuf[5000];
   sprintf(junkbuf, "%s%.4500s",
	   roll?"logonly: ":"", msg);

   if (roll)
      roll->add_line(junkbuf);
   else
      cfht_log(CFHT_MAIN, CFHT_LOGONLY, (char*)msg);
}

LineBuffer::LineBuffer()
{
   bufp = buf[bufnum=0];
   fd = -1;
   command_counter=0;
   command_result=0;
   lastnewline=0;
   found_prompt=FALSE;
}

const char*
LineBuffer::check_line()
{
   int bytes;
   char* newline = 0;
   extern const char *ps1, *ps2, *psok, *psfailed, *pscont; /* === hack */
   extern int ps1len, ps2len, psoklen, psfailedlen, pscontlen;
   
   if (fd==-1) return 0; /* no more input */
   
   *bufp='\0';
   newline = strpbrk(buf[bufnum],"\r\n");
   
   if (!newline)
   {
      void* belpos;

      bytes=read(fd, bufp, CMD_MAXLEN-(bufp-buf[bufnum]));
      
      if (bytes==-1)
      {
	 if (errno!=EAGAIN)
	 {
	    char junkbuf[1024];
	    int save_errno = errno;
	    
	    sprintf(junkbuf, "read(%d) failed; closing file descriptor", fd);
	    log_error(junkbuf, save_errno);
	    close(fd);
	    fd = -1;
	    return 0;
	 }
	 if (!ps2len && command_result && *command_result==BUSY)
	 {
	    if (roll) roll->add_line("debug: (ready/no more data)");
	    if (*(buf[bufnum]))
	    {
	       if (roll) roll->add_line("debug: still in buffer:");
	       if (roll) roll->add_line(buf[bufnum], CFHT_DEBUG);
	    }
	    found_prompt = TRUE;
	    *command_result = PASS;
	    if (command_counter && *command_counter>=0) (*command_counter)--;
	 }
      }

      if (bytes==0)
      {
	 close(fd);
	 fd = -1;
	 return 0;
      }
      
      if (bytes>0)
      {
	 while ((belpos = memchr(bufp, (int)'\a', bytes))!=0)
	 {
	    /*
	     * Strip out any bel characters and make the terminal beep
	     */
	    int len_after_bel = bufp + bytes - (char*)belpos - 1;
	    
	    if (len_after_bel>0)
		     memmove(belpos, (char*)belpos+1, len_after_bel);
	    bytes--;
	    fputc('\a', stderr); /* Make the terminal beep, now */
	 }
	 
	 if (bytes)
	 {
	    bufp += bytes;
	    bufp[0] = '\0';
	    bufp[1] = '\0';
	    
	    if (bufp - buf[bufnum] == CMD_MAXLEN &&
		strpbrk(buf[bufnum],"\r\n") == 0)
	    {
	       cfht_logv(CFHT_MAIN, CFHT_LOGONLY, "LineBuffer overflow");
	       while (read(fd, buf[(bufnum+1)%2], CMD_MAXLEN)>0) ;
	       newline = bufp-1;
	    }
	    else
	    {
	       newline = strpbrk(buf[bufnum], "\r\n");
	    }
	 }
      }
   }
   
   if (command_counter && command_result && found_prompt==FALSE)
   {
      if (strncmp(buf[bufnum], ps1, ps1len)==0)
      {
 	 if (roll) roll->add_line("debug: (ready/ps1)");
	 found_prompt = TRUE;
	 if (command_result && *command_result==BUSY) *command_result = PASS;
      }
      else if (strncmp(buf[bufnum], ps2, ps2len)==0)
      {
 	 if (roll) roll->add_line("debug: (ready/ps2)");
	 found_prompt = TRUE;
	 if (command_result && *command_result==BUSY) *command_result = PASS;
      }
      else if (strncmp(buf[bufnum], psok, psoklen)==0)
      {
 	 /* === if (roll) roll->add_line("debug: (ready/ok)"); */
	 found_prompt = TRUE;
	 if (command_result && *command_result==BUSY) *command_result = PASS;
      }
      else if (strncmp(buf[bufnum], psfailed, psfailedlen)==0)
      {
	 if (roll) roll->add_line("debug: (failed)");
	 found_prompt = TRUE;
	 if (command_result && *command_result==BUSY) *command_result = FAIL;
      }
      if (found_prompt==TRUE && command_counter && *command_counter>=0)
	 (*command_counter)--;
   }
   
   if (newline)
   {
      char* rtn = buf[bufnum];
      
      /* ignore \r's at the beginning of a line and \n's immediately after \r's 
       * but not a \r\n at the beginning of a line.  sigh
       */
      if (newline==rtn)
      {
	 if (*newline=='\r' ||  /* \r at beginning of line */
	     lastnewline=='\r') /* \n after a \r that was not at beginning of line */
	    rtn=0;
	 lastnewline = 0;
      }
      else
      {
	 lastnewline = *newline;
      }

      *newline++ = '\0';
      
      bufnum = (bufnum + 1) % 2;
      strcpy(buf[bufnum], newline);
      bufp = buf[bufnum] + strlen(buf[bufnum]);
      
      found_prompt = FALSE;
      
      if (!rtn) return check_line(); /* recursively call for next line */

      /*
       * === This is gross... checking for the ok> and failed> again
       * could just strip it off the first time we found it, but that
       * is more difficult.  This is safer.
       */
      if (rtn)
      {
	 int pslen = 0;
	 if (strncmp(rtn, ps1, ps1len)==0) pslen = ps1len;
	 else if (strncmp(rtn, ps2, ps2len)==0) pslen = ps2len;
	 else if (strncmp(rtn, psok, psoklen)==0) pslen = psoklen;
	 else if (strncmp(rtn, psfailed, psfailedlen)==0) pslen = psfailedlen;
	 else if (strncmp(rtn, pscont, pscontlen)==0) pslen = pscontlen;
	 if (pslen)
	 {
	    if (pslen < 2)
	    {
	       rtn[pslen-1]='>';
	       rtn += pslen-1;
	    }
	    else if (rtn[pslen-1]==' ')
	    {
	       rtn[pslen-2]='>';
	       rtn += pslen-2;
	    }
	    else if (rtn[pslen]!=' ')
	    {
	       rtn[pslen-2]='>';
	       rtn[pslen-1]=' ';
	       rtn += pslen-2;
	    }
	    else
	    {
	       rtn[pslen-1]='>';
	       rtn += pslen-1;
	    }
	    /*
	     * hide lines that just say '> [alarm]' (This should be discontinued)
	     * hide lines that started with an @
	     */
	    if (strcmp(rtn, "> [alarm]")==0 ||
		strncmp(rtn, "> @", 3)==0) rtn = 0;
	 }

	 return rtn?rtn:check_line();
      }
   }

   return 0;
}


CommandPipe::CommandPipe(const char* name, int wronly=0)
{
   command_fd = -1;
   autoremove = 0;
   pipe_name = name;
   if (pipe_name)
   {
      if (!wronly && unlink(pipe_name)==0)
      {
	 if (roll) roll->add_line("warning: removed stale command pipe");
      }
      if (!wronly && 
	  (mknod(pipe_name, S_IFIFO|S_IWUSR|S_IRUSR|S_IWGRP|S_IWOTH, 0)!=0 ||
	   chmod(pipe_name,         S_IWUSR|S_IRUSR|S_IWGRP|S_IWOTH)!=0)) /* %%% OTH? */
      {
	 log_error("failed to create command pipe", errno);
      }
      else if ((command_fd = 
		open(pipe_name, 
		     (wronly?O_WRONLY:O_RDWR)|O_NONBLOCK
		     ,0))==-1)
	 /*
	  * NOTE: the pipe is opened for writing as well because
	  * even though we don't want to write to it, we want to
	  * be able to select() on it, and that doesn't work right
	  * unless someone has it open for writing already.
	  */
      {
	 log_error("failed to open command pipe for writing", errno);
      }
      if (!wronly) { set_fd(command_fd); autoremove=1; }
   }
}

void
CommandPipe::shutdown()
{
   if (autoremove)
   {
      unlink(pipe_name);
      autoremove = 0;
   }
}

CommandPipe::~CommandPipe()
{
   shutdown();
   if (command_fd!=-1)
   {
      close(command_fd);
      command_fd = -1;
   }
}

void
CommandPipe::send(const char* command)
{
   if (command_fd!=-1)
   {
      if ((*command && 
	   cli_write(command_fd, command, strlen(command), WRITE_MSEC)==FAIL) ||
	  cli_write(command_fd, "\n", 1, WRITE_MSEC)==FAIL)
      {
	 log_error("write(commandPipe) failed", errno);
      }
   }
   if (roll) roll->add_line("warning: Nowhere to send commands");
}

LogPipe::LogPipe(const char* name)
{
   log_fd = -1;
   autoremove = 0;
   pipe_name = name;
   if (pipe_name)
   {
      int create = 0;
      struct stat stbuf;
      
      memset(&stbuf, 0, sizeof(stbuf));
      
      if (stat(pipe_name, &stbuf)==0)
      {
	 if (!S_ISFIFO(stbuf.st_mode))
	 {
	    char nametxt[1024];
	    /*
	     * If the file existed, but was not a named pipe,
	     * rename it / remove it so it can be re-created.
	     */

	    sprintf(nametxt, "%.1000s.TXT", pipe_name);
	    /*
	     * Before moving "pipe_name" to *.TXT, make sure *.TXT can
	     * be removed.  If not, try renaming the file to *.pid.TXT.
	     */
	    if (unlink(nametxt) != 0 && errno != ENOENT)
	    {
	       sprintf(nametxt, "%.1000s.%u.TXT",
		       pipe_name, (unsigned)getpid());
	    }
	    if (rename(pipe_name, nametxt) != 0)
	    {
	       log_error("failed to rename bogus text log file", errno);
	       if (unlink(pipe_name) != 0)
		  log_error("failed to remove bogus text log file", errno);
	    }
	    else
	       if (roll) roll->add_line("logonly: moved text logfile to *.TXT (should be pipe)");
	    create = 1;
	 }
	 else
	 {
	    /*
	     * Open existing log pipe for writing (with non-blocking i/o) to
	     * test if anyone else is already reading it.  This _should_ fail.
	     * If it doesn't fail, it means the log pipe is already in use
	     * by another director, possibly.
	     */
	    log_fd = open(pipe_name, O_WRONLY | O_NONBLOCK, 0);
	    if (log_fd != -1)
	    {
	       if (roll) roll->add_line("logonly: log pipe alread in use!");
	       (void) close(log_fd); /* Ignore if this fails. */
	       log_fd = -1;
	       return;
	    }
	    if (roll) roll->add_line("logonly: log pipe exists; won't create or remove it");
	 }
      }
      else create = 1;
      
      if (create)
      {
	 if (mknod(pipe_name, S_IFIFO|S_IWUSR|S_IRUSR|S_IWGRP, 0)!=0)
	 {
	    log_error("failed to create log pipe", errno);
	    return;
	 }
	 autoremove = 1;
      }
      
      if ((log_fd = open(pipe_name, O_RDWR | O_NONBLOCK, 0))==-1)
	 /*
	  * NOTE: the pipe is opened for writing as well because
	  * even though we don't want to write to it, we want to
	  * be able to select() on it, and that doesn't work right
	  * unless someone has it open for writing already.
	  */
	 log_error("failed to open log pipe", errno);

      set_fd(log_fd);
   }
}

LogPipe::~LogPipe()
{
   if (log_fd!=-1) (void) close(log_fd); /* Ignore failure here. */
   if (autoremove) unlink(pipe_name);
}

#define PIPE_OUTPUT_SIDE 0
#define PIPE_INPUT_SIDE 1

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>     
#include <string.h>

ProcessPipe::ProcessPipe(const char** argv)
{
   int i, argc = 0;
   const char** a = argv;

   /*
    * Make own copy of argv so caller can free theirs.
    */
   while (*a) { argc++; a++; }
   argv_ = (char**)malloc(sizeof(char*)*(argc + 1));
   for (i=0; i<argc; i++)
      argv_[i] = strdup(argv[i]);
   argv_[i] = 0;

   type = PIPE_NOT_USED;
   child_input[PIPE_INPUT_SIDE] = -1;
   child_output[PIPE_OUTPUT_SIDE] = -1;
   child_stderr[PIPE_OUTPUT_SIDE] = -1;
   set_fd(-1);
   err.set_fd(-1);
   restart();
}

ProcessPipe::~ProcessPipe()
{
   char** a = argv_;

   shutdown();
   
   while (*a) free(*a++);
   free(argv_);
}

extern void flush_to_screen(void); /* %%% */
extern void flush_inputs(void);

void
ProcessPipe::shutdown()
{
   if (type==PIPE_TO_PROCESS)
   {
      if (signal(0)==0)
      {
	 int wait_sec = 8;

	 flush_inputs();
	 roll->add_line("status: director: sending TERM signal to agent process");
	 flush_to_screen();
	 signal(SIGTERM);
	 while (1)
	 {
	    do_sleep(1);
	    flush_inputs();
	    flush_to_screen();

	    /*
	     * If the child exited... %%% HACK!
	     */
	    if (cfht_signal_event(SIGCHLD))
	    {
	       const char* s = postmortem();
	       
	       if (s) roll->add_line(s);
	    }
	    
	    if (signal(0)!=0)
	    {
	       roll->add_line("logonly: director: agent process responded to TERM signal");
	       flush_to_screen();
	       break;
	    }
	    if (wait_sec--) continue;
	    /*
	     * Normally, the process will have exited by now.  But if it
	     * doesn't, it means the program refuses to terminate cleanly.
	     * make SURE it's gone by killing it with SIGKILL 8 seconds
	     * later...
	     */
	    roll->add_line("status: director: sending KILL signal to agent process");
	    flush_to_screen();
	    signal(SIGKILL);
	    flush_inputs();
	    flush_to_screen();
	    break;
	 }
      }
   }
   if (child_input[PIPE_INPUT_SIDE]!=-1)
   {
      close(child_input[PIPE_INPUT_SIDE]);
      child_input[PIPE_INPUT_SIDE]=-1;
   }
   if (child_output[PIPE_OUTPUT_SIDE]!=-1)
   {
      close(child_output[PIPE_OUTPUT_SIDE]);
      child_output[PIPE_OUTPUT_SIDE]=-1;
      set_fd(-1);
   }
   if (child_stderr[PIPE_OUTPUT_SIDE]!=-1)
   {
      close(child_stderr[PIPE_OUTPUT_SIDE]);
      child_stderr[PIPE_OUTPUT_SIDE]=-1;
      err.set_fd(-1);
   }
}

extern BOOLEAN waiting;

PASSFAIL
ProcessPipe::restart(void)
{
   char junkbuf[1024];
   char* colon = strchr(argv_[0],':');
   char* brace;
   const char* options = 0;
   const char* argv0 = argv_[0];
   struct stat stbuf;

   shutdown();

   commands_pending = 0;
   command_result = BUSY;
   child = (pid_t)-1;
   type = PIPE_NOT_USED;

   if (roll)
   {
      int i = 0;

      sprintf(junkbuf, "titlebar: %.500s", argv_[0]);
      roll->add_line(junkbuf);
      sprintf(junkbuf, "status: director: starting agent `");
      while (argv_[i] && (strlen(junkbuf)+strlen(argv_[i])+4) < sizeof(junkbuf))
      {
	 if (i) strcat(junkbuf, " ");
	 strcat(junkbuf, argv_[i]);
	 i++;
      }
      strcat(junkbuf, "'");
      roll->add_line(junkbuf);
   }

   if (*argv0 == '{' && (brace = strchr(argv0, '}')))
   {
      options = argv0 + 1;
      argv0 = brace + 1;
   }

   memset(&stbuf, 0, sizeof(stbuf));

   if (!colon && stat(argv0, &stbuf)==0)
   {
      if (S_ISDIR(stbuf.st_mode))
      {
	 if (roll) roll->add_line("error: director: can't run a directory!");
	 return FAIL;
      }
      
      if (S_ISCHR(stbuf.st_mode))
      {
	 extern const char* cli_newline; /* === */
	 /*
	  * Reading from device instead of process
	  */
	 
	 type = PIPE_TO_DEVICE;
	 
	 child_stderr[PIPE_OUTPUT_SIDE] = -1;
	 if ((child_input[PIPE_INPUT_SIDE] =
	      child_output[PIPE_OUTPUT_SIDE] =
	      open(argv0, O_RDWR | O_NONBLOCK | O_NOCTTY))==-1)
	 {
	    log_error("failed to open agent device", errno);
	    do_sleep(2);
	    return FAIL;
	 }
	 if (setserial(child_input[PIPE_INPUT_SIDE], options) < 0)
	 {
	    log_error("failed to set all serial options (continuing)", errno);
	 }
	 
	 if (roll) roll->add_line("status: connected to a serial port");
	 do_sleep(1);
	 if (cli_write(child_input[PIPE_INPUT_SIDE], cli_newline, 1, WRITE_MSEC)==FAIL)
	 {
	    log_error("writing to serial port failed", errno);
	    do_sleep(3);
	 }
	 commands_pending = -1;
	 command_result = PASS;
      }
   }
   
   if (command_result != PASS)
   {
      /*
       * Reading from local process through a pipe
       */
      type = PIPE_TO_PROCESS;
      
      pipe(child_input);
      pipe(child_output);
      pipe(child_stderr);
      
      if ((child = fork()) == (pid_t)-1)
      {
	 log_error("fork failed", errno);
	 return FAIL;
      }
      
      if (child == (pid_t)0)
      {
	 extern const char* logpipename;
	 char junkbuf[1024];
	 
	 /*
	  * This is the child thread.  Create a new ProcessPipe group
	  * so we don't catch signals meant only for the parent.
	  */
	 setsid();
	 
	 alarm(0);
	 cfht_signal(SIGALRM, CFHT_SIG_IGN);
	 cfht_signal_unblock(SIGALRM);
	 cfht_signal(SIGALRM, CFHT_SIG_DFL);
	 cfht_signal(SIGINT, CFHT_SIG_DFL);
	 cfht_signal(SIGTSTP, CFHT_SIG_DFL);
	 cfht_signal(SIGQUIT, CFHT_SIG_DFL);
	 
	 putenv("COLORTERM=dumb");
	 putenv("TERM=dumb");
	 if (logpipename)
	 {
	    sprintf(junkbuf, "CFHTLOGU=%s", logpipename);
	    putenv(junkbuf);
	 }
	 
	 /*
	  * Set up the stdio pipes.
	  */
	 close(STDOUT_FILENO);
	 dup(child_output[PIPE_INPUT_SIDE]);
	 
	 close(STDERR_FILENO);
	 dup(child_stderr[PIPE_INPUT_SIDE]);
	 
	 close(STDIN_FILENO);
	 dup(child_input[PIPE_OUTPUT_SIDE]);
	 
	 close(child_input[PIPE_INPUT_SIDE]);
	 close(child_output[PIPE_OUTPUT_SIDE]);
	 close(child_stderr[PIPE_OUTPUT_SIDE]);
	 
	 execvp( colon?"runon":argv0, (char**)argv_ );
	 perror( argv0 );
	 _exit(EXIT_FAILURE);
      }
      
      /*
       * This is the parent thread.
       */
      close(child_input[PIPE_OUTPUT_SIDE]);
      close(child_output[PIPE_INPUT_SIDE]);
      close(child_stderr[PIPE_INPUT_SIDE]);

      waiting = TRUE; /* %%% Major hack, so main loop knows to wait for initial prompt */
   }

   set_command_counter(&commands_pending, &command_result);
   err.set_command_counter(&commands_pending, &command_result);

   set_fd(child_output[PIPE_OUTPUT_SIDE]);
   err.set_fd(child_stderr[PIPE_OUTPUT_SIDE]);

   if (fcntl(child_input[PIPE_INPUT_SIDE], F_SETFL,
	     fcntl(child_input[PIPE_INPUT_SIDE], F_GETFL, 0)|O_NONBLOCK)==-1)
   {
      log_error("fcntl(child_input ... NON_BLOCK) failed", errno);
      return FAIL;
   }
   return PASS;
}

const char*
ProcessPipe::check_line()
{
   const char* s = LineBuffer::check_line();

   if (!s) return 0;
   if (!strncmp(s, "signalmap:", 10)) return check_line();
   return s;
}

int
ProcessPipe::signal(int signo)
{
   int rtn = -1;

   switch (type)
   {
      case PIPE_TO_PROCESS:
      {
	 if (child!=-1 && child!=0)
	    /*
	     * The "-child" means the whole process group will
	     * get the signal, in order to be consistent with what
	     * happens when signals are send via rcmd()
	     */
	    if ((rtn=kill(-child, signo))==-1)
	    {
	       log_error("failed to signal process", errno);
	    }
	 /* === else print some kind of warning */
	 break;
      }
      default:
      {
	 if (get_fd()!=-1)
	 {
#define CONTROL(c) (1-'A'+c)
	    unsigned char sigchar = 0;

	    /* === clean up */
	    if (signo==SIGINT) sigchar = CONTROL('C');
	    if (signo==SIGTSTP) sigchar = CONTROL('Z');
	    if (signo==SIGQUIT) sigchar = CONTROL('\\');
	    if (sigchar)
	       if ((rtn = cli_write(child_output[PIPE_OUTPUT_SIDE], &sigchar, 1, WRITE_MSEC))==FAIL)
	       {
		  log_error("write(sigchar) failed", errno);
	       }
	    /* === else warn that we can't send that signal */
	 }
      }
   }
   return rtn;
}

PASSFAIL
ProcessPipe::send(const char* command)
{
   extern const char* cli_newline; /* === */

   if (child_input[PIPE_INPUT_SIDE]==-1)
   {
      if (roll) roll->add_line("error: nobody to talk to!");
      return FAIL;
   }

   if ((*command && 
	cli_write(child_input[PIPE_INPUT_SIDE], command, strlen(command), WRITE_MSEC)==FAIL) ||
       cli_write(child_input[PIPE_INPUT_SIDE], cli_newline, 1, WRITE_MSEC)==FAIL)
   {
      log_error("write(ProcessPipe) failed", errno);
      return FAIL;
   }

   commands_pending++;
   command_result = BUSY;

   return PASS;
}


const char*
ProcessPipe::postmortem()
{
   /*
    * tries to conduct an autopsy on an exited process and
    * returns a string describing what happened, or 0 if the
    * process is apparently still running.
    */
   static char rtn[80];
   int status;
   int waitrtn;

   if ( child == (pid_t)-1) return "director: no child process";

   waitrtn = waitpid( child, &status, WNOHANG );
   
   if (waitrtn==0)
      return 0;
   
   if (waitrtn==-1)
   {
      log_error("director: waitpid failed", errno);
      return 0;
   }
   
   if (waitrtn!=child)
   {
      log_error("director: unexpected return from waitpid", errno);
      return 0;
   }
   
   if (!WIFEXITED(status))
   {
      if (WIFSIGNALED(status))
      {
	 child = -1;
	 if (WTERMSIG(status)==SIGTERM)
	    sprintf(rtn, "warning: director: agent exited on TERM signal");
	 else
	    sprintf(rtn, "error: director: agent exited on signal #%d", 
		    WTERMSIG(status));
	 return rtn;
      }
      else if (WIFSTOPPED(status))
      {
	 return "error: director: agent process stopped";
      }
      else
      {
	 return "error: director: unknown problem with child process";
      }
   }
   else
   {
      if (WEXITSTATUS(status)!=EXIT_SUCCESS)
      {
	 child = -1;
	 sprintf(rtn, "error: director: agent exited with exitcode %d", 
		 WEXITSTATUS(status));
	 return rtn;
      }
      /* else, normally exiting child... */
      child = -1;
      return "agent exited normally";
   }
}

/* end of Pipes.cc */
