/*                                             -*- c-file-style: "Ellemtel" -*-

`Roll.cc' - See Roll.hh

This file is part of version 2.12 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2000, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include "Roll.hh"

void do_sleep(int seconds); /* temporarily disable the interval timer and do
			     * a regular sleep 
			     */

#define ADVANCE(ptr) (ptr = (ptr+1)%ROLL_LINES)

void
Roll::init_RollLine(RollLine& r)
{
   r.str[0]='\0'; r.str[LINE_WIDTH]='\0';
   r.serial = 0; r.length = 0;
}
      
void
Roll::set_RollLine(RollLine& r, const char* s, int eatspace=0)
{
   // === Clones watching this may get confused during fast updates
   if (eatspace) if (*s==' ') s++;			// eat 1 space
   r.length = strlen(s);				// get length
   if (r.length > LINE_WIDTH) r.length=LINE_WIDTH;	// trim if too long

   strncpy(r.str, s, r.length);
   r.str[r.length] = '\0';
   r.serial++;
}
      
void
Roll::set_RollPtr(const char* s, int logtype, int eatspace)
{
   int next = (shm->tail+1)%ROLL_LINES;
   RollPtr& r = shm->roll[next];
   
   if (eatspace) if (*s==' ') s++;			// eat 1 space
   r.length = strlen(s);				// get length
   r.length_handle = 0;					// director3 feature
   r.timestamp = 0;					// director3 feature
   if (r.length > LINE_WIDTH) r.length=LINE_WIDTH;	// trim if too long

   if (logtype==CFHT_ERROR) fputc('\a', stderr);	// ring bell for errors

   /*
    * Advance the head pointer if wrapped all the way
    * around and filled all the slots in roll[]
    */
   if (next==shm->head) ADVANCE(shm->head);

   /*
    * Find place in buf[] to start writing the new string
    */
   if (r.length+1 > ROLL_MEM - shm->nextofs)
      /*
       * If it doesn't fit at the end, clear any remaining
       * entries that were using the last part of buf[] and
       * write the new string at buf[0].
       */
   {
      while (shm->head != shm->tail &&
	     shm->roll[shm->head].offset >= shm->nextofs)
	 ADVANCE(shm->head);
      shm->nextofs = 0;
   }
   r.offset = shm->nextofs;
   /*
    * Save new offset that will be used on next call to set_RollPtr()
    */
   shm->nextofs += r.length+1;

   /*
    * Find out how many old messages were clobbered
    */
   while (shm->head != shm->tail &&
          r.offset   <= shm->roll[shm->head].offset &&
          shm->nextofs > shm->roll[shm->head].offset)
      ADVANCE(shm->head);
   
   /*
    * Make sure the head string is still always and empty line
    */
   shm->roll[shm->head].length = 0;
   shm->roll[shm->head].length_handle = 0;
   shm->roll[shm->head].logtype = -1;
   shm->buf[shm->roll[shm->head].offset] = '\0';

   /*
    * Copy the string into place and save other records
    */
   strncpy(&(shm->buf[r.offset]), s, r.length);
   shm->buf[r.offset + r.length] = '\0';
   r.logtype = logtype;
   
   /*
    * Don't incrememnt tail until the data is actually there,
    * or else clones could read the line prematurely.
    */
   if (next!=-1) shm->tail = next;
}


Roll::Roll(int roll_key, clone_mode_t clone_mode)
{
   /*
    * Constructor for Roll class attaches to the shared memory block
    * If must_clone is zero, Roll will automatically determine if it
    * is a clone monitoring another director's RollShm, or if it should
    * actually allocate it.  If must_clone is non-zero, having to
    * allocate the Shm is treated as an error.
    */
   int i, shmkey, shmflg;
   
   shmkey = roll_key;	/* shmkey and roll_key are usually the same, but
			 * shmkey may get hashed to a different value if
			 * there is a collision with some other program.
			 */
   /*
    * Do not allow "must_clone" programs (director -k) to have write
    * access or ability to create the shared memory segment
    */
   if (clone_mode == CLONE_ONLY) shmflg = 00444;
   else shmflg = 00644|IPC_CREAT;
   
   int tries = 50;
   do
   {
      /*
       * Get the id for the correct shared memory block.
       */
      if (roll_key==-1)
      {
	 shmid = shmget(IPC_PRIVATE, sizeof(RollShm), shmflg);
      }
      else while (shmkey!=-1 &&
		  (shmid = shmget(shmkey, sizeof(RollShm), shmflg))==-1)
      {
	 shmkey += 1024; /* hash to different slot if collision */
	 if (shmkey == roll_key || !tries--) shmkey=-1; /* hash table full? */
      }
      
      if (shmid==-1)
      {
	 cfht_logv(CFHT_MAIN, CFHT_ERROR,
		   "User id %d is not running director on this host", roll_key);
	 perror("shmget");
	 fprintf(stderr,
		 "User id %d does not appear to be running director on this host.\n",
		 roll_key);
	 exit(EXIT_FAILURE);
      }
      
      switch (nattch())
      {
	 case -1: perror("shmctl"); exit(EXIT_FAILURE);
	 case 0: clone_=0; break;
	 default:
	 {
	    if (clone_mode==MASTER_ONLY)
	    {
	       cfht_logv(CFHT_MAIN, CFHT_ERROR,
			 "User id %d is already running director", roll_key);
	       fprintf(stderr,
		       "User id %d is already running director", roll_key);
	       exit(EXIT_FAILURE);
	    }
	    clone_=1;
	 }
      }
      
      /*
       * Got a valid id.  Now attach the memory segment
       */
      if ((shm =
	   (RollShm*)shmat(shmid, 0, SHM_RDONLY*clone_))
	  ==(void*)-1)
      {
	 perror("shmat");
	 exit(EXIT_FAILURE);
      }
      if (!clone_) shm->roll_key = roll_key;
   } while (shm->roll_key != roll_key);
   
   if (!clone_)
      /*
       * Initialize the shared memory RollShm structure.
       */
   {
      for (i=0; i<INFO_LINES; i++) init_RollLine(shm->info[i]);
      init_RollLine(shm->statusbar);
      init_RollLine(shm->titlebar);
      init_RollLine(shm->instant);
      
      shm->magic = DIRECTOR_MAGIC;
      shm->version = 2; /* === don't forget to update! */
      shm->tail = 0;
      shm->head = 0;
      shm->roll[0].offset = 0;
      shm->roll[0].length = 0;
      shm->roll[0].length_handle = 0;
      shm->roll[0].timestamp = 0;
      shm->roll[0].logtype = -1;
      shm->buf[0] = '\0'; /* head string is always an empty line */
      shm->nextofs = 1; /* first real string will be written at buf[1] */
      memset(shm->roll[0].reserved, 0, sizeof(RollPtr::reserved));
      memset(shm->reserved, 0, sizeof(RollShm::reserved));
   }
   statusbar_serial_disp = 0;
   rewind(50); /* catch up by showing the last 50 lines */
}

Roll::~Roll()
{
   /*
    * Destructor for roll class detaches from shared memory
    * If there are clones, the master process also waits for
    * clones to exit before removing the shared memory id.
    */
   if (!clone_) shm->time_to_exit = 1; /* signal clones it's time to exit */
   
   if (shmdt((char*)shm)==-1)
      perror("shmdt");
   
   if (!clone_)
   {
      int tries=10;
      
      while (nattch()>0 && tries--) do_sleep(1);
      if (nattch()!=0)
	 fprintf(stderr,"warning: problem detaching shared memory!\n");
      
      if (shmctl(shmid, IPC_RMID, 0)==-1)
	 perror("shmctl");
   }
}

int
Roll::is_clone()
{
   /*
    * Returns 0 if master process
    *         1 if clone and master process is still running
    *        -1 if clone and master process is trying to exit
    */
   return (shm->time_to_exit)?-1:clone_;
}

int
Roll::nattch()
{
   /*
    * Get number of processes attached to shm segment.  Returns -1
    * on error from shmctl()
    */
   struct shmid_ds shmbuf;
   
   if (shmctl(shmid, IPC_STAT, &shmbuf)==-1) return -1;
   else return shmbuf.shm_nattch;
}
	 
void
Roll::rewind(int lines)
{
   /*
    * Moves the disp index back so that get_line() will
    * return `lines' previous lines from the current position.
    * May go back more than `lines' if there are debug messages
    * inbetween.
    */
   int maxlines = ROLL_LINES/2;
   int logtype;

   if (lines) disp = shm->tail;

   while (maxlines-- && lines)
   {
      disp = (disp+ROLL_LINES-1)%ROLL_LINES;
      if ((shm->head<=shm->tail && shm->head<=disp && disp<=shm->tail) ||
	  (shm->head>shm->tail && !(shm->tail<disp && disp<shm->head)))
      {
	 /*
	  * valid position
	  */
	 logtype = shm->roll[disp].logtype;
	 if (logtype != CFHT_DEBUG && logtype != CFHT_LOGONLY) lines--;
      }
      else lines--;
   }
   titlebar_serial_disp = shm->titlebar.serial - 1;
   instant_serial_disp = shm->instant.serial - 1;
   if (statusbar_serial_disp!=0)
      statusbar_serial_disp = -1;	// leave 0 i to avoid initialization
   // of statusbar.  Statusbar only appears
   // when first true "statusbar:" message arrives.
   for (int i=0; i<9; i++) info_serial_disp[i] = shm->info[i].serial - 1;
}

RollPtr*
Roll::get_line()
{
   if (shm->time_to_exit || disp==shm->tail) return 0;
   
   disp++;
   disp %= ROLL_LINES;
   if ((shm->head<=shm->tail && shm->head<=disp && disp<=shm->tail) ||
       (shm->head>shm->tail && !(shm->tail<disp && disp<shm->head)))
   {
      /*
       * valid position
       */
      return &shm->roll[disp];
   }
   else
   {
      /*
       * out of range, return the empty string in head
       */
      return &shm->roll[shm->head];
   }
}

const char*
Roll::get_str(RollPtr* rp)
{
   if (!rp) return NULL;
   else return &(shm->buf[rp->offset]);
}
   
RollLine*
Roll::get_statusbar()
{
   if (statusbar_serial_disp!=shm->statusbar.serial)
   {
      statusbar_serial_disp = shm->statusbar.serial;
      return &shm->statusbar;
   }
   else return 0;
}

RollLine*
Roll::get_titlebar()
{
   if (titlebar_serial_disp!=shm->titlebar.serial)
   {
      titlebar_serial_disp = shm->titlebar.serial;
      return &shm->titlebar;
   }
   else return 0;
}

RollLine*
Roll::get_instant()
{
   /*
    * Returns a pointer to the titlebar line if changed, else 0.
    */
   if (instant_serial_disp!=shm->instant.serial)
   {
      instant_serial_disp = shm->instant.serial;
      return &shm->instant;
   }
   else return 0;
}

RollLine*
Roll::get_info(int n)
{
   /*
    * Returns a pointer to the n-th info line if changed, else 0.
    * n should be in the range 1..INFO_LINES
    */
   n--;
   if (n>=INFO_LINES || n<0) return 0;
   if (info_serial_disp[n]!=shm->info[n].serial)
   {
      info_serial_disp[n] = shm->info[n].serial;
      return &shm->info[n];
   }
   else return 0;
}

void
Roll::add_line(const char* s, int logtype, int nolog, int no_echo)
{
   if (clone_!=0) return; /* === clones are not allowed to write to the roll */
   
   /*
    * First check for message types that go to special screen areas.
    * If found, next is set to -1 to indicate that the feedback roll
    * should not be advanced at the end of add_line(), and nolog is
    * set to 1 to indicate this message should not go to cfht_log().
    */
   
   if (strncmp(s, "statusbar:", 10)==0)
   { set_RollLine(shm->statusbar, s+10); nolog=1; }
   
   else if (strncmp(s, "titlebar:", 9)==0)
   { set_RollLine(shm->titlebar, s+9, 1); nolog=1; }
   
   else if (strncmp(s, "instant:", 8)==0)
   { set_RollLine(shm->instant, s+8, 1); nolog=1; }
   
   else if (strncmp(s, "info", 4)==0 && s[4]>='1' && s[5]==':')
   { set_RollLine(shm->info[(s[4]-'1')%INFO_LINES], s+6, 1); nolog=1; }
   
   /*
    * The rest go to the feedback area, displayed with various colors.
    */
   else if (*s=='>')	// === need to have a type for prompt messages!
   { if (no_echo) return; set_RollPtr(s); nolog=1; }
   
   else if (strncmp(s, "\033&v2S", 5)==0) // unhack hpterm escape sequence. yuk.
      set_RollPtr(s+=5, logtype=CFHT_ERROR, 0);
   
   else if (strncmp(s, "error:", 6)==0)
      set_RollPtr(s+=6, logtype=CFHT_ERROR, 1);
   
   else if (strncmp(s, "\033&v3S", 5)==0)
      set_RollPtr(s+=5, logtype=CFHT_WARN, 0);
   
   else if (strncmp(s, "warning:", 8)==0)
      set_RollPtr(s+=8, logtype=CFHT_WARN, 1);
   
   else if (strncmp(s, "status:", 7)==0)
      set_RollPtr(s+=7, logtype=CFHT_STATUS, 1);
   
   else if (strncmp(s, "logonly:", 8)==0)
      set_RollPtr(s+=8, logtype=CFHT_LOGONLY, 1);
   
   else if (strncmp(s, "debug:", 6)==0)
      set_RollPtr(s+=6, logtype=CFHT_DEBUG, 1);
   
   else if (logtype>=0)
      set_RollPtr(s, logtype);
   
   else if (*s=='*')
   { set_RollPtr(s, CFHT_HIGHLIGHT, 1); nolog=1; }
   
   else
   { set_RollPtr(s); nolog=1; }
   
   /*
    * Don't log messages to cfht_log that came from there in the
    * first place, and don't log messages that weren't of type
    * error/warning/status/logonly/debug.
    */
   if (!nolog) cfht_log(CFHT_MAIN, logtype, (char*)(*s==' '?s+1:s));
}

void
Roll::instant_off()
{
   if (shm->instant.length) set_RollLine(shm->instant, "");
}

void
Roll::set_infosize(int lines)
{
   if (!is_clone()) shm->infosize = lines;
}

int
Roll::get_infosize()
{
   return shm->infosize;
}

/* end of Roll.cc */
