/*                                             -*- c-file-style: "Ellemtel" -*-

`builtins.cc' - builtin commands that director handles itself

This file is part of version 2.12 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2000, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdlib.h>

#include "cli/cli.h"

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0
#endif

static Command local[];		/* contains an entry for each command */
static Command remote[];	/* contains an entry for each command */

/*
 * Commands
 *
 */

static PASSFAIL
com_clock( const char* arg ) {
   BOOLEAN f = cli_bool(arg);
   curs->titleclock(f);
   if (f==TRUE) curs->print("Title clock is ON");
   else         curs->print("Title clock is OFF");
   return PASS;
}

static PASSFAIL
com_jumpscroll( const char* arg )
{
   BOOLEAN f = cli_bool(arg);
   curs->jumpscroll(f);
   if (f==TRUE) 
      curs->print("Jumpscroll is ON (warning: scrollbar may not function correctly!)");
   else    
      curs->print("Jumpscroll is OFF");
   return PASS;
}

static PASSFAIL
com_debug( const char* arg )
{
   curs->debug(cli_bool(arg));
   roll->rewind(LINES);
   return PASS;
}

static PASSFAIL
com_verbose( const char* arg ) 
{
   curs->verbose(cli_bool(arg));
   roll->rewind(LINES); 
   return PASS;
}

static PASSFAIL
com_infosize( const char* arg )
{
   int lines = atoi(arg);
   
   if (lines < 0) return FAIL;
   if (lines > INFO_LINES) lines = INFO_LINES;
   curs->infosize(lines);
   roll->set_infosize(lines); /* This does nothing in clones */
   roll->rewind(0);
   return PASS;
}

static PASSFAIL
com_source( const char* arg )
{
   if (!arg || !*arg)
   {
      roll->add_line("status: Syntax: source <filename>");
      return FAIL;
   }

   return scripter.add(arg);
}

static PASSFAIL
com_builtin( const char* arg )
{
   if (arg);
   /* === can't use cli help because can't print to stdout here */
   roll->add_line("");
   roll->add_line("*---[ Builtin Director Commands ]-----------------------------------------*");
   roll->add_line("termhelp      : display help screen for operating this terminal window");
   roll->add_line("debug on|off  : toggles debug output (also requires verbose for displaying)");
   roll->add_line("verbose on|off: toggles displaying of verbose messages");
   roll->add_line("clock on|off  : controls display of titlebar clock");
/* roll->add_line("jumpscroll    : set to on for (really) slow terminal emulators"); === */
   roll->add_line("infosize N    : displays N lines of the info panel (N=0..9)");
   roll->add_line("cancel (or ^G): cancels queued but not yet executing commands");
   roll->add_line("abort  (or ^C): sends SIGINT to the agent");
   roll->add_line("stop   (or ^Z): sends SIGTSTP to the agent");
   roll->add_line("quit   (or ^|): sends SIGQUIT to the agent");
   roll->add_line("sigsend [NAME]: sends SIG[NAME] to the agent");
   roll->add_line("ready         : immediately returns pass if ready for command, fail if busy");
   roll->add_line("running       : immediately returns pass unless director is shutting down");
   roll->add_line("say <message> : use to send messages to other users with director clones");
   roll->add_line("agent ...     : starts, restarts, or stops an agent running in director");
   roll->add_line("autorestart on|off: tells director whether or not to restart agents that die");
   roll->add_line("source <file> : reads batch commands from <file>");
   roll->add_line("*-------------------------------------------------------------------------*");
   return PASS;
}

static PASSFAIL
com_termhelp( const char* arg )
{
   if (arg);
   roll->add_line("*---[ Terminal Window Tips ]----------------------------------------------*");
   roll->add_line(" o Resizing window will cause feedback area to fill available space");
   /*
    * These might only apply to rxvt
    */
   roll->add_line(" o Shift-numpad-plus and Shift-numpad-minus change font size (rxvt only)");
   roll->add_line(" o Shift-page-up and Shift-page-down scroll back the window (rxvt only)");

   roll->add_line(" o Up-arrow and Down-arrow allow editing/repeating previous commands");
   roll->add_line(" o Other cursor keys (home, end, left, right) do their thing");
   roll->add_line(" o Most commonly used emacs bindings work also (but not ^Y!)");
   roll->add_line(" o Command/file completion (usually bound to esc/tab) NOT implemented");
   roll->add_line(" o Pressing Control-L will clear and redraw the entire window");
   roll->add_line("*-------------------------------------------------------------------------*");
   return PASS;
}

static PASSFAIL
com_license( const char* arg )
{
   /*
    * WARNING: Do not remove this command, modify the text or alter the
    * license conditions without prior permission from the original author!
    *
    * See also: Start of "INPUT LOOP" in director.cc
    */
   if (arg);
   roll->add_line("  Director version "VERSION_STRING" is written by Sidik Isani <isani@cfht.hawaii.edu> for the Canada-France-Hawaii Telescope.  It is free software;  you can redistribute it and/or modify it under the terms of the GNU General Public License, version 2, as published by the Free Software Foundation.  If you do modify the program, please make sure that this is clearly indicated at startup on any copies that you redistribute.  Please read the GPL for more information.");
   roll->add_line("");
   roll->add_line("  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.");
   roll->add_line("");
   roll->add_line("  You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.");
   roll->add_line("");
   roll->add_line("  Official versions of this program, including the source code, can be obtained at http://rpm.cfht.hawaii.edu/director/ or send email to <daprog@cfht.hawaii.edu>.");
   /*
    * WARNING: Do not remove this command, modify the text or alter the
    * license conditions without prior permission from the original author!
    */
   /* === TODO: Put entire GPL here instead? */
   return PASS;
}

static PASSFAIL
com_agent( const char* args )
{
   static char** cargv = NULL;
   static int cargc = 0;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }

   cargv = cli_argv(&cargc, args);

   if ((cargc < 2 ||
	strcmp(cargv[0], "start")) &&
	(cargc < 1 || strcmp(cargv[0], "restart")))
   {
      roll->add_line("status: Syntax: agent start [host:]program     or     agent restart");
      return FAIL;
   }

   if (strcmp(cargv[0], "start")==0)
   {
      if (process) delete process;
      process = new ProcessPipe((const char**)&cargv[1]);
      return PASS; /* === did it really? need a process.ok() method to check! */
   }
   else if (strcmp(cargv[0], "restart")==0)
   {
      if (!process)
      {
	 roll->add_line("warning: director: no process to restart");
	 return FAIL;
      }
      flush_inputs();
      roll->add_line("status: director: restarting process connection . . .");
      flush_to_screen();
      if (process->restart()==PASS)
      {
	 roll->add_line("status: director: successfully restarted connection");
	 return PASS;
      }
      else
      {
	 roll->add_line("error: director: unable to restart connection");
	 return FAIL;
      }
   }
   else
   {
      if (process) delete process;
      return PASS;
   }
}

static PASSFAIL
com_say(const char* arg)
{
   static char msgbuf[2000]; /* === */
   static char* arg1 = 0;
   struct passwd* pwd = 0;

   pwd = getpwuid(getuid());

   if (arg!=0) free(arg1);
   arg1 = cli_arg1(arg);

   sprintf(msgbuf, "say_ * %s: ``%s''", pwd?pwd->pw_gecos:"Unknown", arg1);
   commandPipe->send(msgbuf); /* === krusty */
   return PASS;
}

static PASSFAIL
com_say_(const char* arg)
{
   /*
    * Internal version of say command that expects the User Name: ... to
    * be appended to the string.
    */
   roll->add_line(arg);
   return PASS;
}

static Command local[] = {
   { "clock",     com_clock,     "[on|off] controls titlebar clock" },
   { "debug",     com_debug,     "[on|off] shows/hides debug messages" },
   { "verbose",   com_verbose,   "[on|off] shows/hides extra messages" },
   { "infosize",  com_infosize,  "[n] shows n lines of info panel" },
   { "jumpscroll",com_jumpscroll,"[on|off] turns on/off jumpscroll for slow terminals" },
   { "say",       com_say,       "send message to other users" },
   { 0, 0, 0 }
};

static class SigNames
{
   public:
      const char* signames[NSIG+1];

      SigNames() {
	 for (int s=1; s<=NSIG; s++) signames[s]=0;
	 
	 signames[SIGILL] = "ILL";
	 signames[SIGTRAP] = "TRAP";
	 signames[SIGABRT] = "ABRT";
#ifdef SIGIOT
	 signames[SIGIOT] = "IOT";
#endif
#ifdef SIGEMT
	 signames[SIGEMT] = "EMT";
#endif
	 signames[SIGFPE] = "FPE";
	 signames[SIGKILL] = "KILL";
#ifdef SIGSYS
	 signames[SIGSYS] = "SYS";
#endif
#ifdef SIGPWR
	 signames[SIGPWR] = "PWR";
#endif
#ifdef SIGVTALRM
	 signames[SIGVTALRM] = "VTALRM";
#endif
#ifdef SIGPROF
	 signames[SIGPROF] = "PROF";
#endif
#ifdef SIGIO
	 signames[SIGIO] = "IO";
#endif
#ifdef SIGWINCH
	 signames[SIGWINCH] = "WINCH";
#endif
	 signames[SIGSTOP] = "STOP";
	 signames[SIGTSTP] = "TSTP";
	 signames[SIGCONT] = "CONT";
	 signames[SIGTTIN] = "TTIN";
	 signames[SIGTTOU] = "TTOU";
#ifdef SIGURG
	 signames[SIGURG] = "URG";
#endif
#ifdef SIGLOST
	 signames[SIGLOST] = "LOST";
#endif
	 signames[SIGBUS] = "BUS";
	 signames[SIGCHLD] = "CHLD";
	 signames[SIGTERM] = "TERM";
	 signames[SIGUSR2] = "USR2";
	 signames[SIGUSR1] = "USR1";
	 signames[SIGQUIT] = "QUIT";
	 signames[SIGSEGV] = "SEGV";
	 signames[SIGHUP] = "HUP";
	 signames[SIGINT] = "INT";
	 signames[SIGPIPE] = "PIPE";
	 signames[SIGALRM] = "ALRM";
      }
} sgnm;

static PASSFAIL
com_sigsend(const char* arg)
{
   int s;

   if (!strncmp(arg,"SIG",3)) arg+=3;

   for (s=1; s<=NSIG; s++)
      if (sgnm.signames[s] && !strcmp(arg, sgnm.signames[s]))
      {
	 if (process)
	 {
	    scripter.cancel();
	    pendBuffer.cancel();
	    return (process->signal(s)==0?PASS:FAIL);
	 }
	 else if (roll->is_clone())
	 {
	    char cmd[80];

	    sprintf(cmd, "sigsend %s", arg);
	    commandPipe->send(cmd);
	    return PASS;
	 }
	 else
	 {
	    roll->add_line("warning: no process to signal");
	    return FAIL;
	 }
      }

   roll->add_line("error: unsupported signal", CFHT_WARN);
   return FAIL;
}

static PASSFAIL
com_ready(const char* arg)
{
   if (arg);
   if (process && process->commands_pending<0) return PASS;
   else return FAIL;
}

static PASSFAIL
com_running(const char* arg)
{
   if (arg);
   if (roll && roll->is_clone()>=0) return PASS;
   else return FAIL;
}

static PASSFAIL
com_autorestart(const char* arg)
{
   BOOLEAN f = cli_bool(arg);
   autorestart = cli_bool(arg)==TRUE?2*RESTART_THRESH:0;
   if (f==TRUE) roll->add_line("status: Autorestart is ON");
   else         roll->add_line("status: Autorestart is OFF");
   return PASS;
}

static PASSFAIL com_abort(const char* arg) { if (arg); return com_sigsend("SIGINT"); }
static PASSFAIL com_stop(const char* arg) { if (arg); return com_sigsend("SIGTSTP"); }
static PASSFAIL com_quit(const char* arg) { if (arg); return com_sigsend("SIGQUIT"); }
static PASSFAIL com_cancel(const char* arg) { if (arg); pendBuffer.cancel(); return PASS; }

static Command remote[] = {
   { "cancel",  com_cancel, "aborts queued but not yet executed commands" },
   { "abort",   com_abort,  "sends SIGINT to the current agent" },
   { "stop",    com_stop,   "sends SIGTSTP to the current agent" },
   { "pause",   com_stop,   "Synonym for stop" },
   { "quit",    com_quit,   "sends SIGQUIT to the current agent" },
   { "break",   com_quit,   "Synonym for quit" },
   { "sigsend", com_sigsend,"[NAME] sends SIGNAME to the current agent" },
   { "sendsig", com_sigsend,"Synonym for sigsend" },
   { "source",  com_source,  "[file] read batch commands from file" },
   { "agent",   com_agent,   "start|stop [host:]<agentname>" },
   { "ready",   com_ready,   "Silently returns pass or fail (for clicmd)"},
   { "running", com_running, "Silently returns pass or fail (for clicmd)"},
   { "say_",    com_say_,    "(send message to other users)" },
   { "builtin", com_builtin, "shows list of builtin commands" },
   { "license", com_license, "displays warranty and copying information" },
   { "termhelp",com_termhelp,"display terminal window tips" },
   { "autorestart", com_autorestart, "tells director whether or not to restart agents that die"},
   { 0, 0, 0 } /* This terminates the command list */
};

/* end of builtins.cc */
