/*                                             -*- c-file-style: "Ellemtel" -*-

`setserial.c' - See setserial.h

This file is part of version 2.12 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2000, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <termios.h>
#include <errno.h>
#if defined(HPUX) || defined(hpux)
#include <sys/termiox.h>
#endif

static int
gnu_cfmakeraw(struct termios* t)
{
   t->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP
		   |INLCR|IGNCR|ICRNL|IXON|IXOFF
#ifdef IXANY
		   |IXANY
#endif
      );
   t->c_oflag &= ~OPOST;
   t->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
   t->c_cflag &= ~(CSIZE|PARENB);
   t->c_cflag |= CS8|CREAD;
   t->c_cc[VMIN] = 1;
   t->c_cc[VTIME] = 0;
   return 0;
}

static void
gnu_cfsetspeed(struct termios* tio, speed_t s)
{
   cfsetispeed(tio, s);
   cfsetospeed(tio, s);
}

int
setserial(int fd, const char* options)
{
   int rtn;
   struct termios tio, tioorig;

   if ((rtn = tcgetattr(fd, &tioorig))<0) return rtn;

   tio = tioorig;
   gnu_cfmakeraw(&tio);

   /*
    * Turn off flow control by default
    */
#if defined(CCTS_OFLOW) && defined(CRTS_IFLOW)
   tio.c_cflag &= ~(CCTS_OFLOW|CRTS_IFLOW);
#elif defined(CRTSCTS)
   tio.c_cflag &= ~CRTSCTS;
#if defined(CRTSXOFF)
   tio.c_cflag &= ~CRTSXOFF;
#endif
#elif defined(TCGETX)
   {
      struct termiox tx;
      if (ioctl(fd, TCGETX, &tx)==0)
      {
	 tx.x_hflag &= ~(CTSXON|RTSXOFF);
	 ioctl(fd, TCSETX, &tx);
      }
   }
#else
#warn setserial does not know how to toggle hardware flow control on this system
#endif

   tio.c_cc[VSTOP] = 19; /* ^S */
   tio.c_cc[VSTART] = 17; /* ^Q */

   if (options)
   {
      char* p = strdup(options);
      char* next;
      
      if ((next = strchr(p, '}'))) *next = '\0';

      while (p && *p)
      {
	 if ((next = strchr(p, ','))) *next++ = '\0';

	 /*
	  * Hack to allow for "B9600" instead of "9600" etc.
	  */
	 if (*p=='B') p++;
	 else if (*p=='I') p++;
	 else if (*p=='C' && strncmp(p, "CTS", 3)) p++;

	 if (0) ;
#ifdef B50
	 else if (!strcmp(p, "50")) gnu_cfsetspeed(&tio, B50);
#endif
#ifdef B75
	 else if (!strcmp(p, "75")) gnu_cfsetspeed(&tio, B75);
#endif
#ifdef B110
	 else if (!strcmp(p, "110")) gnu_cfsetspeed(&tio, B110);
#endif
#ifdef B134
	 else if (!strcmp(p, "134")) gnu_cfsetspeed(&tio, B134);
#endif
#ifdef B150
	 else if (!strcmp(p, "150")) gnu_cfsetspeed(&tio, B150);
#endif
#ifdef B200
	 else if (!strcmp(p, "200")) gnu_cfsetspeed(&tio, B200);
#endif
	 else if (!strcmp(p, "300")) gnu_cfsetspeed(&tio, B300);
#ifdef B600
	 else if (!strcmp(p, "600")) gnu_cfsetspeed(&tio, B600);
#endif
#ifdef B900
	 else if (!strcmp(p, "900")) gnu_cfsetspeed(&tio, B900);
#endif
	 else if (!strcmp(p, "1200")) gnu_cfsetspeed(&tio, B1200);
	 else if (!strcmp(p, "1800")) gnu_cfsetspeed(&tio, B1800);
	 else if (!strcmp(p, "2400")) gnu_cfsetspeed(&tio, B2400);
#ifdef B3600
	 else if (!strcmp(p, "3600")) gnu_cfsetspeed(&tio, B3600);
#endif
	 else if (!strcmp(p, "4800")) gnu_cfsetspeed(&tio, B4800);
#ifdef B7200
	 else if (!strcmp(p, "7200")) gnu_cfsetspeed(&tio, B7200);
#endif
	 else if (!strcmp(p, "9600")) gnu_cfsetspeed(&tio, B9600);
	 else if (!strcmp(p, "19200")) gnu_cfsetspeed(&tio, B19200);
	 else if (!strcmp(p, "38400")) gnu_cfsetspeed(&tio, B38400);
#ifdef B57600
	 else if (!strcmp(p, "57600")) gnu_cfsetspeed(&tio, B57600);
#endif
#ifdef B115200
	 else if (!strcmp(p, "115200")) gnu_cfsetspeed(&tio, B115200);
#endif
#ifdef B230400
	 else if (!strcmp(p, "230400")) gnu_cfsetspeed(&tio, B230400);
#endif
#ifdef B460800
	 else if (!strcmp(p, "460800")) gnu_cfsetspeed(&tio, B460800);
#endif
	 else if (!strcmp(p, "N81"))
	 {
	    tio.c_cflag &= ~CSIZE;
	    tio.c_cflag |= CS8;
	    tio.c_cflag &= ~PARENB;
	    tio.c_cflag &= ~CSTOPB;
	 }
	 else if (!strcmp(p, "N72"))
	 {
	    tio.c_cflag &= ~CSIZE;
	    tio.c_cflag |= CS7;
	    tio.c_cflag &= ~PARENB;
	    tio.c_cflag |= CSTOPB;
	 }
	 else if (!strcmp(p, "E71"))
	 {
	    tio.c_cflag &= ~CSIZE;
	    tio.c_cflag |= CS7;
	    tio.c_cflag |= PARENB;
	    tio.c_cflag &= ~PARODD;
	    tio.c_cflag &= ~CSTOPB;
	 }
	 else if (!strcmp(p, "O71"))
	 {
	    tio.c_cflag &= ~CSIZE;
	    tio.c_cflag |= CS7;
	    tio.c_cflag |= PARENB;
	    tio.c_cflag |= PARODD;
	    tio.c_cflag &= ~CSTOPB;
	 }
	 else if (!strcmp(p, "LOCAL")) tio.c_cflag |= CLOCAL;
	 else if (!strcmp(p, "-LOCAL") ||
		  !strcmp(p, "-CLOCAL")) tio.c_cflag &= ~CLOCAL;
	 else if (!strcmp(p, "SOFTFLOW"))
	 {
	    tio.c_iflag |= IXON|IXOFF;
	 }
	 else if (!strcmp(p, "XON"))
	 {
	    tio.c_iflag |= IXON;
	 }
	 else if (!strcmp(p, "XANY"))
	 {
	    tio.c_iflag |= IXON;
#ifdef IXANY
	    tio.c_iflag |= IXANY;
#endif
	 }
	 else if (!strcmp(p, "XOFF"))
	 {
	    tio.c_iflag |= IXOFF;
	 }
	 else if (!strcmp(p, "HARDFLOW"))
	 {
#if defined(CCTS_OFLOW) && defined(CRTS_IFLOW)
	    tio.c_cflag |= (CCTS_OFLOW|CRTS_IFLOW);
#elif defined(CRTSCTS)
	    tio.c_cflag |= CRTSCTS;
#if defined(CRTSXOFF)
	    tio.c_cflag |= CRTSXOFF;
#endif
#elif defined(TCGETX)
	    {
	       struct termiox tx;
	       if (ioctl(fd, TCGETX, &tx)==0)
	       {
		  tx.x_hflag |= (CTSXON|RTSXOFF);
		  ioctl(fd, TCSETX, &tx);
	       }
	    }
#endif
	 }
	 else if (!strcmp(p, "CTS_OFLOW") ||
		  !strcmp(p, "CTSXON") ||
		  !strcmp(p, "RTSCTS"))
	 {
#if defined(CCTS_OFLOW)
	    tio.c_cflag |= (CCTS_OFLOW);
#elif defined(CRTSCTS)
	    tio.c_cflag |= CRTSCTS;
#elif defined(TCGETX)
	    {
	       struct termiox tx;
	       if (ioctl(fd, TCGETX, &tx)==0)
	       {
		  tx.x_hflag |= CTSXON;
		  ioctl(fd, TCSETX, &tx);
	       }
	    }
#endif
	 }
	 else if (!strcmp(p, "RTS_IFLOW") ||
		  !strcmp(p, "RTSXOFF"))
	 {
#if defined(CCTS_OFLOW)
	    tio.c_cflag |= (CRTS_IFLOW);
#elif defined(CRTSXOFF)
	    tio.c_cflag |= CRTSXOFF;
#elif defined(CRTSCTS)
	    tio.c_cflag |= CRTSCTS;
#elif defined(TCGETX)
	    {
	       struct termiox tx;
	       if (ioctl(fd, TCGETX, &tx)==0)
	       {
		  tx.x_hflag |= RTSXOFF;
		  ioctl(fd, TCSETX, &tx);
	       }
	    }
#endif
	 }
	 else rtn = -1;
	 p = next;
      }
      free(p);
   }

   if (rtn==-1)
   {
      /*
       * At least one of the settings was invalid, but try to
       * set the rest and then return EINVAL.
       */
      rtn = tcsetattr(fd, TCSAFLUSH, &tio);
      if (rtn < 0) return rtn; else
      {
	 errno = EINVAL;
	 return -1;
      }
   }
   else return tcsetattr(fd, TCSAFLUSH, &tio);
}
