/*                                             -*- c-file-style: "Ellemtel" -*-

`CliQueue.cc' - See CliQueue.hh

This file is part of version 3 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */


#include "CliQueue.hh"

#include "cli/cli.h"
#include "cli/clidefs.h"
#include "Roll.hh"
extern Roll* roll;

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <pwd.h>
#include <sys/types.h>
#include <dirent.h> /* %%% Goes away with scripter agent */
#include <sys/stat.h> /* %%% " */
#include <unistd.h>   /* %%% " */

#include "Pipes.hh"

#include "director.hh"

static const char*
do_tilde_substitution(const char* orig)
{
   static char buf[1024];
   char user[10];
   char* userp = user;
   char* bufp = buf;
   const char* origp = orig;
   size_t len, ulen = 0, slen = 0;
   struct passwd* pwd;

   while (1)
   {
      const char* subst;

      /*
       * Copy one character at a time until a ~ after a space is found
       */
      while (*origp &&
	     (*origp!='~' || origp==orig || *(origp-1)>' ') &&
	     slen < sizeof(buf)-1)
      { *bufp++ = *origp++; slen++; }

      if (*origp++ != '~') break; /* end of string or buf full */

      /*
       * Get what could be a possible username
       */
      for (userp = user; 
	   *origp > ' ' && *origp != '/' && ulen<9;
	   *userp++ = *origp++, ulen++);
      *userp = '\0'; /* terminate username string */

      if (ulen==0)
	 subst = cli_gethome();	// empty means use default username
      else if (ulen<9 && (pwd=getpwnam(user)))
	 subst = pwd->pw_dir;	// found in passwd table, substitute
      else
      { *bufp++ = '~'; slen++; subst = user; } // otherwise leave alone

      slen += (len = strlen(subst));
      if (slen > sizeof(buf)-1) break; /* overflow */
      strcpy(bufp, subst);
      bufp += len;
   }
   *bufp = '\0'; /* terminate the string */
   return buf; /* return pointer to static area */
}

CliQueueNode::CliQueueNode(const char* newline, int level, pid_t pid)
{
   no_wait_ = no_fail_ = no_echo_ = shell_escape_ = script_dir_ = 0;
   output_ = OUTPUT_DIRECTOR; /* By default, send output only the to the feedback window */
   if (!newline) newline = "# NULL passed to CliQueueNode";
   while (*newline && (strchr(CHARS_SPECIAL" \t", *newline) ||
		       !strncmp(newline, DIRECTOR_CLICMD, DIRECTOR_CLICMD_LEN)))
      /* === The way this is being detected is kind of hack-like
       * and doesn't work if there's a valid command called "clicmdsomething"
       */
   {
      switch (*newline++)
      {
	 case CHAR_NO_WAIT:	no_wait_ = 1; break;
	 case CHAR_NO_ECHO:	no_echo_ = 1; break;
	 case CHAR_NO_FAIL:	no_fail_ = 1; break;
	 case CHAR_SYSTEM_ESC:	shell_escape_ = 1; break;
	 case ' ':
	 case '\t': break;
	 default:
	 if (!strncmp(newline-1, DIRECTOR_CLICMD, DIRECTOR_CLICMD_LEN))
	 {
	    newline += DIRECTOR_CLICMD_LEN - 1;
	 }
      }
   }

   /*
    * cli_arg1() could be strdup() instead below, but that wouldn't
    * handle a line in a startup file correctly that says:
    *
    *   clicmd "agent start directest"
    *
    * The code above removes the 'clicmd ' from the string, but
    * the quotes still need to be removed so director doesn't
    * search for a command called '"agent'.
    */
   line_ = line_orig_ = cli_arg1(do_tilde_substitution(newline));
   handle_ = "";
   /*
    * Check if an agent handle was given on the command line
    */
   {
      char* p = line_;
      int len;

      while (*p && *p!='.' && !isspace(*p)) p++;
      if (*p=='.')
      {
	 *p='\0';
	 handle_ = line_;
	 line_ = p+1;
      }

      p = line_;
      while (*p && !isspace(*p)) p++;
      len = p - line_;
      command_ = (char*)malloc(len + 1);
      if (len) strncpy(command_, line_, len);
      command_[len] = '\0';
   }

   if (!shell_escape_ &&
       !strcmp(command_, DIRECTOR_SOURCE))
      source_=SOURCE_NEED_TO_LOAD;
   else
      source_=SOURCE_NONE;

   pid_ = pid;
   level_ = level;
   result_ = no_fail_?PASS:FAIL; /* until we're told otherwise */
   prev_ = next_ = next_stack_ = prev_stack_ = 0;
   exit_message_ = 0;
   process_ = 0;

   if (pid)
   {
      char tmp[600];

      sprintf(tmp, "%.500s/"CAPPIPE, cli_gethome(), (int)pid);
      fd_capture_ = open(tmp, O_WRONLY|O_NONBLOCK);

      if (fd_capture_==-1)
      {
	 sprintf(tmp, "%.500s/"DUPPIPE, cli_gethome(), (int)pid);
	 fd_capture_ = open(tmp, O_WRONLY|O_NONBLOCK);
	 if (fd_capture_!=-1) output_ = OUTPUT_DUP;
      }
      else
      {
	 output_ = OUTPUT_CAP;
      }
   }
   else
   {
      fd_capture_ = -1;
   }
}

CliQueueNode::~CliQueueNode()
{
   static char junkbuf[1024];

   fd_capture_close();

   if (pid_ > 0)
   {
      if (kill(pid_, result_==PASS?SIGUSR1:SIGUSR2)==-1)
      {
	 sprintf(junkbuf,
		 "%s: Can't signal pid %d to tell it command `%s' %s.  (error was: `%s')",
		 errno==ESRCH?"logonly":"error",
		 (int)pid_, line_, result_==PASS?"passed":"failed", strerror(errno));
	 if (roll) roll->add_line(junkbuf);
#ifndef NO_CFHTLOG
	 else cfht_log(CFHT_MAIN, CFHT_ERROR, junkbuf);
#endif
      }
   }
   /* line_ just points into line_orig_, so it shouldn't be freed! */
   if (line_orig_) { free(line_orig_); line_orig_ = 0; }
   if (command_) { free(command_); command_ = 0; }
		    
   if (exit_message_)
   {
      if (roll) roll->add_line(exit_message_);
      free(exit_message_);
      exit_message_=0;
   }
   if (process_)
   {
      process_->remove_command();
      process_ = 0;
   }
}

void
CliQueueNode::set_exit_message(const char* exit_message)
{
   if (exit_message_) free(exit_message_);
   exit_message_ = strdup(exit_message);
}

void
CliQueue::add_line(CliQueueNode* newNode)
{
   /*
    * Setting level to 99999 would just cause this command to be inserted into
    * a level all by itself, at the top of the stack.
    */
   if (newNode->level_ > level_) newNode->level_ = ++level_;

   if (!head)
   {
      /*
       * If the entire queue is currently empty, add the new command
       * as the only node at level 0, regardless of the value of `level'
       */
      head = newNode;
      level_ = 0;
      return;
   }

   CliQueueNode* this_stack_level = head;
   int level = newNode->level_;

   while (level && this_stack_level->next_stack_)
   {
      /*
       * Try to find the appropriate place for this `level'
       */
      level--;
      this_stack_level = this_stack_level->next_stack_;
   }

   if (!level)
   {
      /*
       * A queue at the apropriate level already exists.  Find
       * the end of the line and add the new node.
       */
      while (this_stack_level->next_)
	 this_stack_level = this_stack_level->next_;
      this_stack_level->next_ = newNode;
      newNode->prev_ = this_stack_level;
   }
   else
   {
      /*
       * Nothing currently exists at this level, so create
       * a new level off of the deepest existing level.
       */
      this_stack_level->next_stack_ = newNode;
      newNode->prev_stack_ = this_stack_level;
   }
}

CliQueueNode*
CliQueue::check_line()
{
   CliQueueNode* p = head;

   if (!p)
   {
      if (level_ != -1)
      {
	 /* should never happen */
	 roll->add_line("warning: correcting internal command queue corruption");
	 level_ = -1;
      }
      return 0; 
   }
   while (p->next_stack_) p = p->next_stack_;

   /*
    * %%% Hack: Check the ~/.director/bin/ directory for scripts
    * which need to be run or sourced.  In director-4, agents will
    * be able to register their command set with director, and should
    * have a priority level.  In that version of director, we should
    * re-implement this using a special "scripter" agent, with a
    * high priority, that registers whatever it finds in
    * ~/.director/bin/ (or other configurable paths) and remove the
    * functionality from director.  This agent will have its own way
    * of figuring out when it needs to rescan the directory for new
    * scripts or have its own equivalent of a "rehash" command (maybe
    * just when it is restarted by director) but at least all of that
    * gross-ness will not need to be in director anymore.
    */
   if (p->source_ == SOURCE_NONE && !strcmp(p->handle(), ""))
   {
      static time_t time_rehash = 0;
      static char* script[100]; /* List of executables in the SCRIPTDIR */
      static char* source[100]; /* List of director "source" files in SCRIPTDIR */
      static int scripts = 0;
      static int sources = 0;
      struct dirent* entry;
      time_t time_now = time(0);
      int i;

      /*
       * Rescan the directory if more than 30 seconds have elapsed.
       */
      if (time_now > time_rehash + 30)
      {
	 char junkbuf[1024];

	 time_rehash = time_now;

	 /*
	  * Free the old script and source lists.
	  */
	 while (scripts) free(script[--scripts]);
	 while (sources) free(source[--sources]);

	 /*
	  * Open and read the script directory.
	  */
	 sprintf(junkbuf, "%.500s/"SCRIPTDIR, cli_gethome());
	 DIR* dir = opendir(junkbuf);
	 if (dir)
	 {
	    const char* savedir = cli_getcwd();
	    chdir(junkbuf);

	    while ((entry = readdir(dir)) != 0)
	    {
	       if (strcmp(entry->d_name, ".") &&
		   strcmp(entry->d_name, ".."))
	       {
		  struct stat stbuf;

		  if (stat(entry->d_name, &stbuf)==0)
		  {
		     if (!S_ISFIFO(stbuf.st_mode) &&
			 !S_ISDIR(stbuf.st_mode) &&
			 (stbuf.st_mode & S_IRUSR))
		     {
			if (stbuf.st_mode & S_IXUSR)
			{
			   script[scripts++] = strdup(entry->d_name);
			}
			else
			{
			   source[sources++] = strdup(entry->d_name);
			}
		     }
		     else if (roll && director_debug)
		     {
			sprintf(junkbuf, "debug: Ignoring strange file `%.500s'", entry->d_name);
			roll->add_line(junkbuf);
		     }
		  }
		  else
		  {
		     if (roll)
		     {
			sprintf(junkbuf, "logonly: Can't stat `%.500s'", entry->d_name); /* === perror? */
			roll->add_line(junkbuf);
		     }
		     /*
		      * Add it anyway.  This error typically occurs when a
		      * file in the script directory is a sym-link to a script
		      * on another filesystem that is temporarily having
		      * (NFS) problems.  If we don't add it to scripts here,
		      * it ends up being sent to the agent, resulting in
		      * non-sense error messages.
		      */
		     script[scripts++] = strdup(entry->d_name);
		  }
	       }
	    }
	    chdir(savedir);
	    closedir(dir);
	 }
      }

      for (i=0; i<scripts; i++)
      {
	 if (!strcmp(p->command(), script[i]))
	 {
	    p->shell_escape_ = 1; /* === Change all these to bool? */
	    p->script_dir_ = 1;
	 }
      }
      for (i=0; i<sources; i++)
      {
	 if (!strcmp(p->command(), source[i]))
	 {
	    p->source_ = SOURCE_NEED_TO_LOAD;
	    p->script_dir_ = 1;
	 }
      }
   }

   switch (p->source_)
   {
      case SOURCE_NEED_TO_LOAD:
      {
	 p->source_ = SOURCE_PASS;
	 builtin_source(p);
	 return check_line();
	 break;
      }
      case SOURCE_FAIL:
      {
	 if (!p->no_fail()) {
	    if (roll) roll->add_line("logonly: Canceling command queue (SOURCE_FAIL)");
	    cancel(p);
	 }
	 else               remove(p);
	 return check_line();
	 break;
      }
      case SOURCE_PASS:
      {
	 remove(p);
	 return check_line();
	 break;
      }
      case SOURCE_NONE:
      {
	 if (p->process_!=0)
	 {
	    /*
	     * If the current line has already been assigned to a process,
	     * check now to see if the process has reported completion yet.
	     */
	    PASSFAILBUSY result = p->process_->pend();
	    
	    if (result==BUSY) return 0;
	    roll->instant_off();
	    if (result==PASS) p->pass();
	    if (result!=PASS && !p->no_fail())
	    {
	       if (roll) roll->add_line("logonly: Canceling command queue (SOURCE_NONE)");
	       cancel(p);
	    }
	    else
	       remove(p);
	    return check_line(); /* get a new command off the stack */
	 }
      }
   }


   return p;
}

void
CliQueue::remove(CliQueueNode* p)
{
   if (!p) return;

   /*
    * This could possibly happen if a shell script continues
    * to fire off sub-level commands in the background after
    * the main parent has already exited.
    */
   if (p->next_stack_)
   {
      if (roll)
	 roll->add_line("warning: command queue corrupt; trashing dangling commands...");
      p->next_stack_->prev_stack_ = 0;
      cancel(p->next_stack_);
   }

   /*
    * This should never happen (if there were previous commands,
    * they should have been executed before this one.)  Try to
    * recover anyway, but things are likely to fail if this situation
    * ever occurs
    */
   if (p->prev_)
   {
      if (roll)
	 roll->add_line("warning: command queue corrupt; trashing skipped commands...");
      remove(p->prev_);
   }

   /*
    * Finally, the normal cases:
    */

   if (p==head) head = p->next_;

   /*
    * Remove this node from the stack's links
    */
   if (p->prev_stack_) p->prev_stack_->next_stack_ = p->next_;

   if (p->next_)
   {
      /*
       * Remove this node from the queue's links
       */
      p->next_->prev_ = 0;
      p->next_->prev_stack_ = p->prev_stack_;
   }
   else
   {
      level_--;
   }

   delete p;
   if (head==0) check_line(); /* only to check for queue corruption */
}

int
CliQueue::cancel(CliQueueNode* p)
{
   int count = 0;
   CliQueueNode* next = 0;
   static char junkbuf[1024];

   if (!p) p = head; /* cancel everything if no specific node given */
   if (!p) return 0; /* nothing left */
   if (p->next_stack_) count += cancel(p->next_stack_);

   if (!p->no_echo() && roll && !p->process_ && p->source_==SOURCE_NONE)
   {
      /*
       * If this line had not yet been assigned to any process, it
       * means the user hasn't seen the command echo on the screen
       * yet, so print it now and indicate that it never executed.
       */
      sprintf(junkbuf, "> %.500s    *** canceled ***", p->line());
      roll->add_line(junkbuf, CLI_LOG_PROMPT);
   }

   next = p->next_;
   remove(p);
   count++;

   if (next) count += cancel(next);

   return count;
}

void
CliQueue::dump(int logtype, CliQueueNode* p, int indent)
{
   static char junkbuf[1024];
   static char junkbuf2[1024];

   if (!p)
   {
      if (head)
      {
	 roll->add_line("# CliQueue:", logtype);
	 p = head;
      }
      else
      {
	 roll->add_line("# CliQueue: Empty", logtype);
	 return;
      }
   }

   sprintf(junkbuf, "#   {%06d.%d}%*s%s`%s%s%.100s'",
	   (int)p->pid_, p->level_, indent*4, "", 
	   p->process_?" P-> ":
	   ((p->source_&&(p->next_stack_->source_!=SOURCE_NONE))?" S-> ":"     "),
	   p->no_echo_?"@":"", p->no_fail_?"-":"",
	   p->line_?p->line_:"[INCONSISTENT: deleted!]");
   if (p->exit_message_) strcat(junkbuf, " [has exit message]");
   if (p->next_stack_==0 && p->prev_==0)
   {
      if (!p->process_) strcat(junkbuf," [next-up for processing]");
      else
      {
	 if (*(p->process_->handle())=='(') /* === better way to flag this */
	    sprintf(junkbuf2, " [waiting on PID %d]",
		    (int)p->process_->childpid());
	 else
	    sprintf(junkbuf2, " [waiting on agent `%s']",
		    p->process_->handle());
	 strcat(junkbuf, junkbuf2);
      }
   }
   roll->add_line(junkbuf, logtype);

   if (p->next_stack_) 
   {
      if (p->next_stack_->prev_stack_ != p)
	 roll->add_line("#   [INCONSISTENT: next_stack_->prev_stack_ broken]", logtype);
      dump(logtype, p->next_stack_, indent+1);
   }
   if (p->next_)
   {
      if (p->next_->prev_ != p)
	 roll->add_line("#   [INCONSISTENT: next_->prev_ broken]", logtype);
      dump(logtype, p->next_, indent);
   }
}


void
CliQueue::builtin_source(CliQueueNode* p)
{
   FILE* scriptFile;
   int lineno = 0;
   const char* fn = p->line();
   char* filename;
   char scriptLine[1024];
   char junkbuf[1024]; /* === merge these scratch buffers? */
   
   if (!strncmp(fn, DIRECTOR_SOURCE, DIRECTOR_SOURCE_LEN))
      fn += DIRECTOR_SOURCE_LEN;
   while (*fn==' ' || *fn=='\r') fn++;
   filename = cli_arg1(fn);

   if (p->script_dir_)
   {
      char* tmp = filename;
      filename = (char*)malloc(strlen(cli_gethome()) + strlen(SCRIPTDIR) + strlen(tmp) + 3);
      sprintf(filename, "%s/"SCRIPTDIR"/%s", cli_gethome(), tmp);
      free(tmp);
   }

   if ((scriptFile = fopen(filename, "r"))==0)
   {
      sprintf(junkbuf,
	      "error: source[%d]: cannot run `%.500s' -- %.100s",
	      p->level()+1, filename, strerror(errno));
      if (roll) roll->add_line(junkbuf);
   }
   else
   {
      if (roll && !p->no_echo())
      {
	 sprintf(junkbuf,
		 "status: source[%d]: Now executing commands from `%s'",
		 p->level()+1, filename);
	 roll->add_line(junkbuf);
	 sprintf(junkbuf,
		 "status: source[%d]: Leaving `%s'",
		 p->level()+1, filename);
	 p->set_exit_message(junkbuf);
      }
      
      while (fgets(scriptLine, sizeof(scriptLine), scriptFile))
      {
	 /*
	  * Successfully read a line from the file.
	  */
	 char* last;
	 char* first = scriptLine;
	 
	 lineno++;
	 
	 while (*first && *first<=' ') first++;
	 last = first + strlen(first) - 1;
	 while (last >= first && *last<=' ') *last-- = '\0';
	 if (*first && *first!='#')
	    add_line(new CliQueueNode(first, p->level()+1));
      }
      
      fclose(scriptFile);
   }
   free(filename);
}

char
CliQueue::state()
{
   if (roll->is_clone()==1)
   {
      return roll->get_prompt_state();
   }
   else
   {
      static char last_rtn = '\0';
      char rtn = ':';
      CliQueueNode* p = head;
      
      if (!p) rtn = '>';
      else while (p)
      {
	 if (p->source_!=SOURCE_NONE && p->next_stack_!=NULL)
	 { rtn = 'S'; break; }
	 else if (p->process_ && p->process_->pend()==BUSY)
	 { rtn = '*'; break; }
	 p = p->next_stack_;
      }
      if (rtn != last_rtn)
      {
	 roll->set_prompt_state(rtn);
	 last_rtn = rtn;
      }
      return rtn;
   }
}
