/*                                             -*- c-file-style: "Ellemtel" -*-

`Curs.cc' - See Curs.hh

This file is part of version 3 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>

/*
 * === Fix this so we use #define _XOPEN_SOURCE instead.
 * But then we need to get rid of strdups?
 */
#ifdef LINUX
extern "C" {
   char *crypt(const char *key, const char *salt);
}
#endif

#include "cli/cli.h"

#include "director.hh"

#include "Curs.hh"

#include "Roll.hh"
extern Roll* roll; /* === added just for scrollback */

#define CONTROL(c) (1-'A'+c)
#define ESCAPE (27)

#include "Curs_dialog.c"

class History
/*
 * Keeps track of command lines as they are entered so the
 * user can scroll back and edit previous commands.  The size
 * of the history (number of commands it can hold before it
 * starts throwing away the oldest ones) is set at compile-time
 * with the define below.  The lines themselves, however, are
 * copied with strdup and can be arbitrarily long.
 */
{
   private:
      char histname[1024];
      char* hist[HISTORYSIZE];
      int hist_pos; // the current command the user is editing
      int hist_end; // last command (the one to jump to with ENTER)
      int clone_;

      void del(int pos) {
	 if (hist[pos]!=0) { free(hist[pos]); hist[pos]=0; } /* %%% core dumped here with -p? */
      }

   public:
      History() {
	 sprintf(histname, "%.800s/"HISTORYFILE, cli_gethome());

	 FILE* histfile = fopen(histname, "rb");
	 char histline[1024];

	 clone_ = 0;
	 hist_pos = 0;
	 hist_end = 0;
	 for (int i=0; i<HISTORYSIZE; i++) hist[i]=0;

	 if (histfile==0) return;

	 /*
	  * Re-read old history file if there was one found.
	  */
	 while (fgets(histline, 1023, histfile))
	 {
	    int len = strlen(histline);
	    if (len>1)
	    {
	       histline[len-1]='\0';
	       add(histline);
	    }
	 }
	 fclose(histfile);
      }

      ~History() {
	 int i;
	 FILE* histfile;

	 if (clone_) return;

	 /*
	  * If can't open history file, silently ignore it.
	  */
	 if ((histfile = fopen(histname, "wb")) == 0)
	 {
	    perror(histname);
	    return;
	 }
	 
	 /*
	  * Otherwise, write entire history to HISTORYFILE
	  */
	 for (i=(hist_end+1)%HISTORYSIZE; i!=hist_end; i=(i+1)%HISTORYSIZE)
	    if (hist[i])
	    { fputs(hist[i], histfile); fputc('\n', histfile); }
	 fclose(histfile);
	 for (i=0; i<HISTORYSIZE; i++) del(i);
      }

      void clone(int onoff=1) { clone_ = onoff; }

      void save(const char* s) {
	 char* saved_s;
	 char* end_s;
	 /*
	  * This is called whenever the user moves off of a line
	  * without hitting enter.  Just update the current history
	  * entry so that whatever edits they had made as still
	  * there if they scroll back.
	  */

	 while (*s==' ') s++; /* trim white space from front */

	 saved_s = strdup(s); /* make a copy of the string */
	 end_s = saved_s + strlen(saved_s)-1;
	 while (end_s >= saved_s && *end_s==' ')
	    *end_s--='\0'; /* trim white space from end */
	 
	 del(hist_pos);
	 hist[hist_pos] = saved_s;
      }
      
      const char* add(const char* s) {
	 /*
	  * This is called when the user hits enter.  The return
	  * value is a pointer to the copied version of the command
	  * and the history position is moved to the end of the
	  * list.
	  */
	 int hist_prev = (hist_end + HISTORYSIZE - 1) % HISTORYSIZE;
	 int hist_next = (hist_end + 1) % HISTORYSIZE;
	 char* saved_s;
	 char* end_s;

	 hist_pos = hist_end;

	 if (!s) return "";

	 while (*s==' ') s++; /* trim white space from front */
	 if (!*s) return "";

	 saved_s = strdup(s); /* make a copy of the string */
	 end_s = saved_s + strlen(saved_s)-1;
	 while (end_s >= saved_s && *end_s==' ')
	    *end_s--='\0'; /* trim white space from end */
	 
	 if (hist[hist_prev]!=0 &&
	     strcmp(hist[hist_prev], saved_s)==0)
	 {
	    /*
	     * don't save duplicate commands in the history
	     */
	    free(saved_s);
	    return hist[hist_prev];
	 }

	 del(hist_pos);
	 
	 hist[hist_pos] = saved_s;
	 hist_pos = hist_end = hist_next;
	 return saved_s;
      }
      
      const char* prev() {
	 /*
	  * Go to previous history item.  Returns a pointer
	  * to the saved string, or 0 if already at the top.
	  */
	 int hist_prev;
	 
	 hist_prev = (hist_pos + HISTORYSIZE - 1) % HISTORYSIZE;
	 if (hist_prev == hist_end) return 0;
	 if (hist[hist_prev]==0) return 0;
	 else return hist[hist_pos=hist_prev];
      }
      
      const char* next() {
	 /*
	  * Go to the next history item.  Returns a pointer
	  * the the save string, or 0 if already at the bottom.
	  */
	 int hist_next;
	 
	 if (hist_pos == hist_end) return 0;
	 hist_next = (hist_pos + 1) % HISTORYSIZE;
	 if (hist[hist_next]==0) return 0;
	 else return hist[hist_pos=hist_next];
      }
} history;

/*
 * To avoid problems when resizing the window, ignore
 * re-size events that are too small, as defined here:
 */
#define MIN_COLS 10
#define MIN_LINES 3

/*
 * First, some simple macro definitions to check for
 * proper return codes from various curses and form
 * library calls.  These assume that cfhtlog and and
 * feedback window are both ready.
 */

#ifdef NO_CFHTLOG

/*
 * These are error/warning handlers for failures of internal curses
 * calls.  Corrupted termcaps, buggy code, or memory allocation probs
 * might cause these, but I've never seen them pop up yet.
 */

#define want_E_OK(fn, desc) {\
  int r;\
  if ((r=(fn))!=E_OK) {\
    wprintw(feedback, "warning: %s failed (return=%d)\n", desc, r);\
  }\
}

#define want_not_NULL(fn, desc) {\
  if ((fn)==NULL) {\
    wprintw(feedback, "%s failed", desc);\
  }\
}

#define want_not_ERR(fn, desc) {\
  if ((int)(fn)==ERR) {\
    wprintw(feedback, "error: %s failed", desc);\
  }\
}

#else

static int _cr_errors=20;
/*
 * use this counter to prevent flooding the logs with errors
 * related only to curses failures.  Print only the first 20.
 */

/*
 * Same as above, but versions that also log a message to cfht_log.
 */
#define want_E_OK(fn, desc) {\
  int r;\
  if ((r=(fn))!=E_OK) {\
    if (_cr_errors && _cr_errors--)\
      cfht_logv(CFHT_MAIN, CFHT_WARN, "%s failed (return=%d)", desc, r);\
    wprintw(feedback, "warning: %s failed (return=%d)\n", desc, r);\
  }\
}

#define want_not_NULL(fn, desc) {\
  if ((fn)==NULL) {\
    if (_cr_errors && _cr_errors--)\
      cfht_logv(CFHT_MAIN, CFHT_WARN, "%s failed", desc);\
    wprintw(feedback, "%s failed", desc);\
  }\
}

#define want_not_ERR(fn, desc) {\
  if ((int)(fn)==ERR) {\
    if (_cr_errors && _cr_errors--)\
      cfht_logv(CFHT_MAIN, CFHT_WARN, "%s failed", desc);\
    wprintw(feedback, "error: %s failed", desc);\
  }\
}

#endif

/*
 * These could be defined like the above except they would print
 * errors instead of warnings, to distinguish between bad problems
 * and _really_ bad problems.  Right now, they are the same though.
 */
#define need_E_OK     want_E_OK
#define need_not_NULL want_not_NULL

/*
 * If the curses library would detect EPIPE correctly, then this
 * wrapper for wnoutrefresh might actually cause director to clean
 * up properly when a window is closed.  It seems, however, that
 * the real place the EPIPE is detected is in the fputs() for the
 * titlebar.  If nothing is changing in the titlebar, there could
 * be problems!
 */
static void
refresh_or_hup(WINDOW* win)
{
   if (wnoutrefresh(win) == ERR)
      raise(SIGHUP);
}

/*
 * These colors are out of order to try and help hpterms.  On
 * xterms, it doesn't matter, but on hpterms ncurses seems to
 * have trouble setting the colors.
 */
typedef enum { COLOR_NORM=6, 
	       COLOR_INFO=2,
	       COLOR_ERROR=3,
	       COLOR_STATUS=4,
	       COLOR_STATUSBAR=5,
	       COLOR_VERBOSE=1,
	       COLOR_WARN=7,
	       COLOR_HIGHLIGHT=8,
	       COLOR_BETA=9,
	       COLOR_PROMPT=10,
	       COLOR_SHADEIN=11,
	       COLOR_SHADEOUT=12,
	       COLOR_SHADOW=13,
	       COLOR_COMPLETE=14,
	       COLOR_STATUSGAP=15,
	       COLOR_HIDDEN=16,
	       COLOR_SHADEIN_YELLOW=17,
	       COLOR_SHADEOUT_YELLOW=18,
	       COLOR_COMPLETE_YELLOW=19,
	       COLOR_COMPLETE_RED=20 } color_t;

#define ATTR_NORM (COLOR_PAIR(COLOR_NORM)|A_NORMAL)
#define ATTR_INFO (COLOR_PAIR(COLOR_INFO)|A_STANDOUT)
#define ATTR_ERROR (COLOR_PAIR(COLOR_ERROR)|A_REVERSE)
#define ATTR_STATUS (COLOR_PAIR(COLOR_STATUS)|A_BOLD)
#define ATTR_STATUSBAR_MONO (COLOR_PAIR(COLOR_STATUSBAR)|A_STANDOUT)
#define ATTR_STATUSBAR (COLOR_PAIR(COLOR_STATUSBAR)|A_BOLD)
#define ATTR_STATUSGAP (COLOR_PAIR(COLOR_STATUSGAP))
#define ATTR_VERBOSE (COLOR_PAIR(COLOR_VERBOSE))
#define ATTR_WARNING (COLOR_PAIR(COLOR_WARN)|A_BOLD)
#define ATTR_PROMPT (COLOR_PAIR(COLOR_PROMPT)|A_BOLD)
#define ATTR_HIDDEN (COLOR_PAIR(COLOR_HIDDEN))
#define ATTR_SHADEIN_YELLOW (COLOR_PAIR(COLOR_SHADEIN_YELLOW)|A_BOLD|A_STANDOUT)
#define ATTR_SHADEOUT_YELLOW (COLOR_PAIR(COLOR_SHADEOUT_YELLOW)|A_BOLD|A_STANDOUT)
#define ATTR_SHADEIN (COLOR_PAIR(COLOR_SHADEIN)|A_BOLD)
#define ATTR_SHADEOUT (COLOR_PAIR(COLOR_SHADEOUT)|A_BOLD)
#define ATTR_SHADOW (COLOR_PAIR(COLOR_SHADOW)|A_BOLD)
#define ATTR_BETA  (COLOR_PAIR(COLOR_BETA)|A_STANDOUT)
#define ATTR_HIGHLIGHT (COLOR_PAIR(COLOR_INFO)|A_STANDOUT|A_BOLD)
#define ATTR_COMPLETE_YELLOW (COLOR_PAIR(COLOR_COMPLETE_YELLOW)|A_STANDOUT)
#define ATTR_EMPTY_YELLOW (COLOR_PAIR(COLOR_COMPLETE_YELLOW)|A_BOLD)
#define ATTR_COMPLETE_RED (COLOR_PAIR(COLOR_COMPLETE_RED)|A_REVERSE)
#define ATTR_EMPTY_RED (COLOR_PAIR(COLOR_COMPLETE_RED))
#define ATTR_COMPLETE (COLOR_PAIR(COLOR_COMPLETE)|A_REVERSE)
#define ATTR_EMPTY (COLOR_PAIR(COLOR_COMPLETE))

Curs::Curs(int infolines, int keep_bgcolor)
{
   const char* term;
   /* 
    * This gets run at the point the variable is declared.
    */

   /*
    * Set everything to zero
    */
   feedback = popup = statbar = prompt = info = 0;
   form = 0;
   roll_ofs = 0;
   field[0] = field[1] = 0;
   statbar_showing_ = 0;
   prompt_showing_ = 1;
   infolines_ = infolines;
   keep_bgcolor_ = keep_bgcolor;
   pendsize_ = 0;
   titleclock_ = 0;
   verbose_ = 0;
   debug_ = 0;
   clone_ = 0;
   jumpscroll_ = -1;
   input_buffer_empty_ = 1;
   instant_ = strdup("");

   if (!(term=getenv("COLORTERM")) || !*term)
      if (!(term=getenv("TERM")) || !*term)
	 putenv((char*)(term="TERM=vt100"));
   if (strncmp(term,"hp",2)==0)
      always_refresh_ = 1;
   else
      always_refresh_ = 0;

   if (strncmp(term,"xterm",5)==0 ||
       strncmp(term,"rxvt",5)==0)
      titlebar_ok_ = 1;
   else
      titlebar_ok_ = 0;

   /*
    * Create the window and take over the terminal
    */
   resize(); /* init(); */
}

Curs::~Curs()
{
   cleanup();
   if (instant_) free(instant_);
   if (roll_ofs) free(roll_ofs);
}


void
Curs::cleanup(void)
{
   if (form!=0)
   {
      want_E_OK(unpost_form(form), "unpost_form");
      want_E_OK(free_form(form), "free_form");
      form = 0;
   }
   if (field[0]!=0)
   {
      want_E_OK(free_field(field[0]), "free_field");
      field[0] = 0;
   }
   endwin();
   history.clone(clone_);
}

static int initialized = FALSE;

void
Curs::init(int clear_feedback)
{
   const char* saveline = 0;

   /*
    * NOTE NOTE NOTE: The order of the following steps is very
    * important to the curses library.  If things are not initialized
    * in the correct order, you may get an error returned from the
    * appropriate curses call, but not always!  So test this thoroughly
    * if anything needs to be adjusted between here and the main input
    * loop.  See the ncurses man pages for more on all of these calls.
    */

   if (initialized==FALSE)
   {
      int i;

      initialized = TRUE;

/*
 * ncurses version >= 4.2 can use terminfo fallbacks built in to the library.
 * See the INSTALL file in the ncurses package.  Currently, we should try to
 * build the following terminal types into libncurses at CFHT:
 *
 * vt100 linux xterm rxvt hp ansi
 *
 * If we don't have those fallbacks, the following provides an even cruder
 * way to make sure that director at least has builtin support for vt100+color.
 */
#if ( NCURSES_VERSION_MAJOR < 4 || \
    ( NCURSES_VERSION_MAJOR == 4 && NCURSES_VERSION_MINOR < 2 ) )
#define RXVT_TERMINFO
#endif
#ifdef RXVT_TERMINFO
      if (getenv("TERM")
	  && (!strcmp(getenv("TERM"), "rxvt")))
      {
#include "terminfo-rxvt.h"
	 
	 SCREEN* tmp = newterm(0, stdout, stdin); /* Memory leak */
	 if (!tmp)
	 {
	    char tienv[128];
	    char* tiname = tienv + 9;
	    int tifd;
	    
	    sprintf(tienv, "TERMINFO=/tmp/terminfo.%d", (int)geteuid());
	    putenv(strdup(tienv));
	    mkdir(tiname, 0755);
	    strcat(tiname, "/r");
	    mkdir(tiname, 0755);
	    strcat(tiname, "/rxvt");
	    tifd = open(tiname, O_RDWR|O_CREAT|O_EXCL, 0644);
	    if (tifd)
	    {
	       write(tifd,
		     terminfo_rxvt,
		     sizeof(terminfo_rxvt));
	    }
	 }
	 else
	 {
	    endwin();
	    delscreen(tmp);
	 }
      }
#endif

      initscr(); cbreak(); noecho(); nonl(); start_color();
      /*
       * Create the main feedback window
       */
      if (infolines_<0) infolines_=0;
      if (infolines_>9) infolines_=9;
      if (infolines_>LINES-3) infolines_=LINES-3;
      feedback_size_ = LINES - statbar_showing_ - prompt_showing_ - infolines_;
      roll_ofs = (int*)malloc(sizeof(int) * feedback_size_);
      for (i = 0; i < feedback_size_; i++) roll_ofs[i] = -1;

      need_not_NULL(feedback = 
		    newwin(feedback_size_, 0, 0, 0),
		    "newwin");
   }
   else
   {
      int i;

      instant("");
      form_driver(form, REQ_BEG_FIELD);
      saveline = history.add(field_buffer(field[0], 0));
      cleanup();

      /*
       * Delete the old statbar and prompt windows (they will be
       * re-created below)
       */
      if (statbar) { delwin(statbar); statbar=0; }
      if (info)    { delwin(info); info=0; }
      if (prompt)  { delwin(prompt); prompt = 0; }
      if (popup)   { delwin(popup); popup = 0; }

      /*
       * Re-calculate the number of lines allocated to the feedback area
       */
      if (infolines_<0) infolines_=0;
      if (infolines_>9) infolines_=9;
      if (infolines_>LINES-3) infolines_=LINES-3;
      i = feedback_size_;
      feedback_size_ = LINES - statbar_showing_ - prompt_showing_ - infolines_;
      roll_ofs = (int*)realloc(roll_ofs, sizeof(int) * feedback_size_);
      /*
       * Initialize only the new entries with '-1' (is the correct? ===
       * won't the whole screen get re-drawn anyway?)
       */
      for (; i < feedback_size_; i++) roll_ofs[i] = -1;

      /*
       * Keep the feedback window and just re-size it appropriately
       * so that the contents are not erased.
       */
      wresize(feedback, feedback_size_, COLS);
   }

   /*
    * If there is enough room, create the window for the popup messages
    */
   popup_width_ = COLS*8/10;
   if (feedback_size_ >= 4 && popup_width_ >= 20)
   {
      popup_x_ = (COLS - popup_width_)/2;
      popup_y_ = (feedback_size_ - 3)/3;
   }
   else
   {
      popup_x_ = popup_y_ = -1;
   }

   /*
    * Create the windows for status bar and command prompt.
    */
   if (statbar_showing_)
      need_not_NULL(statbar= newwin(1, 0, feedback_size_, 0), "newwin");
   if (prompt_showing_)
      need_not_NULL(prompt = newwin(1, 0, feedback_size_+statbar_showing_, 0), "newwin");

   if (infolines_)
      need_not_NULL(info =
		    newwin(infolines_, 0, 
			   feedback_size_+statbar_showing_+prompt_showing_,
			   0), "newwin");

   if (prompt)
   {
      nodelay(prompt, TRUE);
      intrflush(prompt, TRUE);
      keypad(prompt, TRUE);
   }
   leaveok(feedback, TRUE);
   if (statbar) leaveok(statbar, TRUE);
   if (info)    leaveok(info, TRUE);
   wmove(feedback, feedback_size_-1, 0);

   if (has_colors()==TRUE)
   {
      int bg = -1;

      /*
       * Unless the -b option was given on the command line (meaning
       * do not clobber user's background color or pixmap) just set
       * a default black background for everything.
       */
      if (!keep_bgcolor_) bg = COLOR_BLACK;
      else use_default_colors();

      want_not_ERR(init_pair(COLOR_NORM, COLOR_WHITE, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_WARN, COLOR_YELLOW, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_ERROR, COLOR_RED, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_STATUS, COLOR_GREEN, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_VERBOSE, COLOR_YELLOW, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_STATUSBAR,COLOR_WHITE,COLOR_BLUE),"init_pair");
      want_not_ERR(init_pair(COLOR_STATUSGAP,COLOR_BLACK,COLOR_BLUE),"init_pair");
      want_not_ERR(init_pair(COLOR_INFO, COLOR_BLACK, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_HIGHLIGHT,COLOR_BLACK, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_BETA, COLOR_MAGENTA, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_PROMPT, COLOR_CYAN, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_HIDDEN, COLOR_BLUE, bg), "init_pair");
      want_not_ERR(init_pair(COLOR_SHADOW, COLOR_BLACK, COLOR_BLACK), "init_pair");
      want_not_ERR(init_pair(COLOR_SHADEIN, COLOR_BLACK, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_SHADEOUT, COLOR_WHITE, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_SHADEIN_YELLOW, COLOR_YELLOW, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_SHADEOUT_YELLOW, COLOR_YELLOW, COLOR_YELLOW), "init_pair");
/* %%% */
      want_not_ERR(init_pair(COLOR_COMPLETE_YELLOW, COLOR_YELLOW, COLOR_BLACK), "init_pair");
      want_not_ERR(init_pair(COLOR_COMPLETE_RED, COLOR_RED, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_COMPLETE, COLOR_BLUE, COLOR_WHITE), "init_pair");
#if 0
      want_not_ERR(init_pair(COLOR_COMPLETE_DKGREEN, COLOR_GREEN, COLOR_WHITE), "init_pair");
      want_not_ERR(init_pair(COLOR_COMPLETE_GREEN, COLOR_GREEN, COLOR_BLACK), "init_pair");
      want_not_ERR(init_pair(COLOR_COMPLETE_YELLOW, COLOR_YELLOW, COLOR_BLUE), "init_pair");
      want_not_ERR(init_pair(COLOR_COMPLETE_CYAN, COLOR_CYAN, COLOR_BLACK), "init_pair");
#endif
   }

   /*
    * Set the default attributes of each screen area.
    */

   want_not_ERR(wattrset(feedback, ATTR_NORM), "wattrset");
   if (prompt) want_not_ERR(wattrset(prompt, ATTR_PROMPT), "wattrset");
   if (statbar)
   {
      if (has_colors()==TRUE)
      {
	 want_not_ERR(wattrset(statbar, ATTR_STATUSBAR),
		      "wattrset");
      }
      else
      {
	 want_not_ERR(wattrset(statbar, ATTR_STATUSBAR_MONO), 
		      "wattrset");
      }
   }

   if (info)
   {
      want_not_ERR(wattrset(info, ATTR_INFO),
		   "wattrset");
   }

   /*
    * Create the form's fields
    */

   field[1] = NULL;
   need_not_NULL((field[0]=
		  new_field(/*height=*/1, /*width=*/COLS-3,
			    /*toprow=*/0, /*leftcol=*/2,
			    /*offscreen=*/0, /*nbuffers=*/0)),
		 "new_field");
   want_E_OK(field_opts_off(field[0], O_BLANK), "field_opts_off(O_BLANK)");
   want_E_OK(field_opts_off(field[0], O_AUTOSKIP), "field_opts_off(O_AUTOSKIP)");
   want_E_OK(field_opts_off(field[0], O_STATIC), "field_opts_off(O_STATIC)");
   want_E_OK(set_max_field(field[0], 0), "set_max_field");

   /*
    * Create the form
    */

   need_not_NULL((form = new_form(field)), "new_form");

   if (has_colors())
   {
      want_E_OK(set_field_fore(field[0], A_BOLD), "set_field_fore");
      want_E_OK(set_field_back(field[0], ATTR_PROMPT), "set_field_back");
   }

   /*
    * Associate it with the window prompt
    */

   need_E_OK(set_form_win(form, prompt), "set_form_win");

   /*
    * Post it to the screen
    */

   need_E_OK(post_form(form), "post_form");

   /*
    * If this was a resize, and the current input line was
    * saved, restore it now.
    */
   if (saveline != 0)
   {
      form_driver(form, REQ_FIRST_FIELD);
      set_field_buffer(field[0], 0, saveline);
      form_driver(form, REQ_END_LINE);
   }

   wbkgdset(feedback, ATTR_NORM);

   if (clear_feedback) wclear(feedback);
   if (statbar)        wclear(statbar);
   if (info)           wclear(info);

   mvwaddch(prompt, 0, 0, '>'|A_BOLD);

   repaint();

   if (statbar) statusbar("");

   if (info)
      for (int i=0; i<infolines_; i++) infoline("",i+1);

   curs_set(1); /* Make sure cursor is visible */
   curs_set(2); /* Try to select large cursor */
   update();
}

void
Curs::scroll_feedback(int lines)
{
   int i;

   for (i = 0; i < feedback_size_ - 1; i++)
      roll_ofs[i] = roll_ofs[i + 1];
   roll_ofs[i] = -1;

   idlok(feedback, TRUE);
   scrollok(feedback, TRUE);
   wscrl(feedback, lines);
   scrollok(feedback, FALSE);
   idlok(feedback, FALSE);
}


void
Curs::infosize(int infolines)
{
   if (infolines>infolines_) scroll_feedback(infolines - infolines_);
   infolines_=infolines;
   init(0);
}

void
Curs::repaint(void)
{
   clearok(curscr, TRUE);
   if (statbar) { redrawwin(statbar); refresh_or_hup(statbar); }
   if (info)    { redrawwin(info);    refresh_or_hup(info); }
   redrawwin(feedback);
   if (popup) { redrawwin(popup); }
   /* refresh_or_hup(curscr); %%% */
   update();
}

void
Curs::update(char prompt_char)
{
   prompt_char_ = prompt_char;
#ifdef BETA
   if (!debug_)
   {
      want_not_ERR(wattrset(feedback, ATTR_BETA), "wattrset");
      mvwaddnstr(feedback, 0, 0,
		 " BETA VERSION; See `Invoking Director' in"
		 " http://software/director/ for info.      ",
		 COLS);
      want_not_ERR(wattrset(feedback, ATTR_NORM), "wattrset");
   }
#endif

   if (roll && roll->scrollback_active())
   {
      wmove(feedback, feedback_size_-1, 0);
      wclrtoeol(feedback);
      want_not_ERR(wattrset(feedback, ATTR_WARNING), "wattrset");
      waddnstr(feedback,
		 " PgDn for more...",
		 COLS-1);
      want_not_ERR(wattrset(feedback, ATTR_NORM), "wattrset");
   }

   refresh_or_hup(feedback);

   if (popup && !(roll && roll->scrollback_active()))
   {
      /*
       * Copy the characters from the feedback window that should appear
       * to fall under the "shadow" cast by the popup box.
       */
      static chtype s[200];
      int i;

      mvwinchnstr(feedback, popup_y_ + 3, popup_x_, s, 199);
      mvwaddchnstr(popup, 3, 0, s, 199);
      for (i=0; i<3; i++)
      {
	 mvwinchnstr(feedback, popup_y_ + i, popup_x_ + popup_width_ - 2, s, 2);
	 mvwaddchnstr(popup, i, popup_width_ - 2, s, 2);
      }
      draw_shadow(popup, 0, 0, 3, popup_width_ - 2, ATTR_SHADOW);
      touchwin(popup); /* Make sure the whole popup window is re-copied. */
      refresh_or_hup(popup);
   }

   if (prompt_char!='\0')
      mvwaddch(prompt, 0, 0, prompt_char|A_BOLD);
   mvwaddch(prompt, 0, 1, (data_behind(form)==TRUE)?('<'|A_STANDOUT):(' '|A_BOLD));
   mvwaddch(prompt, 0, COLS-1, (data_ahead(form)==TRUE)?('>'|A_STANDOUT):(' '));

   want_E_OK(pos_form_cursor(form),"pos_form_cursor");

   if (prompt_char!='>' && prompt_char!='S' && input_buffer_empty_)
   {
      int y, x;

      getyx(prompt, y, x);

      if (x==2)
      {
	 mvwaddch(prompt, 0, 0, ' '|A_BOLD);
	 wmove(prompt, 0, 0);
      }
   }

   refresh_or_hup(prompt);
   doupdate();
}

static PASSFAIL
autosize_vt100(short unsigned int* rows, short unsigned int* cols)
{
   char buf[200];
   int stdin_flags;
   struct termios tio, tioorig;
   int tmp = 1, size = 0;
   int wait = 10 * 4; /* Wait up to 10 seconds */
   int lrows, lcols;
   int tty = 0;

   if (tcgetattr(STDIN_FILENO, &tioorig)>=0)
   {
      tty = 1;

      /*
       * Turn off echo and put the terminal in raw mode
       */
      
      tio = tioorig;
      tio.c_iflag &= ~(ICRNL | IUCLC);
      tio.c_lflag &= ~(ICANON | ECHO);
      tio.c_cflag |= CS8|CREAD;
      tio.c_cc[VMIN] = 1;
      tio.c_cc[VTIME] = 0;
      if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &tio)!=0) return FAIL;
   }

   if ((stdin_flags = fcntl(STDIN_FILENO, F_GETFL, 0))<0
       || fcntl(STDIN_FILENO, F_SETFL, stdin_flags|O_NONBLOCK)<0)
   {
      if (tty) tcsetattr(STDIN_FILENO, TCSAFLUSH, &tioorig);
      return FAIL;
   }

   /*
    * Flush
    */
   while (read(STDIN_FILENO, buf, sizeof(buf))>0);

   /*
    * Cause VT100's to measure their rows and columns (requires
    * setting the full screen as the scrolling region and moving
    * to the lower left corner.)
    */
   write(STDOUT_FILENO, "\0337\033[r\033[999;999H\033[6n", 19);

   /*
    * Wait up to 4 seconds for the response
    */
   while (size < 10 && tmp != 0 && wait > 0)
   {
      struct timeval tv = { 0, 250000 };
      if ((tmp = read(STDIN_FILENO, buf+size, 10-size))==-1)
      {
	 SELECT(0, 0, 0, 0, &tv);
	 wait--;
      }
      else
      {
	 size += tmp;
	 if (size && buf[size-1]=='R') break;
      }
   }

   fcntl(STDIN_FILENO, F_SETFL, stdin_flags);
   if (tty) tcsetattr(STDIN_FILENO, TCSAFLUSH, &tioorig);

   if (size && buf[size-1]=='R' && sscanf(buf, "\033[%d;%dR", &lrows, &lcols)==2)
   {
      *rows = lrows;
      *cols = lcols;
      return PASS;
   }

   /*
    * Default to 80x24 and return failure.
    */
   *rows = 24;
   *cols = 80;
   return FAIL;
}


void
Curs::resize(void)
{
   struct winsize size;
   
   /*
    * Check the new window size and make sure everything will fit.
    */
   if (TEMP_FAILURE_RETRY(ioctl(STDOUT_FILENO, TIOCGWINSZ, &size))<0 &&
       /*
	* Should really check first to see that we are on a VT100-type
	* terminal.  This will cause some garbage to appear in HPTerms,
	* but it should get cleaned up when curses redraws everything.
	* (Unfortunately, on HPTerms where ncurses doesn't have control
	* of the pseudo terminal, the size will be wrong though.)
	*/
       autosize_vt100(&size.ws_row, &size.ws_col)==FAIL)
   {
#ifndef NO_CFHTLOG
      cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		"Failed to resize terminal properly; assuming 80x24");
#endif
      size.ws_col = 80;
      size.ws_row = 24;
   }
   
   if (size.ws_col>=MIN_COLS && size.ws_row>=MIN_LINES)
   {
      static char lines_env[80];
      static char columns_env[80];
      
      sprintf(lines_env, "LINES=%d", size.ws_row);
      putenv(lines_env);
      sprintf(columns_env, "COLUMNS=%d", size.ws_col);
      putenv(columns_env);

      if (initialized==TRUE)
	 resizeterm(size.ws_row, size.ws_col);

      init();
   }
   else
   {
      /*
       * Ignore resizes that would make the window too small.
       */
      if (initialized==FALSE) init();
   }
   if (jumpscroll_==0) jumpscroll_=-1; /* momentarily enable jumpscroll */
}

int
Curs::confirm(const char* s)
{
   int ch;
   chtype* savedline = (chtype*)malloc((COLS+1)*sizeof(chtype));

   instant("");
   mvwinchnstr(prompt, 0, 0, savedline, COLS);
   want_not_ERR(wattrset(prompt, ATTR_WARNING), "wattrset");
   mvwaddnstr(prompt, 0, 0, s, COLS);
   beep();
   if (roll) roll->scrollback_cancel();
   refresh_or_hup(prompt); update(); /* %%% */
   ch = wgetch(feedback);
   want_not_ERR(wattrset(prompt, ATTR_PROMPT), "wattrset");
   mvwaddchnstr(prompt, 0, 0, savedline, COLS);
   clearok(prompt,1);
   refresh_or_hup(prompt); update(); /* %%% */
   free(savedline);
   if (ch=='y' || ch=='Y') return 1;
   else return 0;
}

int
Curs::password(const char* prompt_string, const char* passwd)
{
   int ch;
   chtype* savedline = (chtype*)malloc((COLS+1)*sizeof(chtype));
   char passwd_entered[32];
   int passwd_length = 0;
   int rtn = 0;

   instant("");
   mvwinchnstr(prompt, 0, 0, savedline, COLS);
   want_not_ERR(wattrset(prompt, ATTR_WARNING), "wattrset");
   mvwaddnstr(prompt, 0, 0, prompt_string, COLS);
   beep();
   wrefresh(prompt);
   while (passwd_length < 31 && (ch = wgetch(feedback))>=' ')
      passwd_entered[passwd_length++] = char(ch);
   passwd_entered[passwd_length] = '\0'; /* Null terminate the string */

   want_not_ERR(wattrset(prompt, A_NORMAL), "wattrset");
   mvwaddchnstr(prompt, 0, 0, savedline, COLS);
   clearok(prompt,1);
   wrefresh(prompt);
   free(savedline);
   if (!strcmp(passwd, crypt(passwd_entered, passwd))) rtn = 1;
   memset(passwd_entered, 0, sizeof(passwd_entered));
   return rtn;
}

void
Curs::pending(const char* s)
{
#if 0
   /* %%% */
   pendsize_++;

   wsetscrreg(feedback, 0, feedback_size_-1);
   scroll_feedback();
   wmove(feedback, feedback_size_-1, 0);
   wattrset(feedback, A_BOLD);
   wprintw(feedback, "* %s", s);
   wattrset(feedback, COLOR_PAIR(COLOR_WARN));
   wprintw(feedback, "   (press ^G or cancel to undo)");
   wattrset(feedback, ATTR_NORM);
   refresh_or_hup(feedback);
   if (jumpscroll_==0) update();
#else
   if (s) ;
#endif
}

void
Curs::print(int offset, const char *s, const char* prefix, int clear_pending, time_t timestamp)
{
   static char* line = 0;
   static int last_maxx = 0;
   const char* p = s;
   int indent = 1;
   int maxx, maxy, sb = roll?roll->scrollback_active():0;

   getmaxyx(feedback, maxy, maxx);

   if (last_maxx != maxx)
   {
      last_maxx = maxx;
      if (line) free(line);
      line = (char*)malloc(maxx+1);
   }

#if 0
   /* %%% */
   if (clear_pending && *s=='>' && pendsize_>0)
   {
      pendsize_--;
      wsetscrreg(feedback, 0,
		 feedback_size_-1-(sb?1:pendsize_));
   }
   else
   {
      wsetscrreg(feedback, 0,
		 feedback_size_-1-(sb?1:pendsize_));
      scroll_feedback();
   }
   wmove(feedback, feedback_size_-1-(sb?1:pendsize_), 0);

   wclrtoeol(feedback);
#else
   if (clear_pending) ;
#endif
   scroll_feedback();
   roll_ofs[feedback_size_ - 1] = offset;
   wmove(feedback, feedback_size_-1-(sb?1:pendsize_), 0);

   if (*s)
   {
      if (prefix && !has_colors())
      {
	 wprintw(feedback, "%s", (char*)prefix);
	 indent = strlen(prefix);
      }

#if 0
/* %%% */      
      if (*s=='>') wattrset(feedback, ATTR_PROMPT);
#endif
      
      if (verbose_ && maxx > 20 && timestamp!=0)
      {
	 struct tm* tm = localtime(&timestamp);

	 wprintw(feedback, "%02d:%02d:%02d ",
		 tm->tm_hour, tm->tm_min, tm->tm_sec);
      }
   }

   for (p=s; *p; p++)
   {
      /*
       * Note: This routine could be made more efficient.
       * It adds the characters in the string one at a time to the
       * curses screen (but only updates the screen at the end)
       */
      int x, y, charsleft;
      const char* nextsp;
      char c = (*p)&0x7F; /* mask off lower 7-bits */

      /*
       * The DEL character, or any control characters other than TAB
       * and control-H are illegal and are replaced by a `*'.
       */
      if ((c<' ' && c!='\t' && c!=CONTROL('H')) || c==127) c=c+'0';

      if (!(nextsp = strpbrk(p+1," \t"))) charsleft = strlen(p+1);
      else charsleft = nextsp-p;

      getyx(feedback, y, x);
      
      if (x==maxx-1 || (c<=' ' && charsleft >= (maxx-x-1)))
      {
	 int i;

	 /*
	  * This is where word-wrapping happens.  Wrap to the next line
	  * and indent to the proper column.
	  */
	 waddch(feedback, '\r');
	 scroll_feedback();
	 for (i=0; i<indent+1; i++)
	    waddch(feedback, ' ');
	 if (c!=' ') waddch(feedback, c);
      }
      else
	 waddch(feedback, c);
   }

   refresh_or_hup(feedback);
   /*
    * Update the screen so the user sees the new line
    * (except in "jumpscroll" mode where it is legal to
    * buffer up several lines and only update the screen
    * during idle time.)
    */
   if (jumpscroll_==0) update();
}

void
Curs::message(int offset, const char* s, int logtype, int clear_pending,
	      time_t timestamp, int length_handle)
{
   int c = ATTR_NORM;	// color to pass to ncurses' wattrset()
   const char* p = "";	// prefix to print if no colors available

   switch (logtype)
   {
      case CLI_LOG_DEBUG:   if (!debug_) return; /* suppress */
      case CLI_LOG_START:
      case CLI_LOG_DONE:
      case CLI_LOG_VERBOSE: if (!verbose_) return; /* suppress */
                                          c=ATTR_VERBOSE; break;
      case CLI_LOG_ERROR:   p="error: ";  c=ATTR_ERROR; break;
      case CLI_LOG_WARNING: p="warning: ";c=ATTR_WARNING; break;
      case CLI_LOG_STATUS:                c=ATTR_STATUS; break;
      case CLI_LOG_HIGHLIGHT:             c=ATTR_HIGHLIGHT; break;
      case CLI_LOG_HIDDEN:  if (!verbose_) return;
     					  c=ATTR_HIDDEN; break;
      case CLI_LOG_PROMPT:		  c=ATTR_PROMPT; break;
   }

   /*
    * Change colors, print the message, and change back to "normal" color
    */
   wattrset(feedback, c);
   print(offset, verbose_?s:s+length_handle, p, clear_pending, timestamp);
   wattrset(feedback, ATTR_NORM);
}


void
Curs::titleclock(int onoff)
{
   if (titleclock_ != onoff)
   {
      titleclock_ = onoff;
      titlebar();
   }
}


const char*
Curs::titlename(const char* u)
{
   static char* u_ = strdup("???");

   if (u)
   {
      if (u_) free(u_);
      u_ = strdup(u);
   }
   return u_;
}


void
Curs::idle()
{
   titlebar(); /* this updates the clock */
   if (prompt_char_!='S' && prompt_char_!='*')
   {
      if (popup && popup_timeout_--==0)
	 progress("");
   }
}


void
Curs::titlebar(const char* s)
{
#define TITLESIZE 1024
   static char* s_ = 0;
   static char timestr[100] = "";
   static char current_[TITLESIZE] = "";
   static char last_[TITLESIZE] = "";
   static int last_state_ = 0;
   int new_state_ = debug_|(verbose_<<1)|(titleclock_<<2);
   time_t date;

   if (!titlebar_ok_) return;
   if (!s && !titleclock_ && last_state_==new_state_) return;

   last_state_ = new_state_;

   if (titleclock_)
   {
      /*
       * Update the clock in the status line
       */
      time(&date);
      strftime(timestr, 99, "%B %d %H:%M:%S  -  ", localtime(&date));
   }
   else timestr[0]='\0';

   if (s)
   {
      if (s_) free(s_);
      s_ = strdup(s);
   }

   if (!s_) s_ = strdup("[no title]");

   sprintf(current_, "\033]2;%.100s%.100s  -  %.100s%.100s%.100s%.100s\a",
	   timestr, titlename(), s_,
	   clone_?" CLONE":"",
	   debug_?" DEBUG":"",
	   verbose_?" VERBOSE":"");
   if (strcmp(current_, last_)==0) return;
   /*
    * Only output the escape sequence if the title actually changed
    */
   strcpy(last_, current_);
   if (fputs(current_, stderr) == EOF)
      /*
       * Simulate "hangup" conditions when not running on a pseudo-terminal
       * if update to titlebar fails.
       */
      raise(SIGHUP);
}


int
Curs::statusbar(const char *s)
{
   int rtn = 0;
   int col = 0;

   if (!statbar)
   {
      rtn = 1;
      scroll_feedback();
      statbar_showing_ = 1;
      init(0);
   }

   if (always_refresh_) clearok(statbar, TRUE);
   wmove(statbar, 0, 0);
   while (*s && ++col < COLS)
   {
      if (*s=='|')
      {
	 if (has_colors()==TRUE)
	 {
	    want_not_ERR(wattrset(statbar, ATTR_STATUSGAP),
			 "wattrset");
	 }
	 waddch(statbar, ACS_VLINE);
	 if (has_colors()==TRUE)
	 {
	    want_not_ERR(wattrset(statbar, ATTR_STATUSBAR),
			 "wattrset");
	 }
      }
      else
      {
	 waddch(statbar, *s);
      }
      s++;
   }

   whline(statbar, (int)' ', COLS);
   refresh_or_hup(statbar);
   update();

   return rtn;
}

void
Curs::infoline(const char* s, int n)
{
   int y, x;

   if (!info || n>infolines_ || n<1) return;
   if (always_refresh_) clearok(info, TRUE);
   mvwaddnstr(info, n-1, 0, s, COLS);
   getyx(info, y, x);
   if (x>0 || !*s)
      whline(info, (int)' ', COLS);
   refresh_or_hup(info);
   update();
}

void
Curs::progress(const char* s)
{
   static int n = 0;
   const char* progress_chars = "\\|/-";
   int len = strlen(s);
   char* pct = 0;
   int done = 0;
   int shadein  = ATTR_SHADEIN;
   int shadeout = ATTR_SHADEOUT;
   int empty = ATTR_EMPTY;
   int complete = ATTR_COMPLETE;
   int color_complete = COLOR_COMPLETE;

   /*
    * if s is NULL, delete the popup window, if any.
    */
   if (!s || !*s)
   {
      if (popup)
      {
	 delwin(popup); popup = 0;
	 /*
	  * Make sure the parts of the feedback window underneath the popup
	  * are properly re-drawn.  The popup is 4 lines, starting at popup_y_.
	  */
	 touchline(feedback, popup_y_, 4);
	 refresh_or_hup(feedback);
	 update();
      }
      return;
   }

   /*
    * === Hack.  Need way for agent to change color.
    */
   if (strstr(s, "Exposure")!=0)
   {
#if 0
      shadein  = ATTR_SHADEIN_YELLOW;
      shadeout = ATTR_SHADEOUT_YELLOW;
#endif
      empty = ATTR_EMPTY_YELLOW;
      complete = ATTR_COMPLETE_YELLOW;
      color_complete = COLOR_COMPLETE_YELLOW;
   }
   else if (strstr(s, "Reading")!=0)
   {
      empty = ATTR_EMPTY_RED;
      complete = ATTR_COMPLETE_RED;
      color_complete = COLOR_COMPLETE_RED;
   }

   /*
    * If there's not enough room on the screen for the progress meter,
    * ignore it and don't show the messages.
    */
   if (popup_x_==-1) return;

   /*
    * Strip out the percentage done, if given.
    */
   if (!strcmp(s+len-2, "%)") && (pct = strrchr(s, '(')))
   {
      done = atoi(pct+1);
      while (pct > s && *(pct-1)<=' ') pct--;
      len = pct - s;
   }
   if (len > popup_width_ - 10) len = popup_width_ - 10;

   /*
    * Create the popup window, if needed.
    */
   if (!popup)
   {
      need_not_NULL(popup =
		    newwin(4, popup_width_, popup_y_, popup_x_), "newwin");
      
      leaveok(popup, TRUE);
      want_not_ERR(wattrset(popup, 0), "wattrset");
      wclear(popup);
      mvwhline(popup, 0, 0, (int)' '|shadeout, popup_width_ - 2);
      mvwhline(popup, 1, 0, (int)' '|shadeout, popup_width_ - 2);
      mvwhline(popup, 2, 0, (int)' '|shadeout, popup_width_ - 2);
      draw_box(popup, 0, 1, 3, popup_width_ - 4, shadeout, shadein);
   }

   /* %%% */
   {
      wattrset(popup, empty);
      mvwhline(popup, 1, 2, (int)' '|empty, popup_width_ - 6);
      mvwprintw(popup, 1, 2 + (popup_width_ - 10 - len)/2, "%.*s", len, s);
      wattrset(popup, 0);
   }

   popup_timeout_ = 2;

   if (pct)
   {
      mvwprintw(popup, 1, popup_width_ - 8, "%d%%", done);
      wmove(popup,1,2);
      wchgat(popup, (popup_width_ - 6)*done/100,
             complete, color_complete, NULL);
      wmove(popup,1,2+(popup_width_-6)*done/100);
      wchgat(popup,
             (popup_width_ - 6) - (popup_width_ - 6)*done/100,
             empty, color_complete, NULL);
   }
   wattrset(popup, shadein);
   mvwhline(popup, 0, 2, ACS_HLINE, popup_width_ - 6);
   mvwprintw(popup, 0, ((popup_width_ - 6)-(7+4))/2,
	     "%c Working %c",
	     progress_chars[n%4],
	     progress_chars[n%4]);
   n++;
   wattrset(popup, 0);
   update();
}

void
Curs::instant(const char* s)
{
   static chtype* savedline = 0;
   static int savedsize = 0;

   if (*s==' ') s++;
   if (strcmp(s, instant_)==0) return;

   int len = strlen(s);

   if (len==0)
   {
      free(instant_); instant_ = strdup("");

      if (savedline)
      {
	 mvwaddchnstr(prompt, 0, 2, savedline, savedsize>COLS-2?(COLS-2):savedsize);
	 clearok(prompt,1);
	 refresh_or_hup(prompt); update(); /* %%% */
	 free(savedline);
	 savedline = 0;
      }
   }
   else if (input_buffer_empty_)
   {
      if (savedline)
      {
	 if (*savedline) instant("");
	 free(savedline);
      }
      free(instant_); instant_ = strdup(s);
      savedline = (chtype*)malloc((COLS+1)*sizeof(chtype));
      mvwinchnstr(prompt, 0, 2, savedline, savedsize=COLS-2);

      want_not_ERR(wattrset(prompt, ATTR_WARNING), "wattrset");
      mvwaddch(prompt, 0, 2, ' ');
      waddch(prompt, '(');
      waddnstr(prompt, s, COLS-5);
      waddch(prompt, ')');
      want_not_ERR(wattrset(prompt, ATTR_PROMPT), "wattrset");
      refresh_or_hup(prompt); update(); /* %%% */
   }
}


const char*
Curs::check_line()
{
   static int insert_mode = 1; /* 1 if edit mode is "insert" */
   static int escape = 0; /* 1 if reading an escape sequence */
   static char escape_seq[1024];
   int ch, result = E_OK;
   const char* rtn = 0;
/* === TODO: static char* kill_ring = 0; */
	 
   do
   {
      ch = wgetch(prompt);
      
      if (ch!=ERR) 
      {
	 input_buffer_empty_ = 0;
	 instant("");
      }

      if (escape)
      {
	 escape_seq[escape-1] = ch;
	 if (!(('0'<=ch && ch<=';') ||
	       ch=='(' || ch=='[' || ch==ESCAPE))
	    /* filter out escape sequences not handled by curses */
	 {
	    escape_seq[escape]='\0';
	    escape=0;
	    if (!strcmp(escape_seq, "[5~"))
	       ch = KEY_PPAGE;
	    else if (!strcmp(escape_seq, "[6~"))
	       ch = KEY_NPAGE;
	    else if (!strcmp(escape_seq, "[3~"))
              ch = KEY_DC;
	    else
	    {
	       beep();
	       if (roll) roll->scrollback_cancel();
	       wprintw(feedback, "\nUnrecognized escape sequence: `%s'\n", escape_seq);
	       refresh_or_hup(feedback);
	       ch = ERR;
	    }
	 }
	 else
	 {
	    if (escape<1022) escape++;
	 }
      }

      if (!escape) switch (ch)
      {
	 case ERR: break; /* no chars were ready */
	 
	 case KEY_ENTER:
	 case CONTROL('M'):
	 case CONTROL('J'):
	 {
	    form_driver(form, REQ_BEG_FIELD);
	    rtn = history.add(field_buffer(field[0], 0));
	    form_driver(form, REQ_FIRST_FIELD);
	    set_field_buffer(field[0], 0, "");
	    if (roll) roll->scrollback_cancel();
	    break;
	 }
	 case ESCAPE: escape=1; break;
	 case KEY_CANCEL: return "cancel";
	 case KEY_HELP: return "help";
	 case KEY_EXIT:
	 case KEY_CLOSE:
	 case KEY_PRINT:
	 case KEY_BREAK: /* exit the program ? */
	 case KEY_SUSPEND:
	 /*
	  * If stty is correct, these should never be seen
	  */
	 case CONTROL('Z'):
	 if (confirm("[^Z] Are you sure you want to send a stop? (Y/N)"))
	    return "stop";
         break;
	 case CONTROL('C'): 
	 if (confirm("[^C] Are you sure you want to send an abort? (Y/N)"))
	    return "abort";
         break;
	 case CONTROL('\\'):
	 if (confirm("[^\\] Are you sure you want to send a quit? (Y/N)"))
	    return "quit";
         break;
	 case CONTROL('G'): return "cancel";
	 /*
	  * === the following keys are still left unbound
	  */
	 case CONTROL('I'):
	 case CONTROL('O'):
	 case CONTROL('Q'):
	 case CONTROL('R'):
	 case CONTROL('S'):
	 case CONTROL('T'):
	 case CONTROL('W'):
	 case CONTROL('X'):
	 case CONTROL(']'):
	 case CONTROL('^'):
	 case CONTROL('_'):
	 waddstr(feedback, "\r(unimplemented key stroke)\n");
	 refresh_or_hup(feedback);
	 break;
	 case KEY_IC:
	 case KEY_EIC:
	 {
	    insert_mode ^= 1;
	    result=form_driver(form, insert_mode?REQ_INS_MODE:REQ_OVL_MODE);
	    break;
	 }
	 
	 case KEY_CLEAR: /* refresh the screen */
	 case KEY_REFRESH:
	 case CONTROL('L'):
	    repaint();
#if defined(SIGWINCH) && (SIGWINCH != 0)
	    raise(SIGWINCH);
	    /* Use ^L to simulate SIGWINCH conditions for
	     * director running in the Java Telnet Applet
	     * but without a real pseudo-terminal.  For this
	     * purpose, the JTA has been hacked to send a
	     * ^L since it cannot send a SIGWINCH.
	     */
#endif
	    break;
	 case CONTROL('U'): result=form_driver(form, REQ_CLR_FIELD); break;
	 case KEY_EOL: /* kill to end of line; save killed region for a yank */
	 case CONTROL('K'):
	 {
	    result=form_driver(form, REQ_CLR_EOL);
	    break;
	 }
	 case CONTROL('Y'):
	 {
	    break;
	 }
	 case KEY_NPAGE:
	 case CONTROL('V'): if (roll) roll->scrollback_change(LINES, LINES/2); break;
	 case KEY_PPAGE:
	 case KEY_A3:       if (roll) roll->scrollback_change(LINES, -LINES/2); break;
	 case KEY_HOME:
	 case KEY_A1:
	 case CONTROL('A'): result=form_driver(form, REQ_FIRST_FIELD); break;
	 case KEY_END:
	 case KEY_C1:
	 case CONTROL('E'): result=form_driver(form, REQ_END_LINE); break;
	 case KEY_LEFT:
	 case CONTROL('B'): result=form_driver(form, REQ_LEFT_CHAR); break;
	 case KEY_RIGHT:
	 case CONTROL('F'): result=form_driver(form, REQ_RIGHT_CHAR); break;
	 case KEY_UP:
	 case CONTROL('P'):
	 {
	    form_driver(form, REQ_BEG_FIELD);
	    history.save(field_buffer(field[0], 0));
	    {
	       const char* h = history.prev();
	       
	       if (!h) beep();
	       else
	       {
		  set_field_buffer(field[0], 0, h);
		  result=form_driver(form, REQ_END_LINE);
	       }
	    }
	    break;
	 }
	 case KEY_DOWN:
	 case CONTROL('N'):
	 {
	    form_driver(form, REQ_BEG_FIELD);
	    history.save(field_buffer(field[0], 0));
	    {
	       const char* h = history.next();
	       
	       if (!h) beep();
	       else
	       {
		  set_field_buffer(field[0], 0, h);
		  result=form_driver(form, REQ_END_LINE);
	       }
	    }
	    break;
	 }
	 case KEY_BACKSPACE:
	 case KEY_DC:
	 case 127:
	 case CONTROL('H'): result=form_driver(form, REQ_DEL_PREV); break;
	 case CONTROL('D'): result=form_driver(form, REQ_DEL_CHAR); break;
	 /* REQ_PREV_WORD */
	 /* REQ_NEXT_WORD */
	 default:
	 if (' '<=ch && ch<='~') result=form_driver(form, ch);
	 else
	 {
	    beep();
	    /* %%% Messes up screen
	    wprintw(feedback, "\nunbound key[%d]!\n", (int)ch);
            refresh_or_hup(feedback); */
	    /* %%% This and two others need "update()" */
	 }
      }
      
      if (result != E_OK) beep();
	
      if (ch!=ERR) update();

   } while (rtn==0 && ch!=ERR);

   if (rtn) input_buffer_empty_ = 1;
   else input_buffer_empty_ = ((mvwinch(prompt, 0, 2)&A_CHARTEXT)<=' '); /* === ineff. */
   return rtn;
}
/* end of Curs.cc */
