
/*  MODIFIED FOR DIRECTOR by Sidik Isani (isani@cfht.hawaii.edu) */

/*
 *  util.c
 *
 *  ORIGINAL AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
 *  MODIFIED FOR LINUX KERNEL CONFIG BY: William Roadcap (roadcap@cfw.com)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <ncurses/ncurses.h>

/*
 * Draw a rectangular box with line drawing characters
 * SI: I would have just created a subwindow and used wborder() which
 *     already part of ncurses, but this seems quicker.
 */
static void
draw_box (WINDOW * win, int y, int x, int height, int width,
          chtype box, chtype border)
{
    int i, j;

    wattrset (win, 0);
    for (i = 0; i < height; i++) {
        wmove (win, y + i, x);
        for (j = 0; j < width; j++)
            if (!i && !j)
                waddch (win, border | ACS_ULCORNER);
            else if (i == height - 1 && !j)
                waddch (win, border | ACS_LLCORNER);
            else if (!i && j == width - 1)
                waddch (win, box | ACS_URCORNER);
            else if (i == height - 1 && j == width - 1)
                waddch (win, box | ACS_LRCORNER);
            else if (!i)
                waddch (win, border | ACS_HLINE);
            else if (i == height - 1)
                waddch (win, box | ACS_HLINE);
            else if (!j)
                waddch (win, border | ACS_VLINE);
            else if (j == width - 1)
                waddch (win, box | ACS_VLINE);
            else
                waddch (win, box | ' ');
    }
}

/*
 * Draw shadows along the right and bottom edge to give a more 3D look
 * to the boxes
 */

static void
draw_shadow (WINDOW * win, int y, int x, int height, int width, chtype shadow_attr)
{
    int i;

    if (has_colors ()) {        /* Whether terminal supports color? */
      /* wattrset (win, shadow_attr); */
        wmove (win, y + height, x + 2);
        for (i = 0; i < width; i++)
            waddch (win, (winch (win) & A_CHARTEXT) | shadow_attr);
        for (i = y + 1; i < y + height + 1; i++) {
            wmove (win, i, x + width);
            waddch (win, (winch (win) & A_CHARTEXT) | shadow_attr);
            waddch (win, (winch (win) & A_CHARTEXT) | shadow_attr);
        }
        wnoutrefresh (win);
    }
}
