/*                                             -*- c-file-style: "Ellemtel" -*-

`Pipes.cc' - See Pipes.hh

This file is part of version 3 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <pwd.h>

#include "setserial.h"
#include "cli/cli.h"
#include "cli/clidefs.h"

/*
 * Timeout in milliseconds after which failure to write
 * to a pipe should cause a failure. %%% Causes problems on slow machines!
 */
#define WRITE_MSEC 6000

#include "CliQueue.hh"

void do_sleep(int seconds); /* temporarily disable the interval timer and do
			     * a regular sleep 
			     */


#include "Pipes.hh"
#include "Roll.hh" /* needed for just a few little things */
extern Roll* roll;

/*
 * Auto-restart settings:
 * ---------------------
 *
 * Don't autorestart if agent dies more than 10 times in
 * a period of about 30 seconds.
 */
#define RESTART_MAX 5
#define RESTART_PERIOD 30

static char junkbuf[5000];
static ProcessPipe* start_ = 0; /* Keeps track of most recently (re)started
				 * agent.  This is so that "agent (re)start"
				 * commands can return BUSY and cause director
				 * to continue waiting for the new agent to
				 * be ready (i.e., when its first prompt
				 * appears.)
				 */

void
LineBuffer::log_error(const char* msg)
{
   int errno_ = errno;

   sprintf(junkbuf, "%s`%.100s' (fd %d) %.4500s (%.400s)",
	   roll?"error: ":"", get_id(), get_fd(), msg, strerror(errno_));

   if (roll)
      roll->add_line(junkbuf);
#ifndef NO_CFHTLOG
   else
      cfht_log(CFHT_MAIN, CFHT_ERROR, (char*)msg);
#else
   /* Error message lost! */
#endif
}

void
LineBuffer::log_debug(const char* msg)
{
   if (!director_debug) return;
   sprintf(junkbuf, "%s`%.100s' (fd %d) %.4500s",
	   roll?"debug: ":"", get_id(), get_fd(), msg);

   if (roll)
      roll->add_line(junkbuf);
#ifndef NO_CFHTLOG
   else
      cfht_log(CFHT_MAIN, CFHT_DEBUG, (char*)msg);
#endif
}

LineBuffer::LineBuffer()
{
   bufp = buf[bufnum=0];
   fd = -1;
   resultp=0;
   lastnewline=0;
   found_prompt=FALSE;
   id = 0;
   mtype = -1;
   prompts = 0;
   timeoutp = 0;
   timeout_restart_ = 0;
   timeout_initial_ = 0;
}

LineBuffer::~LineBuffer()
{
   if (id) { free(id); id=0; }
   if (prompts) { free(prompts); prompts=0; }
}

const char*
LineBuffer::check_line(int counteract_pagers)
{
   int bytes;
   char* newline = 0;
   int trunc = 0;
   extern const char *psfailed; /* === hack */
   extern int psfailedlen;
   
   if (fd==-1) return 0; /* no more input */
   
   *bufp='\0';
   newline = strpbrk(buf[bufnum],"\r\n");

   if (!newline)
   {
      char* belpos;

      bytes=read(fd, bufp, CMD_MAXLEN-(bufp-buf[bufnum]));
      
      {
	 int i;
	 for (i=0; i<bytes; i++)
	 {
	    if ((bufp[i]<' ' || bufp[i]>'~') && !strchr("\t\r\n\a", bufp[i]))
	       bufp[i]+='?'; /* === Show better */
	 }
      }

      if (bytes==-1)
      {
	 if (errno!=EAGAIN)
	 {
	    log_error("read failed; closing file descriptor");
	    (void) close(fd); /* If this fails too, ignore the error. */
	    fd = -1;
	    return 0;
	 }
	 if (timeout_expired() && resultp && *resultp==BUSY)
	 {
	    log_debug("ready - last command timed out");
	    found_prompt = TRUE;
	    *resultp = PASS;
	    if (*(buf[bufnum]))
	    {
#if 0
	       /* %%% */
	       if (roll) roll->add_line("debug: still in buffer:");
	       if (roll) roll->add_line(buf[bufnum], CFHT_DEBUG);
#endif
	       *bufp++ = '\n';
	       return check_line(); /* %%% OK? */
	    }
	 }
      }
      else timeout_restart(); /* Does nothing if no timeout. */

      if (bytes==0)
      {
	 if (close(fd) != 0)
	    log_error("close failed");
	 fd = -1;
	 return 0;
      }
      
      if (bytes>0)
      {
	 /*
	  * Unsolicited data on connections with no prompt string specified
	  * cause the status to return to "BUSY" again.  This way if a
	  * partial line is received when nothing was expected, the code
	  * above will at least print the debug message "still in buffer"
	  */
	 if (timeout_isset() && resultp) *resultp = BUSY;

	 while ((belpos = (char*)memchr(bufp, (int)'\a', bytes))!=0)
	 {
	    /*
	     * Strip out any bel characters and make the terminal beep
	     */
	    int len_after_bel = bufp + bytes - belpos - 1;
	    
	    if (len_after_bel>0)
		     memmove(belpos, belpos + 1, len_after_bel);
	    bytes--;
	    fputc('\a', stderr); /* Make the terminal beep, now */
	 }
	 
	 if (bytes)
	 {
	    bufp += bytes;
	    bufp[0] = '\0';
	    bufp[1] = '\0';
	    
	    if (bufp - buf[bufnum] == CMD_MAXLEN &&
		strpbrk(buf[bufnum],"\r\n") == 0)
	    {
#ifndef NO_CFHTLOG
	       cfht_logv(CFHT_MAIN, CFHT_WARN, "LineBuffer overflow");
#endif
	       while (read(fd, buf[(bufnum+1)%2], CMD_MAXLEN)>0) ;
	       newline = bufp-1;
	    }
	    else
	    {
	       newline = strpbrk(buf[bufnum], "\r\n");
	    }
	 }
      }
   }
   
   if (found_prompt==FALSE && prompts && *prompts)
   {
      const char* ps = prompts;
      const char* ps_next;
      char pdelim;

      pdelim = *ps++;
      ps_next = strchr(ps, pdelim);

      while (pdelim && ps_next && found_prompt==FALSE)
      {
	 if (strncmp(buf[bufnum], ps, ps_next - ps)==0)
	 {
	    found_prompt = TRUE;
	    if (ps_next[-1]==' ') trunc = ps_next - ps - 2;
	    else trunc = ps_next - ps - 1;
	    if (strncmp(DEFAULT_CONT_PROMPT, ps, ps_next - ps)!=0)
	    {
	       /*
		* Any prompt EXCEPT for the cont> prompt should be
		* treated as a signal that the most recent command
		* has completed successfully.
		*/
	       if (resultp && *resultp==BUSY)
	       {
		  *resultp = PASS;
		  log_debug("ready - last command passed");
	       }
	    }
#if VERBOSE
	    else log_debug("ready - continuing prompt");
#endif
	 }
	 ps = ps_next + 1;
	 ps_next = strchr(ps, pdelim);
      }
      if (found_prompt==FALSE)
      {
	 if (strncmp(buf[bufnum], psfailed, psfailedlen)==0)
	 {
	    found_prompt = TRUE;
	    trunc = psfailedlen - 1;
	    if (resultp && *resultp==BUSY)
	    {
	       *resultp = FAIL;
	       log_debug("ready - last command FAILED");
	    }
	 }
         /*
	  * === Pager programs are not compatible with director's feedback
	  * window, so attempt to automatically counteract accidental
	  * use of a pager program.  This can only be done by looking
	  * for pager "NEXT PAGE" prompts and sending a carriage return.
	  */
	 else if (counteract_pagers && !newline &&
		  strncmp(buf[bufnum], "Type <CR> to continue", 21)==0)
	 {
	    return (const char*)-1; /* %%% */
	 }
      }
   }

   /*
    * Agents send various types of prompt strings to director, but
    * director converts them all into '> '.  The variable `trunc'
    * indicates how much longer than '> ' the actual prompt string
    * was.  For example, if it was 'ok> ' instead, this will cause
    * everything to be shifted by two characters.
    */
   if (trunc > 0)
   {
      memmove(buf[bufnum], buf[bufnum] + trunc,
	      bufp - buf[bufnum] - trunc + 2);
      bufp -= trunc;
      if (newline) newline -= trunc;
   }

   if (newline)
   {
      char* rtn = buf[bufnum];
      
      /* ignore \r's at the beginning of a line and \n's immediately after \r's 
       * but not a \r\n at the beginning of a line.  sigh
       */
      if (newline==rtn)
      {
	 if (*newline=='\r' ||  /* \r at beginning of line */
	     lastnewline=='\r') /* \n after a \r that was not at beginning of line */
	    rtn=0;
	 lastnewline = 0;
      }
      else
      {
	 lastnewline = *newline;
      }

      *newline++ = '\0';
      
      bufnum = (bufnum + 1) % 2;
      strcpy(buf[bufnum], newline);
      bufp = buf[bufnum] + strlen(buf[bufnum]);
      
      found_prompt = FALSE;
      
      if (!rtn) return check_line(); /* recursively call for next line */
      
      /*
       * hack to skip lines that started with an @
       */
      if (rtn && strncmp(rtn, "> @", 3)==0)
	 return check_line();

      return rtn;
   }

   return 0;
}


CommandPipe::CommandPipe(const char* name, int wronly=0)
{
   command_fd = -1;
   autoremove = 0;
   if (name)
   {
      set_id(name);
      if (!wronly && unlink(name)==0)
      {
	 if (roll) roll->add_line("warning: removed stale command pipe");
      }
      if (!wronly && 
	  (mknod(name, S_IFIFO|S_IWUSR|S_IRUSR|S_IWGRP|S_IWOTH, 0)!=0 ||
	   chmod(name,         S_IWUSR|S_IRUSR|S_IWGRP|S_IWOTH)!=0)) /* %%% OTH? */
      {
         log_error("failed to create command pipe");
      }
      else if ((command_fd = 
		open(name, 
		     (wronly?O_WRONLY:O_RDWR)|O_NONBLOCK
		     ,0))==-1)
	 /*
	  * NOTE: the pipe is opened for writing as well because
	  * even though we don't want to write to it, we want to
	  * be able to select() on it, and that doesn't work right
	  * unless someone has it open for writing already.
	  */
      {
         log_error("failed to open command pipe for writing");
      }
      if (!wronly) { set_fd(command_fd); autoremove=1; }
   }
}


CommandPipe::~CommandPipe()
{
   if (command_fd!=-1)
   {
      (void) close(command_fd); /* Ignore error on close here. */
      if (autoremove && get_id()) unlink(get_id());
   }
}

void
CommandPipe::send_line(const char* command)
{
   if (!command) { roll->add_line("warning: CommandPipe::send_line(NULL)!"); return; }
   if (command_fd==-1) { roll->add_line("error: CommandPipe::send_line() command_fd==-1"); return; }

   sprintf(junkbuf, "%.4000s\n", command); /* %%% Before I was sending the
					    * command and then the \n with
					    * a separate write().  This
					    * caused problems with the
					    * pipe being "Busy" for the
					    * second write.  This is not
					    * a proper fix, but reduces
					    * the chances of a problem for
					    * now.
					    */
   if (( /* %%% *command &&  */
      cli_write(command_fd, junkbuf, strlen(junkbuf), WRITE_MSEC)==FAIL)
       /* %%% || cli_write(command_fd, "\n", 1, WRITE_MSEC)==FAIL */)
      log_error("write failed");
}

LogPipe::LogPipe(const char* name)
{
   log_fd = -1;
   autoremove = 0;
   if (name)
   {
      set_id(name);
      int create = 0;
      struct stat stbuf;
      
      memset(&stbuf, 0, sizeof(stbuf));
      
      if (stat(name, &stbuf)==0)
      {
	 if (!S_ISFIFO(stbuf.st_mode))
	 {
	    char nametxt[1024];
	    /*
	     * If the file existed, but was not a named pipe,
	     * rename it / remove it so it can be re-created.
	     */

	    sprintf(nametxt, "%.1000s.TXT", name);
	    /*
	     * Before moving "name" to *.TXT, make sure *.TXT can
	     * be removed.  If not, try renaming the file to *.pid.TXT.
	     */
	    if (unlink(nametxt) != 0 && errno != ENOENT)
	    {
	       sprintf(nametxt, "%.1000s.%u.TXT",
		       name, (unsigned)getpid());
	    }
	    if (rename(name, nametxt) != 0)
	    {
	       log_error("failed to rename bogus text log file");
	       if (unlink(name) != 0)
		  log_error("failed to remove bogus text log file");
	    }
	    else
	       if (roll) roll->add_line("logonly: moved text logfile to *.TXT (should be pipe)");
	    create = 1;
	 }
	 else
	 {
	    /*
	     * Open existing log pipe for writing (with non-blocking i/o) to
	     * test if anyone else is already reading it.  This _should_ fail.
	     * If it doesn't fail, it means the log pipe is already in use
	     * by another director, possibly.
	     */
	    log_fd = open(name, O_WRONLY | O_NONBLOCK, 0);
	    if (log_fd != -1)
	    {
	       if (roll) roll->add_line("logonly: log pipe alread in use!");
	       (void) close(log_fd); /* Ignore if this fails. */
	       log_fd = -1;
	       return;
	    }
	    if (roll) roll->add_line("logonly: log pipe exists; won't create or remove it");
	 }
      }
      else create = 1;
      
      if (create)
      {
	 if (mknod(name, S_IFIFO|S_IWUSR|S_IRUSR|S_IWGRP, 0)!=0)
	 {
            log_error("failed to create log pipe");
	    return;
	 }
	 autoremove = 1;
      }
      
      if ((log_fd = open(name, O_RDWR | O_NONBLOCK, 0))==-1)
	 /*
	  * NOTE: the pipe is opened for writing as well because
	  * even though we don't want to write to it, we want to
	  * be able to select() on it, and that doesn't work right
	  * unless someone has it open for writing already.
	  */
         log_error("failed to open log pipe");

      set_fd(log_fd);
   }
}

LogPipe::~LogPipe()
{
   if (log_fd!=-1) (void) close(log_fd); /* Ignore failure here. */
   if (autoremove && get_id()) unlink(get_id());
}

#define PIPE_OUTPUT_SIDE 0
#define PIPE_INPUT_SIDE 1

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>     

/*
 * This routine is used internally to print log messages about
 * what director is doing to an agent process.
 */
void
ProcessPipe::log(const char* type, const char* msg)
{
   sprintf(junkbuf, "%s: Agent %.100s[%d] %s %.4500s",
	   type?type:"error", handle(), (int)childpid(), 
	   default_agent()?"(default agent)":"",
	   msg?msg:"EMPTY MESSAGE");

   if (roll) roll->add_line(junkbuf);
#ifndef NO_CFHTLOG
   else cfht_log(CFHT_MAIN, CFHT_ERROR, junkbuf);
#else
   /* Message lost! */
#endif
}

ProcessPipe::ProcessPipe(const char** argv, int director_level,
			 const char* handle, const char* prompts,
			 int timeout_i, int timeout_s,
			 int default_mtype,
			 int default_agent, int autorestart)
{
   int argc;

   default_agent_ = default_agent;
   autorestart_ = autorestart;

   if (handle && *handle)
   {
      set_id(handle);
   }
   else
   {
      /*
       * Special case for "sh -c ..." shell escapes.  Set the identifier
       * to the actual command, otherwise error messages say "sh" failed
       * without indicated which command was actually running.
       */
      if (argv[0] && !strcmp(argv[0], "sh") &&
	  argv[1] && !strcmp(argv[1], "-c") &&
	  argv[2] && *argv[2])
      {
	 char* new_id = strdup(argv[2]);
	 char* p = new_id;
	 char* q;

	 if (!strncmp(p, "exec ", 5)) p += 5;
	 if ((q=strchr(p, ' '))) *q = '\0';
	 set_id(p);
	 free(new_id);
      }
   }
   for (argc = 0; argv[argc] != 0; argc++) ;
   argv_ = (char**)malloc(sizeof(char*) * (argc + 1));
   for (argc = 0; argv[argc] != 0; argc++)
      argv_[argc] = strdup(argv[argc]);
   argv_[argc] = 0;

   next_ = 0;
   command_ = 0;
   child = -1;
   type = PIPE_NOT_USED;
   child_input[PIPE_INPUT_SIDE] = -1;
   child_output[PIPE_OUTPUT_SIDE] = -1;
   child_stderr[PIPE_OUTPUT_SIDE] = -1;

   set_fd(-1);
   err.set_fd(-1);
   set_mtype(default_mtype);
   err.set_mtype(default_mtype==-1?CLI_LOG_WARNING:default_mtype);
   set_prompts(prompts);
   err.set_prompts(prompts);
   timeout_init(&command_timeout, timeout_i, timeout_s);
   err.timeout_init(&command_timeout, timeout_i, timeout_s);
   set_resultp(&command_result);
   err.set_resultp(&command_result);

   restart(/*accept_input=*/(handle==0||*handle), director_level);
}

ProcessPipe::~ProcessPipe()
{
   cleanup();
   char** argv = argv_;
   while (*argv) free (*argv++);
   free (argv_);
}

extern void flush_to_screen(void); /* %%% */
extern void flush_inputs(void);

void
ProcessPipe::cleanup()
{
   if (type==PIPE_TO_PROCESS)
   {
      if (signal(0)==PASS)
      {
	 int wait_sec = 8;

	 flush_inputs();
	 log("logonly", "getting the TERM signal");
	 flush_to_screen();
	 signal(SIGTERM);
	 while (1)
	 {
	    flush_inputs();
	    flush_to_screen();

	    /*
	     * If the child exited... %%% HACK!
	     */
	    if (cli_signal_event(SIGCHLD))
	    {
	       const char* s = postmortem();
	       
	       if (s) roll->add_line(s);
	    }
	    
	    if (signal(0)!=PASS)
	    {
	       log("logonly", "responded to TERM signal");
	       flush_to_screen();
	       break;
	    }
	    if (wait_sec--)
	    {
	       do_sleep(1);
	       continue;
	    }
	    /*
	     * Normally, the process will have exited by now.  But if it
	     * doesn't, it means the program refuses to terminate cleanly.
	     * make SURE it's gone by killing it with SIGKILL 8 seconds
	     * later...
	     */
	    log("warning", "getting the KILL signal");
	    flush_to_screen();
	    signal(SIGKILL);
	    flush_inputs();
	    flush_to_screen();
	    break;
	 }
      }
   }
   if (command_)
   {
      command_->assign(0);
      command_ = 0;
   }
   if (child_input[PIPE_INPUT_SIDE]!=-1)
   {
      close(child_input[PIPE_INPUT_SIDE]);
      child_input[PIPE_INPUT_SIDE]=-1;
   }
   if (get_fd()!=-1)
   {
      close(get_fd());
      set_fd(-1);
   }
   if (err.get_fd()!=-1)
   {
      close(err.get_fd());
      err.set_fd(-1);
   }
}


PASSFAIL
ProcessPipe::restart(int accept_input, int director_level)
{
   char junkbuf[1024];
   char* colon = strchr(argv_[0],':');
   char* brace;
   const char* options = 0;
   const char* argv0 = argv_[0];
   struct stat stbuf;

   cleanup();

   /*
    * if p->autorestart_ is found to be 1, it means that the
    * agent was dying too quickly and director gave up on
    * restarting it.  The first manual "restart" triggers
    * director to start trying again from now on.
    */
   if (autorestart_ == 1)
   {
      if (roll)
      {
	 roll->add_line("status: Turning on autorestart"); /* === Give agent name? */
	 flush_to_screen();
      }
      autorestart_ = time(0);
   }

   command_result = BUSY;
   child = (pid_t)-1;
   type = PIPE_NOT_USED;

   if (roll)
   {
      int i = 0;

      if (default_agent_)
      {
	 sprintf(junkbuf, "titlebar: %.500s", argv_[0]);
	 roll->add_line(junkbuf);
      }
      sprintf(junkbuf, "%s: starting agent `", accept_input?"status":"logonly");
      while (argv_[i] && (strlen(junkbuf)+strlen(argv_[i])+4) < sizeof(junkbuf))
      {
	 if (i) strcat(junkbuf, " ");
	 strcat(junkbuf, argv_[i]);
	 i++;
      }
      strcat(junkbuf, "'");
      roll->add_line(junkbuf);
   }

   if (*argv0 == '{' && (brace = strchr(argv0, '}')))
   {
      options = argv0 + 1;
      argv0 = brace + 1;
   }

   memset(&stbuf, 0, sizeof(stbuf));

   if (!colon && stat(argv0, &stbuf)==0)
   {
      if (S_ISDIR(stbuf.st_mode))
      {
	 log_error("can't run a directory!");
	 return FAIL;
      }
      
      if (S_ISCHR(stbuf.st_mode))
      {
	 extern const char* cli_newline; /* === */
	 /*
	  * Reading from device instead of process
	  */
	 
	 type = PIPE_TO_DEVICE;
	 
	 child_stderr[PIPE_OUTPUT_SIDE] = -1;
	 if ((child_input[PIPE_INPUT_SIDE] =
	      child_output[PIPE_OUTPUT_SIDE] =
	      open(argv0, O_RDWR | O_NONBLOCK | O_NOCTTY))==-1)
	 {
	    log_error("failed to open agent device");
	    do_sleep(2);
	    return FAIL;
	 }
	 if (setserial(child_input[PIPE_INPUT_SIDE], options) < 0)
	 {
	    log_error("failed to set all serial options (continuing)");
	 }
	 
	 if (roll) roll->add_line("status: connected to a serial port");
	 do_sleep(1);
	 if (cli_write(child_input[PIPE_INPUT_SIDE], cli_newline, 1, WRITE_MSEC)==FAIL)
	 {
	    log_error("writing to serial port failed");
	    do_sleep(3);
	 }
	 command_result = PASS;
      }
   }

   if (command_result != PASS)
   {
      /*
       * Reading from local process through a pipe
       */
      type = PIPE_TO_PROCESS;
      
      pipe(child_input);
      pipe(child_output);
      pipe(child_stderr);
      
      if ((child = fork()) == (pid_t)-1)
      {
	 log_error("fork failed");
	 return FAIL;
      }
      
      if (child == (pid_t)0)
      {
	 extern const char* logpipename;
	 char junkbuf[1024];
	 
	 /*
	  * This is the child thread.  Create a new ProcessPipe group
	  * so we don't catch signals meant only for the parent.
	  */
	 setsid();
	 
	 alarm(0);
	 cli_signal(SIGALRM, CLI_SIG_IGN);
	 cli_signal_unblock(SIGALRM);
	 cli_signal(SIGALRM, CLI_SIG_DFL);
	 cli_signal(SIGINT, CLI_SIG_DFL);
	 cli_signal(SIGTSTP, CLI_SIG_DFL);
	 cli_signal(SIGQUIT, CLI_SIG_DFL);
	 
	 sprintf(junkbuf, DIRECTOR_LEVEL_ENV"=%d", director_level);
	 putenv(strdup(junkbuf));
	 putenv("COLORTERM=dumb");
	 putenv("TERM=dumb");
	 if (logpipename)
	 {
	    sprintf(junkbuf, "CFHTLOGU=%s", logpipename);
	    putenv(junkbuf);
	 }
	 
	 /*
	  * Set up the stdio pipes.
	  */
	 close(STDOUT_FILENO);
	 dup(child_output[PIPE_INPUT_SIDE]);
	 
	 close(STDERR_FILENO);
	 dup(child_stderr[PIPE_INPUT_SIDE]);
	 
	 close(STDIN_FILENO);
	 dup(child_input[PIPE_OUTPUT_SIDE]);
	 
	 close(child_input[PIPE_INPUT_SIDE]);
	 close(child_output[PIPE_OUTPUT_SIDE]);
	 close(child_stderr[PIPE_OUTPUT_SIDE]);
	 
	 execvp( colon?"runon":argv0, (char**)argv_ );
	 perror( argv0 );
	 _exit(EXIT_FAILURE);
      }
      
      /*
       * This is the parent thread.
       */
      close(child_input[PIPE_OUTPUT_SIDE]);
      close(child_output[PIPE_INPUT_SIDE]);
      close(child_stderr[PIPE_INPUT_SIDE]);
   }

   set_fd(child_output[PIPE_OUTPUT_SIDE]);
   err.set_fd(child_stderr[PIPE_OUTPUT_SIDE]);

   if (!accept_input)
   {
      close(child_input[PIPE_INPUT_SIDE]);
      child_input[PIPE_INPUT_SIDE] = -1;
   }

   else if (fcntl(child_input[PIPE_INPUT_SIDE], F_SETFL,
	     fcntl(child_input[PIPE_INPUT_SIDE], F_GETFL, 0)|O_NONBLOCK)==-1)
   {
      log_error("fcntl(child_input ... NON_BLOCK) failed");
      return FAIL;
   }
   start_ = this; /* Keep track of most recently (re)started agent */
   return PASS;
}

process_line_t*
ProcessPipe::check_line()
{
   static process_line_t pl;
   const char* s = LineBuffer::check_line(1);

   if (s==(char*)-1)
   {
      if (cli_write(child_input[PIPE_INPUT_SIDE], "\r", 1, WRITE_MSEC)==FAIL)
	 log_error("write(counteract_pagers) failed");
      return check_line();
   }

   if (!s) return 0;
   if (!strncmp(s, "signalmap:", 10)) return check_line();

   pl.line = s;
   pl.pid = (int)childpid();
   pl.handle = handle();
   pl.mtype = get_mtype();

   if (*s=='>')
   {
      if (command_ && (command_->no_echo() || command_->output()==OUTPUT_CAP))
      {
	 command_->no_echo_clear();
	 pl.mtype = CLI_LOG_HIDDEN;
      }
      else
      {
	 pl.mtype = CLI_LOG_PROMPT;
      }
   }
   else if (command_ && command_->output()!=OUTPUT_DIRECTOR)
   {
	 /*
	  * %%% Major Hack... but I want to suppress these for now :-/
	  */
      if (strncmp(s, "statusbar:", 10)==0 ||
	  strncmp(s, "titlebar:", 9)==0 ||
	  strncmp(s, "instant:", 8)==0 ||
	  strncmp(s, "progress:", 9)==0 ||
	  (strncmp(s, "info", 4)==0 && s[4]>='1' && s[5]==':') ||
	  strncmp(s, "logonly:", 8)==0 ||
	  strncmp(s, "debug:", 6)==0 ||
	  (cli_write(command_->fd_capture(), s, strlen(s), WRITE_MSEC)!=FAIL &&
	   cli_write(command_->fd_capture(), "\n", 1, WRITE_MSEC)!=FAIL))
      {
	 if (command_->output()==OUTPUT_DUP)
	    return &pl;
	 else
	    return check_line();
      }
      else
      {
	 log_error("write to capture pipe failed; closing it");
	 command_->fd_capture_close();
      }
   }
   return &pl;
}


PASSFAIL
ProcessPipe::signal(int signo)
{
   PASSFAIL rtn = FAIL;

   switch (signo)
   {
      case 0: break; /* This does no harm ... just tests if process still exists. */
      case SIGINT: log_debug("sending SIGINT"); break;
      case SIGTSTP: log_debug("sending SIGTSTP"); break;
      case SIGQUIT: log_debug("sending SIGQUIT"); break;
      case SIGTERM: log_debug("sending SIGTERM"); break;
      default: log_debug("sending miscellaneous signal");
   }

   switch (type)
   {
      case PIPE_TO_PROCESS:
      {
	 if (child!=-1 && child!=0)
         {
            /*
             * The "-child" means the whole process group will
             * get the signal, in order to be consistent with what
             * happens when signals are send via rcmd()
	     *
	     * %%% -child changed back to child ... why?
             */
            if ((kill(child, signo))==-1) /* %%% Document. How does fits writing get aborted now? */
	    {
               log_error("failed to signal process");
	    }
	    else
	    {
	       rtn = PASS;
	    }
         }
	 /* === else print some kind of warning */
	 break;
      }
      default:
      {
	 if (get_fd()!=-1)
	 {
#define CONTROL(c) (1-'A'+c)
	    unsigned char sigchar = 0;

	    /* === clean up */
	    if (signo==SIGINT) sigchar = CONTROL('C');
	    if (signo==SIGTSTP) sigchar = CONTROL('Z');
	    if (signo==SIGQUIT) sigchar = CONTROL('\\');
	    if (sigchar)
	       if (cli_write(child_output[PIPE_OUTPUT_SIDE], &sigchar, 1, WRITE_MSEC)==FAIL)
                  log_error("write(sigchar) failed");
	       else
		  rtn = PASS;
	    /* === else warn that we can't send that signal */
	 }
      }
   }
   return rtn;
}

PASSFAIL
ProcessPipe::send_command(CliQueueNode* command)
{
   extern const char* cli_newline; /* === */
   const char* line;
   int length;

   /*
    * This should never happen
    */
   if (command_)
   {
      log("error", "ProcessPipe::send_line() clearing pending command!");
      command_->assign(0);
      command_ = 0;
   }

   if (!command)
   {
      log("error", "ProcessPipe::send_line(NULL)!");
      return FAIL;
   }

   if (pend()==BUSY)
   {
      log("warning", "ProcessPipe::send_line can't send to BUSY agent");
   }

   command_ = command;

   if (child_input[PIPE_INPUT_SIDE]==-1)
   {
      if (roll)
      {
	 roll->add_line("error: Agent not ready for input.");
	 roll->add_line("error: Maybe \"agent restart ...\" is needed?");
      }
      return FAIL;
   }

   line = command->line();
   length = strlen(line);

   /*
    * Only for agents with a timeout: If the line ends in a backslash,
    * no newline is sent.  Allowing this for regular libcli agents
    * would cause a major problem if someone accidentally typed a '\'
    * at the end of a line because director would wait forever for
    * an answer which isn't coming because no newline was sent.  It
    * is generally safe (and sometimes needed) for agents that have
    * a timeout and take single keystroke commands.
    */
   if (timeout_isset() && length>0 && line[length-1]=='\\')
   {
      if ((*line &&
	   cli_write(child_input[PIPE_INPUT_SIDE], line, length-1, WRITE_MSEC)==FAIL))
      {
	 log_error("write(ProcessPipe) failed");
	 return FAIL;
      }
   }
   else if ((length>0 &&
	     cli_write(child_input[PIPE_INPUT_SIDE], line, length, WRITE_MSEC)==FAIL) ||
	    cli_write(child_input[PIPE_INPUT_SIDE], cli_newline, 1, WRITE_MSEC)==FAIL)
   {
      log_error("write(ProcessPipe) failed");
      return FAIL;
   }

   timeout_reinit();

   command_result = BUSY;
   
   return PASS;
}

PASSFAILBUSY
ProcessPipe::pend()
{
   if (command_result==BUSY && !running())
   {
      log("logonly", "command result was BUSY but agent gone (assuming FAIL)");
      /* commands_pending--; %%% */
      command_result = FAIL;
   }
   return command_result;
}

const char*
ProcessPipe::postmortem()
{
   /*
    * tries to conduct an autopsy on an exited process and
    * returns a string describing what happened, or 0 if the
    * process is apparently still running.
    */
   static char rtn[100];
   int status;
   int waitrtn;

   if ( child == (pid_t)-1) return 0; /* this child already gone or never existed */

   waitrtn = waitpid( child, &status, WNOHANG );
   
   if (waitrtn==0)
      return 0;
   
   if (waitrtn==-1)
   {
      log_error("waitpid failed");
      return 0;
   }
   
   if (waitrtn!=child)
   {
      log_error("unexpected return from waitpid");
      return 0;
   }
   
   command_result = FAIL;

   child = (pid_t)-1;
   /*
    * Doesn't make sense to accept any more input, but
    * don't close the output pipes because there might
    * still be output left to drain.
    */
   if (child_input[PIPE_INPUT_SIDE]!=-1)
   {
      close(child_input[PIPE_INPUT_SIDE]);
      child_input[PIPE_INPUT_SIDE]=-1;
   }

   if (!WIFEXITED(status))
   {
      if (WIFSIGNALED(status))
      {
	 switch (WTERMSIG(status))
	 {
	    case SIGTERM:
	    {
	       if (WTERMSIG(status)==SIGTERM)
		  sprintf(rtn, "warning: `%.50s' exited on termination signal",
			  get_id());
	       break;
	    }
	    case SIGINT:
	    {
	       sprintf(rtn, "error: `%.50s' aborted by user interrupt signal",
		       get_id());
	       break;
	    }
	    default:
	    {
	       sprintf(rtn, "error: `%.50s' exited on signal #%d",
		       get_id(),
		       WTERMSIG(status));
	    }
	 }
	 return rtn;
      }
      else if (WIFSTOPPED(status))
      {
	 sprintf(rtn, "error: `%.50s' was stopped but director has no job control!", get_id());
	 return rtn;
      }
      else
      {
	 sprintf(rtn, "error: child process `%.50s' has unknown problem!",
		 get_id());
	 return rtn;
      }
   }
   else
   {
      switch(WEXITSTATUS(status))
      {
	 case EXIT_SUCCESS:
	 {
	    command_result = PASS;
	    sprintf(rtn, "logonly: `%.50s' exited normally", get_id());
	    break;
	 }
	 case EXIT_FAILURE:
	 {
	    sprintf(rtn, "error: `%.50s' exited with failure", get_id());
	    break;
	 }
	 case 2:
	 {
	    sprintf(rtn, "warning: `%.50s' wrong usage", get_id());
	    break;
	 }
	 default:
	 {
	    sprintf(rtn, "error: `%.50s' exited with exitcode %d", 
		    get_id(), WEXITSTATUS(status));
	 }
      }
      return rtn;
   }
}

void
ProcessPipe::dump(int logtype)
/*
 * Agent dumps are formatted as follows:
 *
 * Agents:
 *   Handle       Status stdi/o/e Type/PID Command line
 *   ------------ ------ -------- -------- --------------------
 *   deticli2     BUSY            remote   titan:deticli2
 *   sh           PASS            zombie   sh -c ls
 *   sh           BUSY            12345    sh -c pwd
 *   temp         FAIL            device   /dev/serial
 *
 */
{
   char junkbuf[1024];
   char typebuf[30];

   if (!roll) return;

   switch (type)
   {
      case PIPE_NOT_USED:   strcpy(typebuf, "???"); break;
      case PIPE_TO_DEVICE:  strcpy(typebuf, "device"); break;
      case PIPE_TO_PROCESS:
      {
	 if (child==(pid_t)-1)
	    strcpy(typebuf, "zombie");
	 else
	    sprintf(typebuf, "%d", (int)child);
	 break;
      }
   }
   sprintf(junkbuf, "# %c%c%12.12s %s %02d/%02d/%02d %8.8s %.100s",
	   autorestart_?'R':' ',
	   default_agent()?'*':' ',
	   handle(),
	   command_result==BUSY? " BUSY ":
	   (command_result==PASS?"  OK  ":
	                         "FAILED"),
	   child_input[PIPE_INPUT_SIDE],
	   get_fd(), err.get_fd(),
	   typebuf, argv_[0]);

   for (int i=1; argv_[i]; i++)
   {
      strcat(junkbuf, " ");
      strcat(junkbuf, argv_[i]); /* === check limits */
   }
   roll->add_line(junkbuf, logtype);
}

void
ProcessList::add_read_fds( FDSet& fds )
{
   for (ProcessPipe* p = head_; p; p = p->next_)
   {
      fds.add_fd(p->get_fd());
      fds.add_fd(p->err.get_fd());
   }
}
   
int
ProcessList::killall(const char* handle)
{
   const char* colon = handle?strchr(handle, ':'):0;
   const char* h = colon?(colon+1):handle;
   ProcessPipe* p = head_;	/* Start at the head of the list */
   ProcessPipe* next = 0;	/* Save the next node */
   ProcessPipe* prev = 0;	/* And the previous if middle node deleted */
   int rtn = 0;

   while (p)
   {
      next = p->next_; /* Save pointer to next node in case we delete p */

      /*
       * Delete the node only if no handle was given or if handle matches
       */
      if (!h || !*h || !strcmp(p->handle(), h))
      {
	 p->cleanup();
	 if (p == head_)
	 {
	    /*
	     * Case 1: deleting the first node in the list.  Move head_.
	     */
	    head_ = next;
	 }
	 else
	 {
	    /*
	     * Case 2: deleting from middle of list.  Reconnect
	     * the previous node's next_ pointer.
	     */
	    if (prev) prev->next_ = next;
	 }
	 delete p; /* This causes the associated process to exit. */
	 rtn ++;
      }
      else
      {
	 prev = p; /* If nothing deleted, save the previous node */
      }
      p = next; /* And move on to the next one */
   }
   return rtn;
}

const char*
ProcessList::postmortem()
{
   const char* s;

   for (ProcessPipe* p = head_; p; p = p->next_)
   {
      if ((s = p->postmortem())) return s;
   }
   return 0;
}

ProcessPipe*
ProcessList::exec(const char* argv[], int director_level,
		  const char* handle, const char* prompts,
		  int timeout_i, int timeout_s, int default_mtype,
		  int default_agent, int autorestart)
{
   ProcessPipe* p = head_;
   char* new_handle = 0;

   /*
    * Kill off any duplicate agents with same name first.
    * Also, if the process being started now has default_agent=1,
    * remove default_agent status from any existing process
    * if needed (but don't kill it unless it had the same handle.)
    */
   while (p)
   {
      ProcessPipe* next = p->next_;
      const char* p_handle = p->handle();

      if (default_agent && p->default_agent())
      {
	 p->default_agent_ = 0;
      }

      if (handle && *handle && *p_handle && !strcmp(p_handle, handle))
      {
	 cleanup(p);
	 p = head_;
      }
      else p = next;
   }

   /*
    * Make this the default ("primary") agent if there are
    * no other agents currently running.  The default agent
    * is the one that gets restarted if someone types just
    * "agent restart" and it also gets the commands which
    * are not prefixed with an "<agentname>.", where <agentname>
    * is the handle of the desired agent.
    */
   if (count() == 0) default_agent = 1;

   p = new ProcessPipe(argv, director_level, handle, prompts,
		       timeout_i, timeout_s, default_mtype,
		       default_agent, autorestart);

   p->next_ = head_;
   if (new_handle) { free(new_handle); new_handle = 0; }
   return ((head_ = p));
}

ProcessPipe*
ProcessList::restart(const char* handle)
{
   const char* colon = handle?strchr(handle, ':'):0;
   const char* h = colon?(colon+1):handle;

   for (ProcessPipe* p = head_; p; p = p->next_)
   {
      if (!h || !*h)
      {
	 if (p->default_agent())
	 {
	    if (p->restart()==FAIL) return 0;
	    else return p;
	 }
      }
      else
      {
	 /* === strcase? fnmatch? */
	 if (!strcmp(h, p->handle()))
	 {
	    if (p->restart()==FAIL) return 0;
	    else return p;
	 }
      }
   }
   return 0; /* Not found */
}

/*
 * This checks if a ProcessPipe has completely exited
 * and all of its output has drained.
 */
#define PROCESS_COMPLETE(p) \
   ((p)->command_result!=BUSY && \
    (((p)->type==PIPE_TO_PROCESS && \
      (p)->child==-1 && (p)->get_fd()==-1 && (p)->err.get_fd()==-1) || \
     ((p)->type!=PIPE_TO_PROCESS && \
      (p)->get_fd()==-1 && (p)->err.get_fd()==-1)))

ProcessPipe*
ProcessList::autorestart()
{
   ProcessPipe* p; 

   for (p = head_; p; p = p->next_)
   {
      if (p->autorestart_ > 1 && PROCESS_COMPLETE(p))
      {
	 time_t now = time(0);

	 if (p->autorestart_ > now + RESTART_MAX * RESTART_PERIOD)
	 {
	    roll->add_line("error: agent dying too quickly; turning off autorestart");
	    roll->add_line("error: restart it manually with `agent restart <name>'"); /* === Put name in? */
	    flush_to_screen();
	    p->autorestart_ = 1;
	 }
	 else
	 {
	    if (p->autorestart_ < now) p->autorestart_ = now;
	    p->autorestart_ += RESTART_PERIOD;
	    roll->add_line("error: agent exited; autorestart in progress");
	    flush_to_screen();
	    if (p->restart() == FAIL)
	    {
	       roll->add_line("error: Restart failed; turning off autorestart");
	       p->autorestart_ = 1;
	    }
	    else return p; /* === This process is now busy. */
	 }
      }
   }
   return 0;
}

void
ProcessList::cleanup(ProcessPipe* force_exit)
{
   ProcessPipe* parent = 0;
   ProcessPipe* p = head_; 
   ProcessPipe* next;

   while (p)
   {
      next = p->next_;
      if (p==force_exit ||
	  (p->autorestart_ == 0 && p->default_agent_ == 0 &&
	   PROCESS_COMPLETE(p)))
      {
	 if (parent) parent->next_ = next;
	 else head_ = next;
	 delete p;
      }
      else parent = p;
      p = next;
   }
}


process_line_t*
ProcessList::check_line()
{
   static process_line_t pl;
   static process_line_t* plp;

   for (ProcessPipe* p = head_; p; p = p->next_)
   {
      if ((plp = p->check_line()))
      {
	 return plp;
      }
      if ((pl.line = p->err.check_line()))
      {
	 pl.pid = (int)p->childpid();
	 pl.handle = p->handle();
	 pl.mtype = p->err.get_mtype();
	 return &pl;
      }
   }
   return 0;
}

int
ProcessList::count()
{
   int c = 0;
   
   for (ProcessPipe* p = head_; p; p = p->next_) c++;
   return c;
}

PASSFAIL
ProcessList::assign(CliQueueNode* command)
{
   if (command->shell_escape())
   {
      const char** argv_ = (const char**)malloc(4*sizeof(char*));
      ProcessPipe* p;

      argv_[0] = "sh";
      argv_[1] = "-c";
      argv_[2] = (char*)malloc(5 + strlen(command->line()) + 1);
      sprintf((char*)argv_[2], "exec %s", command->line());
      argv_[3] = 0;
      if (roll)
      {
	 char junkbuf[1024]; /* === clean these up */
	 sprintf(junkbuf, "%s%.1000s",
		 command->script_dir()?"> ":"> !",
		 command->line());
	 roll->add_line(junkbuf,
			command->no_echo()?CLI_LOG_HIDDEN:CLI_LOG_PROMPT);
      }
      p = exec(argv_, command->level()+1);
      p->assign_command(command);
      command->assign(p);
      free((char*)argv_[2]);
      free((char**)argv_);
      return PASS; /* === detect failure of restart() somehow */
   }
   else
   {
      ProcessPipe* p = head_;

      if (!p)
      {
	 roll->add_line("No agents are running.  Type `builtin' for help on director.",
			CLI_LOG_ERROR);
	 return FAIL;
      }

      /*
       * Check for multiple and try to match the handle
       */
      while (p)
      {
	 if (p->default_agent() && !*(command->handle())) break;
	 if (!strcmp(command->handle(), p->handle())) break;
	 if (p->next_) { p = p->next_; continue; }
	 if (*(command->handle()))
	 {
	    char junkbuf[128];

	    sprintf(junkbuf, "There is no agent running with the handle `%.80s'", command->handle());
	    roll->add_line(junkbuf, CLI_LOG_ERROR);
	    return FAIL;
	 }
	 break;
      }

      if (p->send_command(command)==PASS)
      {
	 command->assign(p);
	 return PASS;
      }
      else return FAIL;
      /* %%% Does cleanup work right? */
   }
}


PASSFAIL
ProcessList::signal(int signo)
{
   ProcessPipe* p = head_;
   PASSFAIL rtn = PASS;

   if (!p)
   {
      roll->add_line("No agents are running.  Type `builtin' for help on director.",
			CLI_LOG_ERROR);
      return FAIL;
   }

   while (p)
   {
      if (p->default_agent() || p->pend()==BUSY)
      {
	 /*
	  * %%% p->pend()==BUSY means the signal gets sent to *any*
	  * agent that was busy running a command.  However, if we
	  * get around to implementing a way of having certain commands
	  * execute asychronously and "in the background", we may not
	  * want these agents to see the signal.  We will probably
	  * want to add something to this *if* statement so that it
	  * skips agents currently running an asynchronous command.
	  */
	 if (p->signal(signo)==FAIL) rtn = FAIL;
      }
      p = p->next_;
   }
   return rtn;
}


void
ProcessList::dump(int logtype)
{
   if (roll)
   {
      int primary = 0;
      ProcessPipe* p = head_;

      roll->add_line("# ProcessList:", logtype);
      
      roll->add_line("#         Handle Status stdi/o/e Type/PID Command line", logtype);
      roll->add_line("#   ------------ ------ -------- -------- -----------------", logtype);
      while (p)
      {
	 p->dump(logtype);
	 primary += p->default_agent();
	 p = p->next_;
      }
      roll->add_line("#   ____________ ______ ________ ________ _________________", logtype);
      if (primary)
      {
	 roll->add_line("#   (*) Indicates primary agent.", logtype);
	 roll->add_line("#   (R) Indicates auto-Restart.", logtype);
      }
      else
      {
	 roll->add_line("#   No primary agent is currently running!", logtype);
      }
   }
}
      

/* end of Pipes.cc */
