/*                                             -*- c-file-style: "Ellemtel" -*-

`Roll.cc' - See Roll.hh

This file is part of version 3 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef CYGWIN32
#define USE_SHM
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#ifdef USE_SHM
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

#include "Roll.hh"

#include "cli/cli.h" /* %%% Move to clidefs? */
#include "cli/clidefs.h"

void do_sleep(int seconds); /* temporarily disable the interval timer and do
			     * a regular sleep 
			     */

#define ADVANCE(ptr) (ptr = (ptr+1)%ROLL_LINES)

void
Roll::init_RollLine(RollLine& r)
{
   r.str[0]='\0'; r.str[LINE_WIDTH]='\0';
   r.serial = 0; r.length = 0;
}
      
void
Roll::set_RollLine(RollLine& r, const char* s, int eatspace=0)
{
   // === Clones watching this may get confused during fast updates
   if (eatspace) if (*s==' ') s++;			// eat 1 space
   r.length = strlen(s);				// get length
   if (r.length > LINE_WIDTH) r.length=LINE_WIDTH;	// trim if too long

   strncpy(r.str, s, r.length);
   r.str[r.length] = '\0';
   r.serial++;
}

void
Roll::set_RollPtr(const char* s, int logtype, int eatspace, const char* handle)
{
   int next = (shm->tail+1)%ROLL_LINES;
   RollPtr& r = shm->roll[next];
   
   time(&r.timestamp);					// save approximate time

   if (eatspace) if (*s==' ') s++;			// eat 1 space

   if (!handle || !*handle) handle="director"; // === speed up?

   r.length_handle = strlen(handle) + 2;		// +2 for the ':' and ' '
   if (r.length_handle > LINE_WIDTH-1) 			// trim if too long
      r.length_handle=LINE_WIDTH-1;
   r.length = r.length_handle + strlen(s);		// get length of whole msg
   if (r.length > LINE_WIDTH) r.length=LINE_WIDTH;	// trim if too long

   if (logtype==ERROR_LOG)
   {
      fputc('\a', stderr);	// ring bell for errors
      scrollback_cancel();	// and make sure it is visible
   }

   /*
    * Advance the head pointer if wrapped all the way
    * around and filled all the slots in roll[]
    */
   if (next==shm->head) ADVANCE(shm->head);

   /*
    * Find place in buf[] to start writing the new string
    */
   if (r.length+1 > ROLL_MEM - shm->nextofs)
      /*
       * If it doesn't fit at the end, clear any remaining
       * entries that were using the last part of buf[] and
       * write the new string at buf[0].
       */
   {
      while (shm->head != shm->tail &&
	     shm->roll[shm->head].offset >= shm->nextofs)
	 ADVANCE(shm->head);
      shm->nextofs = 0;
   }
   r.offset = shm->nextofs;
   /*
    * Save new offset that will be used on next call to set_RollPtr()
    */
   shm->nextofs += r.length+1;

   /*
    * Find out how many old messages were clobbered
    */
   while (shm->head != shm->tail &&
	  r.offset   <= shm->roll[shm->head].offset &&
	  shm->nextofs > shm->roll[shm->head].offset)
      ADVANCE(shm->head);
   
   /*
    * Make sure the head string is still always an empty line
    */
   shm->roll[shm->head].length = 0;
   shm->roll[shm->head].length_handle = 0;
   shm->roll[shm->head].logtype = -1;
   shm->buf[shm->roll[shm->head].offset] = '\0';

   /*
    * Copy the string into place and save other records
    */
   memcpy(&(shm->buf[r.offset]), handle, r.length_handle - 2);
   memcpy(&(shm->buf[r.offset+r.length_handle-2]), ": ", 2);
   memcpy(&(shm->buf[r.offset+r.length_handle]), s, r.length - r.length_handle);
   shm->buf[r.offset + r.length] = '\0';
   r.logtype = logtype;
   
   /*
    * Don't increment tail until the data is actually there,
    * or else clones could read the line prematurely.
    */
   if (next!=-1) shm->tail = next;
}


Roll::Roll(int roll_key, clone_mode_t clone_mode)
{
   /*
    * Constructor for Roll class attaches to the shared memory block
    * If must_clone is zero, Roll will automatically determine if it
    * is a clone monitoring another director's RollShm, or if it should
    * actually allocate it.  If must_clone is non-zero, having to
    * allocate the Shm is treated as an error.
    */
   int i;

   sback_ = 0;

#ifdef USE_SHM
   int shmkey, shmflg;

   switch (roll_key)
   {
      /* %%% Define this instead of using -1 and -2 */
      case -1: shmkey = IPC_PRIVATE; break;
      case 0:  shmkey = 1024; break; /* Special case for root */
      default: shmkey = roll_key;
         /* shmkey and roll_key are usually the same, but
          * shmkey may get hashed to a different value if
          * there is a collision with some other program.
          */
   }

   /*
    * Do not allow "must_clone" programs (director -k) to have write
    * access or ability to create the shared memory segment
    */
   if (clone_mode == CLONE_ONLY) shmflg = 00444;
   else shmflg = 00644|IPC_CREAT;

   int tries = 50;
   do
   {
      /*
       * Get the id for the correct shared memory block.
       */
      while (shmkey!=-2 &&
	     (shmid = shmget(shmkey, sizeof(RollShm), shmflg))==-1)
      {
	 shmkey += 1024; /* hash to different slot if collision */
	 if (shmkey == roll_key || !tries--) shmkey=-2; /* hash table full? */
      }
      
      if (shmid==-1)
      {
	 perror("shmget");
	 fprintf(stderr,
		 "User id %d does not appear to be running director on this host.\n",
		 roll_key);
	 exit(EXIT_FAILURE);
      }
      
      switch (nattch())
      {
	 case -1: perror("shmctl"); exit(EXIT_FAILURE);
	 case 0: clone_=0; break;
	 default:
	 {
	    if (clone_mode==MASTER_ONLY)
	    {
#ifndef NO_CFHTLOG
	       cfht_logv(CFHT_MAIN, CFHT_ERROR,
			 "User id %d is already running director", roll_key);
#endif
	       fprintf(stderr,
		       "User id %d is already running director", roll_key);
	       exit(EXIT_FAILURE);
	    }
	    clone_=1;
	 }
      }
      
      /*
       * Got a valid id.  Now attach the memory segment
       */
      if ((shm =
	   (RollShm*)shmat(shmid, 0, SHM_RDONLY*clone_))
	  ==(void*)-1)
      {
	 perror("shmat");
	 exit(EXIT_FAILURE);
      }
      if (!clone_) shm->roll_key = roll_key;
   } while (shm->roll_key != roll_key);
#else
   if (must_clone)
   {
      fprintf(stderr,"No shared memory support!\n");
      exit(EXIT_FAILURE);
   }
   clone_ = 0;
   shm = (RollShm*)malloc(sizeof(RollShm));
#endif

   if (!clone_)
      /*
       * Initialize the shared memory RollShm structure.
       */
   {
      for (i=0; i<INFO_LINES; i++) init_RollLine(shm->info[i]);
      init_RollLine(shm->statusbar);
      init_RollLine(shm->titlebar);
      init_RollLine(shm->instant);
      init_RollLine(shm->progress);
      
      shm->magic = DIRECTOR_MAGIC;
      shm->version = 3; /* === don't forget to update! */
      shm->tail = 0;
      shm->head = 0;
      shm->roll[0].offset = 0;
      shm->roll[0].length = 0;
      shm->roll[0].length_handle = 0;
      shm->roll[0].timestamp = 0;
      shm->roll[0].logtype = -1;
      shm->buf[0] = '\0'; /* head string is always an empty line */
      shm->nextofs = 1; /* first real string will be written at buf[1] */
      shm->prompt_state = 'I';
      memset(shm->roll[0].reserved, 0, sizeof(shm->roll->reserved));
      memset(shm->reserved, 0, sizeof(shm->reserved));
   }
   statusbar_serial_disp = 0;
   rewind(50); /* catch up by showing the last 50 lines */
}

Roll::~Roll()
{
#ifdef USE_SHM
   /*
    * Destructor for roll class detaches from shared memory
    * If there are clones, the master process also waits for
    * clones to exit before removing the shared memory id.
    */
   if (!clone_) shm->time_to_exit = 1; /* signal clones it's time to exit */
   
   if (shmdt((char*)shm)==-1)
      perror("shmdt");
   
   if (!clone_)
   {
      int tries=10;
      
      while (nattch()>0 && tries--) do_sleep(1);
      if (nattch()!=0)
	 fprintf(stderr,"warning: problem detaching shared memory!\n");
      
      if (shmctl(shmid, IPC_RMID, 0)==-1)
	 perror("shmctl");
   }
#else
   if (shm) { free(shm); shm = 0; }
#endif
}

int
Roll::is_clone()
{
   /*
    * Returns 0 if master process
    *         1 if clone and master process is still running
    *        -1 if clone and master process is trying to exit
    */
   return (shm->time_to_exit)?-1:clone_;
}

int
Roll::nattch()
{
   /*
    * Get number of processes attached to shm segment.  Returns -1
    * on error from shmctl()
    */
#ifdef USE_SHM
   struct shmid_ds shmbuf;
   
   if (shmctl(shmid, IPC_STAT, &shmbuf)==-1) return -1;
   else return shmbuf.shm_nattch;
#else
   return 1;
#endif
}

void
Roll::scrollback_change(int lines, int delta)
{
   int overflow;

   if (shm->time_to_exit || 
       disp!=(shm->tail+ROLL_LINES-sback_)%ROLL_LINES) return;
   /* %%% BUG: gets confused if you scroll back too quickly? */

   if (sback_==0 && delta>0) return;
   sback_ -= delta;
   if (sback_ < 0) { delta += sback_; sback_ = 0; }
   overflow = rewind(-(lines-delta)) - lines/2;
   if (overflow > 0) sback_ -= overflow;
   if (sback_ < 0) sback_ = 0;
}

/*
 * %%% Scrollback message gets left on screen sometimes 
 */
int
Roll::rewind(int lines)
{
   /*
    * Moves the disp index back so that get_line() will
    * return `lines' previous lines from the current position.
    * May go back more than `lines' if there are debug messages
    * inbetween.
    */
   int maxlines = ROLL_LINES/2;
   int logtype;
   int overflow = 0;
      
   if (lines < 0)
      lines *= -1;
   else
      if (lines > 0) disp = shm->tail;

   while (maxlines-- && lines)
   {
      disp = (disp+ROLL_LINES-1)%ROLL_LINES;
      if ((shm->head<=shm->tail && shm->head<=disp && disp<=shm->tail) ||
	  (shm->head>shm->tail && !(shm->tail<disp && disp<shm->head)))
      {
	 /*
	  * valid position
	  */
	 logtype = shm->roll[disp].logtype;
	 if (logtype != DEBUG_LOG && logtype != LOGONLY_LOG && logtype != CLI_LOG_HIDDEN) lines--;
      }
      else
      {	
	 lines--;
	 overflow++;
      }
   }
   titlebar_serial_disp = shm->titlebar.serial - 1;
   instant_serial_disp = shm->instant.serial - 1;
   progress_serial_disp = shm->progress.serial;
   if (statusbar_serial_disp!=0)
      statusbar_serial_disp = -1;	// leave 0 i to avoid initialization
   // of statusbar.  Statusbar only appears
   // when first true "statusbar:" message arrives.
   for (int i=0; i<9; i++) info_serial_disp[i] = shm->info[i].serial - 1;
   
   return overflow;
}

RollPtr*
Roll::get_line()
{
   if (shm->time_to_exit || disp==(shm->tail+ROLL_LINES-sback_)%ROLL_LINES) return 0;
   
   disp++;
   disp %= ROLL_LINES;
   if ((shm->head<=shm->tail && shm->head<=disp && disp<=shm->tail) ||
       (shm->head>shm->tail && !(shm->tail<disp && disp<shm->head)))
   {
      /*
       * valid position
       */
      return &shm->roll[disp];
   }
   else
   {
      /*
       * out of range, return the empty string in head
       */
      return &shm->roll[shm->head];
   }
}

const char*
Roll::get_str(RollPtr* rp)
{
   if (!rp) return NULL;
   else return &(shm->buf[rp->offset]);
}
   
RollLine*
Roll::get_statusbar()
{
   if (statusbar_serial_disp!=shm->statusbar.serial)
   {
      statusbar_serial_disp = shm->statusbar.serial;
      return &shm->statusbar;
   }
   else return 0;
}

RollLine*
Roll::get_titlebar()
{
   if (titlebar_serial_disp!=shm->titlebar.serial)
   {
      titlebar_serial_disp = shm->titlebar.serial;
      return &shm->titlebar;
   }
   else return 0;
}

RollLine*
Roll::get_instant()
{
   /*
    * Returns a pointer to the "instant" line if changed, else 0.
    */
   if (instant_serial_disp!=shm->instant.serial)
   {
      instant_serial_disp = shm->instant.serial;
      return &shm->instant;
   }
   else return 0;
}


RollLine*
Roll::get_progress()
{
   /*
    * Returns a pointer to the "progress" line if changed, else 0.
    */
   if (progress_serial_disp!=shm->progress.serial)
   {
      progress_serial_disp = shm->progress.serial;
      return &shm->progress;
   }
   else return 0;
}


RollLine*
Roll::get_info(int n)
{
   /*
    * Returns a pointer to the n-th info line if changed, else 0.
    * n should be in the range 1..INFO_LINES
    */
   n--;
   if (n>=INFO_LINES || n<0) return 0;
   if (info_serial_disp[n]!=shm->info[n].serial)
   {
      info_serial_disp[n] = shm->info[n].serial;
      return &shm->info[n];
   }
   else return 0;
}

void
Roll::add_line(const char* s, int logtype, int nolog, const char* handle, int pid)
{
   const char* ps = s;

   if (clone_!=0) return; /* === clones are not allowed to write to the roll */

   /*
    * First check for message types that go to special screen areas.
    * If found, next is set to -1 to indicate that the feedback roll
    * should not be advanced at the end of add_line(), and nolog is
    * set to 1 to indicate this message should not go to cfht_log().
    */
   
   if (strncmp(s, "statusbar:", 10)==0)
   { set_RollLine(shm->statusbar, s+10); return; }
   
   else if (strncmp(s, "titlebar:", 9)==0)
   { set_RollLine(shm->titlebar, s+9, 1); return; }
   
   else if (strncmp(s, "instant:", 8)==0)
   { set_RollLine(shm->instant, s+8, 1); return; }

   else if (strncmp(s, "progress:", 9)==0)
   { set_RollLine(shm->progress, s+9, 1); return; }
   
   else if (strncmp(s, "info", 4)==0 && s[4]>='1' && s[5]==':')
   { set_RollLine(shm->info[(s[4]-'1')%INFO_LINES], s+6, 1); return; }
   
   /*
    * The rest go to the feedback area, displayed with various colors.
    */
   else if (strncmp(s, "Z&v2S", 5)==0) // unhack hpterm escape sequence. yuk.
   { ps = s+5; logtype=ERROR_LOG; }
   
   else if (strncmp(s, ERROR_KEY, ERROR_KEY_LEN)==0)
   { ps = s+ERROR_KEY_LEN; logtype=ERROR_LOG; }
   
   else if (strncmp(s, "Z&v3S", 5)==0)
   { ps = s+5; logtype=WARNING_LOG; }
   
   else if (strncmp(s, WARNING_KEY, WARNING_KEY_LEN)==0)
   { ps = s+WARNING_KEY_LEN; logtype=WARNING_LOG; }
   
   else if (strncmp(s, STATUS_KEY, STATUS_KEY_LEN)==0)
   { ps = s+STATUS_KEY_LEN; logtype=STATUS_LOG; }
   
   else if (strncmp(s, START_KEY, START_KEY_LEN)==0)
   { ps = s+START_KEY_LEN; logtype=START_LOG; }
   
   else if (strncmp(s, DONE_KEY, DONE_KEY_LEN)==0) /* %%% Use the stuff in clidefs.h? */
   { ps = s+DONE_KEY_LEN; logtype=DONE_LOG; }
   
   else if (strncmp(s, LOGONLY_KEY, LOGONLY_KEY_LEN)==0)
   { ps = s+LOGONLY_KEY_LEN; logtype=LOGONLY_LOG; }
   
   else if (strncmp(s, DEBUG_KEY, DEBUG_KEY_LEN)==0)
   { ps = s+DEBUG_KEY_LEN; logtype=DEBUG_LOG; }

   else if (logtype!=-1)
   { }

   else if (*s=='*')
   { logtype=CLI_LOG_HIGHLIGHT; nolog=1; } /* Don't log "say" messages. */

   else
   { logtype=CLI_LOG_NORMAL; } /* Save prompts, "white" message, etc. */

   /*
    * Strip "handle:" off of messages that came in through cfht_log,
    * but only if they match the expected handle (it is possible that
    * a program exec's another program and director then picks up
    * messages with a different handle.)
    */
   {
      int handle_len = handle?strlen(handle):0;
      
      while (*ps == ' ') ps++;
      if (handle && !strncmp(ps, handle, handle_len) && ps[handle_len] == ':')
	 ps += handle_len + 1;
      if (!strncmp(ps, "starting:", 9)) /* libcfht puts this on for userlog */
	 ps += 9;
   }

   set_RollPtr(ps, logtype, /*eatspace=*/1, handle);
   /*
    * Don't log messages to cfht_log that came from there in the
    * first place, and don't log messages that weren't of type
    * start/done/error/warning/status/logonly/debug.
    */
#ifndef NO_CFHTLOG
   if (!nolog)
   {
      cfht_log_proxy(CFHT_MAIN,
		     (logtype>=0?logtype:LOGONLY_LOG),
		     (char*)(*ps==' '?ps+1:ps), (char*)handle, pid);
   }
#else
   if (pid) ; /* Avoid gcc warnings */
#endif
}

void
Roll::instant_off()
{
   if (shm->instant.length) set_RollLine(shm->instant, "");
}

void
Roll::set_infosize(int lines)
{
   if (!is_clone()) shm->infosize = lines;
}

int
Roll::get_infosize()
{
   return shm->infosize;
}

void
Roll::set_prompt_state(char p)
{
   if (!is_clone()) shm->prompt_state = p;
}

char
Roll::get_prompt_state(void)
{
   register char c = shm->prompt_state;

   return c?c:'>';
}

/* end of Roll.cc */
