/*                                             -*- c-file-style: "Ellemtel" -*-

`builtins.cc' - builtin commands that director handles itself

This file is part of version 3 of the DIRECTOR cli wrapper.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include "cli/cli.h"

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0
#endif

BOOLEAN director_debug = TRUE;  /* Startup will initialize this */

ProcessPipe* newProcess = 0; /* %%% */

/*
 * Commands
 *
 */

static PASSFAIL
com_clock( const char* arg ) {
   BOOLEAN f = cli_bool(arg);
   curs->titleclock(f);
   return PASS;
}

static PASSFAIL
com_jumpscroll( const char* arg )
{
   BOOLEAN f = cli_bool(arg);
   curs->jumpscroll(f);
   return PASS;
}

static PASSFAIL
com_debug( const char* arg )
{
   director_debug = cli_bool(arg);
   curs->debug(director_debug);
   roll->rewind(LINES*2);
   return PASS;
}

static PASSFAIL
com_verbose( const char* arg ) 
{
   curs->verbose(cli_bool(arg));
   roll->rewind(LINES*2); 
   return PASS;
}

static PASSFAIL
com_infosize( const char* arg )
{
   int lines = atoi(arg);
   
   if (lines < 0) return FAIL;
   if (lines > INFO_LINES) lines = INFO_LINES;
   curs->infosize(lines);
   roll->set_infosize(lines); /* This does nothing in clones */
   roll->rewind(0);
   return PASS;
}

static PASSFAIL
com_builtin( const char* arg )
{
   if (arg);
   /* === can't use cli help because can't print to stdout here */
   roll->add_line("");
   roll->add_line("*---[ Builtin Director Commands ]-----------------------------------------*");
   roll->add_line("termhelp      : display help screen for operating this terminal window");
   roll->add_line("debug on|off  : toggles debug output (also requires verbose for displaying)");
   roll->add_line("verbose on|off: toggles displaying of verbose messages");
   roll->add_line("clock on|off  : controls display of titlebar clock");
/* roll->add_line("jumpscroll    : set to on for (really) slow terminal emulators"); === */
   roll->add_line("infosize N    : displays N lines of the info panel (N=0..9)");
   roll->add_line("cancel (or ^G): cancels queued but not yet executing commands");
   roll->add_line("abort  (or ^C): sends SIGINT to the agent");
   roll->add_line("stop   (or ^Z): sends SIGTSTP to the agent");
   roll->add_line("quit   (or ^|): sends SIGQUIT to the agent");
   roll->add_line("sigsend [NAME]: sends SIG[NAME] to the agent");
   roll->add_line("ready         : immediately returns pass if ready for command, fail if busy");
   roll->add_line("running       : immediately returns pass unless director is shutting down");
   roll->add_line("say [message] : enter chat mode ([message] gets broadcast to other clones)");
   roll->add_line("agent ...     : starts, restarts, or stops an agent running in director");
   roll->add_line("clidump       : displays the tree of commands waiting to be executed");
   roll->add_line("autorestart on|off: tells director whether or not to restart agents that die");
   roll->add_line("source <file> : reads batch commands from <file>");
   roll->add_line("*-------------------------------------------------------------------------*");
   return PASS;
}

static PASSFAIL
com_termhelp( const char* arg )
{
   if (arg);
   roll->add_line("*---[ Terminal Window Tips ]----------------------------------------------*");
   roll->add_line(" o Resizing window will cause feedback area to fill available space");
   /*
    * These might only apply to rxvt
    */
   roll->add_line(" o Shift-numpad-plus and Shift-numpad-minus change font size (rxvt only)");
   roll->add_line(" o Shift-page-up and Shift-page-down scroll back the window (rxvt only)");

   roll->add_line(" o Up-arrow and Down-arrow allow editing/repeating previous commands");
   roll->add_line(" o Other cursor keys (home, end, left, right) do their thing");
   roll->add_line(" o Most commonly used emacs bindings work also (but not ^Y!)");
   roll->add_line(" o Command/file completion (usually bound to esc/tab) NOT implemented");
   roll->add_line(" o Pressing Control-L will clear and redraw the entire window");
   roll->add_line("*-------------------------------------------------------------------------*");
   return PASS;
}

static PASSFAIL
com_license( const char* arg )
{
   /*
    * WARNING: Do not remove this command, modify the text or alter the
    * license conditions without prior permission from the original author!
    *
    * See also: Start of "INPUT LOOP" in director.cc
    */
   if (arg);
   roll->add_line("  Director version "VERSION_STRING" is written by Sidik Isani <isani@cfht.hawaii.edu> for the Canada-France-Hawaii Telescope.  It is free software;  you can redistribute it and/or modify it under the terms of the GNU General Public License, version 2, as published by the Free Software Foundation.  If you do modify the program, please make sure that this is clearly indicated at startup on any copies that you redistribute.  Please read the GPL for more information.");
   roll->add_line("");
   roll->add_line("  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.");
   roll->add_line("");
   roll->add_line("  You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.");
   roll->add_line("");
   roll->add_line("  Official versions of this program, including the source code, can be obtained at http://software.cfht.hawaii.edu/director/ or send email to <daprog@cfht.hawaii.edu>.");
   /*
    * WARNING: Do not remove this command, modify the text or alter the
    * license conditions without prior permission from the original author!
    */
   /* === TODO: Put entire GPL here instead? */
   return PASS;
}

static PASSFAIL
com_clidump( const char* args )
{
   if (args) ;

   cliQueue.dump(CLI_LOG_SUPPRESS);
   return PASS;
}

static const char*
agent_handle( const char* arg )
   /*
    * Converts an argument (could be a full path to
    * a versioned executable) to just the handle-name.
    * For example, all of the following will all return "directest"
    * arg="directest"
    * arg="directest-2.12"
    * arg="/cfht/bin/directest"
    * arg="/cfht/bin/directest-2.12"
    */
{
   static char returnbuf[1024]; /* %%% Pass in a buffer instead? */
   const char* colon;
   const char* brace;
   const char* slash;
   const char* dash;
   const char* handle;

   if (!arg) return 0;

   colon = strchr(arg, ':');
   if (colon) handle = colon+1;
   else handle = arg;

   /* 
    * Strip path name off front and version number off end if needed.
    */
   if ((brace = strchr(handle, '}')))
      handle = brace + 1;
   if ((slash = strrchr(handle, '/')))
      handle = slash + 1;
   if ((dash = strrchr(handle, '-')) &&
       '0' <= dash[1] && dash[1] <= '9')
   {
      char* q;
      
      strncpy(returnbuf, handle, sizeof(returnbuf)-1);
      returnbuf[sizeof(returnbuf)-1] = '\0';
      q = strrchr(returnbuf, '-');
      *q = '\0';
      return returnbuf;
   }
   return handle;
}

static PASSFAIL
com_agent( const char* args )
{
   static char** cargv = NULL;
   static int cargc = 0;
   const char* operation;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   operation = cargv[0];

   /*
    * > agent restart [handlename]
    */
   if (cargc && strcmp(operation, "restart")==0)
   {
      const char* handle;

      if (cargc > 2)
      {
	 roll->add_line("error: Syntax: agent restart [handlename]");
	 return FAIL;
      }

      if (processList.count() == 0)
      {
	 roll->add_line("error: There are currently no agents running");
	 return FAIL;
      }

      if (cargc > 1)
      {
	 handle = agent_handle(cargv[1]);
	 if (strcmp(handle, cargv[1]))
	 {
	    /*
	     * "agent restart ..." is really supposed to be used with
	     * a <handle> as the last argument.  If, instead, it looks
	     * like someone put a full path to a program, it is better
	     * to treat this as an "agent start ..." command.
	     */
	    operation = "start";
	 }
	 flush_inputs();
	 roll->add_line("warning: restarting agent . . .");
	 flush_to_screen();
      }
      else
      {
	 handle = 0;
	 flush_inputs();
	 roll->add_line("warning: restarting primary agent . . .");
	 flush_to_screen();
      }

      if (strcmp(operation, "start"))
      {
	 if ((newProcess = processList.restart(handle))==0)
	 {
	    roll->add_line("error: unable to restart connection");
	    return FAIL;
	 }
	 return BUSY; /* See comments near "return BUSY" for agent start. */
      }
   }

   /*
    * > agent start [options] [host:]agentname
    */
   if (cargc && strcmp(operation, "start")==0)
   {
      BOOLEAN recognized = TRUE;
      int cargopt = 0;
      int mtype = default_mtype;
      const char* handle = 0;
      const char* prompts = default_prompts;
      int timeout_init = default_timeout_init;
      int timeout_sust = default_timeout_sust;
      int default_agent = 0;
      int autorestart = 0;

      /*
       * Not ok to use getopt here!  Eat all recognized options and
       * assume the first non-recognized one is the program.
       */
      
      while (recognized && ++cargopt < cargc)
      {
	 if (!strcmp(cargv[cargopt], "-c"))
	 { /* %%% Not implemented. */ }
	 else if (!strcmp(cargv[cargopt], "-R"))
	    autorestart = 1;
	 else if (!strcmp(cargv[cargopt], "-D"))
	    default_agent = 1;
	 else if (!strcmp(cargv[cargopt], "-T"))
	 {
	    char* colon = 0;

	    if (++cargopt < cargc) colon = strchr(cargv[cargopt], ':');
	    if (!colon)
	    {
	       roll->add_line(
		  "error: agent start -T option requires "
		  "'initial:sustained' values (milliseconds)\n");
	       return FAIL;
	    }
	    timeout_init = atoi(cargv[cargopt]);
	    timeout_sust = atoi(colon+1); /* %%% */
	 }
	 else if (!strcmp(cargv[cargopt], "-H"))
	 {
	    if (++cargopt < cargc) handle = cargv[cargopt];
	 }
	 else if (!strcmp(cargv[cargopt], "-P"))
	 {
	    if (++cargopt < cargc) prompts = cargv[cargopt];
	    if (!prompts || 
		strlen(prompts)<2 || *prompts != prompts[strlen(prompts)-1])
	    {
	       roll->add_line(
		  "error: agent start -P option takes a list of prompt "
		  "strings separated by a delimiter character.  This "
		  "delimiter character should appear at the beginning "
		  "AND end of the argument to -P and between each possible "
		  "prompt string.  Choose any delimiter character that does "
		  "not appear in any of your prompts.  For example: "
		  "'agent start -P /ok>/done>/'");
	       return FAIL;
	    }
	 }
	 else if (!strcmp(cargv[cargopt], "-M"))
	 {
	    char* mname = 0;
	    if (++cargopt < cargc) mname = cargv[cargopt];

	    if (!strcmp(mname, "debug")) mtype = CLI_LOG_DEBUG;
	    else if (!strcmp(mname, "logonly")) mtype = CLI_LOG_VERBOSE;
	    else if (!strcmp(mname, "status")) mtype = CLI_LOG_STATUS;
	    else if (!strcmp(mname, "warning")) mtype = CLI_LOG_WARNING;
	    else if (!strcmp(mname, "error")) mtype = CLI_LOG_ERROR;
	    else
	    {
	       roll->add_line(
		  "error: agent start -M option requires an argument "
		  "with the default message type for feedback from this "
		  "agent.  Valid message types are `debug' `logonly' "
		  "`status' `warning' and `error'.");
	       return FAIL;
	    }
	 }
	 else
	    recognized = FALSE;
      }

      if (cargopt >= cargc)
      {
	 roll->add_line("error: agent start requires [hostname:]agentname");
	 return FAIL;
      }

      /*
       * Guess the handle from the program name if no -H option.
       */
      if (handle == 0) handle = agent_handle(cargv[cargopt]);

      /*
       * Start the new agent process.
       */
      newProcess =
	 processList.exec((const char**)&cargv[cargopt], 1000, /* %%% director_level? */
			  handle, prompts, timeout_init, timeout_sust, mtype,
			  default_agent, autorestart);
      if (!newProcess) return FAIL;
      return BUSY; /* === Special case: this command returns BUSY
		    * to indicate that director should not be
		    * ready for commands until the new agent
		    * has displayed its first prompt.  The caller
		    * (director.cc) must check for this BUSY and
		    * look in the global variable "newProcess" to
		    * decide which process to re-assign the agent start
		    * command.
		    */
   }

   /*
    * > agent list
    */
   if (cargc && strcmp(operation, "list")==0)
   {
      if (cargc != 1)
      {
	 roll->add_line("error: Syntax: agent list");
	 return FAIL;
      }
      processList.dump(CLI_LOG_SUPPRESS);
      return PASS;
   }

   /*
    * > agent stop [handlename]
    */
   if (cargc && strcmp(operation, "stop")==0)
   {
      if (cargc != 2)
      {
	 roll->add_line("error: Syntax: agent stop <handlename>");
	 return FAIL;
      }
      if (processList.killall(agent_handle(cargv[1])) < 1)
      {
	 roll->add_line("error: No processes matching that handle.");
	 return FAIL;
      }
      flush_inputs();
      roll->add_line("warning: Agent now stopped.");
      flush_to_screen();
      return PASS;
   }

   roll->add_line("error: Syntax: agent restart|start|stop|list [host:]program");
   return FAIL;
}

static PASSFAIL
com_say(const char* arg)
{
   static char msgbuf[2000]; /* === */
   static char* arg1 = 0;
   struct passwd* pwd = 0;

   pwd = getpwuid(getuid());

   if (arg!=0) free(arg1);
   arg1 = cli_arg1(arg);

   sprintf(msgbuf, "@say_ * %s: ``%s''", pwd?pwd->pw_gecos:"Unknown", arg1);
   commandPipe->send_line(msgbuf); /* %%% krusty */
   return PASS;
}

static PASSFAIL
com_say_(const char* arg)
{
   /*
    * Internal version of say command that expects the User Name: ... to
    * be appended to the string.
    */
   roll->add_line(arg);
   return PASS;
}

static Command local[] = {
   { "clock",     com_clock,     "[on|off] controls titlebar clock" },
   { "debug",     com_debug,     "[on|off] shows/hides debug messages" },
   { "verbose",   com_verbose,   "[on|off] shows/hides extra messages" },
   { "infosize",  com_infosize,  "[n] shows n lines of info panel" },
   { "jumpscroll",com_jumpscroll,"[on|off] turns on/off jumpscroll for slow terminals" },
   { "builtin", com_builtin, "shows list of builtin commands" },
   { "termhelp",com_termhelp,"display terminal window tips" },
   { "license", com_license, "displays warranty and copying information" },
   { "say",       com_say,       "send message to other users" },
   { "Say",	  com_say,	 "Synonym for say" },
   { "say,",	  com_say,	 "Synonym for say" },
   { "Say,",	  com_say,	 "Synonym for say" },
   { 0, 0, 0 }
};

static class SigNames
{
   public:
      const char* signames[NSIG+1];

      SigNames() {
	 for (int s=1; s<=NSIG; s++) signames[s]=0;
	 
	 signames[SIGILL] = "ILL";
	 signames[SIGTRAP] = "TRAP";
	 signames[SIGABRT] = "ABRT";
#ifdef SIGIOT
	 signames[SIGIOT] = "IOT";
#endif
#ifdef SIGEMT
	 signames[SIGEMT] = "EMT";
#endif
	 signames[SIGFPE] = "FPE";
	 signames[SIGKILL] = "KILL";
#ifdef SIGSYS
	 signames[SIGSYS] = "SYS";
#endif
#ifdef SIGPWR
	 signames[SIGPWR] = "PWR";
#endif
#ifdef SIGVTALRM
	 signames[SIGVTALRM] = "VTALRM";
#endif
#ifdef SIGPROF
	 signames[SIGPROF] = "PROF";
#endif
#ifdef SIGIO
	 signames[SIGIO] = "IO";
#endif
#ifdef SIGWINCH
	 signames[SIGWINCH] = "WINCH";
#endif
	 signames[SIGSTOP] = "STOP";
	 signames[SIGTSTP] = "TSTP";
	 signames[SIGCONT] = "CONT";
	 signames[SIGTTIN] = "TTIN";
	 signames[SIGTTOU] = "TTOU";
#ifdef SIGURG
	 signames[SIGURG] = "URG";
#endif
#ifdef SIGLOST
	 signames[SIGLOST] = "LOST";
#endif
	 signames[SIGBUS] = "BUS";
	 signames[SIGCHLD] = "CHLD";
	 signames[SIGTERM] = "TERM";
	 signames[SIGUSR2] = "USR2";
	 signames[SIGUSR1] = "USR1";
	 signames[SIGQUIT] = "QUIT";
	 signames[SIGSEGV] = "SEGV";
	 signames[SIGHUP] = "HUP";
	 signames[SIGINT] = "INT";
	 signames[SIGPIPE] = "PIPE";
	 signames[SIGALRM] = "ALRM";
      }
} sgnm;

static PASSFAIL
com_sigsend(const char* arg)
{
   int s;

   if (!strncmp(arg,"SIG",3)) arg+=3;

   for (s=1; s<=NSIG; s++)
   {
      if (sgnm.signames[s] && !strcmp(arg, sgnm.signames[s]))
      {
	 if (roll->is_clone())
	 {
	    char cmd[80];

	    sprintf(cmd, "sigsend %s", arg);
	    commandPipe->send_line(cmd);
	    return PASS;
	 }
	 else
	 {
#if 0
	    if (s==SIGINT) /* %%% Only cancel for SIGINT -- NO, let command fail. */
	    {
	       if (roll) roll->add_line("logonly: Canceling command queue (sigsend)");
	       cliQueue.cancel();
	    }
#endif
	    return processList.signal(s);
	 }
      }
   }

   roll->add_line("error: unsupported signal", CLI_LOG_WARNING);
   return FAIL;
}

static PASSFAIL
com_ready(const char* arg)
{
   if (arg);

   if (cliQueue.state()=='>') return PASS;
#if 0
%%% need to rework
   if (process && process->commands_pending<0) return PASS;
   else
#endif
      return FAIL;
}

static PASSFAIL
com_running(const char* arg)
{
   if (arg);
   if (roll && roll->is_clone()>=0) return PASS;
   else return FAIL;
}

static PASSFAIL
com_timerhz(const char* arg)
{
   if (arg)
   {
      unsigned int n = atoi(arg);
      if (n<1 || n>400)
      {
	 if (roll)
	    roll->add_line("error: Valid timer HZ values are between 1 and 400.");
	 return FAIL;
      }
      director_timer_hz = n;
      return PASS;
   }
   else
   {
      roll->add_line("usage: timerhz [1..400] (value is poll rate used by director.)");
      return FAIL;
   }
}

static PASSFAIL
com_autorestart(const char* arg)
{
   if (arg) ;
   roll->add_line("warning: Autorestart must be enabled per-agent by starting with -R");
   return FAIL; /* %%% */
/*
   BOOLEAN f = cli_bool(arg);
   autorestart = cli_bool(arg)==TRUE?2*RESTART_THRESH:0;
   if (f==TRUE) roll->add_line("status: Autorestart is ON");
   else         roll->add_line("status: Autorestart is OFF");
   return PASS;
*/
}

static PASSFAIL
com_abort(const char* arg)
{
   if (arg);
   roll->add_line("warning: > abort");
   return com_sigsend("SIGINT");
}

static PASSFAIL
com_stop(const char* arg)
{
   if (arg);
   roll->add_line("warning: > stop");
   return com_sigsend("SIGTSTP");
}

static PASSFAIL
com_quit(const char* arg)
{
   if (arg);
   roll->add_line("warning: > quit");
   return com_sigsend("SIGQUIT");
}

static PASSFAIL com_cancel(const char* arg) {
   if (arg);
   cliQueue.cancel(); 
   roll->scrollback_cancel();
   return PASS; 
}
/* %%% Also cancels running commands!! don't want this */

static Command remote[] = {
   { "cancel",  com_cancel, "aborts queued but not yet executed commands" },
   { "abort",   com_abort,  "sends SIGINT to the current agent" },
   { "stop",    com_stop,   "sends SIGTSTP to the current agent" },
   { "pause",   com_stop,   "Synonym for stop" },
   { "quit",    com_quit,   "sends SIGQUIT to the current agent" },
   { "break",   com_quit,   "Synonym for quit" },
   { "sigsend", com_sigsend,"[NAME] sends SIGNAME to the current agent" },
   { "sendsig", com_sigsend,"Synonym for sigsend" },
   { "timerhz",	com_timerhz, "Change director's internal polling clock" },
   { "agent",   com_agent,   "restart|start|stop|list [host:]<agentname>" },
   { "clidump", com_clidump, "display the current pending cliqueue" },
   { "ready",   com_ready,   "Silently returns pass or fail (for clicmd)"},
   { "running", com_running, "Silently returns pass or fail (for clicmd)"},
   { "say_",  com_say_,      "(send message to other users)" },
   { "autorestart", com_autorestart, "tells director whether or not to restart agents that die"},
   { 0, 0, 0 } /* This terminates the command list */
};

/* end of builtins.cc */
