/*                                             -*- c-file-style: "Ellemtel" -*-

`cli.h' - Interface file for clients to include

This file is part of version 3 of the DIRECTOR cli client library.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef _INCLUDED_cli
#define _INCLUDED_cli 1

/* **** See directest.c for a skeleton example that uses this stuff **** */

#ifdef VXWORKS
#define NO_CFHTLOG
#endif

#include <stdio.h>
#include <missing_protos.h>
#ifndef NO_CFHTLOG
/* This must be included in <> to avoid getting it into the dependencies. */
/* Otherwise, it causes problems for momma-builds outside of CFHT. */
#include <cfht/cfht.h>
#endif

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#define STDOUT_FILENO 1
#define STDERR_FILENO 2
#endif

#ifdef __cplusplus
extern "C" {
#endif
#undef TIME /* Hack for VxWorks */

#ifndef PASS
#define PASS 0
#define FAIL -1
typedef int PASSFAIL;
typedef void (*PFV)(void);   /* pointer to function that returns void */
typedef void (*PFVint)(int);   /*  ditto, with int parameter */
#endif

/*
 * %%% is this internal stuff?
 */
#ifdef NO_CFHTLOG
#define CLI_LOG_ID      0
#define CLI_LOG_START   1
#define CLI_LOG_STATUS  2
#define CLI_LOG_ERROR   3
#define CLI_LOG_WARNING 5
#define CLI_LOG_DONE    6
#define CLI_LOG_VERBOSE 7
#define CLI_LOG_DEBUG   8
#else
/*
 * Make sure director uses same values as libcfht, if libcfht has been included
 */
#define CLI_LOG_ID      CFHT_LOG_ID
#define CLI_LOG_START   CFHT_START
#define CLI_LOG_STATUS  CFHT_STATUS
#define CLI_LOG_ERROR   CFHT_ERROR
#define CLI_LOG_WARNING CFHT_WARN
#define CLI_LOG_DONE    CFHT_DONE
#define CLI_LOG_VERBOSE CFHT_LOGONLY
#define CLI_LOG_DEBUG   CFHT_DEBUG
#endif

#define CLI_LOG_SUPPRESS -1 /* === Must be an unused value of cfht_log type */
#define CLI_LOG_HIGHLIGHT -10 /* === hack, add field to RollLine struct instead? */
#define CLI_LOG_NORMAL	-11
#define CLI_LOG_PROMPT  -12
#define CLI_LOG_HIDDEN  -13

#if !defined(TRUE) || defined(VXWORKS)
#define TRUE 1
#define FALSE 0
typedef int BOOLEAN;
#endif

/*
 * Macro to put around system calls that might be interrupted by a signal.
 * This is ripped from glibc.  It will keep re-trying systems calls until
 * at least a partial success, or fatal error is encountered.  Note that
 * read might still return FEWER BYTES than you told it to read if it is
 * interrupted!
 */
#ifndef TEMP_FAILURE_RETRY
#define TEMP_FAILURE_RETRY(expression)               \
({ long int __result;                                \
     do __result = (long int) (expression);          \
     while (__result == -1L && errno == EINTR);      \
							__result; })
#endif

/*
 * Macro to use for inserting rcs id's
 */

#ifndef RCSID
#if defined(__GNUC__) && defined(__STDC__)
#define RCSID(string) \
  static const char rcs_id[] __attribute__ ((__unused__)) = "@(#) " string
#else
#if defined(__STDC__)
#define RCSID(string) static char rcs_id[] = "@(#) " string
#else
#define RCSID(string) static char rcs_id[] = string
#endif
#endif
#endif

/*
 * UNUSED attribute to tell Gnu compiler not to warn
 * and define volatile to nothing for compilers where everything
 * is volatile (We hope!)
 */
#ifdef __STDC__
#ifdef __GNUC__
#define UNUSED __attribute__ ((__unused__))
#else
#define UNUSED
#endif
#else
#define UNUSED
#define volatile
#endif

/* cli_cmd.c */

PASSFAIL cli_setprompt(PASSFAIL condition);
/*
 * cli_execute() and cli_system() usually are the ones to take care
 * of calling this.  If, however, you are not calling either of these
 * functions before the next cli_getline(), you should call this
 * yourself, with either PASS or FAIL.  It affects how the next
 * prompt looks, which director uses to determine the status of the
 * last command.
 */

PASSFAIL cli_cmd(const char* line);
/*
 * Passes a command to a running director using the named pipe
 * ~/.director/command.np.  Waits for the command to complete
 * and gets exit status using the `{PID}cmd' notation where PID
 * is your process id, which will receive SIGUSR1 or SIGUSR2.
 * Original handling of USR1 and USR2 is restored before cli_cmd
 * returns.  It returns PASS if SIGUSR1 and FAIL if SIGUSR2 received.
 *
 * All variants of cli_cmd() now fork() a child process and
 * do not mess with the parent's handling of SIGUSR1 and SIGUSR2.
 * If your program is set to ignore SIGCHLD, however, this call
 * temporarily changes that otherwise if the child exits before
 * the main thread can wait on it, the exitcode is lost.  The old
 * handling of SIGCHLD is restored before cli_cmd returns.
 *
 * Note: shell scripts may use the utility "clicmd" to invoke
 *       cli_cmd() for them.
 */
PASSFAIL cli_cmd_nowait(const char* line);
/*
 * Same as above but returns as soon as command is written to pipe
 */
PASSFAIL cli_cap(const char* line);
/*
 * Same as cli_cmd(), but captures the output generated by the command
 * (if possible) and print it in the current window rather than the
 * director window.  All output, including logonly's, will be visible.
 */
PASSFAIL cli_dup(const char* line);
/*
 * Same as cli_cmd(), but copies output to both the director window
 * and the window from which cli_dup was run.  As with cli_cap(),
 * All output, including logonly's, will be visible in the cli_dup
 * window, but they may be hidden in the director window.
 */

/* cli_getline.c */

extern BOOLEAN cli_debug;
#define CLI_DEBUG(code) {if (cli_debug) { code; }}
/*
 * cli_getline() will grab "debug on/off" commands for you and set
 * the cli_debug flag.  It will also set cfht_debug so the cfht_log
 * calls react appropriately.
 */

char* cli_getline(void);
/*
 * Displays a prompt string on stdout ("ok> " or "failed> ", depending
 * on result of the last command) and reads a line from stdin.
 *
 * Possible returns:
 *
 *   A pointer to a null terminated character string read from the user.
 *   NULL and errno==0 indicates that the end of file was reached.
 *   NULL and errno==EINTR indicates a non-blocked signal was trapped.
 */

/* cli_comlist.c */

#ifdef USE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#else
typedef PASSFAIL Function (const char* args);
#endif

typedef struct {
      char* name;
      Function* func;
      char* help;
} Command;

PASSFAIL cli_init(const char* name, Command* comlist, int director);
/*
 * `name' is the name of your program.  Currently it is not used for
 * anything significant.
 *
 * `comlist' is the list of commands that future calls to cli_execute
 * or cli_help will use.  Call cli_init() again if you need to change
 * the command set at any time.
 *
 * `director', if set to anything other than 0, causes cli_init to return
 * FAIL and print an error if anyone runs your program with the wrong
 * version of director, or no director at all.
 */

PASSFAIL cli_parse_signalmap(int* m, const char* s);
/*
 * For use by director to read signal numbers from an agent running on
 * a remote host.  Different system types assign the numbers differently.
 */

PASSFAIL cli_help(const char* arg);
/*
 * Prints help text for the current comlist.  If arg is given, only
 * help for the command given in arg is printed.
 */

Command* cli_search(const char* line, Command* comlist); 
/*
 * This is mainly used by director itself to pre-search its own list
 * of internal commands before calling cli_execute() so it doesn't
 * generate an error when a command is not found.
 */

PASSFAIL cli_execute(const char* line);
/*
 * Expects to be passed a string obtained with cli_getline() and runs
 * the apropriate command from the comlist given by the last cli_init.
 * The command being executed is stripped from "line" and the remainder
 * is sent to the command as a single char* argument.  There are three
 * main ways to handle this argument, depending on the needs of the
 * particular command:
 *
 *   1. Pass it through cli_arg1() for commands that want to treat
 *      the whole rest of the line as 1 argument (e.g. echo ....)
 *      If the enitre string starts and ends with matching quote
 *      chars, they are stripped (e.g. echo ".... .." or echo '... ..')
 *   2. Pass it through cli_argv() to break it up into an array,
 *      Strings in "" or '' are treated as one argument and the
 *      quotes are removed.
 *   3. Deal with *line in original form yourself for other cases.
 *
 * See cli_arg1() and cli_argv() descriptions below:
 */

/* cli_parse.c */

char* cli_strdup(const char* orig);
/*
 * See man page for strdup.  VxWorks 5.2 doesn't have
 * a strdup, so use this call instead for code that
 * needs to be portable.
 */

char* cli_arg1(const char* args);
/*
 * Returns the same string in args, but possibly with "" or ''
 * removed from the whole string.  At the very minimum, this function
 * strdup's the string for you, so you should always free the memory
 * it returns when done.
 */

char** cli_argv(int* argc, const char* args);
/*
 * argc, if not NULL, will be used to store the number of arguments
 * found in args.
 *
 * args must be a string of characters or a zero length string (but
 * cannot be NULL).
 *
 * The function returns an vector of arguments with the last element
 * set to NULL.  All of the arguments are allocated in a single string
 * and the returned array just contains pointers into that string.  Even
 * when there are no arguments, you should always free the memory allocated
 * to argv itself, and also to argv[0] whenever there were arguments.
 * Example:
 *
 * {
 *    static char** cargv = NULL;
 *    static int cargc = 0;
 *
 *    . . .
 *
 *    if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
 *    cargv = cli_argv(&cargc, args);
 * }
 *
 */

BOOLEAN cli_bool(const char* arg);
/*
 * Originally from libcfht's cfht_tob() function...
 *
 * This routine takes a string, and returns the string's boolean value.
 * In addition to the usual "TRUE"/"FALSE" and "YES"/"NO" pairs, many
 * other possibilities are handled.
 * The following strings return TRUE:
 *    "True"        "Yes"        "Up"        "Automatic"
 *    "Enable"      "In"         "1".."9"    "ON"
 *     "OPen"        "COlumn"
 * Case is not significant in the tests.  In the above strings, uppercase
 * is used to denote the characters used in determining if a string is
 * TRUE.  Any string whose first characters do not match one of the above
 * is considered FALSE.
 *
 * Usually called on an argument in the array returned by cli_argv()
 * or the return of cli_arg1().
 */

/* cli_syscalls.c */

PASSFAIL cli_system(const char* command, const char* args);
/*
 * Exec's a command with /bin/sh and passes it `args' as arguments.
 * NOTE: This is not very secure at the moment.  A ';' in the args
 * causes sh to execute the rest of the line as a new command.
 * The two strings `command' and `args' are just catted together
 * with a space in between, so you can leave either of them
 * empty or NULL if you don't need both.
 */

const char* cli_getcwd(void);
/*
 * Returns a string with the current working directory
 */

const char* cli_gethome(void);
/*
 * Returns the current effective user's home directory
 */

PASSFAIL cli_cd(const char* args);
/*
 * Use this to implement your "cd" command to ensure synchronization
 * with director's concept of the current working directory.
 * Tilde expansion will already have been done on args when
 * running with director.
 */

PASSFAIL cli_cd_internal(const char* args);
/*
 * For internal use by director itself, so that director is in the
 * same current working directory as its agents.  Director cannot
 * use cli_cd because it prints messages to stdout.
 */

PASSFAIL cli_write(int fd, const void* data, size_t len, int timeout_msec);
/*
 * Use this call to write to a pipe or other file descriptor with a timeout.
 * The file descriptor should set to O_NONBLOCK.
 * In case of an error, errno is left to that of the select or write
 * that caused it, or cli_write sets it to EAGAIN if there was
 * a timeout.
 */

/* cli_runon.c */

void cli_runon(const char* hostname, int raw, int argc, const char* argv[]);
/*
 * Attempts to re-rexec the current process on a new host but running
 * /usr/local/bin/runon (part of the director package.)  This is a hack
 * so that director can find its session host, and as of version 3.x,
 * the clicmd, clicap, and clidup utilities will use this as well so
 * they no longer have to run on the session host (though that is still
 * preferred).
 */

/* cli_signal.c */

#include <signal.h> /* for SIG_DFL and SIG_IGN */

#define CLI_SIG_TRAP ((PFV)cli_signal_handler)
#if 0
/*
 * This is how it should be, but SIG_IGN and SIG_DFL are not prototyped
 * properly on HP and other Unixes.  So... hope that everyone uses 0 and 1
 * for these, otherwise this breaks!
 */
#define CLI_SIG_DFL  ((PFV)SIG_DFL)
#define CLI_SIG_IGN  ((PFV)SIG_IGN)
#else
/*
 * Avoid compiler warnings this way.
 */
#define CLI_SIG_DFL ((PFV)(0))
#define CLI_SIG_IGN ((PFV)(1))
#endif

void	cli_signal(int signo, PFV handler);
BOOLEAN cli_signal_isblocked(int signo);
void	cli_signal_block(int signo);
void	cli_signal_unblock(int signo);
void    cli_signal_handler(int);
BOOLEAN	cli_signal_event(int signo);
BOOLEAN	cli_signal_peek(int signo);

#ifdef __cplusplus
}
#endif

#endif /* ! _INCLUDED_cli */
