/*                                             -*- c-file-style: "Ellemtel" -*-

`cli_getline.c' - cli_getline and cli_setprompt

This file is part of version 3 of the DIRECTOR cli client library.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#ifndef VXWORKS
#include <sys/param.h>
#endif
#include <sys/stat.h>
#ifndef VXWORKS
#include <sys/time.h>
#endif
#include <errno.h>

#ifdef VXWORKS
#include <selectLib.h>
#endif

#include "cli.h"
#include "clidefs.h"

BOOLEAN cli_debug = FALSE;

static const char* cli_prompt = "\r"DEFAULT_OK_PROMPT" ";

PASSFAIL
cli_setprompt(PASSFAIL condition)
{
   if (condition==PASS) cli_prompt = "\r"DEFAULT_OK_PROMPT" ";
   else cli_prompt = "\r"DEFAULT_FAILED_PROMPT" ";
   return condition;
}

#ifndef VXWORKS
static volatile int sigalrm_flag = 0;

static void
sigalrm_handler(int x)
{
   if (x!=SIGALRM) return; /* can't happen */
   sigalrm_flag = 1;
}

char*
cli_getline()
{
   static char inbuf[2000];
   static char* p = inbuf;
   struct sigaction oldsigalrm, newsigalrm;
   char c;

   sigalrm_flag = 0;
   newsigalrm.sa_handler = sigalrm_handler;
   sigemptyset(&newsigalrm.sa_mask);
#ifdef SA_INTERRUPT
    newsigalrm.sa_flags = SA_INTERRUPT;
#else
    newsigalrm.sa_flags = 0;
#endif /* SA_INTERRUPT */

   sigaction(SIGALRM, &newsigalrm, &oldsigalrm);

#define RESTORE_SIGALRM sigaction(SIGALRM, &oldsigalrm, NULL)

   /*
    * If this is a terminal, echo the prompt and flush output.
    * The "cli_prompt" may contain an "ok>" or a "failed>" to
    * be used to tell director whether the last command passed
    * or failed.
    */
   printf(cli_prompt);
   fflush(stdout);

   /*
    * Reset back to default prompt in case this cli_getline is
    * interrupted by a signal.  We don't want end up reporting
    * the ok> or failed> twice.
    */
   cli_prompt = "\r"DEFAULT_CONT_PROMPT" ";

   /*
    * Wait for a complete input line, end of file, or a signal.
    */
   
   for (;;) switch (read(STDIN_FILENO, &c, 1))
   {
      case -1:
      {
	 RESTORE_SIGALRM;
	 if (errno==EINTR)
	    if (sigalrm_flag)
	    {
	       puts("@alarm");
	       fflush(stdout);
	       /* re-raise the signal for the old handler */
	       kill(getpid(), SIGALRM);
	    }
	    else
	       puts("@interrupt");
	 else perror("[error]:");
	 return NULL;	/* received signal */
      }

      case  0:
      {	
	 RESTORE_SIGALRM;
	 puts("[eof]");
	 errno = 0;
	 return NULL;	/* end of input; missing newline at eof causes
			 * line to be lost.
			 */
      }

      default:
      {
	 if (p >= inbuf + sizeof(inbuf)-2 && c!='\r' && c!='\n')
	 {
	    puts("*** libcli cli_getline() buffer full ***");
	    c='\r';
	 }

	 if (c=='\r' || c=='\n') /* got a complete line */
	 {
	    *p = '\0'; /* terminate the string */

	    /* strip whitespace from end of line */
	    while (--p>=inbuf && *p<=' ') *p = '\0';

	    if (!isatty(STDIN_FILENO))
	    {
	       /*
		* If this is NOT a terminal, then there will be no terminal echo
		* so show a fake prompt and the command.  Also, "mode" commands
		*/
	       if (strcmp(inbuf,"mode")==0 ||
		   strncmp(inbuf,"mode ",5)==0)
	       {
		  /* 
		   * "mode" commands are considered "sensitive", similar to password
		   * entry and are not echoed to the terminal.
		   */
		  puts("*** (service command entered)");
	       }
	       else
		  puts(inbuf);
	    }
	    
	    RESTORE_SIGALRM;
	    p = inbuf;

	    /*
	     * Trap "debug on/off" commands in the library and set the
	     * global cli_debug flag.
	     */
	    if (strncmp(inbuf, "debug ", 6)==0)
	    {
	       BOOLEAN tmp = cli_bool(inbuf + 6);
	       if (tmp != cli_debug)
	       {
		  cli_debug = tmp;
#ifndef NO_CFHTLOG
#if 0
		  cfht_debug = cli_debug; /* %%% Should have this for CFHT */
#endif
#endif
		  printf("logonly: Debugging messages for this agent are now %s\n",
			 cli_debug?"ON":"OFF");
	       }
	       cli_setprompt(PASS);
	       return cli_getline();
	    }

	    return strdup(inbuf);
	 }

	 if (c>' ' || (c==' ' && (p!=inbuf)))
	 {
	    *p++ = c; /* add to line, but skipping leading white space 
		       * and stripping control characters from input
		       */
	 }
      }
   }
}
#else
/*
 * This is a hacked up versions of cli_getline() that responds
 * to signals under VxWorks.  There is no concept of system
 * calls that can be interrupted under VxWorks, so doing a
 * read() as under Unix and waiting for EINTR doesn't work.
 *
 * === Under VxWorks, use this instead for cli_getline:
 */
#ifndef NSIG
#define NSIG _NSIGS
#endif

static struct sigaction sig_save[NSIG];
static BOOLEAN sig_flag[NSIG];

static void
sig_handler(int x)
{
   sig_flag[x] = TRUE;
}

char*
cli_getline()
{
   static char inbuf[2000];
   static char* p = inbuf;
   struct sigaction sa_new;
   char c;
   int i;

   for (i=0; i<NSIG; i++) sig_flag[i] = FALSE;
   sa_new.sa_handler = sig_handler;
   sigemptyset(&sa_new.sa_mask);
#ifdef SA_INTERRUPT
   sa_new.sa_flags = SA_INTERRUPT;
#else
   sa_new.sa_flags = 0;
#endif /* SA_INTERRUPT */

   sigaction(SIGALRM, &sa_new, &sig_save[SIGALRM]);
   /* These are only needed in VxWorks */
   sigaction(SIGINT,  &sa_new, &sig_save[SIGINT]);
   sigaction(SIGQUIT, &sa_new, &sig_save[SIGQUIT]);
   sigaction(SIGTSTP, &sa_new, &sig_save[SIGTSTP]);

#define RESTORE_SIGS \
   sigaction(SIGALRM, &sig_save[SIGALRM], NULL); \
   sigaction(SIGINT,  &sig_save[SIGINT],  NULL); \
   sigaction(SIGQUIT, &sig_save[SIGQUIT], NULL); \
   sigaction(SIGTSTP, &sig_save[SIGTSTP], NULL)

   /*
    * If this is a terminal, echo the prompt and flush output.
    * The "cli_prompt" may contain an "ok>" or a "failed>" to
    * be used to tell director whether the last command passed
    * or failed.
    */
   printf(cli_prompt);
   fflush(stdout);

   /*
    * Reset back to default prompt in case this cli_getline is
    * interrupted by a signal.  We don't want end up reporting
    * the ok> or failed> twice.
    */
   cli_prompt = "\r"DEFAULT_CONT_PROMPT" ";

   /*
    * Wait for a complete input line, end of file, or a signal.
    */
   
   for (;;)
   {
      int rd, sl;
      fd_set fds, efds;
      struct timeval timeout;

      do
      {
	 FD_ZERO(&fds);
	 FD_SET(STDIN_FILENO, &fds);
	 efds = fds;
	 timeout.tv_usec = 500000;
	 timeout.tv_sec = 0;
	 sl = select(STDIN_FILENO + 1, (void*)&fds, 0, (void*)&efds, &timeout);
	 /* timeout only for VxWorks */

	 /* VxWorks only ... hack hack hack */
	 for (i=0; i<NSIG; i++) if (sig_flag[i]==TRUE) { sl = -1; errno = EINTR; break; }
      } while (sl == 0); /* VxWorks hack, no timeout otherwise */

      if (sl == -1) rd = -1;
      else rd = read(STDIN_FILENO, &c, 1);

      switch (rd)
      {
	 case -1:
	 {
	    RESTORE_SIGS;
	    for (i=0; i<NSIG; i++)
	    {
	       if (sig_flag[i]==TRUE)
	       {
		  if (i==SIGALRM) puts("@alarm");
		  else            puts("@interrupt");
		  fflush(stdout);
		  /* re-raise the signal for the old handler */
		  raise(i);
		  return NULL; /* received signal */
	       }
	    }
	    if (errno==EINTR)
	       puts("@interrupt");
	    else
	       perror("[error]:");
	    return NULL;
	 }

	 case  0:
	 {	
	    RESTORE_SIGS;
	    puts("[eof]");
	    errno = 0;
	    return NULL;	/* end of input; missing newline at eof causes
				 * line to be lost.
				 */
	 }

	 default:
	 {
	    if (p >= inbuf + sizeof(inbuf)-2 && c!='\r' && c!='\n')
	    {
	       puts("*** libcli cli_getline() buffer full ***");
	       c='\r';
	    }
	    
	    if (c=='\r' || c=='\n') /* got a complete line */
	    {
	       *p = '\0'; /* terminate the string */
	       
	       /* strip whitespace from end of line */
	       while (--p>=inbuf && *p<=' ') *p = '\0';
	       
	       if (!isatty(STDIN_FILENO))
	       {
		  /*
		   * If this is NOT a terminal, then there will be no terminal echo
		   * so show a fake prompt and the command.  Also, "mode" commands
		   */
		  if (strcmp(inbuf,"mode")==0 ||
		      strncmp(inbuf,"mode ",5)==0)
		  {
		     /* 
		      * "mode" commands are considered "sensitive", similar to password
		      * entry and are not echoed to the terminal.
		      */
		     puts("*** (service command entered)");
		  }
		  else
		     puts(inbuf);
	       }
	       
	       RESTORE_SIGS;
	       p = inbuf;
	       return cli_strdup(inbuf);
	    }
	    
	    if (c>' ' || (c==' ' && (p!=inbuf)))
	    {
	       *p++ = c; /* add to line, but skipping leading white space 
			  * and stripping control characters from input
			  */
	    }
	 }
      }
   }
}
#endif

/* End of cli_getline.c */
