/*                                             -*- c-file-style: "Ellemtel" -*-

`cli_parse.c' - cli_arg1, cli_argv, cli_bool

This file is part of version 3 of the DIRECTOR cli client library.
Read the `License' file for terms of use and distribution.
Copyright 2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#ifndef VXWORKS
#include <sys/param.h>
#endif
#include <sys/stat.h>
#ifndef VXWORKS
#include <sys/time.h>
#endif
#include <errno.h>

#include "cli.h"

char*
cli_strdup(const char* orig)
{
   char* s = malloc(strlen(orig)+1);
   strcpy(s, orig);
   return s;
}

char*
cli_arg1(const char* args)
{
   size_t len;

   if (args==NULL) return NULL;

   if ((*args=='"' || *args=='\'') &&
       (len = strlen(args))>=2 &&
       args[len-1]==*args)
   {
      char *p = cli_strdup(args+1);
      p[len-2] = '\0';
      return p;
   }
   else return cli_strdup(args);
}

char**
cli_argv(int* argc, const char* args)
{
   char* p;     /* temporary working pointer */
   char** cargv;      /* pointers into rtn */
   char quote_char = 0;
   int arg_siz, arg_cnt;

   cargv = (char**)malloc(sizeof(char*) * (arg_siz=32));
   arg_cnt = 0;

   /* strip white space from beginning of line */
   while (*args<=' ' && *args!='\0') args++; 
   if (*args=='"' || *args=='\'') quote_char=*args++;
   p = cli_strdup(args);

   while (*p!='\0')
   {
      cargv[arg_cnt++] = p; /* even if it's an empty line, cargv[0] will never be NULL */

      if (arg_cnt>=arg_siz-1) cargv = realloc(cargv, sizeof(char*) * (arg_siz+=32));

      if (quote_char)
	 while (*p && *p!=quote_char) p++;
      else
	 while (*++p>' ') ;
      if (*p!='\0') *p++='\0';
      while (*p<=' ' && *p!='\0') p++; /* strip white between and after arguments */
      if (*p=='"' || *p=='\'') quote_char=*p++; else quote_char = 0;
   }

   cargv[arg_cnt] = NULL; /* store a NULL at the end of the argument list */

   if (argc!=NULL) *argc = arg_cnt; /* return a copy of arg_cnt if requested */

   if (arg_cnt==0) free(p);

   return cargv; /* return the argument array */
}

BOOLEAN
cli_bool(const char* arg)
{
   char c;

   /* No arg or empty string -> FALSE */
   if (!arg || !arg[0]) return FALSE;

   /*
    * Skip white space and quote characters so we can be lazy and not
    * run arg through cli_arg1() first.
    */
   while ((*arg<=' ' && *arg!='\0') ||
	  (*arg=='"' || *arg=='\'')) arg++;

   c = toupper(arg[0]);
   
   /* Up, Automatic, Enable, In, True, Yes, non-zero -> TRUE */
   if (strchr("UAEITY123456789", c)) return TRUE;

   /* Any other single characters -> FALSE */
   if (!arg[1]) return FALSE;

   /* Open, On -> TRUE */
   if (c == 'O' && strchr("PN", toupper(arg[1]))) return TRUE;

   /* Column -> TRUE */
   if (c == 'C' && toupper(arg[1]) == 'O') return TRUE;

   /* Anything else -> FALSE */
   return FALSE;
}

/* End of cli_parse.c */
