/*                                             -*- c-file-style: "Ellemtel" -*-

`det_ace.c' - ACE control commands      (shutter, filter)

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <string.h>
#include <float.h>
#include <time.h>

#include "detcom.h"

/*
 * 12K ACE state and bit definitions
 */
#define ACE_SVS_RST		0
#define ACE_SVS_RSTSTOW		1
#define ACE_SVS_COCK		2
#define ACE_SVS_COCK_DLY	3
#define ACE_SVS_STOW		4
#define ACE_SVS_RDY		5
#define ACE_SVS_OPEN		6
#define ACE_SVS_OPEN_DLY	7
#define ACE_SVS_O_CHK		8
#define ACE_SVS_INT		9
#define ACE_SVS_CLOSE		10
#define ACE_SVS_CLOSE_DLY	11
#define ACE_SVS_C_CHK		12
#define ACE_SVS_LKOUT1		13
#define ACE_SVS_LKOUT2		14
#define ACE_SVS_TEST_DELAY	15
static const char* ace_svs_state[16] = {
"RESET",
"RSTSTOW",
"COCK",
"COCK_DLY",
"STOW",
"RDY",
"OPEN",
"OPEN_DLY",
"O_CHK",
"INT",
"CLOSE",
"CLOSE_DLY",
"C_CHK",
"LKOUT1",
"LKOUT2",
"TEST_DELAY" };
#define ACE_SVS_ERR_MASK	0x0010
#define ACE_SVS_STATE_MASK	0x000F
#define SH_H0 0x0080
#define SH_H1 0x0100
#define SH_H2 0x0200
#define SH_H3 0x0400
static const char*
diag_12kace_shutter_sensors(unsigned long v)
{
   switch (v&(SH_H0|SH_H1|SH_H2|SH_H3))
   {
      case (SH_H2 | SH_H3): return "Both Blades COCKED, Camera Aperture CLOSED";
      case (SH_H0 | SH_H3): return "Open Blade OPEN, Close Blade COCKED, Camera Aperture OPEN";
      case (SH_H0 | SH_H1): return "Open Blade OPEN, Close Blade CLOSED, Camera Aperture CLOSED";
      case (SH_H3): return "Open Blade NOT FULLY OPEN, Close Blade COCKED, Camera Aperture PARTIALLY BLOCKED";
      case (SH_H0): return "Open Blade OPEN, Close Blade NOT FULLY CLOSED, Camera Aperture PARTIALLY BLOCKED";
      case (0): return "Open Blade NOT FULLY OPEN, Close Blade NOT FULLY CLOSED, Camera Aperture PARTIALLY BLOCKED";
      case (SH_H0 | SH_H1 | SH_H3): return "Error Condition Problem with Reading Hall Sensors H1 or H3";
      case (SH_H0 | SH_H2 | SH_H3): return "Error Condition Problem with Reading Hall Sensors H0 or H2";
      case (SH_H0 | SH_H1 | SH_H2 | SH_H3): return "Error Condition Problem with Reading Hall Sensors H0 or H2 & H1 or H3";
      default: return "Unexpected Error Condition Reading Hall Sensors";
   }
}
static const char*
diag_12kace_master_digin(unsigned long v)
{
  static char buf[128];

  sprintf(buf, "lubM DIG_IN: %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s",
	  (v&0x01)?"CTL_ID":"ctl_id",
	  (v&0x0002)?"SH_CLK0":"sh_clk0",
	  (v&0x0004)?"OPN_STA":"opn_sta",
	  (v&0x0008)?"SH_TSDA":"sh_tsda",
	  (v&0x0010)?"VG_STAT":"vg_stat",
	  (v&0x0020)?"VG_DGSI":"vg_dgsi",
	  (v&0x0040)?"SH_ADCO":"sh_adco",
	  (v&0x0080)?"SH_H0":"sh_h0",
	  (v&0x0100)?"SH_H1":"sh_h1",
	  (v&0x0200)?"SH_H2":"sh_h2",
	  (v&0x0400)?"SH_H3":"sh_h3",
	  (v&0x0800)?"SH_SV0":"sh_sv0",
	  (v&0x1000)?"SH_SV1":"sh_sv1",
	  (v&0x2000)?"SH_SV2":"sh_sv2",
	  (v&0x4000)?"SH_SV3":"sh_sv3",
	  (v&0x8000)?"SH_SV4":"sh_sv4");
  return buf;
}

/*
 * Definitions for GLACE (not 12K)
 */
/*
Bit definitions for the Glace as they appear to the SDSU output bits.

 D0 always Hi
 D1 FW_GO* (* active lo)
 D2 FW_DIR always Hi
 D3 FW_POS0*
 D4 FW_POS1*, ...,
 D8 FW_POS5*
 D9 MODE_SH_STAT (hi=shutter status, lo=fw status)
 D10-D13 UNUSED (set Hi)
 D14 SH_OPEN*
 D15 RESET*

Status bits:
  D0 SH_BUSY*
  D1 SH_CLOSED*
  D2 SH_OPEN*
  D3 HEARTBEAT

  D4 SV0/F0LED* (Bits D4-D11 depend on MODE_SH_STAT 1/0)
  D5 SV1/F1LED*
  D6 SV2/F2LED*
  D7 SV3/F3LED*

  D8 H0/F4LED*
  D9 H1/F5LED*
  D10 H2/SH_OPEN_LED*
  D11 H3/SH_CLOSED_LED*

  D12 H4
  D13 ERROR*
  D14 FW_ON*
  D15 FP_ON
*/

static int shutter_count_ = 0;

int shutter_count(void) { return shutter_count_; }

PASSFAIL
com_shuttercount(const char* args)
{
   if (!args || !*args)
   {
      printf("%d\n", shutter_count_);
   }
   else if (!xstrcasecmp(args, "increment"))
   {
      shutter_count_++;
   }
   else if (!xstrcasecmp(args, "reset"))
   {
      if (detcom_mode != DETCOM_ENGINEERING)
      {
	 printf("error: Observer not permitted to reset shutter counter\n");
	 return FAIL;
      }
      shutter_count_ = 0;
   }
   else
   {
      char* endptr;
      unsigned long val = strtoul(args, &endptr, 10);

      if (!endptr || *endptr != '\0')
      {
	 printf("error: Invalid argument `%s' for shuttercount command\n", args);
	 return FAIL;
      }
      shutter_count_ = val;
   }
   return PASS;
}

static enum
{ SHUTTER_UNKNOWN=-1, SHUTTER_UNSLEEP=0, SHUTTER_SLEEP=1 }
shutter_state = SHUTTER_UNKNOWN; /* State unknown at DetCom startup time. */

typedef enum
{
   SHUTTER_TYPE_NONE=0,
   SHUTTER_TYPE_DSP=1,
   SHUTTER_TYPE_12KACE=2,
   SHUTTER_TYPE_GLACE=3,
} shutter_type_t;

typedef enum
{
   FILTER_TYPE_NONE=0,
   FILTER_TYPE_DSP=1,
   FILTER_TYPE_12KACE=2,
   FILTER_TYPE_GLACE=3,
} filter_type_t;

/*
 * The following give the maximum time in seconds which
 * DetCom should wait for the filter and shutter to be
 * read (i.e., during "filter wait" or "shutter wait".
 * These values are only used if there's no DSP code
 * symbol (FILTER_TIME EQU ... and SHUTTER_TIME EQU ...).
 */
#ifdef CAMERA_CFH12K
/*
 * === NOTE: These defines can go away when all versions of
 * the DSP code contain FILTER_TYPE and SHUTTER_TYPE.
 */
#define DEFAULT_FILTER_TIME 30
#define DEFAULT_SHUTTER_TIME 40
#define DEFAULT_SHUTTER_TYPE SHUTTER_TYPE_12KACE
#define DEFAULT_FILTER_TYPE FILTER_TYPE_12KACE
#define DEFAULT_FILTER_COUNT 4
#else
#ifdef CAMERA_MDM8K
#define DEFAULT_FILTER_TIME 95
#define DEFAULT_SHUTTER_TIME 40
#define DEFAULT_SHUTTER_TYPE SHUTTER_TYPE_GLACE
#define DEFAULT_FILTER_TYPE FILTER_TYPE_GLACE
#define DEFAULT_FILTER_COUNT 6
#else
#define DEFAULT_FILTER_TIME 95
#define DEFAULT_SHUTTER_TIME 40
#define DEFAULT_SHUTTER_TYPE SHUTTER_TYPE_DSP
#define DEFAULT_FILTER_TYPE FILTER_TYPE_NONE
#define DEFAULT_FILTER_COUNT 0
#endif
#endif

static shutter_type_t shutter_type;
static int shutter_time;
static filter_type_t filter_type;
static int filter_time;
static int filter_count;

static void
check_dsp_symbols(void)
{
   static char* dsp_version_ = 0;
   if (!dsp_version_ || strcmp(dsp_version_, dsp_version()))
   {
      if (dsp_version_) free(dsp_version_);
      dsp_version_ = strdup(dsp_version());
      
      printf("logonly: Checking DSP symbols for dsp_version:%s\n", dsp_version_);

      shutter_type = (shutter_type_t)sym_to_number(&leach[0], LUB, "SHUTTER_TYPE");
      shutter_time = sym_to_number(&leach[0], LUB, "SHUTTER_TIME");
      filter_type = (filter_type_t)sym_to_number(&leach[0], LUB, "FILTER_TYPE");
      filter_time = sym_to_number(&leach[0], LUB, "FILTER_TIME");
      filter_count = sym_to_number(&leach[0], LUB, "FILTER_COUNT");
      if (shutter_type == (shutter_type_t)SYM_INVALID)
	 shutter_type = DEFAULT_SHUTTER_TYPE;
      if (shutter_time == SYM_INVALID)
	 shutter_time = DEFAULT_SHUTTER_TIME;
      if (filter_type == (filter_type_t)SYM_INVALID)
	 filter_type = DEFAULT_FILTER_TYPE;
      if (filter_time == SYM_INVALID)
	 filter_time = DEFAULT_FILTER_TIME;
      if (filter_count == SYM_INVALID)
	 filter_count = DEFAULT_FILTER_COUNT;

      switch (filter_type)
      {
	 case FILTER_TYPE_12KACE:
	 {
	    printf("logonly: DSP code supports 12K ACE filter control.\n");
	    break;
	 }
	 case FILTER_TYPE_GLACE:
	 {
	    printf("logonly: DSP code supports GLACE filter control.\n");
	    break;
	 }
	 case FILTER_TYPE_DSP:
	 {
	    printf("logonly: Only ACE filter control is implemented.\n");
	    break;
	 }
	 case FILTER_TYPE_NONE:
	 {
	    /* printf("info2: Filter control not supported by DSP code.\n"); %%% */
	    break;
	 }
      }
   }
}

/*
 * There are defined at the bottom of this file.  Here are
 * the prototypes:
 */
static PASSFAIL shutter_reset(void);
static PASSFAIL shutter_open(void);
static PASSFAIL shutter_close(void);
static PASSFAIL shutter_wait(void);
static PASSFAIL shutter_sleep(void);
static PASSFAIL shutter_unsleep(void);
static PASSFAIL shutter_recock(void);

static PASSFAIL filter_motion(BOOLEAN status);
static void filter_sense(void);
static PASSFAIL filter_request(int slot_requested);

PASSFAIL
com_shutter(const char* args)
{
   static char* arg = 0;
   int len;

   /*
    * === Be careful about calling this function recursively because
    * of the way it frees the last args!
    */
   if (arg) free(arg);
   if (args) arg = cli_arg1(args);
   else arg = strdup("");
   len = strlen(arg);

   com_readout("wait"); /* Wait for controller to be ready */

   check_dsp_symbols();

   /* %%% */
   if (shutter_type == SHUTTER_TYPE_NONE)
   {
      if (xstrcasecmp(arg, "wait"))
	 printf("warning: No shutter configured\n");
      return PASS;
   }

   if (shutter_state == SHUTTER_UNKNOWN)
   {
      /*
       * When DetCom starts up, the first call to com_shutter()
       * should get it into a known state (unsleep).
       */
      if (shutter_unsleep() != PASS) return FAIL;
   }

   if (!xstrcasecmp(arg, "unsleep"))
   {
      if (shutter_state == SHUTTER_UNSLEEP) return PASS;
      return shutter_unsleep();
   }

   if (!xstrcasecmp(arg, "sleep"))
   {
      if (shutter_state == SHUTTER_SLEEP) return PASS; /* Already sleeping */
      return shutter_sleep();
   }

   if (!xstrcasecmp(arg, "recock"))
   {
      return shutter_recock();
   }

   if (!xstrcasecmp(arg, "reset"))
   {
      return shutter_reset();
   }

   if (len && !xstrncasecmp(arg, "status", len))
   {
      switch (shutter_type)
      {
	 case SHUTTER_TYPE_12KACE:
	 {
	    unsigned long dig_in; /* Current values in the digital input register */
	    unsigned long dig_in_svs; /* dig_in >> 11 ... the remaining 5 bits are shutter state */
	    unsigned long ace_svs; /* DSP code saves a snapshot of dig_in_svs here. */
	    
	    dig_in = sym_rdmem("lubM", "DIG_IN") & ~FLAG16;
	    ace_svs = sym_rdmem("lubM", "ACE_SVS") & ~FLAG16;
	    dig_in_svs = dig_in >> 11;
	    
	    printf("ACE_SVS=%ld, DIG_IN=0x%06lx %s\n",
		   ace_svs, dig_in, diag_12kace_master_digin(dig_in));
	    break;
	 }
	 default:
	    printf("warning: No shutter status support for this shutter.\n");
      }
      return PASS;
   }

   if (len && !xstrncasecmp(arg, "wait", len))
   {
      if (shutter_state == SHUTTER_SLEEP) return PASS;
      return shutter_wait();
   }

   if (len && (!xstrncasecmp(arg, "open", len) || !strcmp(arg, "1")))
   {
      if (shutter_state != SHUTTER_UNSLEEP) shutter_unsleep();
      return shutter_open();
   }

   if (len && (!xstrncasecmp(arg, "close", len) || !strcmp(arg, "0")))
   {
      if (shutter_state != SHUTTER_UNSLEEP) shutter_unsleep();
      return shutter_close();
   }
   cli_help("shutter");
   return FAIL;
}

static char* filtername = 0;
static int filterid = -1;
static int filterid_request = -1;
static BOOLEAN filter_in_motion = FALSE;
static BOOLEAN ace_front_panel_on = FALSE;
static time_t filter_time_stop = 0;


int
filter_id(void)
{
   if (filter_in_motion)
      return filterid_request;
   else
      return filterid;
}

const char*
filter_name(void)
{
   if (!filtername) return "Unknown";
   else if (filter_in_motion) return "Filter is moving";
   else if (filterid != filterid_request) return "Error";
   else return filtername;
}

static const char*
filter_name_and_id(void)
{
   static char rtn[128];

   if (filter_in_motion) return "Filter is moving";
   else if (filterid != filterid_request)
   {
      sprintf(rtn, "Error (%d but requested %d)", filterid, filterid_request);
      return rtn;
   }
   sprintf(rtn, "%.100s (slot %d)", filtername?filtername:"Unknown", filterid);
   return rtn;
}

const char*
filter_command(void)
{
   static char rtn[128];

   if (filterid_request == -1) return "wait";
   sprintf(rtn, "%d %.100s", filterid_request, filtername?filtername:"?");
   return rtn;
}

PASSFAIL
com_filter(const char* args)
{
   BOOLEAN filter_was_moving = filter_in_motion;
   int filterid_new;

   /*
    * %%% Until we have a status server, this provides a relatively
    * fast, reliable way for scripts to get the current filter
    * without worrying about whether the par file they see on
    * disk is synchronized and current.
    */
   if (!args || !*args)
   {
      if (filtername)
	 printf("%s\n", filtername);
      else
	 printf("%d\n", filterid_request);
      return PASS;
   }

   if (filter_type == FILTER_TYPE_NONE) return PASS;

   if (!xstrcasecmp(args, "test"))
   {
      if (!filter_in_motion) return PASS;
      filter_sense();
      if (filterid == filterid_request || time(0) > filter_time_stop)
      {
	 return com_filter("wait");
      }

      return PASS;
   }

   com_readout("wait"); /* Wait for controller to be ready */

   check_dsp_symbols();

   filter_sense();

   if (filter_in_motion)
   {
      cli_signal_event(SIGINT);
      while (filterid != filterid_request &&
	     time(0) <= filter_time_stop &&
	     !cli_signal_peek(SIGINT))
      {
	 printf("progress: Filter wheel is moving (current pos=%d).\n",
		filterid);
	 cli_signal_unblock(SIGINT);
	 xusleep(250000);
	 cli_signal_block(SIGINT);
	 filter_sense();
	 if (filterid == -1)
	 {
	    printf("error: Can't read filter position\n");
	    return FAIL;
	 }
	 if (filterid == filterid_request)
	 {
	    filter_motion(FALSE);
	    printf("status: Filter has reached %s.\n",
		   filter_name_and_id());
	 }
      }
      filter_motion(FALSE);
      filter_sense();
      printf("progress: Filter is now %s\n", filter_name_and_id());
      
      if (cli_signal_event(SIGINT))
      {
	 printf("error: Filter motion aborted by user.\n");
	 return FAIL;
      }
      if (filterid != filterid_request)
      {
	 printf("error: Filter motion timed out.\n");
	 /* Fall through and print the requested/current positions */
      }
   }

   if (filterid_request < 0) filterid_request = filterid;
   if (filtername == 0) filtername = strdup("");

   if (!xstrcasecmp(args, "check") ||
       !xstrcasecmp(args, "wait"))
   {
      if (filterid < 0)
      {
	 printf("error: Unable to read filter position.  Last filter: %s (slot %d)\n",
		filtername, filterid_request);
	 return FAIL;
      }
      else if (filterid != filterid_request)
      {
	 printf("error: Filter expected to be in position %s (slot %d), but sensed in "
		"slot %d.  Please issue a new filter command confirming which "
		"position you want.\n", filtername, filterid_request, filterid);
	 return FAIL;
      }
      else
      {
	 if (filter_was_moving || !xstrcasecmp(args, "check"))
	    printf("status: Filter is in position %s\n",
		   filter_name_and_id());
	 return PASS;
      }
   }

   if (!args || args[0] < '0' ||
       atoi(args) >= filter_count)
   {
      cli_help("filter");
      return FAIL;
   }

   filterid_new = *args++ - '0';
   while (*args && *args <= ' ') args++;
   if (filtername) free(filtername);
   filtername = strdup(args);
   if (!*args)
   {
      printf("warning: Doesn't filter %d have a name?  "
	     "Re-enter with a command like `filter %d Halpha'\n",
	     filterid_new, filterid_new);
   }

   /*
    * If the filter is already in position, just double-check
    * (and this can also be used to change the filter name.)
    */
   filterid_request = filterid_new;
   parfile_update("filter changed");

   if (filterid == filterid_request) return com_filter("check");

   printf("status: Trying to move filter to %s (slot %d)...\n",
	  filtername, filterid_request);

   if (filter_request(filterid_request) == FAIL) return FAIL;

   filter_motion(TRUE);
   filter_sense();
   filter_time_stop = time(0) + filter_time;

   return PASS;
}

static double temp = -999.99;
static char temp_comment_[256] = "";
static char temp_status_comment_[256] = "";
static const char* temp_status_ = "detector temp unknown";

/*
 * Defaults for detector temperature formula, if not found in DSP code.
 * Versions in DSP code should be stored as integers, and will be divided
 * by a factor of 1000 by DetCom.
 *
 * Example:
 *
 * DETTEM = DETTEM_C1/1000 + DETTEM_C2/1000 * A_CCDT
 * DETTEM range = DETTEM_MIN/1000 .. DETTEM_MAX/1000
 *
 * DETTEM_C1	EQU	-516000
 * DETTEM_C2	EQU	142
 * DETTEM_MIN	EQU	-91000
 * DETTEM_MAX	EQU	-85000
 */
#define DETTEM_C1  -516.0
#define DETTEM_C2  0.142
#define DETTEM_MIN -91.0
#define DETTEM_MAX -85.0

/* #define CONTEM_C1  -1057.920 */
#define CONTEM_C1  -1035.0
#define CONTEM_C2  0.380
#define CONTEM_MAX 20.0

#define DOUBLE_INVALID DBL_MIN

static double
double_1k(unsigned long symval)
{
   if (symval == SYM_INVALID) return DOUBLE_INVALID;
   /* Convert from 24-bit signed to 32-bit signed. */
   if (symval & 0x800000) symval |= 0xFF000000;
   /* Return as double, divided by 1000 */
   return (double)((signed)symval) / 1000.;
}

static int
ulong_compare(const void* a, const void* b)
{
   if (*(const unsigned long*)a > *(const unsigned long*)b) return 1;
   if (*(const unsigned long*)a < *(const unsigned long*)b) return -1;
   return 0;
}

PASSFAIL
com_temp(const char* args)
{
   static char* dsp_version_ = 0;	/* Last known DSP code version IDs */
   static int dsp_support = 1;		/* DSP supports temperature readout */
   static double dettem_c1, dettem_c2,	/* Constants obtained from the */
      dettem_min, dettem_max;		/*  DSP code for calculating temp */
   static double			/* Constants obtained from the */
      contem_c1[MAX_CONTROLLERS],	/*  DSP code for calculating the */
      contem_c2[MAX_CONTROLLERS],	/*  controller temperature (on the */
      contem_max[MAX_CONTROLLERS];	/*  utility boards. */
   static double temp_last = 0.0;	/* Last seen temperature, deg. C */
   static time_t time_last = 0;		/* Time of the last seen temp. */
   static int warned_rdmem = 0;		/* 0 -> DSP is responding. */
   					/* 1 -> problem with DSP on last try */
   static time_t time_warning = 0;	/* 0 -> Temperature is normal */
					/* or, this stores the timestamp when
					 * the temperature first went out of
					 * range.
					 */
   time_t time_now = time(0);		/* Current time. */
#define A_CCDT_COUNT 5			/* Take median of 5 samples */
   unsigned long a_ccdt[A_CCDT_COUNT];	/* Stores raw temp values from DSP */
   unsigned long
      a_contem[MAX_CONTROLLERS];	/* Stores raw controller temp values */
   double contemp[MAX_CONTROLLERS];	/* Controller temp in deg. C. */
   int quiet = 0;			/* 0 -> Print message even if normal */
   char strbuf[256];			/* Buffer for status string */
   int i;				/* Local loop counter */

   com_readout("wait"); /* Wait for controller to be ready */

   /*
    * "temp check" is mainly intended as a pre-exposure check.  If
    * a recent in-range temperature value is available, it returns
    * immediately.  Otherwise it checks with the DSP again and prints
    * a warning if needed.
    */
   if (args && !strcmp(args, "check"))
   {
      /*
       * Force a new temperature reading if it's been neglected for
       * more than 2 TEMP_PERIODs or if the last known temperature
       * was out of range.  (Usually this will not be the case, if
       * we are running with auto temperature polling when idle.)
       */
      if (time_now < time_last + 2*TEMP_PERIOD &&
	  warned_rdmem == 0 &&
	  time_warning == 0)
      {
	 return PASS; /* Everything's ok.  Skip the temperature check */
      }

      time_warning = 0; /* To force a new warning message */
      quiet = 1;
   }

   /*
    * "temp -q" is for internal use by DetCom.  The main loop calls
    * this at a rate of 1Hz and the code here figures out if it is
    * time to check again or not.
    */
   if (args && !strncmp(args, "-q", 2))
   {
      if (time_now < time_last + TEMP_PERIOD)
      {
	 return PASS; /* Not time to check again yet. */
      }

      quiet = 1;
   }

   /*
    * Read DETTEM symbols only if dsp code has been re-downloaded.
    */
   if (!dsp_version_ || strcmp(dsp_version_, dsp_version()))
   {
      if (dsp_version_) free(dsp_version_);
      dsp_version_ = strdup(dsp_version());

      if (sym_to_addr(&leach[0], LUB, "A_CCDT")==SYM_INVALID)
      {
	 printf("logonly: No DSP support for temperature readout.\n");
	 printf("info2: CCD temperature not supported by DSP code.\n");
	 dsp_support = 0;
      }
      else
      {
	 if (!dsp_support)
	 {
	    dsp_support = 1;
	    printf("logonly: DSP has support for temperature readout.\n");
	 }
      }

      dettem_c1 = double_1k(sym_to_number(&leach[0], LUB, "DETTEM_C1"));
      dettem_c2 = double_1k(sym_to_number(&leach[0], LUB, "DETTEM_C2"));
      dettem_min = double_1k(sym_to_number(&leach[0], LUB, "DETTEM_MIN"));
      dettem_max = double_1k(sym_to_number(&leach[0], LUB, "DETTEM_MAX"));
      /*
       * Use defaults for anything not defined.
       */
      if (dettem_c1 == DOUBLE_INVALID) dettem_c1 = DETTEM_C1;
      if (dettem_c2 == DOUBLE_INVALID) dettem_c2 = DETTEM_C2;
      if (dettem_min == DOUBLE_INVALID) dettem_min = DETTEM_MIN;
      if (dettem_max == DOUBLE_INVALID) dettem_max = DETTEM_MAX;

      sprintf(temp_status_comment_, "Detector temp range (%g..%g)",
	      dettem_min, dettem_max);

      for (i = 0; i < n_controllers; i++)
      {
	 contem_c1[i] = double_1k(sym_to_number(&leach[i], LUB, "CONTEM_C1"));
	 contem_c2[i] = double_1k(sym_to_number(&leach[i], LUB, "CONTEM_C2"));
	 contem_max[i] = double_1k(sym_to_number(&leach[i], LUB, "CONTEM_MAX"));

	 if (contem_c1[i] == DOUBLE_INVALID) contem_c1[i] = CONTEM_C1;
	 if (contem_c2[i] == DOUBLE_INVALID) contem_c2[i] = CONTEM_C2;
	 if (contem_max[i] == DOUBLE_INVALID) contem_max[i] = CONTEM_MAX;
      }
   }

   if (!dsp_support)
   {
      if (!quiet)
	 printf("CCD temperature not supported by DSP code.\n");
      return PASS;
   }

   /*
    * Read the detector temperature probe connected to the utility board.
    * A_CCDT is a raw ADC value.  Since it has a lot of noise, we
    * read 5 times and take the median.
    */
   for (i=0; i<A_CCDT_COUNT; i++)
   {
      a_ccdt[i] = sym_rdmem("lubM", "A_CCDT") & ~FLAG16;

      /*
       * If there's a problem reading A_CCDT...
       */
      if (a_ccdt[i]==(SYM_INVALID&~FLAG16))
      {
	 if (!warned_rdmem)
	 {
	    printf("warning: Can't read A_CCDT from lubM\n");
	    warned_rdmem = 1;
	    temp = -9999.;
	    temp_status_ = "detector temp missing (A_CCDT unavailable)";
	    printf("info2: CCD temperature unavailable.\n");
	 }
	 return FAIL;
      }
      else
      {
	 if (warned_rdmem)
	 {
	    printf("warning: Able to read A_CCDT again\n");
	    warned_rdmem = 0;
	 }
      }
   }

   qsort(a_ccdt,
	 sizeof(a_ccdt)/sizeof(*a_ccdt),
	 sizeof(*a_ccdt), ulong_compare);

   /*
    * Using the constants obtained from the DSP code, calculate
    * the temperature with this fixed formula:
    */
   temp = dettem_c1 + dettem_c2 * a_ccdt[A_CCDT_COUNT/2];

   /*
    * No median averaging is done for the utility board value.
    * It seems to be stable enough.  Just plug into the formula
    * and convert the raw value to degrees Celsius.
    */
   for (i=0; i<n_controllers; i++)
   {
      switch (i)
      {
	 /* %%% Fix this! */
	 case 0: a_contem[0] = sym_rdmem("lubM", "AD_IN") & ~FLAG16; break;
	 case 1: a_contem[1] = sym_rdmem("lubS", "AD_IN") & ~FLAG16; break;
      }
      contemp[i] = contem_c1[i] + contem_c2[i] * a_contem[i];
   }

   sprintf(temp_comment_, "Detector temp deg C = %g + %g * %ld",
	   dettem_c1, dettem_c2, a_ccdt[A_CCDT_COUNT/2]);
   temp_status_ = "ok";

   /*
    * Build the message string.
    */
   sprintf(strbuf, "CCD temperature: %.1f deg. C. -- Controller temperature%s:",
	   temp, n_controllers > 1?"s":"");
   for (i=0; i<n_controllers; i++)
   {
      sprintf(strbuf + strlen(strbuf), " %.1f", contemp[i]);
   }

   printf("info2: %s\n", strbuf);

   if (!quiet)
      printf("%s\n", strbuf);

   if (temp < dettem_min)	temp_status_ = "detector temp is low";
   else if (temp > dettem_max)	temp_status_ = "detector temp is high";

   if (temp < dettem_min || temp > dettem_max)
   {
      if (!time_warning)
      {
	 printf("warning: CCD temperature (%.1f) is out of normal range (%g..%g).\n",
		temp, dettem_min, dettem_max);
	 time_warning = time_now;
      }
   }
   else
   {
      if (time_warning)
      {
	 int minutes = (time_now - time_warning)/60;

	 printf("warning: CCD temperature is ok now (it was out of range for %d minute%s).\n",
		minutes, minutes==1?"":"s");
	 time_warning = 0;
      }
   }
   
   temp_last = temp;
   time_last = time_now;

   return PASS;
}

double      temp_float(void)   { return temp; }
const char* temp_comment(void) { return temp_comment_; }
const char* temp_status(void)  { return temp_status_; }
const char* temp_status_comment(void) { return temp_status_comment_; }

/*
 * Shutter operations:
 * ------------------
 */

static PASSFAIL
shutter_reset()
{
   PASSFAIL rtn = PASS;
   com_cmd("lubM CSH"); /* Ignore failure of this command */
   if (shutter_type == SHUTTER_TYPE_12KACE)
   {
      if (com_cmd("lubM ARS")==FAIL) rtn = FAIL;
      xusleep(1000000);
      if (com_cmd("lubM RRS")==FAIL) rtn = FAIL;
   }
   return rtn;
}

static PASSFAIL
shutter_open(void)
{
   com_shuttercount("increment"); /* Ignore failure here (a warning will appear). */
   return com_cmd("lubM OSH");
}
static PASSFAIL
shutter_close(void)
{
   return com_cmd("lubM CSH");
}

static PASSFAIL
shutter_wait(void)
{
   unsigned long dig_in; /* Current values in the digital input register */
   unsigned long dig_in_svs; /* dig_in >> 11 ... the remaining 5 bits are shutter state */
   unsigned long ace_svs; /* DSP code saves a snapshot of dig_in_svs here. */
   
   time_t time_stop = time(0) + shutter_time; /* Give up after a few secs */
   int shutter_retry = 1;

   if (shutter_type != SHUTTER_TYPE_12KACE)
   {
      CLI_DEBUG(printf("\rdebug: shutter wait command only implemented for CFH12K.\n"));
      return PASS;
   }

   if (sym_to_addr(&leach[0], LUB, "ACE_SVS")==SYM_INVALID)
   {
      printf("warning: shutter wait command does nothing because ACE_SVS undefined.\n");
      return PASS;
   }

shut_retry:
   do
   {
      dig_in = sym_rdmem("lubM", "DIG_IN") & ~FLAG16;
      ace_svs = sym_rdmem("lubM", "ACE_SVS") & ~FLAG16;
      dig_in_svs = dig_in >> 11;
      
      if (dig_in == ((unsigned long)(-1)&~FLAG16))
      {
	 printf("error: Can't read DIG_IN from lubM\n");
	 return FAIL;
      }
      if ((dig_in_svs & ACE_SVS_ERR_MASK)==0 &&
	  (dig_in_svs & ACE_SVS_STATE_MASK)!=ACE_SVS_RDY)
	 printf("progress: Waiting for shutter (Current state is %s).\n",
		ace_svs_state[dig_in_svs&ACE_SVS_STATE_MASK]);
      cli_signal_unblock(SIGINT);
      xusleep(250000);
      cli_signal_block(SIGINT);
   } while (time(0) < time_stop &&
	    (dig_in_svs & ACE_SVS_ERR_MASK)==0 &&
	    (dig_in_svs & ACE_SVS_STATE_MASK)!=ACE_SVS_RDY &&
	    !cli_signal_event(SIGINT));
   
   printf("progress: Shutter is %s.\n", ace_svs_state[dig_in_svs&ACE_SVS_STATE_MASK]);
   
   if ((dig_in_svs & ACE_SVS_ERR_MASK) ||
       (dig_in_svs & ACE_SVS_STATE_MASK) != ACE_SVS_RDY ||
       dig_in_svs != ace_svs || shutter_retry)
   {
      printf("logonly: ACE_SVS=%ld, DIG_IN=0x%06lx (%s)%s\n",
	     ace_svs, dig_in, shutter_retry?"":" after a reset",
	     diag_12kace_master_digin(dig_in));
   }
   
   if ((dig_in_svs & ACE_SVS_ERR_MASK) ||
       (dig_in_svs & ACE_SVS_STATE_MASK) != ACE_SVS_RDY)
   {
      printf("error: Shutter %s in state %s; (%s)\n",
	     (dig_in_svs&ACE_SVS_ERR_MASK)?"failed (ERR bit set)":"control is hung (no ERR bit)",
	     ace_svs_state[dig_in_svs&ACE_SVS_STATE_MASK],
	     diag_12kace_shutter_sensors(dig_in));
      
      if (shutter_retry--)
      {
	 /*
	  * Shutter Troubleshooting Guide
	  * -----------------------------
	  *
	  * From Barry Starr's October 1999 e-mail... some helpful
	  * messages for the poor guy who has to diagnose shutter problems.
	  */
	 printf("Shutter Troubleshooting Wizard Says:\n");
	 if (!(dig_in_svs & ACE_SVS_ERR_MASK))
	 {
	    printf("No error bit set.  Maybe the ACE front panel is on?\n"); /* %%% */
	 }
	 else
	 {
	    switch (dig_in_svs & ACE_SVS_STATE_MASK)
	    {
	       case ACE_SVS_RST:
	       case ACE_SVS_COCK_DLY:
	       case ACE_SVS_LKOUT2:
	       case ACE_SVS_TEST_DELAY:
		  printf("No error condition should occur in this state!  Weird.\n");
		  break;
		  
	       case ACE_SVS_RSTSTOW:
		  printf(
		     "Timeout waiting for SW1 limit switch to close.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Ballscrew is needs lubrication or is broken\n"
		     "2) DC motor not working\n"
		     "3) ACE Driver is BAD\n"
		     "4) Limit switch SW1 is bad\n"
		     "5) Other ACE or system problem\n");
		  break;
		  
	       case ACE_SVS_COCK:
		  printf(
		     "Timeout waiting for both blades to be pushed to right (COCK direction).  Expected hall sensors H2 and H3 to go HI and SW0 to close.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Ballscrew is needs lubrication or is broken\n"
		     "2) DC motor not working\n"
		     "3) ACE Driver is BAD\n"
		     "4) Limit switch SW0 is bad\n"
		     "5) Hall effect H2 or H3 is bad\n"
		     "6) Other ACE or system problem\n"
		     "NOTE: If you find yourself in a continous loop where you go from STOW to COCK to COCK_DLY to STOW then one or more of the shutter latches is not holding the blades in place!\n");
		  break;
		  
	       case ACE_SVS_STOW:
		  printf(
		     "Here's a description of what the shutter was trying to do:\n"
		     "The Ballscrew is driven to the left (STOW Direction) until limit switch W1 says ball nut is fully stowed.  If one of the shutter blades does not stay cocked we immediately return to the COCK state to try and recock.  This will continue until SW1 and Hall Effects are correct or until Timeout.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Ballscrew is needs lubrication or is broken\n"
		     "2) DC motor not working\n"
		     "3) ACE Driver is BAD\n"
		     "4) Limit switch SW1 is bad\n"
		     "5) Hall effects bad\n"
		     "6) Other ACE or system problem\n"
		     "NOTE: If you find yourself in a continous loop where you go from STOW to COCK to COCK_DLY to STOW then one or more of the shutter latches is not holding the blades in place!\n");
		  break;
		  
	       case ACE_SVS_RDY:
		  printf(
		     "One or more of the blades probably slipped while waiting for command to open the shutter.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Problem with Solenoid Latching Mechanism\n"
		     "2) Solenoid not working\n"
		     "3) ACE Driver is BAD\n"
		     "4) Hall effects not working\n"
		     "5) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_OPEN:
		  printf(
		     "Close Blade Slipped when Open Blade Released.  Blade 'Double-Fire'\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Problem with Solenoid Latching Mechanism\n"
		     "2) Hall effects not working\n"
		     "3) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_OPEN_DLY:
		  printf(
		     "Open blade doesn't seem to have released\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Problem with Solenoid Latching Mechanism\n"
		     "2) Hall effects not working\n"
		     "3) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_O_CHK:
		  printf(
		     "Open blade may not be fully open.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Open Blade is Sagging Due to weak spring force holding it fully open\n"
		     "2) Something is interfering with the Open Blade\n"
		     "3) Hall effects not working\n"
		     "4) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_INT:
		  printf(
		     "Aperture didn't stay fully open during integration.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Open Blade is Sagging Due to weak spring force holding it fully open\n"
		     "2) Close Blade has Slipped Due to problem with latching Mechanism\n"
		     "3) Hall effects not working\n"
		     "4) Other ACE or system problem\n");
		  break;
		  
	       case ACE_SVS_CLOSE:
		  printf(
		     "Open blade has sagged.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Open Blade is Sagging Due to weak spring force holding it fully open\n"
		     "2) Hall effects not working\n"
		     "3) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_CLOSE_DLY:
		  printf(
		     "Close blade didn't seem to release.\n"
		     "Problems, in order of likelyhood:\n"
                     "1) Problem with Solenoid Latching Mechanism\n"
		     "2) Hall effects not working\n"
		     "3) Other ACE or system problem\n");
		  break;
		  
	       case ACE_SVS_C_CHK:
		  printf(
		     "Close blade didn't seem to close fully.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Close Blade is Sagging Due to weak spring force holding it fully closed\n"
		     "2) Something is interfering with the Close Blade\n"
		     "3) Hall effects not working\n"
		     "4) Other ACE or system problem\n");
		  break;

	       case ACE_SVS_LKOUT1:
		  printf(
		     "This state is used to delay recocking sequence fully the shutter close command.  It was designed in to allow CCDs to be readout before the recocking sequence.  It turns out that we can read the CCDs during the recock sequence with no penalty so this state should never be seen during operation.  It is used during the 'shuttercheck' utility when a 'shutter open' command is sent.  This causes the Lockout_ signal to be asserted and Open_ signal to be asserted.\n"
		     "Problems, in order of likelyhood:\n"
		     "1) Close Blade is Sagging Due to weak spring force holding it fully closed\n"
		     "2) Hall effects not working\n"
		     "3) Other ACE or system problem\n");
		  break;
	    }
	 }
	 
	 /*
	  * Now go ahead and close/reset the shutter.  This may be the end
	  * of a long exposure, and it may still be salvageable.
	  */
	 printf("error: Attempting to close and reset the shutter. . .\n");
	 shutter_reset();
	 time_stop = time(0) + shutter_time; /* Wait some more */
	 goto shut_retry;
      }
      return FAIL;
   }
   return PASS;
}

static PASSFAIL
shutter_sleep(void)
{
   if (shutter_type == SHUTTER_TYPE_12KACE)
   {
      printf("\rstatus: Placing shutter in sleep mode.\n");
      shutter_state = SHUTTER_UNKNOWN;
      if (shutter_wait() != PASS) return FAIL;
      if (shutter_open() != PASS) return FAIL;
      xusleep(500000);
      if (shutter_close() != PASS) return FAIL;
   }
   shutter_state = SHUTTER_SLEEP;
   return PASS;
}

static PASSFAIL
shutter_unsleep(void)
{
   if (shutter_type == SHUTTER_TYPE_12KACE)
   {
      if (shutter_state == SHUTTER_UNKNOWN)
	 printf("\rlogonly: Making sure shutter is not in sleep mode.\n");
      else
	 printf("\rstatus: Shutter was in sleep mode ... please wait.\n");
      
      if (com_cmd("lubM RCK") != PASS ||
	  shutter_wait() != PASS)
      {
	 shutter_state = SHUTTER_UNKNOWN;
	 return FAIL;
      }
   }
   shutter_state = SHUTTER_UNSLEEP;
   return PASS;
}

static PASSFAIL
shutter_recock(void)
{
   if (shutter_type == SHUTTER_TYPE_12KACE)
   {
      return com_cmd("lubM RCK");
   }
   return PASS;
}

static PASSFAIL
filter_motion(BOOLEAN status)
{
   filter_in_motion = status;

   if (filter_type == FILTER_TYPE_GLACE)
   {
      if (status)
	 return com_wrmem("lubM DIG_OUT OFF 1<<FW_GO");
      else
	 return PASS; /* %%% com_wrmem("lubM DIG_OUT ON  1<<FW_GO"); */
   }
   return PASS;
}

static void
filter_sense_12kace(void)
{
   unsigned long dig_in = sym_rdmem("lubS", "DIG_IN") & ~FLAG16;

   if (dig_in == SYM_INVALID)
   {
      filterid = -1;
   }
   else switch ((dig_in & 0x000F00) >> 8)
   {
      case 0xE: filterid = 0; break;
      case 0xA: filterid = 1; break;
      case 0xC: filterid = 2; break;
      case 0x8: filterid = 3; break;
      case 0xf: filterid = -2; break; /* This is normal when wheel is moving */
      default:
      {
	 printf("logonly: Filter state bits invalid (%lx)\n",
		(dig_in & 0x000F00) >> 8);
	 filterid = -2;
      }
   }
}

static void
filter_sense_glace(void)
{
   unsigned long dig_in;

   filterid = -1;
   ace_front_panel_on = FALSE;

   /*
    * Select filter status to be returned by DIG_IN
    */
   if (com_wrmem("lubM DIG_OUT OFF 1<<SH_STAT")==FAIL)
   {
      printf("info2: Filter position: WRMEM ERROR\n");
      return;
   }
   dig_in = sym_rdmem("lubM", "DIG_IN"); /* %%% FLAG16 */
   if (dig_in == SYM_INVALID) return; /* %%% Change info2 */

   /*
    * If the front panel is on, DetCom is locked out.
    * Status bits are also different, and we must read positon
    * using the F*LED bits.
    */
   if (dig_in & (1<<15))
   {
      ace_front_panel_on = TRUE;
      printf("warning: ACE Front panel is on.\n"); /* %%% */
      switch (((~dig_in) & 0x0003F0) >> 4)
      {
	 case 0x00: filterid = -2; break;
	 case 0x01: filterid = 0; break;
	 case 0x02: filterid = 1; break;
	 case 0x04: filterid = 2; break;
	 case 0x08: filterid = 3; break;
	 case 0x10: filterid = 4; break;
	 case 0x20: filterid = 5; break;
	 default: filterid = -2; break;
      }
      return;
   }
   
   if ((dig_in & (1<<11)))
   {
      filterid = -2;
      return;
   }
   
   switch ((dig_in & 0x000700) >> 8)
   {
      case 0x07: filterid = 0; break;
      case 0x06: filterid = 1; break;
      case 0x05: filterid = 2; break;
      case 0x04: filterid = 3; break;
      case 0x03: filterid = 4; break;
      case 0x02: filterid = 5; break;
      default:
      {
	 /* %%% Print error? */
	 printf("info2: Bogus filter bits 0x%02lx (dig_in=0x%06lx); stopping motion!\n",
		(dig_in & 0x000700)>>8, dig_in);
	 filter_motion(FALSE);
	 filterid = -2;
	 return;
      }
   }
}

static void
filter_sense(void)
{
   switch (filter_type)
   {
      case FILTER_TYPE_12KACE: filter_sense_12kace(); break;
      case FILTER_TYPE_GLACE: filter_sense_glace(); break;
      default:
	 /* That's all we know how to handle right now */
   }
   printf("info2: Filter position: %s\n", filter_name_and_id());
}

static PASSFAIL
filter_request(int slot_requested)
{
   if (ace_front_panel_on)
   {
      printf("error: DetCom cannot control filter with ACE front panel on\n");
      return FAIL;
   }

   switch (filter_type)
   {
      case FILTER_TYPE_GLACE:
      {
	 unsigned long dig_out;
	 unsigned long addr = sym_to_addr(&leach[0], LUB, "DIG_OUT");

	 dig_out = sym_rdmem("lubM", "DIG_OUT") & ~FLAG16;
	 if (dig_out == SYM_INVALID || addr == SYM_INVALID)
	 {
	    printf("error: Can't access DIG_OUT to move filter!\n");
	    return FAIL;
	 }
	 dig_out |= 0x0001F8;
	 dig_out &= ~(0x000008<<slot_requested);
	 wrmem(&leach[0], LUB, addr, dig_out, 0);
	 break;
      }
      case FILTER_TYPE_12KACE:
      {
	 /* %%% */
	 int hack;
	 for (hack=0; hack<2; hack++)
	 {
	    char cmd[32];

	    xusleep(500000);
	    if (com_cmd("lubS RGO")==FAIL) return FAIL;
	    xusleep(500000);
	    sprintf(cmd, "lubS MW%d", slot_requested); /* Start motion */
	    if (com_cmd(cmd)==FAIL) return FAIL;
	 }
	 break;
      }
      default:
      {
	 fprintf(stderr, "error: No filter support.\n");
	 return FAIL;
      }
   }
   return PASS;
}

/* End of det_ace.c */
