/*                                             -*- c-file-style: "Ellemtel" -*-

`det_expose.c' - Exposure related commands (etype, etime, go)

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h> /* === Just for fabs */

#include "cli/cli.h"
#include "detcom.h"
#ifdef DETCOM_IR
#include "time.c" /* Martin Riopel's timer code */
#endif

typedef enum
{
   ETYPE_OBJECT,
   ETYPE_SNAP,
   ETYPE_DITHER,
   ETYPE_BIAS,
   ETYPE_DARK,
   ETYPE_FLAT,
   ETYPE_TFLAT,
   ETYPE_COMPARISON,
   ETYPE_ALIGN,
   ETYPE_FOCUS,
   ETYPE_TSTPAT,
   ETYPE_CNTPAT,
   ETYPE_LED,
   ETYPE_UNKNOWN
} etype_t;

#define MAX_ETYPES 100

int exp_iteration = 0;
struct timeval detcom_obs_eip = { 0, 0 };
struct timeval detcom_obs_don = { 0, 0 };
unsigned long blade_open = 0;
unsigned long blade_close = 0;
BOOLEAN timing_critical = FALSE;

static double etime = 1.0;
static double etime_dsp = -1.;
static char etype = ETYPE_OBJECT;
static volatile BOOLEAN quit = FALSE; /* Quit series of exposures requested? */
#ifdef DETCOM_IR
static float time_begin=0,time_elapsed=0.,time_end=0.,time_rd1=0.,timed_rd1=0.;
#endif

static void
sigquit_handler_expose(void)
{
   printf("warning: Notice: exposure sequence will stop "
	  "after this exposure finishes.  Use `stop' to "
	  "read out now, or `abort' to cancel completely.\n");
   quit = TRUE;
   cli_signal(SIGQUIT, CLI_SIG_IGN);
}

static void
sigquit_handler(void)
{
   printf("warning: Notice: exposure sequence will stop "
	  "after this exposure finishes. "
	  "Use `abort' to cancel immediately.\n");
   quit = TRUE;
   cli_signal(SIGQUIT, CLI_SIG_IGN);
}

char
etype_char(void)
{
   switch(etype)
   {
      case ETYPE_OBJECT:	return 'o';
      case ETYPE_SNAP:		return 's';
      case ETYPE_DITHER:	return 'o';
      case ETYPE_BIAS:		return 'b';
      case ETYPE_DARK:		return 'd';
      case ETYPE_FLAT:		return 'f';
      case ETYPE_TFLAT:		return 'f';
      case ETYPE_COMPARISON:	return 'c';
      case ETYPE_ALIGN:		return 'a';
      case ETYPE_FOCUS:		return 'x';
      case ETYPE_TSTPAT:	return 't';
      case ETYPE_CNTPAT:	return 'c';
      case ETYPE_LED:		return 'l';
   }
   return '?';
}

const char*
etype_command(void)
{
   switch (etype)
   {
      case ETYPE_OBJECT:	return "OBJECT";
      case ETYPE_SNAP:		return "SNAP";
      case ETYPE_DITHER:	return "DITHER";
      case ETYPE_BIAS:		return "BIAS";
      case ETYPE_DARK:		return "DARK";
      case ETYPE_FLAT:		return "FLAT";
      case ETYPE_TFLAT:		return "TFLAT";
      case ETYPE_COMPARISON:	return "COMPARISON";
      case ETYPE_ALIGN:		return "ALIGN";
      case ETYPE_FOCUS:		return "FOCUS";
      case ETYPE_TSTPAT:	return "TSTPAT";
      case ETYPE_CNTPAT:	return "CNTPAT";
      case ETYPE_LED:		return "LED";
      default: return "UNKNOWN";
   }
}

const char*
etype_fits(void)
{
   switch (etype)
   {
      case ETYPE_OBJECT:	return "OBJECT";
      case ETYPE_SNAP:		return "OBJECT";
      case ETYPE_DITHER:	return "OBJECT";
      case ETYPE_BIAS:		return "BIAS";
      case ETYPE_DARK:		return "DARK";
      case ETYPE_FLAT:		return "FLAT";
      case ETYPE_TFLAT:		return "FLAT";
      case ETYPE_COMPARISON:	return "COMPARISON";
      case ETYPE_ALIGN:		return "ALIGN";
      case ETYPE_FOCUS:		return "FOCUS";
      case ETYPE_TSTPAT:	return "TSTPAT";
      case ETYPE_CNTPAT:	return "CNTPAT";
      case ETYPE_LED:		return "LED";
      default: return "UNKNOWN";
   }
}

#define NO_ETIME(et) ((et)==ETYPE_BIAS||(et)==ETYPE_TSTPAT||(et)==ETYPE_CNTPAT)

BOOLEAN
is_testpat(void)
{
   if (etype == ETYPE_TSTPAT)
      return TRUE;
   else
      return FALSE;
}

BOOLEAN
have_etime(void)
{
   if (NO_ETIME(etype))
      return FALSE;
   else
      return TRUE;
}

BOOLEAN
have_shutter(void)
{
   if (!have_etime() || etype==ETYPE_DARK)
      return FALSE;
   else
      return TRUE;
}

const char*
etime_command(void)
{
   static char cmd[40];

   sprintf(cmd, "%0*g%.8s",
	   (etime - (double)((int)(etime+0.05)))<0.05?2:4,
	   etime, NO_ETIME(etype)?"(NA)":"");
   return cmd;
}

static const char*
etime_remain(int seconds)
{
   static char str[40];
   double remain = seconds;

   sprintf(str, "%0*g",
	   (remain - (double)((int)(remain+0.05)))<0.05?2:4,
	   remain);
   return str;
}

double
etime_float(void)
{
   return etime;
}

double
etime_meas(void)
{
   if (NO_ETIME(etype)) return 0.;

   if (etime_dsp==-1.)
      return etime;
   else
      return etime_dsp;
}

static BOOLEAN
etype_valid(etype_t etype)
{
   static BOOLEAN valid[MAX_ETYPES];
   static BOOLEAN initialized = FALSE;

   if (!initialized || etype == ETYPE_UNKNOWN)
   {
      initialized = TRUE;
      memset(valid, 0, sizeof(valid));
      valid[ETYPE_OBJECT] = TRUE;
      valid[ETYPE_SNAP] = TRUE;
      valid[ETYPE_DITHER] = TRUE;
      valid[ETYPE_BIAS] = TRUE;
      valid[ETYPE_FLAT] = TRUE;
      valid[ETYPE_TFLAT] = TRUE;
      valid[ETYPE_COMPARISON] = TRUE;
      valid[ETYPE_ALIGN] = TRUE; /* %%% */
      valid[ETYPE_FOCUS] = TRUE;
      
      if (sym_to_addr(&leach[0], LUB, "OPTIONS") != SYM_INVALID)
      {
	 if (sym_to_number(&leach[0], LUB, "OPT_SH") != SYM_INVALID)
	    valid[ETYPE_DARK] = TRUE;
	 if (sym_to_number(&leach[0], LUB, "OPT_LED") != SYM_INVALID)
	    valid[ETYPE_LED] = TRUE;
      }
      if (sym_to_addr(&leach[0], LTB, "OPTIONS") != SYM_INVALID)
      {
	 if (sym_to_number(&leach[0], LTB, "TSTPAT") != SYM_INVALID)
	    valid[ETYPE_TSTPAT] = TRUE;
	 if (sym_to_number(&leach[0], LTB, "CNTPAT") != SYM_INVALID)
	    valid[ETYPE_CNTPAT] = TRUE;
      }
   }
   return valid[etype];
}

static etype_t
name_to_etype(const char* arg)
   /*
    * This converts a string like "BIAS" "Bias" or "bias"
    * (or even `b') into the one-character code used in
    * fits file names.  It is ok for the string in `arg'
    * to already be a one character code.
    */
{
   char* s;
   etype_t type = ETYPE_UNKNOWN;
   int len;

   if (!arg || !*arg)
   {
      printf("warning: No value given for exposure type!\n");
      return ETYPE_UNKNOWN;
   }
   s = cli_arg1(arg);
   len = strlen(s);

   /*
    * The one-letter syntax is discouraged in scripts because
    * it is not as clear as writing out "OBJECT", for example.
    * Furthermore, some complex exposure types do not have a
    * specific single letter associated with them (such as "DITHER"
    * which generates the same *o.fits files as "OBJECT")  This
    * etype must be entered with "etype dither".
    */
   if (len==1) switch (*s)
   {
      case 'o':
      case 'O': type = ETYPE_OBJECT; break;
      case 'B':
      case 'b': type = ETYPE_BIAS; break;
      case 'D':
      case 'd': type = ETYPE_DARK; break;
      case 'f': type = ETYPE_FLAT; break;
      case 'c': type = ETYPE_COMPARISON; break;
      case 'a': type = ETYPE_ALIGN; break;
      case 'x': type = ETYPE_FOCUS; break;
      case 't': type = ETYPE_TSTPAT; break;
      case 'T': type = ETYPE_CNTPAT; break;
      case 'l': type = ETYPE_LED; break;
   }
   else if (len > 1 || toupper(*s)!='F')
   {
      if (!xstrncasecmp(s, "OBJECT", len))		type = ETYPE_OBJECT;
      else if (!xstrncasecmp(s, "BIAS", len))		type = ETYPE_BIAS;
      else if (!xstrncasecmp(s, "DARK", len))		type = ETYPE_DARK;
      else if (!xstrncasecmp(s, "FLATFIELD", len))	type = ETYPE_FLAT;
      else if (!xstrncasecmp(s, "COMPARISON", len))	type = ETYPE_COMPARISON;
      else if (!xstrncasecmp(s, "ALIGN", len))		type = ETYPE_ALIGN;
      else if (!xstrncasecmp(s, "FOCUS", len))		type = ETYPE_FOCUS;
      else if (!xstrncasecmp(s, "TSTPAT", len))		type = ETYPE_TSTPAT;
      else if (!xstrncasecmp(s, "TESTPAT", len))	type = ETYPE_TSTPAT;
      else if (!xstrncasecmp(s, "CNTPAT", len))		type = ETYPE_CNTPAT;
      else if (!xstrncasecmp(s, "LED", len))		type = ETYPE_LED;
      else if (!xstrncasecmp(s, "SNAP", len))		type = ETYPE_SNAP;
      else if (!xstrncasecmp(s, "DITHER", len))		type = ETYPE_DITHER;
      else if (!xstrncasecmp(s, "TFLAT", len))		type = ETYPE_TFLAT;
   }
   if (type==ETYPE_UNKNOWN)
   {
      printf("warning: `%s' is invalid; "
	     "Use OBJECT BIAS DARK FLAT COMPARISON ALIGN or FOCUS\n", s);
      printf("logonly: Engineering types: TSTPAT CNTPAT and LED also available\n");
   }

   free(s);
   return type;
}

PASSFAIL
com_etime(const char* arg)
{
   double new_etime;
   char* s = cli_arg1(arg);
   char* colon = strchr(s, ':');
   unsigned long n;
   double tgt_min, tgt_max, tgt_stp;

   /*
    * Use defaults if DSP code doesn't specifiy limits for exposure time.
    */
   n = sym_to_number(&leach[0], LUB, "TGT_MIN");
   if (n != SYM_INVALID) tgt_min = (double)n/1000.; else tgt_min = TGT_MIN;
   n = sym_to_number(&leach[0], LUB, "TGT_MAX");
   if (n != SYM_INVALID) tgt_max = (double)n/1000.; else tgt_max = TGT_MAX;
   n = sym_to_number(&leach[0], LUB, "TGT_STP");
   if (n != SYM_INVALID) tgt_stp = (double)n/1000.; else tgt_stp = TGT_STP;

   if (colon)
   {
      new_etime = atoi(s)*60.0 + atof(colon+1);
   }
   else
   {
      new_etime = atof(s);
   }
   free(s);

   /*
    * Round to nearest tgt_stp
    */
   new_etime += tgt_stp/2.;
   new_etime = (int)(new_etime / tgt_stp) * tgt_stp;

   if (new_etime == 0)
   {
      if (etype == ETYPE_BIAS)
      {
	 printf("NOTE: `etime' is ignored when taking BIASes.\n");
	 return PASS;
      }
      printf("error: Please select `etype BIAS' instead.  `etime' will then be ignored.\n");
      return FAIL;
   }

   if (new_etime < tgt_min || new_etime > tgt_max)
   {
      printf("error: Please enter an exposure time from %g to %g seconds. "
	     "Exposure time setting is ignored for BIASes, but a valid time must still be selected. "
	     "For example: `etime 3.0', `etime 240', `etime 4:00'.\n",
	     tgt_min, tgt_max);
      return FAIL;
   }

   if (fabs(etime - new_etime) > 0.0001)
   {
      etime = new_etime;
      parfile_update("etime changed"); /* === Update status server */
      com_preexp("reset");
   }
   return PASS;
}

PASSFAIL
com_etype(const char* arg)
{
   etype_t new_etype = name_to_etype(arg);

   if (new_etype == ETYPE_UNKNOWN)
   {
	 printf("error: Please enter an exposure type (full name) or "
		"the single letter code:\nerror: ");
	 cli_help("etype");
	 return FAIL;
   }

   if (etype == new_etype) return PASS; /* No change */

   etype_valid(ETYPE_UNKNOWN); /* Forces check of DSP code symbols */

   if (etype_valid(new_etype) == FALSE)
   {
      printf("error: DSP code or instrument does not support %s exposures\n",
	     arg);
      return FAIL;
   }

   etype = new_etype;
   parfile_update("etype changed");
   com_preexp("reset");
   return PASS;
}

#ifdef DETCOM_IR
PASSFAIL
com_rac(const char* args)
{
   static char* arg1 = 0;

   if (!args || !*args)
   {
      printf("Reset anomaly correction is currently %s.\n", rac_on?"ON":"OFF");
      return cli_help("rac");
   }
   if (arg1) free(arg1);
   arg1 = cli_arg1(args);
   if (xstrncasecmp(arg1, "tog", 3) != 0 && /* Argument is not "toggle" */
       rac_on == cli_bool(arg1)) /* And no change */
      return PASS;

   if ((rac_on = (!rac_on))) /* Toggle it */
   {
      if (camera_state(CAMERA_STATE_QUERY) == CAMERA_READY)
	 camera_state(CAMERA_RAC_ON);
      parfile_update("reset anomaly correction has been turned ON");
   }
   else
   {
      if (camera_state(CAMERA_STATE_QUERY) == CAMERA_RAC_ON)
	 camera_state(CAMERA_READY);
      parfile_update("reset anomaly correction has been turned OFF");
   }
   timed_rd1=0.;
   return PASS;
}

const char*
rac_command(void)
{
   if (rac_on) return "on";
   else return "off";
}

PASSFAIL
com_stare(const char* args)
       /* 
	* Staring mode command - engineering only - jw
	* When stare_on = TRUE , the shutter is opened before reset-ref and
	* closed after raw
	*/
{
   static char* arg1 = 0;

   if (!args || !*args)
   {
      printf("Staring Mode is currently %s.\n", stare_on?"ON":"OFF");
      return cli_help("dim");
   }
   if (arg1) free(arg1);
   arg1 = cli_arg1(args);
   if (xstrncasecmp(arg1, "tog", 3) != 0 && /* Argument is not "toggle" */
       stare_on == cli_bool(arg1)) /* And no change */
      return PASS;

   if ((stare_on = (!stare_on))) /* Toggle it */
   {
     
     printf("Staring Mode has been turned ON.\n");
     parfile_update("staring mode has been turned ON");
   }
   else
   {
     printf("Staring Mode has been turned OFF.\n");
     parfile_update("staring mode has been turned OFF");
   }
   return PASS;
}

const char*
stare_command(void)
{
   if (stare_on) return "on";
   else return "off";
}
#endif /* DETCOM_IR */

PASSFAIL
com_preexp(const char* arg)
{
   static BOOLEAN preexp_valid = FALSE;
   char tmp[80];
#define do_wrmem(board, address, value) \
   ((sprintf(tmp, "%s %s %d", (board), (address), (value))<0 || \
     com_wrmem(tmp)==FAIL)?FAIL:PASS)

   if (arg && !xstrcasecmp(arg, "reset"))
   {
      if (preexp_valid == TRUE)
      {
	 CLI_DEBUG(printf("debug: preexp status has been reset\n"));
	 preexp_valid = FALSE;
      }
      return PASS;
   }

   if (arg && *arg)
   {
      cli_help("preexp");
      return FAIL;
   }

   if (preexp_valid)
   {
      CLI_DEBUG(printf("debug: com_preexp() -- still valid; skipped\n"));
      return PASS;
   }

   CLI_DEBUG(printf("debug: com_preexp(\"%s\") starting\n", arg?arg:""));

   printf("\rprogress: Pre-exposure DSP setup . . .\n");

   /*
    * Set options bits according to exposure type.  This carefully
    * leaves any unknown bits alone, so if they get messed up,
    * they do not get reset to 0 by this pre-exposure sequence!
    */
   if (com_boot("")==FAIL) /* Check that controller is ready for commands */
   {
      if (detcom_mode==DETCOM_ENGINEERING && NO_ETIME(etype))
      {
	 if (com_tdl("lub 0x55A55A logonly:")==FAIL)
	 {
	    printf("warning: Voltage test failed, probably due to failure to "
		   "communicate with a utility board (see logonly messages above). "
		   "Since engineering mode and etype %s have been selected, "
		   "this exposure will be attempted anyway.\n",
		   etype_command());
	 }
      }
      else
      {
	 return FAIL;
      }
   }

   /*
    * Don't set utility board options if an exposure type has
    * been selected that doesn't require the utility board for
    * anything. (This includes BIAS and TESTPAT types.)
    */
   if (!NO_ETIME(etype))
   {
      /*
       * Set exposure time in msec (==1 for BIAS)
       */
      if (do_wrmem("lubM", "TGT_TIM",
		   NO_ETIME(etype) ? 1 : (int)(etime * 1000))==FAIL)
	 return FAIL;

#ifdef DETCOM_IR
      if (STAR_SH bit exists)
      {
         if (stare_on &&
             (etype==ETYPE_FLAT || etype==ETYPE_DITHER ||
              etype==ETYPE_TFLAT || etype==ETYPE_OBJECT || etype==ETYPE_ALIGN ||
              etype==ETYPE_FOCUS || etype==ETYPE_COMPARISON))
	 { if (com_wrmem("lubM OPTIONS ON  1<<STAR_SH")==FAIL) return FAIL; }
         else
	 { if (com_wrmem("lubM OPTIONS OFF 1<<STAR_SH")==FAIL) return FAIL; }
      }
#endif /* DETCOM_IR */

      if (etype_valid(ETYPE_DARK))
      {
	 if (etype==ETYPE_FLAT || etype==ETYPE_DITHER ||
	     etype==ETYPE_TFLAT || etype==ETYPE_OBJECT || etype==ETYPE_ALIGN ||
	     etype==ETYPE_FOCUS || etype==ETYPE_COMPARISON)
	 { if (com_wrmem("lubM OPTIONS ON  1<<OPT_SH")==FAIL) return FAIL; }
	 else
	 { if (com_wrmem("lubM OPTIONS OFF 1<<OPT_SH")==FAIL) return FAIL; }
      }

      if (etype_valid(ETYPE_LED))
      {
	 if (etype==ETYPE_LED)
	 { if (com_wrmem("lubM OPTIONS ON  1<<OPT_LED")==FAIL) return FAIL; }
	 else
	 { if (com_wrmem("lubM OPTIONS OFF 1<<OPT_LED")==FAIL) return FAIL; }
      }
   }

   if (etype_valid(ETYPE_TSTPAT))
   {
      if (etype==ETYPE_TSTPAT)
      { if (com_wrmem("ltb OPTIONS ON  1<<TSTPAT")==FAIL) return FAIL; }
      else
      { if (com_wrmem("ltb OPTIONS OFF 1<<TSTPAT")==FAIL) return FAIL; }
   }

   if (etype_valid(ETYPE_CNTPAT))
   {
      if (etype==ETYPE_CNTPAT)
      { if (com_wrmem("ltb OPTIONS ON  1<<CNTPAT")==FAIL) return FAIL; }
      else
      { if (com_wrmem("ltb OPTIONS OFF 1<<CNTPAT")==FAIL) return FAIL; }
   }

   /*
    * Download raster parameters to timing boards
    */
   if (raster_to_dsp()==FAIL) return FAIL;
   
   /*
    * Make sure we have a current temperature reading
    */
   com_temp("check");

   /*
    * Indicate the preexp was successful.
    */
   preexp_valid = TRUE;

   printf("\rprogress: Pre-exposure setup complete . . .\n");
   return PASS;
}


PASSFAIL
com_chargeshift(const char* arg)
{
   int i, iterations = 1;

   if (com_boot("")==FAIL) /* Check that controller is ready for commands */
   {
      printf("error: Need to have code booted and power on; chargeshift failed.\n");
      return FAIL;
   }

   if (camera_state(CAMERA_STATE_QUERY) != CAMERA_EXPOSED)
   {
      printf("error: No exposure to shift!\n");
      return FAIL;
   }

   if (arg && *arg) iterations = atoi(arg);
   if (iterations < 1) iterations = 1;

   printf("\rprogress: Shifting Charge on CCDs . . .\n");
   for (i=0; i<iterations; i++)
   {
      if (iterations > 1)
	 printf("\rprogress: Shifting Charge on CCDs . . . (%d%%)\n", i*100/iterations);

      if (com_cmd("ltb SHF")==FAIL) return FAIL;
   }
   printf("\rprogress: Charge Shift Complete . . .\n");
   return PASS;
}


PASSFAIL
com_clean(const char* arg)
{
   PASSFAIL rtn = PASS;
   int i, iterations = 1;

   CLI_DEBUG(printf("debug: com_clean(\"%s\") starting\n", arg?arg:""));

   if (com_preexp("")==FAIL) /* Check that controller is ready for commands */
   {
      printf("error: Need to have code booted and power on; clean failed.\n");
      return FAIL;
   }

   if (camera_state(CAMERA_STATE_QUERY) == CAMERA_POF ||
       camera_state(CAMERA_STATE_QUERY) == CAMERA_UNKNOWN)
   {
      printf("error: Need high voltage power on before cleaning.\n");
      return FAIL;
   }

   if (arg && *arg) iterations = atoi(arg);
   if (iterations < 1) iterations = 1;

   printf("\rprogress: Cleaning . . .\n");
   camera_state(CAMERA_CLEANING);
   
   for (i=0; i<iterations; i++)
   {
      if (iterations > 1)
	 printf("\rprogress: Cleaning . . . (%d%%)\n", i*100/iterations);

      if (n_controllers > 1)
      {
	 if (com_cmd("ltbS CLN")==FAIL) rtn = FAIL;
	 /*
	  * CLN doesn't wait for the actual cleaning to finish, but
	  * no problem since the CLR below to the master does, and
	  * should take the same amount of time to finish.  If these
	  * commands could be sent at the same time, it would be
	  * better to send CLR to both and wait for the two DON's to
	  * come back.
	  */
      }
      if (com_cmd("ltbM CLR DON 8000")==FAIL) rtn = FAIL; /* CLR waits for cleaning to finish. */
      cli_signal_unblock(SIGINT);
      cli_signal_block(SIGINT);
      if (cli_signal_event(SIGINT))
      {
	 printf("\rprogress: Cleaning aborted . . .\n");
	 printf("error: Aborted while cleaning the detector.\n");
	 camera_state(CAMERA_READY);
	 return FAIL;
      }
   }

   printf("\rprogress: Cleaning complete . . . (100%%)\n");
   if (rtn==FAIL)
      camera_state(CAMERA_READY);
   else
      camera_state(CAMERA_CLEANED);

   return rtn;
}


static shm_t* ds_shm = 0;

static void remain_decr(void) { if (ds_shm) ds_shm->etime_remaining--; }

static pid_t go_child = -1;
static BOOLEAN expose_fg = TRUE; /* Do exposure in the "foreground" */

PASSFAIL
com_expose(const char* arg)
{
   struct itimerval itv, itv_save;
   struct sigaction sigalrm_save;
   PASSFAIL rtn = PASS;
   BOOLEAN full_exposure = TRUE;
   BOOLEAN already_done = FALSE;

   CLI_DEBUG(printf("debug: com_expose(\"%s\") starting\n", arg?arg:""));

   /*
    * %%% Document
    */
   if (arg && !strcmp(arg, "fg"))
   {
      if (expose_fg != TRUE)
      {
	 expose_fg = TRUE;
	 printf("status: expose fg - `go' command executes exposure in foreground.\n");
      }
      return PASS;
   }
   if (arg && !strcmp(arg, "bg"))
   {
      if (expose_fg != FALSE)
      {
	 expose_fg = FALSE;
	 printf("status: expose bg - `go' command returns when exposure begins; use `expose wait' to wait for completion.\n");
      }
      return PASS;
   }
   if (arg && (!strcmp(arg, "wait") || !strcmp(arg, "poll")))
   {
      child_exit_t child_exit;
      int loop = 0;

      if (go_child == -1)
      {
	 CLI_DEBUG(printf("\rdebug: go thread not running\n"));
	 return PASS;
      }

      printf("logonly: expose thread %d being waited on.\n", go_child);
 
      ds_shm = im_buf_get(FALSE);
   
      while ((child_exit = pid_wait(go_child,1)) == CHILD_RUNNING)
      {
	 if (!strcmp(arg, "poll")) return PASS;
	 if ((loop++ % 10) == 0) /* Update 1 / sec */
	    printf("\rprogress: Exposure %s in progress.  Time remaining: %s (%d%%)\n",
		   filebase(), etime_remain(ds_shm->etime_remaining),
		   ((int)etime - ds_shm->etime_remaining)*100 / (int)etime);
	 xusleep(100000); /* Wait 1/10 sec */
      }

      go_child = -1; /* Make sure it doesn't get waited on again. */

      switch (child_exit)
      {
	 case CHILD_EXITPASS:
	 camera_state(CAMERA_EXPOSED); /* Make sure detcom knows not to go into idle mode */
	 printf("logonly: expose thread finished successfully.\n");
	 printf("progress: Exposure %s complete . . . (100%%)\n", filebase());
	 com_readout("bg"); /* "expose bg" REQUIRES "readout bg"! */
	 com_readout("auto");
	 return PASS;
	 break;

	 case CHILD_EXITSIG:
	 printf("warning: Exposure interrupted.\n");
	 /* Fall through */
	 case CHILD_EXITFAIL:
	 printf("error: Exposure failed.\n");
	 printf("progress: Exposure %s aborted!\n", filebase());
	 break;

         /*
          * These shouldn't happen:
          */
         case CHILD_ERROR:
         case CHILD_GONE:
         case CHILD_RUNNING:
	 printf("error: expose thread had unknown problem\n");
	 break;
      }
      return FAIL;
   }

   etime_dsp = -1.;
   memset(&detcom_obs_eip, 0, sizeof(detcom_obs_eip));
   memset(&detcom_obs_don, 0, sizeof(detcom_obs_don));
   blade_open = 0;
   blade_close = 0;

   com_readout("wait"); /* Just in case */

#ifndef DETCOM_IR
   filename_next();
   /*
    * Include this if we always want the filename to go to the "next"
    * exposure number as soon as the corresponding "go" is issued.
    * If you comment it out, the exposure number changes when the
    * previous save is completed instead (which is usually the same
    * behavior, except when exposures and save operations overlap.)
    * If a "go" script calls "filename next", all this is moot anyway.
    */
#endif

   /*
    * This waitdata -poll (and one further down in the expose loop)
    * helps to give better feedback of the exposure sequence.  Even
    * though exposure N is now beginning, exposure N-1's readout
    * may still be in the process of being saved to disk.  Calling
    * waitdata -poll checks to see if this is done yet.  As soon as
    * it is, a message will be printed and the "current" exposure
    * number will be updated to match the exposure now in progress.
    */
   if (data_state(DATA_STATE_QUERY)!=DATA_EMPTY
#ifdef DETCOM_IR
       && data_state(DATA_STATE_QUERY)!=DATA_READING_REF
#endif
      )
   {
      if (com_waitdata("-poll"));
      if (data_state(DATA_STATE_QUERY)!=DATA_SAVING)
      {
	 data_state(DATA_EMPTY);
	 filename_next();
      }
   }

   if (etype != ETYPE_LED)
   {
      if (have_shutter())
	 if (com_shutter("unsleep")==FAIL) return FAIL;
      if (com_shutter("wait")==FAIL ||
	  com_filter("wait")==FAIL)
      {
	 if (!have_shutter())
	 {
	    /* Might as well take a bias or dark even if shutter/filter fail */
	    printf("warning: Ignoring the error because this exposure type doesn't use the shutter.\n");
	 }
	 else
	 {
	    return FAIL;
	 }
      }
   }
   /* %%% What about biases?  Could still be noise if filter wheel
    * still moving for example.  Move to preexp?
    */

   printf("progress: Starting exposure. . .\n");

#undef FAKE_EXPOSURES
#ifdef FAKE_EXPOSURES
   if (etype != ETYPE_DARK)
   {
      /*
       * %%% Hack!
       */
      printf("warning: Faking exposure type to BIAS for tests...\n");
      /*
       * Go through the motions, just so the state machine doesn't get unhappy.
       */
      camera_state(CAMERA_EXPOSING);
      camera_state(CAMERA_EXPOSED);
      return PASS;
   }
#endif

   /*
    * Double-check that pre-exposure settings are still intact.
    */
   if (com_preexp("") == FAIL)
      return FAIL;

   /*
    * Send 'SEX' command to master and wait for 'EIP' in response.
    */
   if (com_cmd("-quiet lubM SEX EIP")==FAIL)
   {
      printf("error: Unable to start exposure (power or shutter problem?)\n");
      return FAIL;
   }
   /*
    * Save a timestamp of when the EIP was received.
    */
   gettimeofday(&detcom_obs_eip, 0);

   camera_state(CAMERA_EXPOSING);

   ds_shm = im_buf_get(FALSE);
   ds_shm->etime_remaining = etime;

   if (!strcmp(arg, "start") ||
       (!strcmp(arg, "auto") && expose_fg == FALSE))
   {
      switch (go_child = fork())
      {
	 case -1: perror("error: fork failed"); return FAIL; break;
	 case 0: /* Child thread */ expose_fg = FALSE; break;
	 default:
	 {
	    fprintf(stderr,
		    "logonly: expose thread %d proceeding in background\n",
		    go_child);
	    return PASS;
	 }
      }
   }
   
   itv.it_interval.tv_sec = 1;
   itv.it_interval.tv_usec = 0;
   itv.it_value.tv_sec = 1;
   itv.it_value.tv_usec = 0;

   /*
    * Start a 1 Hz alarm signal for timing the exposure.
    */
   sigaction(SIGALRM, 0, &sigalrm_save);
   cli_signal(SIGALRM, remain_decr);
   setitimer(ITIMER_REAL, &itv, &itv_save);
   cli_signal_unblock(SIGALRM); /* %%% Check signal mask instead? */

   if (quit!=TRUE) cli_signal(SIGQUIT, sigquit_handler_expose);

   while (ds_shm->etime_remaining > 1 && rtn == PASS)
   {
      if (go_child == -1)
      {
	 if (data_state(DATA_STATE_QUERY)!=DATA_EMPTY
#ifdef DETCOM_IR
	     && data_state(DATA_STATE_QUERY)!=DATA_READING_REF
#endif
	    )
	 {
	    if (com_waitdata("-poll"));
	    if (data_state(DATA_STATE_QUERY)!=DATA_SAVING)
	    {
	       data_state(DATA_EMPTY);
	       filename_next();
	    }
	 }
	 printf("\rprogress: Exposure %s in progress.  Time remaining: %s (%d%%)\n",
		filebase(), etime_remain(ds_shm->etime_remaining),
		((int)etime - ds_shm->etime_remaining)*100 / (int)etime);
      }
      
      cli_signal_unblock(SIGINT);
      cli_signal_unblock(SIGQUIT);
      cli_signal_unblock(SIGTSTP);
      
      pause();
      
      cli_signal_block(SIGINT);
      cli_signal_block(SIGQUIT);
      cli_signal_block(SIGTSTP);

      if (ds_shm->etime_remaining > 2)
      {
	 /*
	  * Check for the possible "interrupt" signals
	  */
	 if (cli_signal_event(SIGINT))
	 {
	    printf("\rprogress: Exposure %s aborted!\n", filebase());
	    printf("\rerror: Exposure aborted by user with %d/%d seconds remaining!  Not reading out.\n",
		   ds_shm->etime_remaining, (int)etime);
	    com_cmd("lub AEX");
	    full_exposure = FALSE;
	    rtn = FAIL;
	 }
	 else if (cli_signal_event(SIGTSTP))
	 {
	    printf("\rwarning: User stop.  Skipping last %d/%d seconds of this exposure "
		   "and reading out immediately.\n", ds_shm->etime_remaining,
		   (int)etime);
	    com_cmd("lubM STE");
	    full_exposure = FALSE;
	    ds_shm->etime_remaining = 0;
	 }
	 /*
	  * Check for unsolicited data from any controller.
	  * Depending on the message, try to recover in different ways.
	  */
	 /* %%% Use of ualarm() in leach_poll() conflicts with setitimer. */
#ifdef LINUX
	 else switch (leach_poll(/*verbose=*/1))
	 {
	    case LEACH_POLL_NODATA: break;
	    case LEACH_POLL_GOODRESP: /* shouldn't happen %%% */
	    case LEACH_POLL_BADRESP: /* shouldn't happen */
	    case LEACH_POLL_ERROR: /* %%% Handle */
	    case LEACH_POLL_GARBAGE:
	    case LEACH_POLL_ZEROPIX:
	    {
	       printf("\rwarning: Stray data from controller during exposure.  "
		      "Trying to salvage exposure.  Hit abort if you want to start over.\n");
	       break;
	    }
	    case LEACH_POLL_DON:
	    {
	       printf("\rwarning: DON came back %d seconds earlier than expected.\n",
		      ds_shm->etime_remaining+1);
	       ds_shm->etime_remaining = 0;
	       already_done = TRUE;
	       break;
	    }
	    case LEACH_POLL_ERR:
	    {
	       printf("\rerror: Unexpected ERR from CCD controller.  "
		      "Closing shutter and trying to salvage exposure.\n");
	       /*
		* Error is printed, but com_expose still returns PASS if
		* it was able to close the shutter.
		*/
	       full_exposure = FALSE;
	       ds_shm->etime_remaining = 0;
	       if (com_shutter("wait")==FAIL) rtn = FAIL;
	       else rtn = com_shutter("close");
	       break;
	    }
	    case LEACH_POLL_SYR:
	    {
	       printf("\rerror: Unexpected CCD controller reset!  Not reading out.\n");
	       com_cmd("lub AEX");
	       full_exposure = FALSE;
	       rtn = FAIL;
	       break;
	    }
	 }
#endif
      }
      else
      {
	 if (quit!=TRUE) cli_signal(SIGQUIT, sigquit_handler);
      }
   }
   if (quit!=TRUE) cli_signal(SIGQUIT, sigquit_handler);
   
   if (rtn == PASS)
   {
      if (expose_fg)
      {
	 fprintf(expose_fg?stdout:stderr,
		 "progress: Exposure %s in progress.  Waiting for DSP . . . (100%%)\n",
		 filebase());
      }
	 
      if (!already_done &&
	  com_cmd("-quiet lubM '' DON 4000")==FAIL) /* Wait up to 4 seconds for a DON */
      {
	 printf("\rwarning: Failure at end of exposure.\n");
	 if (com_shutter("wait")==PASS && com_shutter("close")==PASS)
	 {
	    printf("\rwarning: DetCom was able to close shutter immediately.  Exposure is probably OK after all.\n");
	 }
	 else
	 {
	    printf("\rerror: Cannot verify that shutter actually closed.  Trying to reset shutter.\n");
	    com_shutter("reset");
	    rtn = FAIL;
	 }
      }
      /*
       * Save a timestamp of when the DON was received.
       */
      gettimeofday(&detcom_obs_don, 0);
   }

   /*
    * Restore handling of the alarm signal and interval timer.
    */
   setitimer(ITIMER_REAL, &itv_save, 0);
   sigaction(SIGALRM, &sigalrm_save, 0);

   if (go_child == -1)
      camera_state(CAMERA_EXPOSED); /* Make sure detcom knows not to go into idle mode */

   if (rtn==PASS)
   {
      unsigned long tgt_tim;

      if (expose_fg)
	 printf("progress: Reading DSP Exposure Time . . . (100%%)\n");

      tgt_tim = sym_rdmem("lubM", "TGT_TIM") & ~FLAG16;
      if (tgt_tim == (SYM_INVALID&~FLAG16))
      {
	 printf("\rwarning: Unable to get DSP-measured exposure time.\n");
      }
      else
      {
	 if (have_shutter())
	 {
	    static char* dsp_version_ = 0;
	    static unsigned long tgt_h0;
	    static unsigned long tgt_h1;

	    if (!dsp_version_ || strcmp(dsp_version_, dsp_version()))
	    {
	       if (dsp_version_) free(dsp_version_);
	       dsp_version_ = strdup(dsp_version());
	       
	       fprintf(expose_fg?stdout:stderr,
		       "\rlogonly: Checking DSP symbols for shutter feedback\n");

	       tgt_h0 = sym_rdmem("lubM", "TGT_H0") & ~FLAG16;
	       tgt_h1 = sym_rdmem("lubM", "TGT_H1") & ~FLAG16;
	    }

	    if ((tgt_h0 != (SYM_INVALID & ~FLAG16)) &&
		(tgt_h1 != (SYM_INVALID & ~FLAG16)))
	    {
	       blade_open = tgt_h0;
	       blade_close = tgt_h1 - tgt_tim;
	    
	       if (tgt_h0 > tgt_h1 || tgt_h0 > tgt_tim || tgt_tim > tgt_h1)
	       {
		  printf("\rwarning: Shutter ballistic values don't "
			 "make sense (h0=%ld, h1=%ld, tim=%ld)\n",
			 tgt_h0, tgt_h1, tgt_tim);
	       }
	       else if (blade_open > 600 || blade_close > 600 ||
			fabs((double)blade_open - (double)blade_close) > 400.) /* %%% Set these limits */
		  /* === Get these from DSP code */
	       {
		  printf("\rwarning: Shutter blades may be sluggish: open=%ld, close=%ld ms.\n",
			 blade_open, blade_close);
	       }
	       else
	       {
		  fprintf(expose_fg?stdout:stderr,
			  "\rlogonly: Shutter ballistics: time to open=%ld, close=%ld ms.\n",
			  blade_open, blade_close);
	       }
	    }
	    com_shuttercount("increment");
	 }

	 etime_dsp = tgt_tim / 1000.;

	 if (full_exposure)
	 {
	    if (fabs(etime_dsp - etime) > 1)
	    {
	       printf("\rwarning: DSP-measured exposure time (%f) disagrees "
		      "with unix-measured exposure time (%f); using unix value.\n",
		      etime_dsp, etime);
	       etime_dsp = -1.;
	    }
	    else if (fabs(etime_dsp - etime) > 0.1)
	    {
	       printf("\rwarning: DSP-measured exposure time (%f) differs slightly "
		      "from requested time (%f).\n",
		      etime_dsp, etime);
	    }
	 }
	 else
	 {
	    if (etime_dsp > etime)
	    {
	       printf("\rerror: DSP-measured exposure time (%f) after abort or stop "
		      "is greater than requested exposure time (%f)!\n",
		      etime_dsp, etime);
	       etime_dsp = -1.;
	    }
	 }
      }
   }
   if (rtn == PASS && expose_fg)
      printf("progress: Exposure %s complete . . . (100%%)\n", filebase());

   if (!strcmp(arg, "auto") && expose_fg == FALSE)
   {
      _exit(rtn==PASS?EXIT_SUCCESS:EXIT_FAILURE);
      _exit(EXIT_FAILURE);
   }

   return rtn;
}

PASSFAIL
com_go(const char* arg)
   /*
    * The "go" command executes "preexp" "clean" "expose" and "readout",
    * but it may skip "preexp" and "clean" if this is a go in the middle
    * of a focus sequence.  It may skip "expose" if this is a BIAS or
    * TESTPAT readout.  And it may skip "readout" if this is a focus
    * sequence and not the last "go" of the sequence.  The last go is
    * given by giving "go -r".  -r is ignored for other exposure types.
    */
{
   static BOOLEAN first_go = TRUE; /* First go after a readout? */
   BOOLEAN do_focus_readout = FALSE;
   int i, iterations = 1;

   CLI_DEBUG(printf("debug: com_go(\"%s\") starting\n", arg?arg:""));

   if (arg && !xstrncasecmp(arg, "-r", 2))
   {
      do_focus_readout = TRUE;
   }
   else if (arg && *arg)
   {
      if ((iterations = atoi(arg)) <= 0)
      {
	 printf("error: usage: go [-readout] [N]\n");
	 return FAIL;
      }
   }
   
   quit = TRUE;

   if (iterations > 1)
   {
      if (etype == ETYPE_FOCUS)
      {
	 iterations = 1;
	 printf("logonly: Setting iterations to 1 because of FOCUS mode.\n");
      }
      else
      {
	 quit = FALSE;
	 printf("Use `quit' to break out of the sequence.\n");
	 cli_signal(SIGQUIT, sigquit_handler);
	 cli_signal_block(SIGQUIT);
      }
   }

   for (i=0; i<iterations && (iterations==1 || quit==FALSE); i++)
   {
      if (iterations > 1)
      {
	 exp_iteration = i+1;
	 statusbar_update();
      }
      else
      {
	 exp_iteration = 0;
      }

      if (etype!=ETYPE_FOCUS) first_go = TRUE;

      /*
       * === Break this up into pre-exposure and pre-readout stuff?
       * The power test must happen before the clear.  Some of the
       * other stuff doesn't need to happen *each* go in FOCUS mode.
       */
      if (com_preexp("")==FAIL) return FAIL;

#ifdef DETCOM_IR
      set_auto_save_count(exp_iteration, iterations);

      if (!im_buf_new())
      {
         printf("error: Failed to get an SHM buffer\n");
         return FAIL;
      }
#endif

      if (iterations > 1)
	 printf("status: Taking %s exposure %d of %d.\n",
		etype_command(), exp_iteration, iterations);

      if (first_go)
      {
	 /*
	  * A friendly reminder to the user if they aren't getting
	  * all the data on disk (this may be normal if there was
	  * an abort or testpattern readout that no one wants to save.)
	  */
	 switch (data_state(DATA_STATE_QUERY))
	 {
#ifdef DETCOM_IR
	    case DATA_READING_REF:
	    case DATA_READING_RAW:
#endif
	    case DATA_READING:
	    {
	       /*
		* Only generate a warning if the current exposure type
		* is one that collected real photons.
		*/
	       if (!NO_ETIME(etype))
	       {
		  printf("warning: Previous readout was never saved.\n");
		  if (etime > 2. && !NO_ETIME(etype))
		     printf("warning: You have %d seconds to `abort' this "
			    "exposure and still `save' the last readout.\n",
			    (int)etime);
	       }
	       break;
	    }
	    case DATA_ABORTED:
	    {
	       printf("warning: Previous readout was aborted and not saved.\n");
	       break;
	    }
	    case DATA_FAILED:
	    {
	       printf("warning: Previous readout not completely saved.\n");
	       break;
	    }
	    case DATA_EMPTY:  /* Ok ... */
	    case DATA_SAVING:
	    case DATA_SAVED: break;
	    case DATA_STATE_QUERY: /* Won't happen */ break;
	 }
	 if (etype == ETYPE_TSTPAT || etype == ETYPE_CNTPAT)
	 {
	    /* Make the sanity-check state machine happy */
	    camera_state(CAMERA_READY);
	    camera_state(CAMERA_CLEANING);
	    camera_state(CAMERA_CLEANED);
	 }
	 else
	 {
	    /*
	     * Don't add overhead of writing par file during short exposures.
	     */
	    if (etime < 60.0)
	       timing_critical = TRUE;

	    if (camera_state(CAMERA_STATE_QUERY) == CAMERA_CLEANED)
	    {
	       CLI_DEBUG(printf("debug: com_clean() -- already clean; skipped\n"));
	    }
	    else
	    {
	       if (com_clean("")==FAIL) return FAIL;
	    }
	 }
      }
      
      memset(&detcom_obs_eip, 0, sizeof(detcom_obs_eip));
      memset(&detcom_obs_don, 0, sizeof(detcom_obs_don));
      if (!NO_ETIME(etype))
      {
	 /* "etype focus" is incompatible with "expose bg" and nexp>1*/
	 if (etype != ETYPE_FOCUS && iterations <= 1)
	 {
	    if (com_expose("auto")!=PASS) return FAIL;
	 }
	 else
	 {
	    if (com_expose("")!=PASS) return FAIL;
	 }
      }

      if (etype == ETYPE_FOCUS && !do_focus_readout)
      {
#ifdef CAMERA_CFH12K
	 printf("progress: Recocking shutter. . .\n");
	 com_shutter("recock"); /* Usually happens at end of readout, but
				 * in this case there will be no readout before
				 * the next exposure
				 */
#endif
	 first_go = FALSE; /* Don't clean next time since we haven't read out */
      }
      else if (etype == ETYPE_FOCUS || iterations > 1 || expose_fg == TRUE)
      {
	 if (com_readout("auto")==FAIL) return FAIL;
	 timing_critical = FALSE;
	 first_go = TRUE;
      }
      /* else ... "expose wait" starts the readout. */
   }

   if (iterations > 1 && i < iterations)
   {
      printf("status: Quitting after %d of %d exposures\n",
	     i, iterations);
      return PASS; /* This is not considered an error. */
   }

   return PASS;
}
