/*                                             -*- c-file-style: "Ellemtel" -*-

`det_modes.c' - detcom software settings  (filename,modes,.detrc,defaults)

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h> /* for file locking */
#include <math.h> /* for fabs() */
#if defined(CAMERA_CFH12K) || defined(CFHT_OD)
#include <cfht/cfht.h>
#endif
#include "cli/cli.h"
#include "fh/fh.h"
#include "fh/fh_registry.h"
#include "detcom.h"

#define RCFILENAME ".detcomrc"
#define PARFILENAME ".,detcom.par"

/*
 * Synonyms (aliases) available in observing mode.
 * Synonyms aren't advertised in the help, and scripts
 * should stick to using the preferred names instead of
 * these.
 */
#define COMLIST_OBSERVING_SYNONYMS \
   { "?                 ",com_help,       "Synonym for help" },		\
   { "file              ",com_filename,   "Synonym for filename"}, 	\
   { "rdm               ",com_rdmem,      "Synonym for rdmem" },	\
   { "tstpat            ",com_testpat,    "Synonym for testpat" },	\
   { "cln               ",com_clean,	  "Synonym for clean" },	\
   { "clr               ",com_clean,	  "Synonym for clean" },	\
   { "flush             ",com_clean,	  "Synonym for clean" },	\
   { "fits <c> <val>    ",com_header,     "Synonym for header" },	\
   { "mosaic <rspec>    ",com_raster,     "Synonym for raster" },	\
   { "download          ",com_boot,	  "Synonym for boot" }

/*
 * Some additional synonyms that work in only engineering mode.
 */
#define COMLIST_ENGINEERING_SYNONYMS \
   { "command           ",com_cmd,        "Synonym for cmd" },		\
   { "wrm               ",com_wrmem,      "Synonym for wrmem" },	\
   { "ptest             ",com_power,      "Synonym for power" },	\
   { "prexp             ",com_preexp,	  "Synonym for preexp" },	\
   { "voltage           ",com_power,      "Synonym for power" }

/*
 * These commands are not useful to the observer, or perhaps
 * even dangerous to allow outside of engineering mode.
 */
#define COMLIST_ENGINEERING \
   { "wrmem LB addr data",com_wrmem,      "Write DSP memory (optional arg: mask)" }, \
   { "auto <fnc> on|off ",com_auto,       "Toggle auto <fnc>=`save' `clean' or `voltage'"}, \
   { "cmd LB XYZ        ",com_cmd,        "Issue any basic leach command to any board(s)" }, \
   { "pon [LB]          ",com_pon,        "(Issue PON command manually (utility boards only))" }, \
   { "pof [LB]          ",com_pof,        "(Issue POF command manually (utility boards only))" }, \
   { "syr               ",com_syr,        "Send an SYR command to each board" }, \
   { "power [OP] [LB]   ",com_power,      "OP=on:PON off:POF test:PTS, or read voltages" }, \
   { "preexp            ",com_preexp,     "Used by go to set up pre-exposure DSP parameters" }

/*
 * These are normally used for engineering, but are safe
 * enough to leave in for observing mode.  In observing
 * mode, these are un-advertised.
 *
 * There may be some rope here for the observer to hang
 * themselves with (for example, they could change the
 * filename to something manually and take a couple of
 * exposures that overwrite each other.)  But there must
 * not be anything in here which could allow them to damage
 * the equipment.
 *
 * === TODO: Print a warning when overwriting with something like:
 * warning: Backing up existing `ccd.fits' as `ccd.fits.BAK'
 *
 * If observers see this message once, they should realize it is
 * still not too late to salvage the last copy.  If they keep
 * seeing the message, it should be obvious enough that data
 * is being lost.
 */
#define COMLIST_ENGINEERING_SAFE(ad) \
   { "mode <modename>   ",com_mode,	  ad?"Use `mode observing' for normal operations":"()" }, \
   { "tdl [LB [1234]]   ",com_tdl,        ad?"Test Data Link (Board and data pattern optional)":"()" }, \
   { "boot LB <name>    ",com_boot,       ad?"download DSP lod file <name> to specified boards.":"()" }, \
   { "symbol LB [name]  ",com_symbol,     ad?"Print values of DSP code symbols found in .lod file":"()" }, \
   { "shutter [OP]      ",com_shutter,    ad?"Operate shutter; OP=open close recock wait status":"()" }, \
   { "shuttercount <arg>",com_shuttercount,ad?"`reset' or `increment' the shutter counter":"()" }, \
   { "filename <string> ",com_filename,   ad?"Set filename base string for next exposure":"()" }, \
   { "fileno <N>        ",com_fileno,     ad?"Filename with ??? only: next sequence":"()" }, \
   { "testpat           ",com_testpat,    ad?"Verify in-memory data against test pattern":"()" }, \
   { "testccd           ",com_testccd,    ad?"Verify that in-memory data looks like a CCD":"()" }, \
   { "temp              ",com_temp,       ad?"Check CCD temperature":"()" }, \
   { "save              ",com_save,       ad?"Save in-memory data to disk (use w/`auto save off')":"()" }, \
   { "savepreview       ",com_savepreview,ad?"Save in-memory data to disk as binned 8x8 composite":"()" }, \
   { "waitdata          ",com_waitdata,   ad?"Data writing runs in the background; Use this to wait":"()" }, \
   { "abortdata         ",com_abortdata,  ad?"Kills any data writer threads as soon as possible":"()" }, \
   { "rdmem LB addr     ",com_rdmem,      ad?"Read DSP memory (optional args: expected_data, mask)":"()" }, \
   { "expose            ",com_expose,     ad?"Used by go to time an exposure (SEX to lubM)":"()" }, \
   { ad?"expose fg|bg       : Select whether exposure runs in foreground or background":"", 0, 0 }, \
   { "readout           ",com_readout,	  ad?"Used by go to readout CCDs (RDC to ltbM)":"()" }, \
   { ad?"readout fg|bg      : Select whether readouts run in foreground or background":"", 0, 0 }

/*
 * Finally, the set of commands the user normally works with.
 */
#ifdef CAMERA_CFH12K
/*
 * CFH12K Help screen (kind of ugly, since it is hardcoded into %%%
 * DetCom at the moment.
 *
 * %%% What about snap/flux?
 */
#define COMLIST_OBSERVING \
   { "*---[ Detector/Instrument Setup Commands ]-----------------------------------*", 0, 0 }, \
   { "defaults           ",com_defaults,   "Reset filename, etime, etype, and raster settings" }, \
   { "mef on|off         ",com_mef,	  "Save multi-extension (on) or separate FITS files (off)"}, \
   { "header <c> <val>   ",com_header,     "Set keyword `<c>' (comment,observer,object) to `<val>'"}, \
   { "dheader <args>     ",com_dheader,   "(Used internally for DSP code keywords)" }, \
   { "nheader <args>     ",com_nheader,	  "Set any keyword; <args>=`index CARDNAME value comment'" }, \
   { "filter <name>      ",com_filter,     "Moves a new filter into beam (focus is auto-adjusted)" }, \
   { "raster <rspec>     ",com_raster,     "<rspec>=`xc yc xs ys [xb yb]' (See `raster help')" }, \
   { "etype <type>       ",com_etype,	  "Select <type>=OBJECT, BIAS, DARK, FLAT (for dome flats)" }, \
   { "etype <sequence>   :    <sequence>=DITHER, FOCUS, or TFLAT (twilight flats)", 0, 0 }, \
   { "etime <sec>        ",com_etime,	  "Set exposure time; <sec> can be a floating point number" }, \
   { "nexp <number>      : Set the number of exposures in a set or sequence" }, \
   { "startexp <number>  : Set starting exposure number; Default is 1 or resume" }, \
   { "dppattn <pattn>    : Selects dither pattern 2-21; `etype DITHER' only" }, \
   { "*---[ Starting and Stopping Exposures ]--------------------------------------*", 0, 0 }, \
   { "chargeshift [n]    ",com_chargeshift,"(Shift charge on CCDs [n] times)" }, \
   { "clean [n]          ",com_clean,      "(Force [n] extra cleaning passes of the detector now)" }, \
   { "go                 ",com_go,	  "Start `nexp' exposures of current `etype' and `etime'" }, \
   { "stop               : Shorten an exposure (close shutter and readout now)", 0, 0 }, \
   { "break              : Complete current exposure, then break out of sequence", 0, 0 }, \
   { "abort              : Use with caution, and only when all else fails!", 0, 0 }, \
   { "*---[ Telecope Commands, Other Commands ]------------------------------------*", 0, 0 }, \
   { "userfocus 0        : Discard current focus and start at \"nominal\" value", 0, 0 }, \
   { "userfocus <z-mm>   : Move bonnette Z to (abs.) offset from \"nominal\" value", 0, 0 }, \
   { "focusstep <z-mm>   : Sets bonnette Z focus step size; `etype FOCUS' only" }, \
   { "tcoords <coords>   : <coords>=\"ra dec eq [pmra pmdec ['object name']]\"", 0, 0 }, \
   { "tcoords zenith     : Ask Telescope to slew to zenith for next observation", 0, 0 }, \
   { "gcoords <coords>   : Pass coords of next guide star to Observing Assistant", 0, 0 }, \
   { "gcoords select|none: Next guide star (select)ed by O.A. or unguided (none)", 0, 0 }, \
   { "ocoords <args>     : Set up Telescope offset (See `ocoords help')", 0, 0 }, \
   { "exit               ",com_exit,       "(Exit the program)" }, \
   { "help               ",com_help,       "(Displays this help screen of DetCom commands)" }, \
   { "cd [<directory>]   ",cli_cd,         "Change/Show current working directory for images"}, \
   { "ls [<files>]       ",com_ls,         "List files in the current working directory"}, \
   { "say <message>      : Broadcast a message to other users on-line", 0, 0 }
#else

/*
 * Standard DetCom Commands (not including extra CFH12K stuff)
 */
#define COMLIST_OBSERVING \
   { "*---[ Detector/Instrument Setup Commands ]-----------------------------------*", 0, 0 }, \
   { "defaults           ",com_defaults,   "Reset filename, etime, etype, and raster settings" }, \
   { "mef on|off         ",com_mef,	  "Save multi-extension (on) or separate FITS files (off)"}, \
   { "reserve <number>   ",com_reserve,    "Set number of reserved FITS keyword slots."}, \
   { "header <c> <val>   ",com_header,     "Set keyword `<c>' (comment,observer,object) to `<val>'"}, \
   { "dheader <args>     ",com_dheader,   "(Used internally for DSP code keywords)" }, \
   { "nheader <args>     ",com_nheader,	  "Set any keyword; <args>=`index CARDNAME value comment'" }, \
   { "filter <name>      ",com_filter,     "Moves a new filter into beam" }, \
   { "raster <rspec>     ",com_raster,     "<rspec>=`xc yc xs ys [xb yb]' (See `raster help')" }, \
   { "etype <type>       ",com_etype,	  "Select <type>=OBJECT, BIAS, DARK, FLAT" }, \
   { "etime <sec>        ",com_etime,	  "Set exposure time; <sec> can be a floating point number" }, \
   { "*---[ Starting and Stopping Exposures ]--------------------------------------*", 0, 0 }, \
   { "clean [n]          ",com_clean,      "(Force [n] extra cleaning passes of the detector now)" }, \
   { "go [n]            ",com_go,	  "Start (n) exposure(s) with current etime and etype" }, \
   { "stop               : Shorten an exposure (close shutter and readout now)", 0, 0 }, \
   { "break              : Complete current exposure, then break out of sequence", 0, 0 }, \
   { "abort              : Use with caution, and only when all else fails!", 0, 0 }, \
   { "exit              ",com_exit,       "(Exit the program)" }, \
   { "*---[ Other Commands ]-------------------------------------------------------*", 0, 0 }, \
   { "exit               ",com_exit,       "(Exit the program)" }, \
   { "help               ",com_help,       "(Displays this help screen of DetCom commands)" }, \
   { "cd [<directory>]   ",cli_cd,         "Change/Show current working directory for images"}, \
   { "ls [<files>]       ",com_ls,         "List files in the current working directory"}, \
   { "say <message>      : Broadcast a message to other users on-line", 0, 0 }
#endif

static Command comlist_observing[] = {
   /*
    * List of all commands this cli understands in a specific mode.
    * The cli_help() functions specifically keys off of any commands
    * descriptions that start with "Synonym" and skips them when listing
    * the full set of commands.
    */
   { "",                  com_tdl,       "Synonym for tdl" },
   COMLIST_OBSERVING_SYNONYMS,
   COMLIST_ENGINEERING_SAFE(/*advertise=*/0),
   COMLIST_OBSERVING,
   { "*----------------------------------------------------------------------------*", 0, 0 },
   { 0, 0, 0 } /* This terminates the command list */
};


static Command comlist_engineering[] = {
   { "",                  com_tdl,        "Synonym for tdl" },
   COMLIST_OBSERVING_SYNONYMS,
   COMLIST_ENGINEERING_SYNONYMS,
   { "*---[ Engineering Commands ]-------------------------------------------------*", 0, 0 },
   { "* `LB' means a leach board name-- `lub' for utility board and `ltb' for timing "
       "board.  A `0' / `1' (or `M' / `S') can be appended to select only the master "
       "or slave controller.  For example, `lub' selects both utility boards, `lubS' "
       "selects only the slave's utility board.  Just an `M', `S' or `*' can be used "
       "to select all applicable boards on the master,  slave,  or both controllers.", 0, 0 },
   { "* `addr' can be either a symbol name from the DSP source or an address.  (E.g. "
       "`OPTION' or `Y:0000'.)   Address values are assumed to be given in hexadecimal.", 0, 0 },
   { "* `data' can be a symbol as well, but is usually a numeric value.  Unlike addr "
       "the value of `data' is assumed to be in decimal, unless given with a leading "
       "`0x'. A leading `0' implies an octal value (consistent w/ the C strtoul function).", 0, 0 },
   COMLIST_ENGINEERING_SAFE(/*advertise=*/1),
   COMLIST_ENGINEERING,
   COMLIST_OBSERVING,
   { "*----------------------------------------------------------------------------*", 0, 0 },
   { 0, 0, 0 } /* This terminates the command list */
};


static FILE* rcfile = 0;
static char filebase_[80] = "image???$";
static char filebase_undo[80] = "";
static BOOLEAN filebase_used = 0;   /* Set to 1 when filename has been used */
static int filebase_update = 0; /* Increment this to force filebase()
				 * to regenerate the proper filename.
				 * This should be done each time the
				 * file sequence number, exposure type,
				 * mef on/off, or base name changes.
				 */

static void frameno_advance(void);

static PASSFAIL
idle_mode_off(void)
{
   printf("\rlogonly: Turning off idle mode.\n");
   return com_cmd("-quiet ltb STP");
}

static PASSFAIL
idle_mode_on(void)
{
   static BOOLEAN warned = FALSE;
   
   printf("\rlogonly: Turning on idle mode.\n");
   if (com_cmd("-quiet ltb IDL")==PASS)
   {
      if (warned)
      {
	 warned = FALSE;
	 printf("status: Auto clean mode is working again.\n");
      }
      return PASS;
   }
   else
   {
      if (!warned)
      {
	 printf("warning: Can't turn on auto clean mode; will keep trying.\n");
	 warned = TRUE;
      }
      return FAIL;
   }
}

static const char*
camera_state_str(camera_state_t state)
{
   switch (state)
   {
      case CAMERA_STATE_QUERY: return "CAMERA_STATE_QUERY"; /* Shouldn't happen! */
      case CAMERA_UNKNOWN:	return "Error";
      case CAMERA_POF:		return "POF";
      case CAMERA_SWEEPING:	return "Sweeping";
      case CAMERA_CLEANING:	return "Cleaning";
      case CAMERA_CLEANED:	return "Cleaned";
      case CAMERA_EXPOSING:	return "Exposing";
      case CAMERA_EXPOSED:	return "Exposed";
      case CAMERA_READING:	return "Reading";
      case CAMERA_READY:	return "";
      default:	return "Internal error"; /* Shouldn't happen! */
   }
   return "Internal error"; /* Shouldn't happen */
}

camera_state_t
camera_state(camera_state_t new_state)
{
   static camera_state_t camera_state_ = CAMERA_UNKNOWN;
   /*
    * Assume idle mode might be on at startup to make
    * sure it gets turned off before downloading.
    */

   if (new_state == CAMERA_STATE_QUERY) return camera_state_;

   if (camera_state_ == CAMERA_SWEEPING)
   {
      idle_mode_off(); /* Make sure SWEEPING is off before switching states. */
      /* See also: call to idle_mode_on() below. */
   }

   switch (new_state)
   {
      case CAMERA_UNKNOWN: break; /* On error, go to UNKNOWN from any state. */
      case CAMERA_POF:     break; /* A POF can be issued any time too. */
      case CAMERA_READY:
      {
	 if (camera_state_ == CAMERA_EXPOSED)
	 {
	    printf("warning: Previous exposure not read out.\n");
	 }
	 else if (camera_state_ == CAMERA_SWEEPING ||
		  camera_state_ == CAMERA_CLEANING ||
		  camera_state_ == CAMERA_CLEANED)
	 {
	    /*
	     * This is OK.  It can happen after a download.
	     *
	     * printf("warning: Camera should be CLEANED, not READY.\n");
	     */
	 }
	 else if (camera_state_ == CAMERA_EXPOSED ||
		  camera_state_ == CAMERA_EXPOSING)
	 {
	    printf("warning: Previous exposure never completed.\n");
	 }
	 break;
      }
      case CAMERA_SWEEPING:
      {
	 /*
	  * CAMERA_READY/CLEANED -> CAMERA_SWEEPING is the only allowed path.
	  */
	 if (camera_state_ != CAMERA_READY &&
	     camera_state_ != CAMERA_CLEANED)
	 {
	    printf("error: camera state internal error.  Should be READY before SWEEPING.\n");
	    return camera_state_; /* Don't allow the state change! */
	 }
	 if (idle_mode_on()!=PASS) new_state = CAMERA_READY;
	 break;
      }
      case CAMERA_CLEANING:
      {
	 /*
	  * CAMERA_READY -> CAMERA_CLEANING and
	  * CAMERA_CLEANED -> CAMERA_CLEANING is all that makes sense.
	  */
	 if (camera_state_ == CAMERA_EXPOSED)
	 {
	    printf("warning: Flushing un-read exposure.\n");
	 }
	 else if (camera_state_ != CAMERA_READY &&
		  camera_state_ != CAMERA_CLEANED &&
		  camera_state_ != CAMERA_SWEEPING)
	 {
	    printf("error: camera state internal error.  Should be READY before CLEANING.\n");
	 }
	 break;
      }
      case CAMERA_CLEANED:
      {
	 if (camera_state_ != CAMERA_SWEEPING &&
	     camera_state_ != CAMERA_CLEANING &&
	     camera_state_ != CAMERA_READING)
	 {
	    printf("warning: camera state internal error.  CLEANED but not CLEANING (state=%s).\n", camera_state_str(camera_state_));
	 }
	 break;
      }
      case CAMERA_EXPOSING:
      {
	 if (camera_state_ == CAMERA_READY)
	 {
	    printf("warning: Camera may not be properly flushed.\n");
	 }
	 /*
	  * CAMERA_CLEANED -> CAMERA_EXPOSING is the normal path, while
	  * CAMERA_EXPOSED -> CAMERA_EXPOSING is used for focus exposures.
	  */
	 else if (camera_state_ != CAMERA_EXPOSED && /* Focus/multiple exposure */
		  camera_state_ != CAMERA_CLEANED) /* Normal exposure */
	 {
	    printf("error: camera state internal error.  Should be CLEANED before EXPOSING.\n");
	 }
	 break;
      }
      case CAMERA_EXPOSED:
      {
	 /*
	  * CAMERA_EXPOSING -> CAMERA_EXPOSED only makes sense.
	  */
	 if (camera_state_ != CAMERA_EXPOSING)
	 {
	    printf("warning: camera state internal error.  EXPOSED but not EXPOSING (state=%s).\n", camera_state_str(camera_state_));
	 }
	 break;
      }
      case CAMERA_READING:
      {
	 if (camera_state_ == CAMERA_READY)
	 {
	    printf("warning: Reading out but camera not CLEANED or EXPOSED.\n");
	 }
	 /*
	  * CAMERA_EXPOSED -> CAMERA_READING is the normal path, while
	  * CAMERA_CLEANED -> CAMERA_READING might happen for biases.
	  */
	 else if (camera_state_ != CAMERA_EXPOSED &&
		  camera_state_ != CAMERA_CLEANED)
	 {
	    printf("warning: camera state %s instead of EXPOSED before READING.\n",
		   camera_state_str(camera_state_));
	 }
	 break;
      }
      default:
      {
	 printf("error: camera state internal error.  %d invalid state code.\n",
		new_state);
	 new_state = camera_state_;
      }
   }
   if (camera_state_ != new_state)
   {
      camera_state_ = new_state;
      statusbar_update();
      parfile_update(camera_state_str(new_state));
   }
   return camera_state_;
}

static inline const char*
mode_string(detcom_mode_t mode) /* For the statusbar */
{
   switch (detcom_mode)
   {
      case DETCOM_OBSERVING: return ""; break;
      case DETCOM_FAKE: return "FAKE ** "; break;
      case DETCOM_SAFE: return "SAFE ** "; break;
      case DETCOM_ENGINEERING: return "ENGINEERING ** "; break;
      default: return "detcom_mode? "; /* Shouldn't happen */
   }
}

BOOLEAN
file_from_odometer(void)
{
   if (!xstrcasecmp(filebase_, "odometer")) return TRUE;
   else return FALSE;
}

static const char*
rcfile_getname(BOOLEAN final)
{
   static char fname[2][128] = { "", "" };
   int idx = final?0:1;

   if (!*fname[idx])
      sprintf(fname[idx], "%.100s/"RCFILENAME"%s", cli_gethome(), final?"":"%");

   return fname[idx];
}

static const char*
parfile_getname(void)
{
   static char fname[128] = "";

   if (!*fname)
      sprintf(fname, "%.100s/"PARFILENAME, cli_gethome());

   return fname;
}

PASSFAIL
rcfile_update(void)
{
   static int warned_rcfile = 0;
   int n;
   BoardID board;

   if (rcfile)
   {
      printf("\rwarning: Not updating DetCom rcfile (old rcfile still open).\n");
      return FAIL;
   }

   printf("\rlogonly: Updating rcfile\n");

   rcfile = fopen(rcfile_getname(FALSE), "w");

   if (!rcfile)
   {
      if (!warned_rcfile)
      {
	 perror("fopen(\"~/"RCFILENAME"%\", \"w\") failed");
	 warned_rcfile = 1;
      }
      goto rcfile_bail;
   }

   if (fprintf(rcfile,
	       "# ~/"RCFILENAME" - This file is maintained by DetCom to keep state.\n"
	       "# It stores all the current command paramters and is re-read\n"
	       "# each time DetCom is restarted.\n")<0)
   {
      goto rcfile_bail;
   }

   switch (detcom_mode)
   {
      case DETCOM_ENGINEERING:
      {
	 fprintf(rcfile, "mode engineering\n");
	 if (auto_save == FALSE)
	    fprintf(rcfile, "auto save off\n");
	 if (auto_clean == FALSE)
	    fprintf(rcfile, "auto clean off\n");
	 break;
      }
      case DETCOM_OBSERVING:
      {
	 fprintf(rcfile, "mode observing\n");
	 break;
      }
      case DETCOM_FAKE:
      {
	 fprintf(rcfile, "mode fake\n");
	 break;
      }
      default:
      {
	 fprintf(rcfile, "mode safe\n");
      }
   }

   for (n = 0; n < n_controllers; n++)
   {
      for (board=0; board<MAX_BOARDS; board++)
      {
	 const char* name;
	 if ((name = leach[n].dspcode_filename[board])!=0)
	 {
	    if (fprintf(rcfile,
			"download %s %s\n",
			leach[n].name_cmdarg[board],
			name)<0)
	    {
	       goto rcfile_bail;
	    }
	 }
      }
   }

   if (fprintf(rcfile,
	       "download\n" /* Re-check the DSP code and ID bits */
	       "file %s\n"
	       "raster %s%s %s\n"
	       "filter %s\n"
	       "etype %s\n"
	       "etime %s\n"
	       "header observer %s\n"
	       "header object %s\n"
	       "header comment %s\n"
	       "header sequence %s\n"
	       "header _piname %s\n"
	       "header _runid %s\n"
	       "mef %s\n"
	       "shuttercount %d\n"
	       "reserve %d\n"
	       ,
	       filebase_,
	       *raster_id()?"id=":"",
	       raster_id(),
	       raster_command(),
	       filter_command(),
	       etype_command(),
	       etime_command(),
	       fitscard("observer", 0),
	       fitscard("object", 0),
	       fitscard("comment", 0),
	       fitscard("sequence", 0),
	       fitscard("_piname", 0),
	       fitscard("_runid", 0),
	       mef_command(),
	       shutter_count(),
	       detcom_reserve)<0)
   {
      goto rcfile_bail;
   }

   if (fclose(rcfile)!=0)
   {
      rcfile = 0;
      goto rcfile_bail;
   }
   rcfile = 0;

   if (rename(rcfile_getname(FALSE), rcfile_getname(TRUE))!=0)
   {
      goto rcfile_bail;
   }
   else
   {
      if (warned_rcfile)
      {
	 warned_rcfile = 0;
	 printf("status: ~/"RCFILENAME" is ok again.\n");
      }
   }
   return PASS;

rcfile_bail:

   if (!warned_rcfile)
   {
      perror("warning: Can't update ~/"RCFILENAME" startup file");
      warned_rcfile = 1;
   }
   if (rcfile)
      fclose(rcfile);
   rcfile = 0;
   return FAIL;
}

/*
 * === Temporary hack until we figure out how to handle status.
 */
PASSFAIL
parfile_update(const char* why)
{
   static char old_contents[2048] = "";
   struct flock fl;
   int fd, content_len;
   char contents[2048];

   if (rcfile)
   {
      CLI_DEBUG(printf("\rdebug: Parfile update suppressed while reading rcfile\n"));
      return PASS;
   }

   if (timing_critical)
   {
      CLI_DEBUG(printf("\rdebug: Parfile update suppressed (timing_critical)\n"));
      return PASS;
   }

   printf("\rlogonly: Updating parfile (%s)\n", why);

   content_len = 
      sprintf(contents,
	      "# ~/"PARFILENAME" - Used by DetCom to pass status to RPM.\n"
	      "status=\"%.99s\"\n"
	      "file=\"%.99s\"\n"
	      "raster=\"%.99s\"\n"
	      "filter=\"%d\"\n"
	      "etype=%.99s\n"
	      "etime=%.99s\n"
	      "observer=\"%.99s\"\n"
	      "object=\"%.99s\"\n"
	      "comment=\"%.99s\"\n"
	      "sequence=\"%.99s\"\n"
	      "piname=\"%.99s\"\n"
	      "runid=\"%.99s\"\n"
	      "autoclean=%s\n"
	      "autosave=%s\n"
	      "autovoltage=%s\n"
	      "mef=%s\n",
	      camera_state_str(camera_state(CAMERA_STATE_QUERY)),
	      filebase(),
	      *raster_id()?raster_id():raster_command(),
	      filter_id(),
	      etype_command(),
	      etime_command(),
	      fitscard("observer", 0),
	      fitscard("object", 0),
	      fitscard("comment", 0),
	      fitscard("sequence", 0),
	      fitscard("_piname", 0),
	      fitscard("_runid", 0),
	      auto_clean?"on":"off",
	      auto_save?"on":"off",
	      auto_voltage?"on":"off",
	      mef_command());
#ifdef SUNOS
   content_len = strlen(contents);
#endif

   if (strcmp(old_contents, contents) == 0)
   {
      printf("\rlogonly: Parfile is unchanged\n");
      return PASS;
   }

   strcpy(old_contents, contents);

   fd = open(parfile_getname(), O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
   if (fd == -1)
   {
      perror(parfile_getname());
      return FAIL;
   }

   memset(&fl, 0, sizeof(fl));
   fl.l_type = F_WRLCK;
   fl.l_whence = SEEK_SET;
   fl.l_start = 0;
   fl.l_len = 0;

   if (TEMP_FAILURE_RETRY(fcntl(fd, F_SETLKW, &fl)) != 0)
      perror("warning: parfile lock failed");
   else fl.l_type = F_UNLCK;

   if (write(fd, contents, content_len) != content_len)
   {
      perror("parfile write failed");
      close(fd);
      return FAIL;
   }

   if (ftruncate(fd, content_len) != 0)
   {
      perror("parfile ftruncate failed");
      close(fd);
      return FAIL;
   }

   if (fl.l_type == F_UNLCK &&
       TEMP_FAILURE_RETRY(fcntl(fd, F_SETLKW, &fl)) != 0)
      perror("warning: parfile unlock failed");

   if (close(fd)!=0)
   {
      perror("parfile close failed");
      return FAIL;
   }
   return PASS;
}


PASSFAIL
rcfile_reset(void)
{
   rcfile = fopen(rcfile_getname(TRUE), "r");

   if (!rcfile)
   {
      perror("warning: ~/"RCFILENAME);
      printf("warning: Is this the first time you're running DetCom?\n");
      return FAIL;
   }
   else return PASS;
}

char*
rcfile_getline(BOOLEAN download_from_rc)
{
   char buf[256];
   char* newline;
   
   if (!rcfile) return 0;

   if (fgets(buf, sizeof(buf), rcfile)==0)
   {
      if (ferror(rcfile))
	 perror("warning: Error reading ~/"RCFILENAME" startup file");
      fclose(rcfile);
      printf("logonly: Finished reading startup commands from ~/"RCFILENAME"\n");
      printf("progress: Welcome to DetCom\n");
      /*
       * === Make sure first prompt is an ok> prompt, even if startup
       * fails, because otherwise director will abort its startup script.
       */
      cli_setprompt(PASS);
      rcfile = 0;
      return 0;
   }
   if ((newline = strpbrk(buf, "\r\n"))) *newline = '\0';
   if (*buf == '#' || *buf=='\0') return rcfile_getline(download_from_rc);
   if (!download_from_rc &&
       (!strncmp(buf, "boot ", 5) || !strncmp(buf, "download ", 9)))
   {
      CLI_DEBUG(printf("\rdebug: Ignoring `download' command in rc file\n"));
      return rcfile_getline(download_from_rc);
   }
   printf("\rlogonly: rcfile: `%s'\n", buf);
   return strdup(buf);
}

void
statusbar_update(void)
{
   char newstatus[128];
   /*
    * sprintf'ing to this buffer is really not needed except
    * that it helps director keep things straight when more than
    * one thread is printing to the same stdout at the same time.
    */

   sprintf(newstatus,
	   "\rstatusbar: %sfile %.20s%s.fits | raster %.30s | etype %.20s | etime %.20s | %s\n",
	   mode_string(detcom_mode),
	   filebase(), !strcmp(mef_command(),"on")?"":"*",
	   raster_command(),
	   etype_command(),
	   etime_command(),
	   camera_state_str(camera_state(CAMERA_STATE_QUERY)));
   printf(newstatus);
}

/*
 * Default to "safe" mode if someone starts up DetCom without
 * a .detcomrc file so DetCom doesn't automatically power up a
 * system.
 */
detcom_mode_t detcom_mode = DETCOM_SAFE;

PASSFAIL
com_help( const char* args ) { return cli_help( (args&&*args)?args:0 ); }
/*
 * The '0' hack with no args keeps libcli from sticking its own
 * ---[ Command Set ]----- line into the help screen.
 */

PASSFAIL
com_mode( const char* args )
{
   PASSFAIL rtn;

   if (args)
   {
      if (!strcmp(args, "observing"))
      {
	 if (detcom_mode == DETCOM_SAFE)
	 {
	    printf("status: DetCom entering observing mode.\n");
	 }
	 detcom_mode = DETCOM_OBSERVING;
	 return cli_init("detcom", comlist_observing, 3);
      }
      if (!strcmp(args, "safe"))
      {
	 if (detcom_mode != DETCOM_SAFE)
	 {
	    printf("warning: DetCom entering safe mode; "
		   "Polling and auto-power-on disabled\n");
	 }
	 detcom_mode = DETCOM_SAFE;
	 return cli_init("detcom", comlist_observing, 3); /* Same commands */
      }
      if (!strcmp(args, "engineering"))
      {
	 detcom_mode = DETCOM_ENGINEERING;
	 rtn = cli_init("detcom", comlist_engineering, 3);
	 return rtn;
      }
      if (!strcmp(args, "fake"))
      {
	 rtn = cli_init("detcom", comlist_engineering, 3);
	 detcom_mode = DETCOM_FAKE;
	 printf("warning: DetCom now in FAKE MODE!  Most hardware "
		"found to be missing or not functioning will be simulated.  "
		"Attempts to talk to real hardware may still be made!\n");
	 return rtn;
      }
   }
   printf("error: Invalid use of `mode' command.\n");
   return FAIL;
}

PASSFAIL
com_fileno( const char* args )
{
   /*
    * === TODO: Use cli_arg1() and clean up help messages.
    */
   if (!args || !*args || !atoi(args))
   {
      printf("Syntax: fileno <N>  N>1.  Sets next exposure number for filenames with ???\n"
	     "When filename contains ???, existing files will never be overwritten, and file number will autoincrement, but you can use this command to start from a specific number.\n");
      return FAIL;
   }
   filesequence_set(atoi(args));
   statusbar_update();
   parfile_update("file number changed");
   return PASS;
}

void
filename_next(void)
{
   int retries = 99;
   
   /*
    * If it the current filename has been used, grab a new
    * sequence/odometer number (if necessary applicable).
    */
   if (filebase_used == TRUE) frameno_advance();
   
   /*
    * If this is the first frame, whether or not it was marked
    * "used", we set the filebase_used flag anyway, to force the
    * following loop to check that the file doesn't already exist.
    */
   if (filesequence()==1) filebase_used = TRUE;
   
   /*
    * Here's the loop that keeps incrementing the sequence number
    * or odometer until it finds a filename that does not exist
    * in the current directory.
    */
   while (filesequence()>0 && filebase_used == TRUE && retries--)
   {
      char filename_test1[1024];
      char filename_test2[1024];
      struct stat st_buf;
      
      sprintf(filename_test1, "%.200s.fits", filebase()); /* Single FITS */
      sprintf(filename_test2, "%.200s", filebase()); /* Subdirectory */
      if (lstat(filename_test1, &st_buf)==0 ||
	  lstat(filename_test2, &st_buf)==0)
	 /*
	  * Don't use this filename again.  There's already a file on
	  * disk by that name, so keep incrementing the sequence number.
	  */
	 frameno_advance();
      else
	 /*
	  * Filename available.  Break out of loop.
	  */
	 filebase_used = FALSE;
   }
   
   filebase_used = FALSE;
   
   /*
    * From this point on, filebase() will return the filename for
    * the "next" exposure, which is not yet been created.  Once it
    * is created, "filename used" should be called (by the save
    * routine.)
    */
}


PASSFAIL
com_filename( const char* args )
{
   int len;

   if (!args || !*args || strlen(args)>(sizeof(filebase_)-1))
   {
      printf("Syntax: filename <pattern>    Examples of <pattern>:\n"
             "next            - Display what the next filename will be.\n"
	     "odometer        - Fetch file sequence number from a global counter.\n"
	     "undo            - Go back to the previous filename.\n"
	     "bias???.fits    - Replaces ??? with internal auto-incrementing counter.\n"
	     "image????$.fits - Replaces ?'s with counter and $ with exposure type.\n");
      return FAIL;
   }

   if (!strcmp(args, "used"))
   {
      if (filebase_used == TRUE)
      {
	 printf("error: Filename %s used more than once!\n", filebase());
	 return FAIL;
      }
      filebase_used = TRUE;
      return PASS;
   }

   if (!strcmp(args, "next"))
   {
      filename_next();
      printf("%s%s\n", filebase(), !strcmp(mef_command(),"on")?".fits":"");
      return PASS;
   }

   if (!strcmp(args, "undo"))
   {
      if (!*filebase_undo)
      {
	 printf("error: No previous filename change to `undo'\n");
	 return FAIL;
      }
      args = filebase_undo;
   }

   /*
    * Automatically strip ".fits" or "*.fits" or "*" off the end
    * of the filename base if the user entered it on the command line.
    */
   len = strlen(args);
   if (len > 5 && !xstrcasecmp(args+len-5, ".fits")) len -= 5;
   if (len > 1 && args[len-1]=='*') len--;
   if (args != filebase_undo) strcpy(filebase_undo, filebase_);
   strncpy(filebase_, args, len);
   filebase_[len] = '\0';
   filebase_update++;
   filesequence_set(1);
   statusbar_update();
   parfile_update("file name changed");
   return PASS;
}

static long odometer_ = -1;
static long sequence_ = -1;

/*
 * frameno_advance() is called by DetCom whenever a "save" operation
 * has completed and the current filename sequence number should
 * no longer be used.  If a fixed filename is being used, for example
 *
 * filename flux.fits
 *
 * then this command does nothing, but if the filename came from the
 * "odometer" file, or if the filename had a sequence pattern like
 *
 * filename image???
 *
 * then this command simply marks the current number as "used".  When
 * a new number is needed, it finds the current number -1 and gets
 * a new one at that time.  This command will not cause a new number
 * to be fetched from the odometer.
 */
static void
frameno_advance(void)
{
   int update_stuff = 0; /* Print a message showing new filename? */

   /*
    * Case 1: Filename with auto-incrementing pattern.
    */
   if (strchr(filebase_, '?'))
   {
      if (sequence_ != -1) { sequence_ = -1; update_stuff = 1; }
   }

   /*
    * Case 2: Filename from a global odometer file.
    */
   else if (file_from_odometer())
   {
      if (odometer_ != -1) { odometer_ = -1; update_stuff = 1; }
   }

   if (update_stuff)
   {
      statusbar_update();
      parfile_update("update_stuff"); /* %%% */
   }

   /*
    * Case 3: Fixed filename.  Do nothing.
    */
}

/*
 * Return the current file sequence number.  Fetch a new one
 * from the odometer or increment the last counter now, if needed.
 *
 * Returns:
 * -1: Error getting current sequence number.
 *  0: Sequence number doesn't apply to current file naming.
 * >0: The current sequence number to be used.
 */
static int next_sequence_ = 1;

void
filesequence_set(int n)
{
   sequence_ = -1;
   next_sequence_ = n;
   filebase_update++;
}

long
filesequence(void)
{
   /*
    * Case 1: Filename with auto-incrementing pattern.
    */
   if (strchr(filebase_, '?'))
   {
      if (sequence_ == -1) { sequence_ = next_sequence_++; filebase_update++; }
      return sequence_;
   }

   /*
    * Case 2: Filename from a global odometer file.
    */
   else if (file_from_odometer())
   {
      if (odometer_ == -1)
      {
	 printf("progress: Obtaining new filename from the odometer\n");
#ifdef CFHT_OD
	 odometer_ = cfht_od();
#endif
	 filebase_update++;
	 printf("progress: Odometer number is %ld\n", odometer_);
      }
      if (odometer_ == -1)
      {
#ifdef CFHT_OD
	 printf("error: Can't read odometer!  No automatic archiving.\n");
	 printf("error: Switching to default internal file naming scheme.\n");
#endif
	 com_filename("image???$"); /* Set the default filenaming scheme. */
	 return filesequence();
      }
      return odometer_;
   }
   else return 0; /* Fixed filename.  No number used */
}

const char*
filebase(void)
{
   static char filebase_rtn[256];
   static char last_etype = ' ';
   static int last_sequence = -1;
   static int last_odometer = -1;
   static int last_update = -1;
   char* first_question; /* Pointer to where the ??? starts in the filename */
   char* first_dollar;	/* Pointer to $ character (replaced with etype) */
   char new_etype = etype_char();
   int sequence = filesequence();

   if (sequence == 0) return filebase_; /* Case 3: No auto-incrementing */

   /*
    * Case 1: Filename with auto-incrementing pattern.
    */
   if ((first_question = strchr(filebase_, '?')))
   {
      if (last_etype != new_etype || last_sequence != sequence ||
	  last_update != filebase_update)
      {
	 /*
	  * Regenerate filebase_rtn . . .
	  */
	 int width = strspn(first_question, "?"); /* Count number of ? characters */
	 int prefix_len = first_question - filebase_;

	 if (prefix_len > 100) prefix_len = 100;
	 sprintf(filebase_rtn, "%.*s%0*d%.100s",
		 prefix_len, filebase_, width, sequence,
		 first_question + width);
	 /*
	  * Replace the first '$' in the string with the etype character.
	  */
	 if ((first_dollar = strchr(filebase_rtn, '$')))
	 {
	    *first_dollar = new_etype;
	 }

	 /*
	  * Save current sequence and etype so filename string doesn't have
	  * to be regenerated.  Set last_odometer to -1 so switching back to
	  * odometer mode will force a new filename to be generated.
	  */
	 last_etype = new_etype;
	 last_sequence = sequence;
	 last_update = filebase_update;
	 last_odometer = -1;
	 parfile_update("file base changed");
      }
      return filebase_rtn;
   }

   /*
    * Case 2: Filename from a global odometer file.
    */
   if (last_etype != new_etype || last_odometer != sequence)
   {
      /*
       * Regenerate filebase_rtn . . .
       */

      sprintf(filebase_rtn, "%06d%c", sequence, new_etype);

      /*
       * Save current sequence and etype so filename string doesn't have
       * to be regenerated.  Set last_sequence to -1 so switching back to
       * file???.fits mode will force a new filename to be generated.
       */
      last_etype = new_etype;
      last_odometer = sequence;
      last_sequence = -1;
      parfile_update("file base changed");
   }

   return filebase_rtn;
}


PASSFAIL
com_auto( const char* args )
{
   static char** cargv = 0;
   static int cargc = 0;
   BOOLEAN* var = 0;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   if (cargc==1 && !xstrcasecmp(cargv[0], "defaults"))
   {
      /*
       * These are the defaults for observing mode.
       * Only engineering mode can override these now.
       */
      auto_clean = TRUE;
      auto_save = TRUE;
      auto_voltage = FALSE;
      parfile_update("auto defaults");
      return PASS;
   }

   if (cargc>=1)
   {
      if (!xstrcasecmp(cargv[0], "writefits") || !xstrcasecmp(cargv[0], "save"))
	 var = &auto_save;
      else if (!xstrcasecmp(cargv[0], "voltage"))
	 var = &auto_voltage;
      else if (!xstrcasecmp(cargv[0], "clean") ||
	       !xstrcasecmp(cargv[0], "clear") ||
	       !xstrcasecmp(cargv[0], "flush"))
	 var = &auto_clean;
   }

   if (var)
   {
      if (cargc==2)
      {
	 if (!xstrcasecmp(cargv[1], "on") || !strcmp(cargv[1], "1"))
	    *var = TRUE;
	 else if (!xstrcasecmp(cargv[1], "off") || !strcmp(cargv[1], "0"))
	    *var = FALSE;
	 else if (!xstrcasecmp(cargv[1], "toggle"))
	    *var = ~*var;
	 else
	 {
	    printf("error: Expected auto %s `on', `off', or `toggle', not `%s'.\n",
		   cargv[0], cargv[1]);
	    return FAIL;
	 }
	 parfile_update(cargv[0]);
      }
      printf("status: The auto %s feature is %s.\n",
	     cargv[0], *var?"ON":"OFF");
      return PASS;
   }

   printf("Syntax: auto save|voltage|clean [on|off|toggle]");
   return FAIL;
}


int detcom_reserve = DEFAULT_RESERVE;

PASSFAIL
com_reserve(const char* args)
{
   unsigned long new_reserve;
   char* endptr;

   if (args == NULL || *args == '\0')
   {
      printf("%d\n", detcom_reserve);
      return PASS;
   }

   new_reserve = strtoul(args, &endptr, 0);
   if (*endptr == '\0')
   {
      detcom_reserve = new_reserve;
      return PASS;
   }

   printf("error: reserve command takes a single integer argument,\n"
	  "error: which sets the minimum number of blank keyword\n"
	  "error: slots to leave in each FITS header that DetCom produces.\n");
   return FAIL;
}


PASSFAIL
com_header(const char* args)
{
   const char* space = strchr(args, ' ');
   const char* current = fitscard(args, 0);

   if (!current)
   {
      printf("error: You can only change `observer', `object', and `comment'\n");
      printf("error: Syntax: header observer|object|comment \"String for FITS card\"\n");
      return FAIL;
   }

   if (!space)
   {
      printf("Current value is `%s'; Give additional argument to change.\n", current);
      return PASS;
   }

   if (!strcmp(current, space+1))
      return PASS; /* Already the same value. */

   if (!fitscard(args, space+1))
   {
      printf("error: You can only change `observer', `object', and `comment'\n");
      return FAIL;
   }
   if (strncmp(args, "_piname",7)==0 ||
       strncmp(args, "_runid",6)==0)
   {
      com_nheader("reset");
   }
   parfile_update(args);
   return PASS;
}

HeaderUnit dheader = 0;
#if 0
HeaderUnit dheader_ampid[ROC_MAX][ROC_MAX_AMPS];
#endif

PASSFAIL
com_dheader(const char* args)
{
   static int current_controller = 0;
   static char** cargv = 0;
   static int cargc = 0;
   double idx = 0;
#if 0
   int i, j;
#endif

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   if (cargc == 1)
   {
      if (!strcmp(cargv[0], "clear")
          || !strcmp(cargv[0], "reset"))
      {
         printf("logonly: resetting dheader keywords\n");
         if (dheader)
         {
            fh_destroy(dheader);
            dheader = 0;
#if 0
            for (i = 0; i < ROC_MAX; i++)
               for (j = 0; j < ROC_MAX_AMPS; j++)
               {
                  fh_destroy(dheader_ampid[i][j]);
                  dheader_ampid[i][j] = 0;
               }
#endif
         }
         return PASS;
      }
      if (!strcmp(cargv[0], "pixsize"))
      {
	 double pixsize = 0.;
	 fh_get_PIXSIZE(dheader, &pixsize);
	 printf("%g\n", pixsize);
	 if (pixsize == 0.) return FAIL;
	 return PASS;
      }
      if (!strcmp(cargv[0], "show")
          || !strcmp(cargv[0], "list")
          || !strcmp(cargv[0], "print"))
      {
         if (dheader)
         {
            printf("DSP code header keywords:\n");
            fh_show(dheader);
#if 0
            for (i = 0; i < ROC_MAX; i++)
               for (j = 0; j < ROC_MAX_AMPS; j++)
               {
                  if (fh_first(dheader_ampid[i][j]))
                  {
                     printf("Controller %d, Amplifier %d:\n", i, j);
                     fh_show(dheader[i][j]);
                  }
               }
#endif
         }
         return PASS;
      }
   }

   if (cargc == 2 && !strcmp(cargv[0], "controller"))
   {
      current_controller = atoi(cargv[1]);
      return PASS;
   }

   if (cargc != 4 ||
       (idx = strtod(cargv[0], 0)) == 0. ||
       strlen(cargv[1]) > 8)
   {
      printf("error: bad parameters to dheader command.\n");
      printf("dheader commands are usually embedded in DSP lod files and"
             " specify things like read noise and gain.  See nheader for"
             " information on the syntax.  Unlike the nheader command,"
             " the comment field is required, and all headers are sticky"
             " (until new DSP code is loaded.)\n");
      printf("dheader can also be used to set keywords which are different"
             " per amplifier.  In this case, the value field must be a"
             " list of exactly N comma-separated items inside braces, where"
             " N is the number of amplifiers per controller.  The command"
             " \"dheader controller N\" can be used to change the current"
             " controller.\n");
      return FAIL;
   }
   /*
    * From this point, make sure dheader and all dheader_ampid
    * have been created.
    */
   if (dheader == 0)
   {
      dheader = fh_create();
#if 0
      /* %%% */
      for (i = 0; i < ROC_MAX; i++)
         for (j = 0; j < ROC_MAX_AMPS; j++)
            dheader_ampid[i][j] = fh_create();
#endif
   }

   if (cargv[2][1] == '{')
   {
      printf("error: Multi-amp dheader command not yet implemented!\n"); /* %%% Finish! */
      return FAIL;
   }

   fh_set_val(dheader, idx, cargv[1], cargv[2], cargv[3]);
   return PASS;
}

HeaderUnit nheader = 0;
HeaderUnit nheader_nonsticky = 0;

PASSFAIL
com_nheader(const char* args)
{
   HeaderUnit nh;
   static char** cargv = 0;
   static int cargc = 0;
   double idx = 0;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   if (cargc == 1)
   {
      /*
       * %%% This is only used internally, to reset the
       * non-sticky set of nheader keywords whenever a new exposure
       * is starting.
       *
       * %%% Should nheader reset clear these too?
       */
      if (!strcmp(cargv[0], "unstick"))
      {
	 printf("logonly: Clearing non-sticky nheader keywords\n");
	 if (nheader_nonsticky)
	 {
	    fh_destroy(nheader_nonsticky);
	    nheader_nonsticky = 0;
	 }
	 return PASS;
      }

      if (!strcmp(cargv[0], "clear")
	  || !strcmp(cargv[0], "reset"))
      {
	 if (nheader)
	 {
	    printf("logonly: resetting nheader keywords\n");
	    fh_destroy(nheader);
	    nheader = 0;
	 }
	 return PASS;
      }
      if (!strcmp(cargv[0], "show")
	  || !strcmp(cargv[0], "list")
	  || !strcmp(cargv[0], "print"))
      {
	 if (!nheader && !nheader_nonsticky)
	 {
	    printf("No nheader keywords defined.\n");
	    return PASS;
	 }
	 if (nheader)
	 {
	    printf("Sticky header keywords:\n");
	    fh_show(nheader);
	 }
	 if (nheader_nonsticky)
	 {
	    printf("One-time (non-sticky) header keywords:\n");
	    fh_show(nheader_nonsticky);
	 }
	 return PASS;
      }
   }

   if ((cargc != 3 && cargc != 4) ||
       (idx = strtod(cargv[0], 0)) == 0. ||
       strlen(cargv[1]) > 8)
   {
      printf("error: bad parameters to nheader.\n");
      printf("The nheader command takes three or four arguments."
	     "  The first should be a float number >= 10000 which"
	     " determines the order the FITS keywords will appear in a file."
	     "  The second must be a card name of up to 8 characters."
	     " The third argument is the value.  The fourth, if present will be placed "
	     " in the FITS keyword's comment field if there is room.\n");
      printf("NOTE: === TODO: Implement a way to force a keyword to be stored as a string."
	     " this routine automatically detects if the value can be stored as"
	     " a float, integer, or boolean type FITS keyword.\n");
      printf("Use `nheader reset' to clear arbitrary keywords which have been created.\n");
      printf("warning: This command is intended for QSO use, and should not be "
             "used to clobber keywords which the observing system already creates!\n");
#if 0
      printf("NOTE: If you want to force the keyword to be stored as"
	     " a string, add single quotes ('...') around the value field"
	     " given to this command, otherwise the command will attempt"
	     " to store numeric values as numeric fits keywords, and T/F"
	     " as boolean FITS keywords.  So, for example, to store the"
	     " string 'T  ' in a fits card, you should use:\n");
      printf("  nheader 25000.1 MYSTRING 'T' 'This is stored as a string'\n");
#endif
      return FAIL;
   }

   if (idx < 0)
   {
      if (!nheader_nonsticky)
	 nheader_nonsticky = fh_create();
      nh = nheader_nonsticky;
      idx = fabs(idx);
   }
   else
   {
      if (!nheader)
	 nheader = fh_create();
      nh = nheader;
   }

   if (idx < 10.)
   {
      printf("error: header index values below 10 are reserved for internal use.\n");
      return FAIL;
   }

   fh_set_val(nh, idx, cargv[1], cargv[2], cargc>3?cargv[3]:0);
   return PASS;
}

PASSFAIL
com_defaults(const char* args)
{
   if (args && *args)
   {
      cli_help("defaults");
      return FAIL;
   }
   
   com_etype("OBJECT");
   com_etime("10"); /* === Or set min etime? */
   com_raster("FULL");
   com_auto("defaults");
   com_filename("odometer");
   com_nheader("reset");
   
   return PASS;
}

const char*
fitscard(const char* name, const char* val)
/*
 * name - must always contain the string name (lower case) of the fits card
 * val - if NULL, just return the current value, else set to val
 *       (and print a message)
 * always returns the current value, or NULL if name was not found.
 */
{
   static char* comment  = 0;
   static char* sequence = 0;
   static char* observer = 0;
   static char* object   = 0;
   static char* piname   = 0;
   static char* runid    = 0;
   char** which = 0;

   if (strncmp(name, "obj", 3)==0)	{ name="object";   which = &object; }
   if (strncmp(name, "obs", 3)==0)	{ name="observer"; which = &observer; }
   if (strncmp(name, "c", 1)==0)	{ name="comment";  which = &comment; }
   if (strncmp(name, "s", 1)==0)	{ name="sequence"; which = &sequence; }
   if (strncmp(name, "_piname",7)==0)	{ name="piname";   which = &piname; }
   if (strncmp(name, "_runid",6)==0)	{ name="runid";    which = &runid; }

   if (which && (*which)==0)
      *which = strdup("");

   if (which && val && (strcmp((*which), val)))
   {
      char* p;

      free((*which));
      (*which) = cli_arg1(val);
      /*
       * Since the routines to handle these values further down do not
       * do adequate encoding of special characters (like the " character)
       * just replace them with an '_' to avoid problems for now. %%%
       */
      for (p = *which; *p; p++)
      {
	 if (*p < ' ' || *p > '~' || strchr("\"$\\!`", *p))
	    *p = '_';
      }
      printf("New `%s' keyword will be `%s'\n", name, (*which));
   }
   return (which?(*which):0);
}

PASSFAIL
com_exit( const char* args UNUSED ) { done = TRUE; return PASS; }
/*
 * The exit command just sets a flag to tell the main while loop to
 * exit cleanly on the next iteration
 */

PASSFAIL
com_ls( const char* arg ) { return cli_system("ls -F", arg); }

/* End of det_modes.c */
