/*                                             -*- c-file-style: "Ellemtel" -*-

`det_raster.c' - Raster setting commands	  (raster, fraster)

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "cli/cli.h"
#include "detcom.h"

/*
 * Two rasters need to be saved here: the last one used in a readout,
 * and the one selected for the next readout.  DetCom does not keep
 * track of an array of numbered raster choices.  It is up to a GUI
 * or script to remember this array and pass in all parameters each
 * time it wants to select a raster.  In the future, it may be possible
 * to define variables in DetCom that store raster definitions, so
 * you could issue a command like:
 *
 * `raster myraster'
 *
 * For now, the following hardcoded names are recognized:
 *
 * `raster FULL'      (supercedes deti's `raster 1')
 * `raster FULL BIN2' (supercedes deti's `raster 6')
 * `raster FULL BIN4' (supercedes deti's `raster 7')
 */

static raster_t raster_exposed_; /* Raster in effect during last exposure */
static raster_t raster_; /* New raster */
static BOOLEAN raster_ok = FALSE;

const char* raster_command(void)
{ 
   if (raster_ok) return raster_.command;
   else return "FULL";
}

const char* raster_id(void)
{
   if (raster_ok) return raster_.id;
   else return "FULL";
}

const raster_t* raster_exposed(void)
{
   return raster_exposed_.npix?(&raster_exposed_):(&raster_);
}

PASSFAIL
raster_to_dsp(void)
{
   char tmp[80];

   if (!raster_ok)
   {
      printf("error: No raster selected.\n");
      return FAIL;
   }

#define do_wrmem(board, address, value) \
   ((sprintf(tmp, "%s %s %d", (board), (address), (value))<0 || \
     com_wrmem(tmp)==FAIL)?FAIL:PASS)

   if (do_wrmem("ltb", "NSR", raster_.nsr)==FAIL ||
       do_wrmem("ltb", "NPR", raster_.npr)==FAIL ||
       do_wrmem("ltb", "NSSKIP", raster_.nsskip)==FAIL ||
       do_wrmem("ltb", "NPSKIP", raster_.npskip)==FAIL ||
       do_wrmem("ltb", "NSSKIP2", raster_.nsskip2)==FAIL ||
       do_wrmem("ltb", "NSOVR", raster_.nsovr)==FAIL ||
       do_wrmem("ltb", "NPOVR", raster_.npovr)==FAIL ||
       do_wrmem("ltb", "NSBIN", raster_.nsbin)==FAIL ||
       do_wrmem("ltb", "NPBIN", raster_.npbin)==FAIL)
   {
      printf("error: Failed to set DSP raster paramters.\n");
      return FAIL;
   }
   raster_exposed_ = raster_;
   return PASS;
}

PASSFAIL
com_raster(const char* args)
{
#define X_CENTER	0
#define Y_CENTER	1
#define X_SIZE		2
#define Y_SIZE		3
#define X_BIN		4
#define Y_BIN		5
#define RSPEC_ARGS	6
   static char** cargv = 0;
   static int cargc = 0;
   int arg = 0;
   int ri = 0, rspec[RSPEC_ARGS] = { 0, 0, 0, 0, 1, 1 };
   int x_os = 0, y_os = 0;
   raster_t tmp;
   shm_t* shm = im_buf_get(FALSE);
   int cols_per_amp = 0, rows_per_amp = 0,
      rows_overscan = 32, cols_overscan = 32, /* %%% Major hack.  Pass through */
      pixels_per_amp = 0;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   strcpy(tmp.id, "");

   if (shm)
   {
      cols_per_amp = shm->cols_per_amp;
      rows_per_amp = shm->rows_per_amp;
      pixels_per_amp = cols_per_amp * rows_per_amp;
   }

   if (cargc == 1 && !strcmp(cargv[0], "help"))
   {
   printf("Using the `raster' command:\n"
	  "\traster <xc> <yc> <xs> <ys> [<xb> <yb>]\n"
	  "\traster CENTER <xs> <ys> [<xb> <yb>]\n"
	  "\traster FULL [BIN2|BIN4|BIN8]\n"
	  "Where <xc> <yc> is the center of a region of size <xs> by <ys> "
	  "pixels.  The current detector has a maximum image area of %d by %d "
	  "pixels, so the center must be in the range [1:%d,1:%d], and the size must "
	  "remain within this as well.\n"
	  "The 4 parameters can be replaced by `FULL' to choose the full "
	  "detector.  Or, the first 2 parameters can be replaced by `CENTER', "
	  "followed by <xs> <ys>.\n"
	  "If present, the last two arguments give x and y binning factors "
	  "<xb> <yb>, or the single word `BIN2' `BIN4' or "
	  "`BIN8' can be used for equal binning in x and y.\n"
	  "The current raster selection appears on the status bar.\n",
	  cols_per_amp - cols_overscan,
	  rows_per_amp - rows_overscan,
	  cols_per_amp - cols_overscan,
	  rows_per_amp - rows_overscan);
      return PASS;
   }

   /*
    * Parse the command line arguments.
    */

   while (ri < RSPEC_ARGS && arg < cargc)
   {
      /*
       * === lookup cargv[arg] as a variable, else...
       */
      if (ri==0 && !xstrncasecmp(cargv[arg], "id=", 3))
      {
	 if (strlen(cargv[arg])+1 > sizeof(tmp.id))
	 {
	    printf("error: raster ID string too long\n");
	    return FAIL;
	 }
	 strcpy(tmp.id, cargv[arg++] + 3);
      }
      else if (ri==0 && !xstrcasecmp(cargv[arg], "CENTER"))
      {
	 arg++;
	 ri += 2;
	 rspec[X_CENTER] = (cols_per_amp-cols_overscan)/2;
	 rspec[Y_CENTER] = (rows_per_amp-rows_overscan)/2;
      }
      else if (ri==0 && !strcasecmp(cargv[arg], "FULL"))
      {
	 arg++;
	 ri += 4;
	 rspec[X_CENTER] = (cols_per_amp-cols_overscan)/2;
	 rspec[Y_CENTER] = (rows_per_amp-rows_overscan)/2;
	 rspec[X_SIZE] = (cols_per_amp-cols_overscan);
	 rspec[Y_SIZE] = (rows_per_amp-rows_overscan);
      }
      else if (ri==4 && !strcasecmp(cargv[arg], "BIN2"))
      {
	 arg++; ri += 2;
	 rspec[X_BIN] = 2; rspec[Y_BIN] = 2;
      }
      else if (ri==4 && !strcasecmp(cargv[arg], "BIN4"))
      {
	 arg++; ri += 2;
	 rspec[X_BIN] = 4; rspec[Y_BIN] = 4;
      }
      else if (ri==4 && !strcasecmp(cargv[arg], "BIN8"))
      {
	 arg++; ri += 2;
	 rspec[X_BIN] = 8; rspec[Y_BIN] = 8;
      }
      else rspec[ri++] = atoi(cargv[arg++]);
   }

   if (ri==5)
   {
      printf("warning: Setting both x and y binning factor to %d.\n",
	     rspec[Y_BIN] = rspec[X_BIN]);
      ri++;
   }

   if (ri == 0)
   {
      /* Just print help message, or maybe print current raster? */
   }
   else if (ri!=4 && ri!=6)
   {
      printf("error: Not enough arguments in raster specification.\n");
   }
   else if (arg < cargc)
   {
      printf("error: Too many arguments in raster specification.\n");
   }
   else if (rspec[X_BIN]<1 || rspec[X_BIN]>cols_per_amp || rspec[X_BIN]>rspec[X_SIZE] ||
	    rspec[Y_BIN]<1 || rspec[Y_BIN]>rows_per_amp || rspec[Y_BIN]>rspec[Y_SIZE])
   {
      printf("error: Invalid binning factor(s)\n");
   }
   /*
    * Could let the following checks come out in the wash after
    * the conversion to leach raster coordinates, but this provides
    * a clearer error message to the user on where they went wrong.
    */
   else if (rspec[X_CENTER]<1 || rspec[X_CENTER]>cols_per_amp ||
	    rspec[Y_CENTER]<1 || rspec[Y_CENTER]>rows_per_amp)
   {
      printf("error: Center coordinates must be xc:[1..%d] yc:[1..%d].\n",
	     cols_per_amp, rows_per_amp);
   }
   else
   {
      if (rspec[X_BIN]>16 || rspec[Y_BIN]>16)
      {
	 printf("warning: Unusually large binning factor selected (%d X %d).\n",
		rspec[X_BIN], rspec[Y_BIN]);
      }

      /*
       * Still not sure this is a valid raster specification,
       * but time to do some more calculations before checking
       * that the region fits on the chip . . .
       */

      /*
       * Change user raster coordinates to leach raster coordinates.
       * This is generic for a single output detector.
       */
      tmp.nsskip = rspec[X_CENTER] - (rspec[X_SIZE] - 1)/2 - 1;
      tmp.npskip = rspec[Y_CENTER] - (rspec[Y_SIZE] - 1)/2 - 1;
      tmp.nsskip2 = cols_per_amp - cols_overscan - tmp.nsskip - rspec[X_SIZE];

      /*
       * Automatically add the complete overscan regions iff the raster
       * is adjacent to the first overscan column and/or row.
       */
      switch (cols_per_amp - (tmp.nsskip + rspec[X_SIZE] + cols_overscan))
      {
	 case 0:
	 {
	    CLI_DEBUG(printf("debug: Region will include overscan columns.\n"));
	    x_os = cols_overscan;
	    break;
	 }
	 case 1:
	 {
	    printf("warning: Region is one pixel away from overscan columns.  "
		   "Image will not include overscan columns.\n");
	    break;
	 }
      }

      switch (rows_per_amp - (tmp.npskip + rspec[Y_SIZE] + rows_overscan))
      {
	 case 0:
	 {
	    CLI_DEBUG(printf("debug: Region will include overscan rows.\n"));
	    y_os = rows_overscan;
	    break;
	 }
	 case 1:
	 {
	    printf("warning: Region is one pixel away from overscan rows.  "
		   "Image will not include overscan rows.\n");
	    break;
	 }
      }
      
      tmp.nsr = rspec[X_SIZE] / rspec[X_BIN];
      tmp.npr = rspec[Y_SIZE] / rspec[Y_BIN];
      tmp.nsovr = x_os / rspec[X_BIN];
      tmp.npovr = y_os / rspec[Y_BIN];
      tmp.nsbin = rspec[X_BIN];
      tmp.npbin = rspec[Y_BIN];
      tmp.npix = (tmp.nsr + tmp.nsovr) * (tmp.npr + tmp.npovr);

      /*
       * Do the final sanity checks.
       */
      if (tmp.nsskip < 0 || tmp.nsr < 1 ||
	  tmp.nsskip + tmp.nsr > cols_per_amp - cols_overscan)
      {
	 printf("error: Raster region too wide for chip.\n");
      }
      else if (tmp.npskip < 0 || tmp.npr < 1 ||
	       tmp.npskip + tmp.npr > rows_per_amp - rows_overscan)
      {
	 printf("error: Raster region too tall for chip.\n");
      }
      else if (tmp.npix > pixels_per_amp)
      {
	 /*
	  * This one shouldn't happen, given the other checks above, but . . .
	  */
	 printf("error: Raster defines more pixels that total pixels per chip! (%d > %d)\n",
		tmp.npix, pixels_per_amp);
      }
      else
      {
	 /*
	  * Valid raster.  Store it.
	  */

	 raster_ = tmp;

	 if (rspec[X_CENTER] == (cols_per_amp-cols_overscan)/2 &&
	     rspec[Y_CENTER] == (rows_per_amp-rows_overscan)/2)
	 {
	    if (rspec[X_SIZE] == (cols_per_amp-cols_overscan) &&
		rspec[Y_SIZE] == (rows_per_amp-rows_overscan))
	    {
	       strcpy(raster_.command, "FULL");
	    }
	    else
	    {
	       sprintf(raster_.command, "CENTER %d %d",
		       rspec[X_SIZE], rspec[Y_SIZE]);
	    }
	 }
	 else
	 {
	    sprintf(raster_.command, "%d %d %d %d",
		    rspec[X_CENTER], rspec[Y_CENTER],
		    rspec[X_SIZE], rspec[Y_SIZE]);
	 }
	 if (rspec[X_BIN]!=1 || rspec[Y_BIN]!=1)
	 {
	    if (rspec[X_BIN]==2 && rspec[Y_BIN]==2)
	    {
	       strcat(raster_.command, " BIN2");
	    }
	    else if (rspec[X_BIN]==4 && rspec[Y_BIN]==4)
	    {
	       strcat(raster_.command, " BIN4");
	    }
	    else if (rspec[X_BIN]==8 && rspec[Y_BIN]==8)
	    {
	       strcat(raster_.command, " BIN8");
	    }
	    else
	    {
	       sprintf(raster_.command + strlen(raster_.command),
		       " %d %d", rspec[X_BIN], rspec[Y_BIN]);
	    }
	 }
	 raster_ok = TRUE;
	 parfile_update("raster changed");
	 com_preexp("reset");

	 return PASS;
      }
   }
   printf("error: Try `raster help' for information on this command.\n");
   return FAIL;
}

/* End of det_raster.c */
