/*                                             -*- c-file-style: "Ellemtel" -*-

`det_readout.c' - Readout related commands  (readout, built-in descrambler func)

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "cli/cli.h"
#include "detcom.h"
#include "leachcont.h"

/*
 * Descrambler routines
 */

static unsigned int next_amp[MAX_CONTROLLERS];
static unsigned short* im_buf_ptr[MAX_CONTROLLERS][MAX_AMPS_PER_CONTROLLER];
static shm_t* ds_shm = 0;

static void
descrambler_init(unsigned int pixels_per_amp)
{
   int i, n, max_pixels;

   ds_shm = im_buf_get(FALSE);
   max_pixels = ds_shm->rows_per_amp * ds_shm->cols_per_amp;

   if (pixels_per_amp > max_pixels)
   {
      printf("error: internal error: %d exceed maximum expected pixels, %d!\n",
	     pixels_per_amp, max_pixels);
      pixels_per_amp = max_pixels;
   }

   for (n=0; n<n_controllers; n++)
   {
      next_amp[n] = 0;
      ds_shm->pixels_read[n] = 0;
      ds_shm->pixels_avail[n] = 0;
      ds_shm->pixels_expected[n] = pixels_per_amp * ds_shm->n_amps_per_controller;
      ds_shm->pixels_per_amp[n] = pixels_per_amp;
      for (i=0; i<ds_shm->n_amps_per_controller; i++)
      {
	 im_buf_ptr[n][i] = ds_shm->buf_base + ds_shm->buf_ofs[n][i];
      }
   }
}


static void
descrambler_exit(int zeropad)
{
   unsigned int word1, total;
   int n, amp, amp_count;
   unsigned int amps_per_controller;
   
   if (ds_shm == 0) return;

   amps_per_controller = ds_shm->n_amps_per_controller;

   /*
    * Fill any unused regions with zero
    */
   for (n=0; n<n_controllers; n++)
   {
      word1 = ds_shm->pixels_expected[n] - ds_shm->pixels_read[n];
      if (word1 && zeropad)
      {
	 printf("logonly: Filling %s's buffer with zeroes from %d..%d\n",
		leach[n].name_contr, ds_shm->pixels_read[n], ds_shm->pixels_expected[n]);
	 for (amp_count = 0, amp = next_amp[n];
	      amp_count < amps_per_controller;
	      amp_count++, amp = (amp + 1)%amps_per_controller)
	 {
	    total = (word1 + amps_per_controller - 1)/amps_per_controller;
	    if (!word1--) break;
	    
	    memset(im_buf_ptr[n][amp], 0, total*sizeof(short));
	    im_buf_ptr[n][amp] += total;
	 }
      }
      ds_shm->pixels_avail[n] = ds_shm->pixels_expected[n]; /* Signal readout complete */
   }
}

#ifdef LINUX
#define PIXVAL_OFFSET 0 /* %%% lotuspci driver already offset the values */
#else
#define PIXVAL_OFFSET 32768
#endif

static void
descrambler(int controller, unsigned short* scrambled, unsigned int words)
{
   unsigned short* dest;
   unsigned short* src;
   unsigned int amp, amp_count, word1, total, todo;
   unsigned int amps_per_controller;
   
   if (ds_shm == 0) return;

   amps_per_controller = ds_shm->n_amps_per_controller;

   if (controller < 0 || controller >= n_controllers)
   {
      printf("error: Internal error; "
	     "invalid controller number (%d) passed to descrambler.\n",
	     controller);
      return;
   }

   if (ds_shm->pixels_read[controller] + words > ds_shm->pixels_expected[controller])
   {
      printf("warning: Descrambler overflow!\n");
      words = ds_shm->pixels_expected[controller] - ds_shm->pixels_read[controller];
   }

   word1 = words;

   for (amp_count = 0, amp = next_amp[controller];
	amp_count < amps_per_controller;
	amp_count++, amp = (amp + 1)%amps_per_controller)
   {
      dest = im_buf_ptr[controller][amp];
      src = scrambled + amp_count;
      todo = total = (word1 + amps_per_controller - 1)/amps_per_controller;
      if (!word1--) break;

      if (amps_per_controller == 1)
      {
	 /*
	  * Optimization for single amplifier systems.
	  */
	 while (todo--)
	    *dest++ = *src++ - PIXVAL_OFFSET;
      }
      else
      {
	 while (todo--)
	 {
	    *dest++ = *src - PIXVAL_OFFSET;
	    src += amps_per_controller;
	 }
      }

      im_buf_ptr[controller][amp] += total;
   }

   /*
    * Update counters so they are ready for the next call
    */
   next_amp[controller] += words;
   next_amp[controller] %= amps_per_controller;
   ds_shm->pixels_read[controller] += words;
   ds_shm->pixels_avail[controller] += words;
}


PASSFAIL
com_readout(const char* args)
{
   static pid_t readout_child = -1;
   static BOOLEAN readout_fg = TRUE; /* Don't return PASS until readout complete */
   int npix = raster_exposed()->npix;
   BOOLEAN send_rdc = TRUE; /* === For future use */

   Devp devs[MAX_CONTROLLERS];
   int n;

   /*
    * %%% Document:
    */
   if (args && !strcmp(args, "fg"))
   {
      if (readout_fg != TRUE)
      {
	 readout_fg = TRUE;
	 printf("status: readout fg - `go' command returns when readout completes.\n");
      }
      return PASS;
   }
   if (args && !strcmp(args, "bg"))
   {
      if (readout_fg != FALSE)
      {
	 readout_fg = FALSE;
	 printf("status: readout bg - `go' command returns when readout begins.\n");
      }
      return PASS;
   }

   if (args && !strcmp(args, "wait"))
   {
      PASSFAIL rtn = PASS;
      int maxtries = 10000;
      int zeropad = 1;
      child_exit_t child_exit;

      com_expose("wait"); /* "readout wait" implies "expose wait" first. */
      if (readout_child == -1)
      {
	 CLI_DEBUG(printf("\rdebug: readout thread not running\n"));
	 return PASS;
      }

      printf("logonly: readout thread %d being waited on\n", readout_child);
      printf("progress: Waiting for readout to finish\n");
      /* (Normally the user won't see the particular progress: message above,
       * because the readout thread is sending constant progress: messages
       * which over-write it.  Just in case a readout thread has frozen,
       * sending this progress: message before the wait ensures that there's
       * a message on the screen indicating that things are now frozen
       * waiting for a readout to finish.)
       */

      /*
       * These signals should already be blocked, but make absolutely
       * sure that they are blocked so that readouts don't get
       * interrupted.  Interrupting a readout often leaves the SDSU
       * in a very bad state, and could be dangerous for the CCDs.
       */
      cli_signal_block(SIGINT);
      cli_signal_block(SIGQUIT);
      cli_signal_block(SIGTSTP);

      child_exit = pid_wait_nointr(readout_child);
      readout_child = -1; /* Make sure it doesn't get waited on again. */

      printf("logonly: readout thread finished; checking image buffer\n");
      /*
       * Check if shared memory shows the right number of pixels.
       */
      for (n=0; n<n_controllers; n++)
      {
	 if (ds_shm->pixels_read[n] != ds_shm->pixels_expected[n])
	 {
	    printf("error: %s returned %d pixels; expected %d (%d %s pixels)\n",
		   leach[n].name_contr,
		   ds_shm->pixels_read[n], ds_shm->pixels_expected[n],
		   abs(ds_shm->pixels_expected[n] - ds_shm->pixels_read[n]),
		   (ds_shm->pixels_read[n] < ds_shm->pixels_expected[n])?"missing":"extra");
	    rtn = FAIL;
	    /* === Will never see the "extra" pixels message the way this
	     * is set up.  Maybe the code in fpbox should just call the
	     * descrambler and make it overflow so the descrambler can
	     * at least add up how many extra pixels were on each controller.
	     */
	 }
      }

      switch (child_exit)
      {
	 case CHILD_EXITPASS:
	    printf("logonly: readout thread completed successfully.\n");
	    descrambler_exit(zeropad);
	    /*
	     * %%% This is a hack.  I need to figure out why EXITSIG is not
	     * being returned (instead, EXITPASS comes back and the pixel
	     * count in shared memory is wrong.  So that, the wrong pixel
	     * count in shared memory, is how an aborted readout is detected.
	     */
	    if (rtn == PASS)
	       break;
	 case CHILD_EXITSIG:
	    zeropad = 0;
	    com_abortdata("");
	    printf("error: readout thread interrupted\n");
	    /* Fall through */
	 case CHILD_EXITFAIL:
	    descrambler_exit(zeropad);
	    printf("progress: Attempting to stop the flow of pixels.\n");
	    printf("logonly: readout thread failed; sending AEX to utility boards\n");
	    printf("error: Aborting readout.\n");
	    com_cmd("-quiet lub AEX");
	    /* Normal to have some garbage here, so ignore it */
	    while (leach_poll(/*verbose=*/0)!= LEACH_POLL_NODATA && maxtries--);
	    printf("progress: Testing Data Link\n");
	    if (com_tdl("ltb 0x55A55A logonly:")==FAIL)
	       printf("warning: Communication with controller lost!\n");
	    printf("progress: Readout not successful!\n");
	    break;

	 /*
	  * These shouldn't happen:
	  */
	 case CHILD_ERROR:
	 case CHILD_GONE:
	 case CHILD_RUNNING:
	    descrambler_exit(zeropad);
	    printf("error: readout thread had unknown problem\n");
	    break;
      }

      /*
       * Automatically check for correct testpattern if testpat etype.
       */
      if (is_testpat())
      {
	 if (com_testpat("") != PASS) rtn = FAIL;
      }

#if 0
      /*
       * %%% Want this for speed, where possible.  However, EEV1 seems
       * to act very strangely if we skip the clean!
       */
      if (rtn == PASS && !xstrncasecmp(raster_exposed()->command, "FULL", 4))
      {
	 CLI_DEBUG(printf("\rdebug: camera state CLEANED after FULL read\n"));
	 camera_state(CAMERA_CLEANED);
      }
      else
#endif
      {
	 camera_state(CAMERA_READY); /* But not necessarily clean */
      }
      return rtn;
   }

   if (args && !strcmp(args, "auto"))
   {
      printf("debug: com_readout(auto) called from com_expose\n");
      args = 0;
   }
   else if (detcom_mode == DETCOM_OBSERVING)
   {
      printf("error: Use `go' instead of `readout'.  A separate `readout'"
	     " command is only available in engineering mode for testing.\n");
      return FAIL;
   }

   printf("progress: Waiting for exposure\n");

   com_expose("wait");

   for (n=0; n<n_controllers; n++)
      devs[n] = leach[n].dev;        /* === */

   printf("progress: Waiting for previous exposure to finish saving\n");

   com_waitdata(""); /* Don't start re-using buffers until the previous
		      * exposure is completely saved off to disk.
		      */

   printf("progress: Preparing DetCom for readout. . .\n");

   descrambler_init(npix);

   /*
    * Only generate a warning if the exposure type collected real photons.
    */
   if (have_etime()) switch (data_state(DATA_STATE_QUERY))
   {
      case DATA_STATE_QUERY: /* Won't happen. */
      case DATA_SAVED: /* last exposure successfully saved */ break;
      case DATA_EMPTY: /* no previous data in buffers */ break;
      case DATA_FAILED:  printf("warning: Last readout not completely saved (overwriting).\n"); break;
      case DATA_ABORTED: printf("warning: Last readout aborted and removed (re-using filename).\n"); break;
      case DATA_READING: printf("warning: Last readout never saved (re-using filename).\n"); break;
      case DATA_SAVING:  printf("error: Last readout still being saved!?\n"); break;
      /* DATA_SAVING should never happen here because waitdata takes care of it */
   }

   data_state(DATA_EMPTY);
   filename_next(); /* By the time of the readout, we definitely
		     * want the new exposure number to be displayed.
		     */
   data_state(DATA_READING);

   if (auto_save) com_save("");
   else
      printf("Auto save is OFF.  Use `save' command to save data to disk.\n");

   /*
    * Set up EDT cards for ring buffer mode.  This needs to be done a
    * second or two BEFORE pixels start flowing because it can take
    * some time to allocate buffers, and we need to be ready for pixels
    * immediately.  This call also sets a flag which causes the fpbox
    * to latch the last word of data.  It doesn't actually get sent
    * down the fiber until the lio_read16_parallel command is issued.
    */
   if (lio_read16_init(n_controllers, devs, ds_shm->pixels_expected[0])!=LIO_OK)
   {
      printf("error: Unable to set up ring buffer mode\n");
      com_abortdata("");
      descrambler_exit(/*zeropad=*/0);
      return FAIL;
   }

   camera_state(CAMERA_READING);

   if (send_rdc)
   {
      com_cmd((args && *args)?args:"ltbM RDC ''");
      printf("progress: Starting readout. . .\n");

      /* There must be no immediate response to this command.
       * To verify that this is working correctly, uncomment:
      printf("TEST !!! Delaying for 5 seconds after sending RDC...\n");
      xusleep(5000000);
      printf("Done.\n");
      */
   }

   switch (readout_child = fork())
   {
      case -1: perror("error: fork failed"); return FAIL; break;
      case 0:
      {
	 /*
	  * Begin the ring buffer read in the child thread.
	  */
	 if (lio_read16_parallel(n_controllers, devs, descrambler,
				 ds_shm->pixels_expected[0],
				 /*response_msec=*/25000,
				 /*timeout_msec=*/300000) < 0)
	    /* === Make these args configurable, or in .h file? */
	    _exit(EXIT_FAILURE);
	 _exit(EXIT_SUCCESS);
      }
   }

   if (readout_fg == TRUE)
   {
      return com_readout("wait");
   }
   else
   {
      printf("logonly: readout thread %d proceeding in background\n",
	     readout_child);
      return PASS;
   }
}

/* End of det_readout.c */
