/*                                             -*- c-file-style: "Ellemtel" -*-

`detcom.h' - Global data decl., camera geometry, and headers for det_*.c

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef _INCLUDED_detcom
#define _INCLUDED_detcom 1

#include <sys/time.h>

#include "leachcont.h"
#include "misc.h"
#include "fh/fh.h"

/* TOMSEC stands for "TimeOut in MilliSEConds" */

#define TOMSEC_FILE 60*1000
/*
 * If we aren't able to write anything to the FITS file for
 * over 60 seconds at a time, return an error.  Even if one
 * byte gets through every 60 seconds, this timeout will not
 * occur.
 */

/* #define DATA_WRITE_MAX (16*1024*1024) %%% */
#define DATA_WRITE_MAX (16*2048*4096)
#define DATA_WRITE_MIN (1024*1024)
/*
 * When writing the actual pixel data to file or to the
 * DCA, the write operations will use block sizes between
 * the two values above.  If DATA_WRITE_MAX and DATA_WRITE_MIN
 * are not the same, the block size can vary depending on
 * how much data is ready at the time each write() operation
 * is starting.  Values are in pixels.
 */

#define MAX_CONTROLLERS		2 /* Must be >= NCTL in ltbM DSP code */ /* %%% Add this to docs */
#define MAX_AMPS_PER_CONTROLLER	6 /* Must be >= NAMP in ltbM DSP code */
#define DEFAULT_NXPX 2048	/* Default amplifier scan size */
#define DEFAULT_NYPX 4096	/* (2K x 4K if NXPX, PYPX missing.) */
#define DEFAULT_NXOV 32	/* Default columns of overscan if no NXOV */
#define DEFAULT_NYOV 32 /* Default rows of overscan if no NYOV */

/*
 * Default number of fits keyword slots to leave (in addition to
 * whatever keywords DetCom will insert) if no "reserve" command is issued
 * to override.  A "reserve" command is sticky and saved in .detcomrc.
 */
#define DEFAULT_RESERVE 200

/*
 * These machines require byte-swapping:
 */
#if (defined(i386) || defined(__i386__) || defined(__alpha) || \
     defined(vax) || defined(__vax__) || (defined(mips) && defined(MIPSEL)))
#define DETCOM_BSWAP
#endif

/* detcom.c */

#define SHM_KEY_DETCOM 1212 /* Same as 12kcom.  Only want one at a time. */
#define SHM_FILENAME "/tmp/detcom_memory_mapped_readout_buffer"

typedef struct
{
   unsigned int n_controllers;
   unsigned int n_amps_per_controller;
   unsigned int rows_per_amp;
   unsigned int cols_per_amp;

   int etime_remaining; /* Only for "expose bg" mode */

   unsigned int pixels_read[MAX_CONTROLLERS];
   unsigned int pixels_avail[MAX_CONTROLLERS];
   unsigned int pixels_expected[MAX_CONTROLLERS];
   unsigned int pixels_per_amp[MAX_CONTROLLERS];
   /*
    * During a readout, pixels_read and pixels_avail are the same
    * thing.  Only at the end of an incomplete readout will
    * pixels_avail be set to the full expected amount (that is,
    * pixels_expected) with missing pixels set to zero and
    * pixels_read will contain the true number actually read
    * from the camera.
    */
   unsigned long  buf_ofs[MAX_CONTROLLERS][MAX_AMPS_PER_CONTROLLER];
   unsigned short buf_base[2]; /* Actually much larger than this */
} shm_t;

extern int n_controllers; /* Actual number of controllers currently open */
extern BOOLEAN done;	/* flag to set when program should exit */
extern BOOLEAN auto_save;
extern BOOLEAN auto_voltage;
extern BOOLEAN auto_clean;
extern LeachDev leach[MAX_CONTROLLERS];
extern int exp_iteration;

shm_t* im_buf_get(BOOLEAN remove);
shm_t* im_buf_cfg(unsigned int n_controllers,
		  unsigned int n_amps_per_controller,
		  unsigned int rows_per_amp,
		  unsigned int cols_per_amp);

/* det_modes.c */

typedef enum {
   DETCOM_SAFE,		/* No lowlevel commands, nothing automatic either. */
   DETCOM_OBSERVING,	/* No lowlevel commands.  Automatic power on, etc. */
   DETCOM_ENGINEERING,	/* Full lowlevel access, nothing automatic. */
   DETCOM_FAKE		/* Not implemented. */
} detcom_mode_t;

extern detcom_mode_t detcom_mode;
extern int detcom_reserve;

typedef enum {
   CAMERA_STATE_QUERY=0,/* Just get the current state */
   CAMERA_UNKNOWN=60,	/* Unknown state (probably because no communications) */
   CAMERA_POF=61,	/* HighV off (don't CLEAN/READOUT, shutter disarmed) */
   CAMERA_READY=62,	/* Ready for next exposure, PON but not sweeping yet */
   CAMERA_SWEEPING=63,	/* Controllers in constant clearing mode (IDL) */
   CAMERA_CLEANING=64,	/* Cleaning CCDs */
   CAMERA_CLEANED=65,	/* CCDs clean (either from CLEANING or SWEEPING) */
   CAMERA_EXPOSING=66,	/* Shutter open (exposure in progress) */
   CAMERA_EXPOSED=67,	/* Shutter closed, still need to read out */
   CAMERA_READING=68	/* Readout in progress */
} camera_state_t;

camera_state_t camera_state(camera_state_t new_state);

PASSFAIL parfile_update(const char* why);

PASSFAIL rcfile_update(void);
PASSFAIL rcfile_reset(void);
char*    rcfile_getline(BOOLEAN download_from_rc);

void statusbar_update(void);
void filename_next(void);

extern HeaderUnit dheader;
extern HeaderUnit nheader;
extern HeaderUnit nheader_nonsticky;

PASSFAIL com_help( const char* args );
PASSFAIL com_mode( const char* args );
PASSFAIL com_fileno( const char* args );
PASSFAIL com_filename( const char* args );
PASSFAIL com_auto( const char* args );
PASSFAIL com_reserve( const char* args );
PASSFAIL com_header( const char* args );
PASSFAIL com_dheader( const char* args );
PASSFAIL com_nheader( const char* args );
PASSFAIL com_defaults( const char* args );

PASSFAIL com_exit( const char* args );
PASSFAIL com_ls( const char* arg );

const char* fitscard(const char* name, const char* val);
void filesequence_set(int n);
long filesequence(void);
const char* filebase(void);
BOOLEAN file_from_odometer(void);

/* det_expose.c */

/*
 * Exposure timing limits, if not defined in utility board DSP code.
 * Units here are in seconds.  In DSP code, the units are milliseconds.
 */
#define TGT_MIN 0.1
#define TGT_MAX 14400.0
#define TGT_STP 0.1

extern unsigned long blade_open, blade_close;
extern struct timeval detcom_obs_eip, detcom_obs_don;
extern BOOLEAN timing_critical;

BOOLEAN is_testpat(void);
BOOLEAN have_etime(void);
BOOLEAN have_shutter(void);

char etype_char(void);
const char* etype_fits( void );
const char* etype_command( void );
const char* etime_command( void );
double etime_float(void);
double etime_meas(void);

PASSFAIL com_etime( const char* args );
PASSFAIL com_etype( const char* args );
PASSFAIL com_go( const char* args );
PASSFAIL com_preexp( const char* args );
PASSFAIL com_expose( const char* args );
PASSFAIL com_chargeshift( const char* arg );
PASSFAIL com_clean( const char* args );

/* det_raster.c */

typedef struct
{
      char id[20];
      char command[100];
      int nsskip;
      int npskip;
      int nsskip2;
      int nsr;
      int npr;
      int nsovr;
      int npovr;
      int nsbin;
      int npbin;
      int npix;
} raster_t;

PASSFAIL raster_to_dsp( void );
const char* raster_id( void );
const char* raster_command( void );
const raster_t* raster_exposed( void );
PASSFAIL com_raster( const char* args );

/* det_readout.c */

PASSFAIL com_readout( const char* args );

/* det_data.c */

typedef enum {
   DATA_STATE_QUERY=0, /* Just get the current state */
   DATA_EMPTY=31,	/* Nothing in data buffers */
   DATA_READING=32,	/* A "readout" command was/is in progress */
   DATA_SAVING=33,	/* A "save" command currently busy */
   DATA_SAVED=34,	/* Save command completed */
   DATA_FAILED=35,	/* Last save failed or partially aborted */
   DATA_ABORTED=36	/* Last save command aborted */
} data_state_t;

data_state_t data_state(data_state_t new_state);

PASSFAIL com_testpat( const char* args );
PASSFAIL com_testccd( const char* args );
PASSFAIL com_abortdata( const char* args );
PASSFAIL com_waitdata( const char* args );
PASSFAIL com_save( const char* args );
PASSFAIL com_savepreview( const char* args );
PASSFAIL com_writedca( const char* args );
PASSFAIL com_mef( const char* args );
const char* mef_command( void );

/* det_ace.c */

#define TEMP_PERIOD 15		/* Poll every 15 seconds */

int filter_id(void);
int shutter_count(void);
const char* filter_name(void);
const char* filter_command(void);

PASSFAIL com_shuttercount(const char* args);
PASSFAIL com_shutter( const char* args );
PASSFAIL com_filter( const char* args );
PASSFAIL com_temp( const char* args );
double temp_float( void );
/*
 * Returns the last temperature reading.
 *
 * This can go into the DETTEMP FITS header.
 */

const char* temp_comment( void );
/*
 * Returns the formula used for the last temperature calculation.
 * This string is stored in the FITS header comment, in case there
 * was a problem with the calculation or the constants in the
 * formula needed to be adjusted.  This gives some record of when
 * this happened.
 *
 * This can go into the DETTEM FITS header comment field.
 */

const char* temp_status( void );
/*
 * Returns the status of the last temperature check.
 *
 * This can go into the DETSTAT FITS header.
 */

const char* temp_status_comment( void );
/*
 * Returns a string describing the normal range of temperatures.
 *
 * This can go into the DETSTAT FITS header comment field.
 */
#endif /* !_INCLUDED_detcom */
