/*                                             -*- c-file-style: "Ellemtel" -*-

`leach_cmd.c' - Basic leach commands       (tdl, syr, general "cmd").

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>

#include "cli/cli.h"
#include "leachcont.h"
#include "detcom.h" /* === Why? */

typedef enum { TDL_NOT_TESTED,
	       TDL_PASS,
	       TDL_FAIL,
	       TDL_NO_RESPONSE,
	       TDL_WRONG_RESPONSE,
	       TDL_PARTIAL
             } tdl_t;

#define TDL_DEFAULT 0xFFFFFFFF

static unsigned long
stow3(const char* s)
{
   char a = s[0];
   char b = s[1];
   char c = s[2];

   return STOW3(a, b, c);
}

static PASSFAIL
send_message( int n, BoardID board, const char* cmdstr, int verbose )
{
   unsigned long data[2];

   if (strlen(cmdstr)!=3)
   {
      printf("error: Leach commands must be three letters.\n");
      return FAIL;
   }

   LMSG2_SET(data, board, stow3(cmdstr));

   if (verbose)
      CLI_DEBUG(printf("\rdebug: issuing %s command to %s\n",
		       cmdstr, leach[n].name_board[board]));
   if (lio_write24(leach[n].dev, data, (sizeof(data)/sizeof(long)))==LIO_ERR)
   {
      printf("error: lio_write24 failed for %s command to %s\n",
	     cmdstr, leach[n].name_board[board]);
      return FAIL;
   }
   return PASS;
}

static leach_poll_t
wait_response( int n, BoardID board, const char* expect, int verbose, int to_msec)
{
   LREAD resp[4];

   if (strlen(expect)!=3)
   {
      printf("error: Expected leach responses must be three letters.\n");
      return LEACH_POLL_ERROR;
   }

   switch (LIO_LREAD(leach[n].dev, resp, 4, 2, to_msec)) /* Read response */
   {
      case 2:
      {
	 if ((resp[0]&0xFFFF)!=0x0002)
	 {
	    printf("%s: wrong header in response from %s\n",
		   verbose?"warning":"logonly",
		   NAMESTR(leach[n].name_board[board]));
	    return LEACH_POLL_GARBAGE;
	 }
	 else if (LREAD_MASK(resp[1])==LREAD_MASK((stow3(expect))))
	 {
	    return LEACH_POLL_GOODRESP;
	 }
	 else if (LREAD_MASK(resp[1])==LREAD_MASK((STOW3('E','R','R'))))
	 {
	    printf("%s: %s returned `ERR'\n",
		   verbose?"warning":"logonly",
		   NAMESTR(leach[n].name_board[board]));
	    return LEACH_POLL_ERR;
	 }
	 else
	 {
#ifndef LIO_16BIT_READ
	    printf("%s: %s returned 0x%06lX (`%c%c%c')\n",
		   verbose?"warning":"logonly",
		   NAMESTR(leach[n].name_board[board]),
		   resp[1],
		   TOCHAR((resp[1]&0xFF0000)>>16),
		   TOCHAR((resp[1]&0xFF00)>>8),
		   TOCHAR(resp[1]&0x00FF));
#else
	    printf("%s: %s returned 0x**%04X (`*%c%c')\n",
		   verbose?"warning":"logonly",
		   NAMESTR(leach[n].name_board[board]),
		   resp[1]&0xFFFF, TOCHAR((resp[1]&0xFF00)>>8), TOCHAR(resp[1]&0x00FF));
#endif
	    return LEACH_POLL_BADRESP;
	 }
      }
      break;
      case LIO_ERR:
      {
	 return LEACH_POLL_NODATA;
      }
      break;
      default:
      {
	 printf("%s: wrong length response from %s\n",
		verbose?"warning":"logonly",
		NAMESTR(leach[n].name_board[board]));
	 return LEACH_POLL_GARBAGE;
      }
   }
}


static tdl_t
tdl( int n, BoardID board, unsigned long pattn )
{
   unsigned long data[3][3];
   LREAD resp[4];
   int i, j;

   LMSG3_SET(data[0], board, stow3("TDL"), 0xA5AAA5);
   LMSG3_SET(data[1], board, stow3("TDL"), 0x5A555A);
   LMSG3_SET(data[2], board, stow3("TDL"), 0x55A55A);

   if (pattn!=TDL_DEFAULT) data[0][2] = pattn;

   leach_poll(/*verbose=*/1); /* Flush any previously unread response */

   for (j=0; j<((pattn!=TDL_DEFAULT)?1:3); j++)
   {
      for (i=0; i<((pattn!=TDL_DEFAULT)?1:3); i++)
      {
	 int noresp_retry = NORESP_RETRY;
	 int length;

noresp_tdl:
	 lio_write24(leach[n].dev, data[i], 3);
	 switch ((length = LIO_LREAD(leach[n].dev, resp, 4, 2, TOMSEC_LEACH_FAST)))
	 {
	    case 2:
	    {
	       if ((resp[0]&0xFFFF)!=0x0002 ||
		   LREAD_MASK(resp[1])!=LREAD_MASK(data[i][2]))
	       {
#ifndef LIO_16BIT_READ
		  printf("logonly: Bad response to TDL(iter=%d,pttn=0x%06lX) "
			 "from %s (got:0x%06lX 0x%06lX)\n",
			 j, data[i][2], NAMESTR(leach[n].name_board[board]),
			 resp[0], resp[1]);
#else
		  printf("logonly: Bad response to TDL(iter=%d,pttn=0x%06lX) "
			 "from %s (got:0x**%04X 0x**%04X)\n",
			 j, data[i][2], NAMESTR(leach[n].name_board[board]),
			 resp[0]&0xFFFF, resp[1]&0xFFFF);
#endif
		  return TDL_WRONG_RESPONSE;
	       }
	       if (noresp_retry != NORESP_RETRY)
	       {
		  printf("warning: TDL to %s required %d retr%s\n",
			 NAMESTR(leach[n].name_board[board]),
			 NORESP_RETRY - noresp_retry,
			 (NORESP_RETRY - noresp_retry) != 1?"ies":"y");
	       }
	    }
	    break;
	    case LIO_ERR:
	    {
	       CLI_DEBUG(printf("debug: No response to TDL(iter=%d,pttn=0x%06lX) command to %s\n",
				j, data[i][2], NAMESTR(leach[n].name_board[board])));
	       if (noresp_retry--)
	       {
		  if (NORESP_DELAY_USEC) xusleep(NORESP_DELAY_USEC);
		  goto noresp_tdl;
	       }
	       if (i || j) return TDL_PARTIAL;
	       else return TDL_NO_RESPONSE;
	    }
	    break;
	    default:
	    {
	       int ii;

	       printf("logonly: Bad response length %d to TDL(iter=%d,pttn=0x%06lX) from %s\n",
		      length, j, data[i][2], NAMESTR(leach[n].name_board[board]));
	       /*
		* It might help to show the actual data, for troubleshooting.
		*/
	       for (ii=0; ii<length; ii++)
		  printf("logonly: Response[%d] = 0x%06lX\n", ii, (unsigned long)(resp[ii]));

	       return TDL_FAIL;
	    }
	 }
      }
   }
   return TDL_PASS;
}

static BOOLEAN resetfiber_needed_all = TRUE;
static BOOLEAN resetfiber_needed[MAX_CONTROLLERS];

static PASSFAIL
resetfiber( int n )
{
   int tries = 2;
   int gen;

   printf("logonly: Resetting host board and fiber interface for controller %d.\n",
	  n);

   while (tries--)
   {
      for (gen = 2; gen >= 1; gen--)
      {
	 if (lio_reset(leach[n].dev, gen) == LIO_OK &&
	     (tdl(n, LTB, 0x123456) == TDL_PASS ||
	      tdl(n, LTB, 0x654321) == TDL_PASS))
	 {
	    printf("status: Controller %d %s GEN%d SDSU.\n",
		   n, gen==1?"is or emulates":"is", gen);
	    return PASS;
	 }
      }
      if (detcom_mode == DETCOM_OBSERVING)
      {
	 /*
	  * Try to reset the timing board if it is not responding.
	  * DSP code will need to be reloaded if this works.
	  */
	 printf("warning: Attempting to reset controller %d timing board.\n", n);
	 lio_reset_peer(leach[n].dev);
      }
      else
      {
	 printf("warning: Try `syr' to attempt controller reset.\n");
      }
   }

   if (lio_reset(leach[n].dev, 2) == LIO_OK)
   {
      printf("warning: Assuming controller %d is GEN2 SDSU.\n", n);
   }
   else
   {
      printf("warning: Assuming controller %d is GEN1 SDSU.\n", n);
      lio_reset(leach[n].dev, 1);
   }

   return FAIL;
}

PASSFAIL
com_tdl( const char* args )
{
   static char** cargv = 0;
   static int cargc = 0;
   BOOLEAN all_passed = TRUE;
   BOOLEAN ctl_no_response[MAX_CONTROLLERS];
   BOOLEAN all_no_response = TRUE;
   tdl_t r[MAX_CONTROLLERS][MAX_BOARDS];
   unsigned long pattn = TDL_DEFAULT;
   lb_set_t lbset;
   int n;
   PASSFAIL rtn;
   char* endptr;
   const char* message_ok = "status:";
   const char* message_fail = "error:";

   com_readout("wait");

   for (n=0; n<n_controllers; n++)
   {
      ctl_no_response[n] = TRUE;
      if (resetfiber_needed_all) resetfiber_needed[n] = TRUE;
   }
   resetfiber_needed_all = FALSE;

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   if (cargc>=1)
   {
      lbset = lb_parse(cargv[0]);
   }
   else
   {
      /*
       * Default is to TDL all boards on all controllers
       */
      LBZERO(&lbset);
      for (n=0; n<n_controllers; n++)
      {
	 LBSET(&lbset, n, LUB);
	 LBSET(&lbset, n, LTB);
      }
   }

   if (cargc>=2)
   {
      pattn = strtoul(cargv[1], &endptr, 0);

      if (!endptr || *endptr!='\0')
      {
	 printf("error: `%s' is not a valid TDL pattern.\n", cargv[1]);
	 return FAIL;
      }
   }

   if (cargc>=3) message_ok = cargv[2];
   if (cargc>=4) message_fail = cargv[3];

   for (n=0; n<n_controllers; n++)
   {
      r[n][LTB] = r[n][LUB] = TDL_NOT_TESTED;

      if (LBISSET(&lbset, n, LTB) ||
	  LBISSET(&lbset, n, LUB))
      {
	 if (resetfiber_needed[n])
	 {
	    /* Auto-detect gen1/gen2 */
	    resetfiber_needed[n] = FALSE;
	    resetfiber(n);
	 }

	 if (LBISSET(&lbset, n, LTB))
	 {
	    if ((r[n][LTB] = tdl(n, LTB, pattn))!=TDL_PASS) all_passed = FALSE;
	    if (r[n][LTB]!=TDL_PASS) all_passed = FALSE;
	    if (r[n][LTB]!=TDL_NO_RESPONSE) ctl_no_response[n] = all_no_response = FALSE;
	 }
	 if (LBISSET(&lbset, n, LUB))
	 {
	    if ((r[n][LUB] = tdl(n, LUB, pattn))!=TDL_PASS) all_passed = FALSE;
	    if (r[n][LUB]!=TDL_PASS) all_passed = FALSE;
	    if (r[n][LUB]!=TDL_NO_RESPONSE) ctl_no_response[n] = all_no_response = FALSE;
	 }
      }
   }

   if (all_passed == TRUE && cargc<1)
   {
      if (n_controllers > 0)
	 printf("%s All boards passed data link test.\n", message_ok);
      else
	 printf("warning: No controllers found for data link test.\n");
      return PASS;
   }

   if (pattn != 0xFFFF && pattn != 0xFFFFFF)
   {
      int check_master = LIO_OK, check_slave = LIO_OK;

      if (LBISSET(&lbset, 1, LUB) ||
	   LBISSET(&lbset, 1, LTB))
      {
	 check_master = lio_check(leach[0].dev);
      }

      if (n_controllers > 1 &&
	  (LBISSET(&lbset, 2, LUB) ||
	   LBISSET(&lbset, 2, LTB)))
      {
	 check_slave =  lio_check(leach[1].dev);
      }

      if (check_master!=LIO_OK || check_slave!=LIO_OK)
      {
	 const char* msg = " for both controllers";
	 if (n_controllers<2) msg="";
	 else if (check_master==LIO_OK) msg = " for the slave controller";
	 else if (check_slave==LIO_OK)  msg = " for the master controller";

	 printf("warning: Note: All of the interface card's data lines%s are high. "
		"This typically happens when the `fpbox' is switched off or not properly "
		"connected to the detector host.\n", msg);
      }
   }
 
   if (all_no_response && cargc<1)
   {
      printf("%s TDL failed to get any responses.\n", message_fail);
      resetfiber_needed_all = TRUE;
      com_preexp("reset");
      return FAIL;
   }

   rtn = PASS;

   for (n=0; n<n_controllers; n++)
   {
      int i;
      int board;

      if (ctl_no_response[n] && cargc<1)
	 resetfiber_needed[n] = TRUE;

      for (i=0; i<2; i++)
      {
	 board = (i?LUB:LTB);
	 if (r[n][board]==TDL_NOT_TESTED)
	 {
	    CLI_DEBUG(printf("debug: %s not tested\n",
			     NAMESTR(leach[n].name_board[board])));
	 }
	 else if (r[n][board]!=TDL_PASS)
	 {
	    rtn = FAIL;
	    printf("%s %s TDL failed", message_fail,
		   NAMESTR(leach[n].name_board[board]));
	    switch (r[n][board])
	    {
	       case TDL_NO_RESPONSE: printf(" (no response)\n"); break;
	       case TDL_WRONG_RESPONSE: printf(" (wrong response)\n"); break;
	       case TDL_PARTIAL: printf(" (not all patterns worked)\n"); break;
	       default: printf("\n");
	    }
	 }
	 else
	 {
	    printf("%s %s passed data link test.\n", message_ok,
		   NAMESTR(leach[n].name_board[board]));
	 }
      }
   }
   if (rtn != PASS)
      com_preexp("reset");
   return rtn;
}


PASSFAIL
com_syr( const char* args ) /* === Clean up */
{
   unsigned long syra[] = { 0x000202, STOW3('S','Y','R') };
   unsigned long syrb[] = { 0x000302, STOW3('S','Y','R') };
   LREAD resp[200];
   int n;

   for (n=0; n<n_controllers; n++)
   {
      lio_reset_peer(leach[n].dev);
      sleep(3);
      lio_write24(leach[n].dev, syra, sizeof(syra)/sizeof(long));
      LIO_LREAD(leach[n].dev, resp, sizeof(resp)/sizeof(LREAD), 0, TOMSEC_LEACH_SYR);
      lio_write24(leach[n].dev, syrb, sizeof(syrb)/sizeof(long));
      LIO_LREAD(leach[n].dev, resp, sizeof(resp)/sizeof(LREAD), 0, TOMSEC_LEACH_SYR);
#ifdef LIO_16BIT_READ
      /* Default leach code does not support RM2 */
      memset(leach[n].has_rm2, 0, sizeof(leach[n].has_rm2));
#endif
   }
   com_preexp("reset");
   return PASS;
}


PASSFAIL
com_cmd( const char* args )
{
   static char** cargv = 0;
   static int cargc = 0;
   char* boardspec = 0;
   char* cmdname = 0;
   char* expect = 0;
   PASSFAIL rtn = PASS;
   lb_set_t lbset;
   char* p;
   int n;
   int quiet = 0;
   int board;
   int tomsec = TOMSEC_LEACH_SLOW;

   CLI_DEBUG(printf("debug: com_cmd(\"%s\") starting\n", args?args:""));

   if (cargv) { if (cargv[0]) free(cargv[0]); free(cargv); }
   cargv = cli_argv(&cargc, args);

   if (cargc>=1 && !strcmp(cargv[0], "-quiet"))
   {
      quiet = 1;
   }

   if (cargc - quiet >= 2)
   {
      boardspec = cargv[quiet];
      cmdname = cargv[quiet + 1];
      expect = "DON";
   }
   if (cargc - quiet >= 3)
   {
      expect = cargv[quiet + 2];
   }
   if (cargc - quiet >= 4)
   {
      tomsec = atoi(cargv[quiet + 3]);
   }

   if (!boardspec || !cmdname ||
       (strlen(cmdname)!=3 && strlen(cmdname)!=0))
   {
      printf("error: example: cmd ltbM CLR [DON [millsecond_timeout]]\n");
      return FAIL;
   }

   lbset = lb_parse(boardspec);

   for (p = cmdname; *p; p++) *p = toupper(*p);

   for (n=0; n<n_controllers; n++)
   {
      for (board = 0; board < 4 /* %%% */; board++)
      {
	 if ((board==LUB || board==LTB) && /* %%% */
	     LBISSET(&lbset, n, board))
	 {
	    int noresp_retry = NORESP_RETRY;

noresp_cmd:
	    if (*cmdname && send_message(n, board, cmdname, /*verbose=*/!quiet)==FAIL)
	       rtn = FAIL;
	    else if (*expect)
	    {
	       CLI_DEBUG(printf("debug: Waiting for response from %s\n",
				NAMESTR(leach[n].name_board[board])));

	       switch (wait_response(n, board, expect, !quiet, tomsec))
	       {
		  case LEACH_POLL_GOODRESP:
		  CLI_DEBUG(printf("debug: Good response from %s\n",
				   NAMESTR(leach[n].name_board[board])));
		  break;
		  case LEACH_POLL_NODATA:
		  printf("%s: no response from %s\n",
			 (!quiet && noresp_retry==0)?"warning":"logonly", /* Move other warnings here too */
			 NAMESTR(leach[n].name_board[board]));
		  if (noresp_retry--)
		  {
		     if (NORESP_DELAY_USEC) xusleep(NORESP_DELAY_USEC);
		     goto noresp_cmd;
		  }
		  /* Fall through and fail */
		  case LEACH_POLL_BADRESP:
		  case LEACH_POLL_ERR:
		  case LEACH_POLL_DON:
		  case LEACH_POLL_SYR:
		  case LEACH_POLL_ERROR:
		  case LEACH_POLL_ZEROPIX:
		  case LEACH_POLL_GARBAGE:
		  CLI_DEBUG(printf("debug: Bad response from %s\n",
				   NAMESTR(leach[n].name_board[board])));
		  rtn = FAIL; break;
	       }
	    }
	 }
      }
   }
   if (rtn==FAIL && !quiet)
   {
      printf("error: `cmd %s' failed\n", args);
   }
   return rtn;
}

#define POLL_MAXWORDS 5000

leach_poll_t
leach_poll(int verbose) /* === TODO: Combine this with wait_response? */
  /*
   * The communications channel to the leach system should always
   * be ready for new data.  This routine checks if any new data
   * has arrived.  Such unsolicited data should be cause to re-check
   * TDL to the affected to controller(s) and check IDs to make sure
   * DSP code is still downloaded.  (But right now, this just prints
   * a message that unsoliced data was received.)
   *
   * Returns FAIL if unsolicited data was found.
   */
{
   LREAD buf[POLL_MAXWORDS];
   LREAD* bufp;
   int n;
   int words;
   leach_poll_t rtn = LEACH_POLL_NODATA;

   for (n=0; n<n_controllers; n++)
   {
      /* timeout=0 makes this act like a poll */
      words = LIO_LREAD(leach[n].dev, buf, sizeof(buf)/sizeof(LREAD), 0, 0);

      if (words > 0)
      {
	 if (rtn < LEACH_POLL_GARBAGE) rtn = LEACH_POLL_GARBAGE;
      }

      for (bufp = buf; words > 0; )
      {
	 int len = bufp[0]&0xFF;

	 if ((bufp[0]&0xFF00)==0x0000 && /* Proper leach header destined for host */
	     len>=2 && len<=words &&     /* With logical # of words */
	     (len==words || len<=6))      /* Any apparent message in the
					  * middle of a block of garbage
					  * that is more than 6 words is
					  * treated as garbage too.
					  */
	 {
	    if (len==1 && bufp[0]==0)
	    {
	       if (rtn < LEACH_POLL_ZEROPIX) rtn = LEACH_POLL_ZEROPIX;
	       printf("%s: Stray zero pixel from %s\n",
		      verbose?"warning":"logonly", leach[n].name_contr);
	    }
	    /*
	     * The message actually has a header that makes sense
	     * if this condition is met.  First check for some common
	     * single word responses.
	     */
	    else if (len==2)
	    {
	       if (LREAD_MASK(bufp[1])==LREAD_MASK(STOW3('S','Y','R')))
	       {
		  if (rtn < LEACH_POLL_SYR) rtn = LEACH_POLL_SYR;
		  printf("%s: `SYR' from %s (system reset)\n",
			 verbose?"warning":"logonly", leach[n].name_contr);
	       }
	       else if (LREAD_MASK(bufp[1])==LREAD_MASK(STOW3('D','O','N')))
	       {
		  if (rtn < LEACH_POLL_DON) rtn = LEACH_POLL_DON;
		  printf("%s: Unsolicited `DON' from %s\n",
			 verbose?"warning":"logonly", leach[n].name_contr);
	       }
	       else if (LREAD_MASK(bufp[1])==LREAD_MASK(STOW3('E','R','R')))
	       {
		  if (rtn < LEACH_POLL_DON) rtn = LEACH_POLL_ERR;
		  printf("%s: Unsolicited `ERR' from %s\n",
			 verbose?"error":"logonly", leach[n].name_contr);
	       }
	       else
	       {
#ifndef LIO_16BIT_READ
		  printf("%s: Unsolicited leach message from %s 0x%06lX (%c%c%c)\n",
			 verbose?"warning":"logonly", leach[n].name_contr,
			 bufp[1], 
			 TOCHAR((bufp[1]&0xFF0000)>>16),
			 TOCHAR((bufp[1]&0xFF00)>>8),
			 TOCHAR(bufp[1]&0x00FF));
#else
		  printf("%s: Unsolicited leach message from %s 0x**%04X (*%c%c)\n",
			 verbose?"warning":"logonly", leach[n].name_contr,
			 bufp[1], 
			 TOCHAR((bufp[1]&0xFF00)>>8),
			 TOCHAR(bufp[1]&0x00FF));
#endif
	       }
	    }
	    else
	    {
	       printf("%s: Unsolicited %d-word message (including proper header) from %s\n",
		      verbose?"warning":"logonly", len, leach[n].name_contr); /* === Show actual data */
	    }
	    words -= (len);
	    bufp += len;
	 }
	 else
	 {
#define MAX_SHOW 75 /* Old director versions had a problem with 100! */
	    int i;
	    LREAD* bufp_save = bufp;
	    
	    printf("%s: Unsolicited garbage from %s%s %s%d*words",
		   verbose?"warning":"logonly",
		   leach[n].name_contr, words>6?"\nlogonly:":":",
		   words==POLL_MAXWORDS?"all ":"", words);
	    for (i = ((words>MAX_SHOW)?MAX_SHOW:words); i--; bufp++)
	    {
#ifndef LIO_16BIT_READ
	       printf(" 0x%06lX", *bufp);
#else
	       printf(" 0x**%04X", *bufp);
#endif
	    }
	    if (words>MAX_SHOW) printf("...\n");
	    else
	    {
	       printf(" ( ");
	       for (i = words, bufp = bufp_save; i--; bufp++)
	       {
#ifndef LIO_16BIT_READ
		  printf("%c%c%c ",
			 TOCHAR((*bufp&0xFF0000)>>16),
			 TOCHAR((*bufp&0xFF00)>>8),
			 TOCHAR(*bufp&0x00FF));
#else
		  printf("*%c%c ",
			 TOCHAR((*bufp&0xFF00)>>8),
			 TOCHAR(*bufp&0x00FF));
#endif
	       }
	       printf(")\n");
	    }
	    bufp = buf + words;
	    words = 0;
	 }
      }
   }
   return rtn;
}


/* End of leach_cmd.c */
