/*                                             -*- c-file-style: "Ellemtel" -*-

`leach_parse.c' - Leach specific parsing functions.

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include <ctype.h>

#include "leachcont.h"
#include "detcom.h" /* === Why? */

lb_set_t
lb_parse(const char* s)
   /*
    * Parses a character string and returns a bit mask (type lb_set_t)
    * indicating which leach boards and/or controllers were selected
    * by the string.  See the error printf before for examples.
    */
{
   lb_set_t rtn = 0;
   BoardID boards = LB_ALL;
   int controller = -1;
   const char* p = s;
   int n;

   /*
    * Figure out which boards are selected.
    */
   if (*p=='*')                        { p++; }
   else if (!xstrncasecmp(p, "all", 3)) { p+=3; }
   else if (!xstrncasecmp(p, "hs", 2))  { p+=2; boards = HS; }
   else if (!xstrncasecmp(p, "lvb", 3)) { p+=3; boards = LVB; }
   else if (!xstrncasecmp(p, "ltb", 3)) { p+=3; boards = LTB; }
   else if (!xstrncasecmp(p, "lub", 3)) { p+=3; boards = LUB; }

   /*
    * Figure out which controllers are selected.
    */
   if (*p=='0' || toupper(*p)=='M')      { p++; controller=0; }
   else if (*p=='1' || toupper(*p)=='S') { p++; controller=1; }

   if (*p!=0 || p==s)
   {
      printf("warning: `%s' is not a valid leach board specification; none selected.\n"
	     "Here are some examples of valid leach board names:\n"
	     "   `*' or ALL   = All relevant boards.\n"
             "   LUB or lub   = All (both if N_CONTROLLERS=2) utility boards.\n"
	     "   LUBM or lub0 = Only the master (`controller 0') utility board.\n"
	     "   ltbS or ltb1 = Only the slave (`controller 1') timing board only.\n"
	     "   M or 0 or *M = All relevant boards on master (`controller 0').\n"
	     "   S or 1 or *S = All relevant boards on slave (`controller 1').\n", s);
      return rtn;
   }

   for (n=0; n<n_controllers; n++)
   {
      if (controller==-1 || controller==n)
      {
	 switch (boards)
	 {
	    case HS:  LBSET(&rtn,n,HS); break;
	    case LVB: LBSET(&rtn,n,LVB); break;
	    case LTB: LBSET(&rtn,n,LTB); break;
	    case LUB: LBSET(&rtn,n,LUB); break;
	    default:  LBSET(&rtn,n,HS);  LBSET(&rtn,n,LVB);
                      LBSET(&rtn,n,LTB); LBSET(&rtn,n,LUB);
	 }
      }
   }
   return rtn;
}

/* End of leach_parse.c */
