/*                                             -*- c-file-style: "Ellemtel" -*-

`leachcont.h' - Leach-specific timeouts, conversion functions, and headers.

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef _INCLUDED_leachcont
#define _INCLUDED_leachcont 1

#include <time.h>
#include "cli/cli.h"
#include "lio.h"

/* TOMSEC stands for "TimeOut in MilliSEConds" */

#define TOMSEC_LEACH_FAST 150
/*
 * Responses to "fast" leach commands are expected to be virtually instantaneous.
 * Once data has begun flowing, the timeout TOMSEC_LEACH_READ takes effect.
 */

#define TOMSEC_LEACH_SLOW 3000
/*
 * Responses to "slow" leach commands, like PON, may take up to this amount
 * of time to begin sending their response.
 * Once data has begun flowing, the timeout TOMSEC_LEACH_READ takes effect.
 */

#define TOMSEC_LEACH_SYR 3000
/*
 * Anything data that begins within the 3 seconds after an SYR command
 * is taken as the response.  Once characters begin flowing, TOMSEC_LEACH_READ
 * takes effect though, so this does not mean you will necessarily get
 * all characters received within 3 seconds after the command.
 * Once data has begun flowing, the timeout TOMSEC_LEACH_READ takes effect.
 */

#define TOMSEC_LEACH_READ 10
/*
 * A gap of this many milliseconds in the stream of data coming from
 * the leach system causes and of the lio_read* routines to return
 * whatever they've read so far.
 */

#define NO_VALUE 0xFFFFFFFF
/*
 * Since leach words are 24-bits, this value will never occur, and
 * is used to indicate that no value has be read yet.
 */

#define FLAG16 0x80000000
/*
 * If the high byte equals 0x80, this is used to signal the fact
 * that the data is the result of a 16-bit read which hides the
 * upper 8-bits of the 24-bit leach word.
 */

#define MEM_RETRY 6
/*
 * Number of retries for RDM for WRM commands
 */

/* string-to-word macro for packing 3 chars */
#define STOW(x) ((unsigned long)((x)[0]<<16)|((x)[1]<<8)|((x)[2]))
#define STOW3(a,b,c) ((unsigned long)((a)<<16)|((b)<<8)|(c))
#define LEACH_MEMBASE(c) ((toupper(c)=='P')?0x100000:\
                          (((toupper(c)=='X')?0x200000:\
                           ((toupper(c)=='Y')?0x400000:0))))
#define MEMBASE(a) ((((a)&0x00F00000)==0x100000)?'P':\
                     ((((a)&0x00F00000)==0x200000)?'X':\
                      ((((a)&0x00F00000)==0x400000)?'Y':'?')))
#define MEMMASK(a) ((a)&0x000FFFFF)

#define LMSG2_SET(data,board,cmd) { data[0]=((board)<<8)|2; data[1]=(cmd); }
#define LMSG3_SET(data,board,cmd,arg) { data[0]=((board)<<8)|3; data[1]=(cmd); data[2]=(arg); }
#define LMSG4_SET(data,board,cmd,a1,a2) { data[0]=((board)<<8)|4; data[1]=(cmd); data[2]=(a1); data[3]=(a2); }
#define NAMESTR(s) ((s)?(s):"(null)")

/*
 * Addresses >= to this are assumed to be in ROM (or EPROM).
 * The download routine skips these _DATA blocks.
 */
#define LEACH_P_ROM 0x1000
#define LEACH_X_ROM 0x1000
#define LEACH_Y_ROM 0x1000

#define TOCHAR(x) (((char)(x)<' '||(char)(x)>'~')?'.':(char)(x))
/* === ? */

#define LBZERO(lbset)                   (*lbset = 0)
#define LBSET(lbset,controller,board)   (*lbset |= (1<<((board)+(controller)*8)))
#define LBISSET(lbset,controller,board) ((*lbset & (1<<((board)+(controller)*8)))!=0)

typedef int lb_set_t;

typedef enum { LB_ALL=-1, HS=0, LVB=1, LTB=2, LUB=3 } BoardID; /* === Move me */
#define MAX_BOARDS 4

typedef struct
{
   char* symname;
   char* symval;
} LeachSym;


typedef struct
{
   Devp dev;
   const char* name_contr;		/* Name of whole controller, for human readable
					 * messages.  Example: `leach' `master' `slave'
					 */
   const char* name_board[MAX_BOARDS];	/* Name of board (and controller) for human
					 * readable messages.
					 * Examples: `master utility board'
					 *           `slave timing board'.
					 */
   const char* name_cmdarg[MAX_BOARDS];	/* Name of board in acceptable format for command
					 * line arguments.  Examples: `lubM' `ltbS'
					 */
   char* dspcode_filename[MAX_BOARDS]; /* === FREE ? */
   time_t dspcode_timestamp[MAX_BOARDS];
   int dspcode_ok[MAX_BOARDS];
   int dspcode_force[MAX_BOARDS]; /* %%% Hack to force downloading in some situations */
#ifdef LIO_16BIT_READ
   int has_rm2[MAX_BOARDS];
#endif
   int sym_count[MAX_BOARDS];
   LeachSym* symbols[MAX_BOARDS];
} LeachDev;

/*
 * FUNCTION DECLARATIONS
 */

/* leach_parse.c */

lb_set_t lb_parse(const char* s);

/* leach_cmd.c */

typedef enum { LEACH_POLL_NODATA=0,
	       LEACH_POLL_GARBAGE=1,
	       LEACH_POLL_ZEROPIX=2,
	       LEACH_POLL_DON=3,
	       LEACH_POLL_ERR=4,
	       LEACH_POLL_SYR=5,
	       LEACH_POLL_GOODRESP=6, /* If "ERR" or "DON" is the desired response, GOODRESP takes precedence */
	       LEACH_POLL_BADRESP=7,
	       LEACH_POLL_ERROR=8 /* Command syntax error or system error */
             } leach_poll_t;

leach_poll_t leach_poll( int verbose );
/*
 * Poll for unsolicited data from any controller.  If nothing is
 * detecter, POLL_NODATA is returned.  Otherwise, the highest
 * POLL_xxxx return code that is applicable is returned.
 * (For example, if one controller had a DON and the other some
 * garbage followed by SYR, the return code would be SYR.
 */

PASSFAIL com_resetfiber( const char* args );
PASSFAIL com_tdl( const char* args );
PASSFAIL com_syr( const char* args );
PASSFAIL com_cmd( const char* args );

/* leach_power.c */

PASSFAIL com_pon( const char* args );
PASSFAIL com_pof( const char* args );
PASSFAIL com_power( const char* args );
PASSFAIL voltage_check(int n, const char* mtype_ok, const char* mtype_err);

/* leach_mem.c */

/*
 * Returned by sym_* functions if symbol not found or wrong type.
 * A logonly message giving the reason will be generated.
 */
#define SYM_INVALID ~((unsigned long)0)

unsigned long sym_to_number(LeachDev* dev, int board, const char* sym_name);
unsigned long sym_to_addr(LeachDev* dev, int board, const char* sym_name);
unsigned long sym_rdmem( const char* board_name, const char* sym_name);
PASSFAIL rdmem( LeachDev* leach, BoardID board,
		unsigned long addr, unsigned long* data,
		int verify, int verbose );
PASSFAIL wrmem( LeachDev* leach, BoardID board,
		unsigned long addr, unsigned long data,
		int verify_first );

PASSFAIL com_wrmem( const char* args );
PASSFAIL com_rdmem( const char* args );
/*
 * The following two are for keeping stats on how many times
 * wrm/rdm retries were needed before com_wrmem/com_rdmem were
 * able to return PASS.  Counter is reset to 0 each time
 * cmd_retries_reset() is called.
 */
void mem_retries_reset(void);
int  mem_retries(void);

/* leach_dsp.c */

typedef enum { DSP_SYM_UNKNOWN,
	       DSP_SYM_ADDRESS,
	       DSP_SYM_NUMBER,
	       DSP_SYM_ANY } dsp_sym_t;

const char* sym_lookup( LeachDev* leach, BoardID board,
			const char* name, dsp_sym_t symtype);
PASSFAIL com_symbol( const char* args );
PASSFAIL com_boot( const char* args );
const char* dsp_version( void );
/*
 * Returns a string suitable for the CONSWV FITS header.
 */

#endif
