/*                                             -*- c-file-style: "Ellemtel" -*-

`libedt.h' - Library from EDT that makes doing DMA with their cards easier.

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */
/* DRV_LABEL:  Copyright (c) by Engineering Design Team, Inc.  1995 */

#ifndef INCLUDE_edtlib_h
#define INCLUDE_edtlib_h
/* Copyright (c) 1995 by Engineering Design Team, Inc. */

#ifdef __STDC__
#pragma ident "@(#)libedt.h	1.29 03/25/98 EDT"
#else
    static char SccsId_libedt_h[] = "@(#)libedt.h	1.29 03/25/98 EDT" ;
#endif

#ifndef _DRVLIB_H
#define	_DRVLIB_H

#include <sys/types.h>
#include <sys/ioccom.h>
#ifdef SYSV
#include <thread.h>
#include <synch.h>
#else
#include <sys/asynch.h>
#endif

/*
 * SBus Device struct
 */
typedef struct {
	int	fd;		   /* file descriptor of opened device     */
	int	size;		   /* buffer size                          */
	int     nbufs ;	   	   /* number of multiple buffers	   */
	void  **ring_buffers ;	   /* pointer to mulitple buffer pointers  */
	void   *ring_buffer_tids ; /* pointer to mulitple thread IDs	   */
	int	nextbuf ;	   /* for edt_next_data			   */
	int	nextwbuf ;	   /* for edt_next_writebuf                */
	u_int	donecount ;
	int	last_buffer_waited ; /* lockstep enabled                   */
	int	ring_buffer_count ;
	int	ring_buffer_size ;
	int	ring_buffers_allocated ;
	int	ring_buffers_configured ;
	int	ring_buffer_setup_error ;
	int	write_flag ;
	int	bufnum ;
#ifdef SYSV
	mutex_t bufnum_mutex ;
	sema_t  bufnum_sema ;
#endif
} EdtDev;

#define EDT_READ 0
#define EDT_WRITE 1
#define IBUF_BASE 0x2000

#ifdef _STDC_
EdtDev *edt_open(char *device_name, int unit) ;
int edt_close(EdtDev *edt_p) ;
int edt_read(EdtDev *edt_p, void *buf, int size) ;
int edt_write(EdtDev *edt_p, void *buf, int size) ;
int edt_configure_ring_buffers(EdtDev *edt_p, int bufsize, int numbufs, int write_flag, void **bufarray) ;
int edt_configure_internal_ring_buffers(EdtDev *edt_p, int bufsize,int numbufs);
int edt_disable_ring_buffers(EdtDev *edt_p) ;
int edt_start_buffers(EdtDev *edt_p, int count) ;
int edt_stop_buffers(EdtDev *edt_p) ;
void *edt_wait_for_buffers(EdtDev *edt_p, int count) ;
void *edt_wait_for_next_buffer(EdtDev *edt_p) ;
int  edt_ring_buffer_overrun(EdtDev *edt_p) ;
void *edt_check_for_buffers(EdtDev *edt_p, int count) ;
void *edt_next_writebuf(EdtDev *edt_p) ;
void **edt_buffer_addresses(EdtDev *edt_p) ;
int edt_abort_dma(EdtDev *edt_p) ;
int edt_reset_ring_buffers(EdtDev *edt_p) ;
int edt_abort_current_dma(EdtDev *edt_p) ;
int edt_done_count(EdtDev *edt_p) ;
int edt_byte_count(EdtDev *edt_p) ;
#else
extern EdtDev *edt_open() ;
extern int edt_close() ;
extern int edt_read() ;
extern int edt_write() ;
extern int edt_configure_ring_buffers() ;
extern int edt_configure_internal_ring_buffers();
extern int edt_disable_ring_buffers() ;
extern int edt_start_buffers() ;
extern int edt_stop_buffers() ;
extern void *edt_wait_for_buffers() ;
extern void *edt_wait_for_next_buffer() ;
int  edt_ring_buffer_overrun() ;
extern void *edt_check_for_buffers() ;
void *edt_next_writebuf() ;
extern void **edt_buffer_addresses() ;
extern int edt_abort_dma() ;
extern int edt_reset_ring_buffers() ;
extern int edt_abort_current_dma() ;
extern int edt_done_count() ;
extern int edt_byte_count() ;
#endif

#ifdef PCI_DRIVER

#include "edt_ioctl.h"

#else
 
#ifndef X
#   ifdef _SYS_IOCCOM_H
#       define X       'X'
#   else
#       ifdef __STDC__
#          define X       'X'
#       endif
#   endif
#endif

#define EDTS_BURST_SIZE        	_IOW(X, 200, u_short)
#define EDTG_BURST_SIZE       	_IOR(X, 201, u_short)
#define	EDTS_DEBUG_LEVEL	_IOW(X, 202, u_short)
#define	EDTG_DEBUG_LEVEL	_IOR(X, 203, u_short)
#define	EDTS_DEBUG_INTR		_IOW(X, 204, u_short)
#define	EDTG_DEBUG_INTR		_IOR(X, 205, u_short)
#define	EDTS_MINPHYS		_IOW(X, 206, u_int)
#define EDTG_MINPHYS		_IOR(X, 207, u_int)
#define EDTG_DONE		_IOR(X, 208, u_int)
#define EDTG_QUEUE		_IOR(X, 209, u_int)
#define EDTG_AIO_COUNT		_IOR(X, 210, u_int)
#define EDTG_IO_COUNT		_IOR(X, 211, u_int)
#define EDTS_DMATHROT           _IOW(X, 212, u_int)
#define EDTG_DMATHROT           _IOR(X, 213, u_int)
#define EDTS_MAXDMASIZE         _IOW(X, 214, u_int)
#define EDTG_MAXDMASIZE         _IOR(X, 215, u_int)
#define EDTS_TERMINATE		_IO (X, 216)
#define EDT_FREE_BUF		_IO (X, 217)
#define EDT_KEEP_BUF		_IO (X, 218)
#define	EDT_STARTAIO		_IOW(X, 219, u_int)
#define	EDTS_NBUFIO		_IOW(X, 220, u_int)
#define	EDTG_NBUFIO		_IOR(X, 221, u_int)
#define EDTS_EODMA_SIG          _IOW(X, 222, u_short)
#define EDTS_FREEZE		_IO (X, 223)
#define EDTS_THAW		_IO (X, 224)
#ifdef TRACE
#define EDTG_TRACE		_IOWR(X, 225, caddr_t)
#define EDT_TRACECLR		_IO (X, 226)
#define EDT_TRACEMARK		_IO (X, 227)
#endif /* TRACE */
#define EDT_NODMASYNC		_IO (X, 228)
#define EDT_DMASYNC		_IO (X, 229)
#define EDTS_ALLOW 		_IOW(X, 230, u_short)

/* set read/write timeout */
#define  EDTS_RTIMEOUT          _IOW(X, 231, u_short)
#define  EDTS_WTIMEOUT          _IOW(X, 232, u_short)

/* 
 * set short xfer timeout mode 
 * - no error on timeout if set to nonzero
 * this will cause a read or write system call to return xfer size instead
 * of -1 on a timeout, in addition to having the xfer size in 
 * the aio_result structure for asynch i/o
 *
 * if short_xfer > 1, reset timeout on start of each transfer
 * if only set to 1 and R or WTIMEOUT is set, then timeout will
 * only be started when a read or write occurs when I/O is not
 * already in progress. (If a transfer completes and no other asynch
 * request are queued, 	the timeout is cancelled).
 */
#define  EDTS_SHORT_XFER	_IOW(X, 233,u_short)

/*
 * Driver semaphore implementation.
 */
#define	 EDTS_APPBUFS_COMPLETED	_IOW(X, 234, u_int)
#define	 EDTG_DEVBUFS_COMPLETED	_IOR(X, 235, u_int)
#define	 EDTG_DONECOUNT		_IOR(X, 235, u_int)
#define  EDTS_WAKEUP_DONECOUNT	_IOW(X, 236, u_int)
#define  EDTS_LOCKSTEP		_IOW(X, 237, u_int)

/*
 * return number of bytes transferred
 */
#define	 EDTG_BYTECOUNT		_IOR(X, 238, u_int)

/* Late addition to lockstep stuff */
#define  EDT_LOCKOFF		_IO (X, 239)

				     /* 240 is reserved for KATO */

#define  EDTS_BYTECOUNT         _IOW(X, 241, u_int)
#define  EDTS_IGNORE_SIGNALS    _IOW(X, 242, int)

/* set read/write timeout */
#define  EDTG_RTIMEOUT          _IOW(X, 243, u_short)
#define  EDTG_WTIMEOUT          _IOW(X, 244, u_short)

/* set long read/write timeout */
#define  EDTS_RLTIMEOUT          _IOW(X, 245, u_long)
#define  EDTS_WLTIMEOUT          _IOW(X, 246, u_long)

#define  EDTS_AUTO_BYTECOUNT     _IOW(X, 247, u_int)

#define	 EDTG_APPBUFS_COMPLETED	 _IOR(X, 234, u_int)

/* set/get customer requirement flag */
#define  EDTS_CUSTOMER           _IOW(X, 181, u_int)
#define  EDTG_CUSTOMER           _IOW(X, 180, u_int)

/* set association of a buffer address and it's order in ring buffer */
#define  EDTS_LSEEK              _IOW(X, 182, u_int *)

/* Have driver avoid dma splits; may be expensive when starting a ring buffer */
#define EDT_NODMASPLIT		_IO (X, 183)
#define EDT_DMASPLIT		_IO (X, 184)

/* Internal ring-buffer mode settings (IBUF) */
#define  EDTS_IBUF_SIZE		_IOW(X, 199, u_int)
#define  EDTG_IBUF_SIZE		_IOR(X, 198, u_int)
#define  EDTS_IBUF_NUM		_IOW(X, 197, u_int)
#define  EDTG_IBUF_NUM		_IOR(X, 196, u_int)
#define  EDTS_IBUF_DIR		_IOW(X, 195, u_int)
#define  EDTG_IBUF_DIR		_IOR(X, 194, u_int)
#define  EDTS_IBUF_MODE		_IOW(X, 193, u_int)
#define  EDTG_IBUF_MODE		_IOR(X, 192, u_int)

/* Set/get recycle_dma mode.  Recycle DMA memory when set, don't
 * free and realloc.
 */
#define  EDTS_RECYCLE_DMA	_IOW(X, 191, u_int)
#define  EDTG_RECYCLE_DMA	_IOR(X, 190, u_int)

#ifdef SYSV
/* 64-bit donecount */
#define  EDTG_LLDONECOUNT	_IOR(X, 189, u_longlong_t)

/* Manual DMA SYNC operations */
#define  EDTS_SYNC_RING_BUFFER	_IOW(X, 188, u_int)
#endif
#define EDT_ABORT_CURRENT	_IO(X, 187)
#define EDT_ABORT_DMA		_IO(X, 186)
#define EDTG_RFTAB		_IOWR(X, 185, caddr_t)
#define EDT_WAITRDY		_IO(X, 179) 

#define EDTG_TEST		_IOR(X, 185, caddr_t)
#define EDT_CLEAR_DONECOUNT     _IO (X, 178)

#define  EDTG_LOCKSTEP		_IOR(X, 177, u_int)

#define EDT_SHARE_RING_BUFFERS  _IOW(X, 176, u_int)
#define EDT_FREE_SHARED_RING_BUFFERS  _IO (X, 175)
#define EDT_RESET_RING_BUFFERS  _IO (X, 176)
#define drv_SET_BUFFER  	_IOW(X, 172, int)
#define EDT_SET_BUFFER  	_IOW(X, 172, int)

#endif /* PCI_DRIVER */

#endif /* !_DRVLIB_H */
#endif
