/*                                             -*- c-file-style: "Ellemtel" -*-

`lio_fake.c' - Read/write routines that use named pipes, for testing.

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#include "lio.h"

static Devp devs_used[MAX_CONTROLLER_SEARCH];
static int devs_initialized = 0;

Devp lio_open(const char* fn)
{
   int i;
   Devp rtn = 0;
   
   if (!devs_initialized)
   {
      memset(devs_used, 0, sizeof(devs_used));
      devs_initialized = 1;
   }
   
   if (fn)
   {
      char* tmp = strdup(fn);
      int n, len;
      
      len = strlen(tmp);
      n = tmp[len-1] - '0';
      tmp[len-1] = '\0';
      rtn = edt_open(tmp, n);
      free(tmp);
      return rtn;
   }
   else
   {
      for (i=0; i<MAX_S16D_SEARCH; i++)
      {
	 if (devs_used[i]==0 && (rtn = edt_open("s16d", i))!=0)
	 {
	    printf("debug: Found EDT card at /dev/s16d%d\n", i);
	    devs_used[i] = rtn;
	    return rtn;
	 }
      }
   }
   return 0;
}

int lio_close(Devp devp) { return 0; }

/*
 * This closes the device previously returned by lio_open.  The
 * return code is the same as the return code from close().
 */

int lio_check(Devp devp) { return 0; }
/*
 * Tests the connection between the sbus card and the lio.
 * A non-zero value is returned if the box appears to be connected.
 * If it fails, a warning is printed explaining possible problems,
 * and 0 is returned.
 */

void lio_reset(Devp devp) { }
/*
 * Set proper operating paramters for the EDT driver and send an
 * init pulse to the lio.
 */

void lio_write24(Devp devp, unsigned long* data, int nwords) { }
/*
 * Send 24-bit data to the lio.  The data is actually sent as 2 16-bit
 * words which the lio will re-assemble, but higher level code need
 * not be concerned with this.
 */

int lio_read16(Devp devp, unsigned short* buf, int maxwords, int response_msec)
{ return 0; }
/*
 * Read up to maxwords into buf and return the actual number of
 * words that were written, or -1.  Because of a hardware limitation
 * in the lio, the upper 8 bits of each 24-bit value are lost.
 * response_msec is the maximum number of milliseconds it could take
 * before data begins arriving.  Once data has started flowing, a
 * much shorter timeout takes effect.
 */

int lio_read16ring_parallel(int n_controllers,	/* Number of controllers */
			    Devp devp[],		/* Array of devices to read */
			    read16_handler handler,	/* Takes args: (controller,buffer,words) */
			    int maxwords,		/* Size in words of each buffer */
			    int response_seconds,	/* Maximum time before response should begin */
			    int max_seconds)		/* Maximum time for entire operation */
{ return 0; }

/*
 * This can be used to "prime" ring buffer mode by calling it in the following sequence:
 *
 * lio_read16ring_init(...);
 * lio_write24(...some command that causes an immediate response...);
 * lio_read16ring_parallel(...);
 */
int lio_read16ring_init(int n_controllers,		/* Number of controllers */
			Devp devp[])			/* Array of devices to read */
{ return 0; }
