/*                                             -*- c-file-style: "Ellemtel" -*-

`misc.c' - See misc.h.

This file is part of version 3 of the CFHT Detector Host Command Interface.
Read the `License' file for terms of use and distribution.
Copyright 1998-2001, Canada-France-Hawaii Telescope, daprog@cfht.hawaii.edu.

___This header automatically generated from `Index'. Do not edit it here!___ */

#ifndef xusleep
#include <sys/time.h>
#include <sys/types.h>

int
xusleep(unsigned int useconds)
{
   struct timeval tv;
   tv.tv_sec  = (unsigned long)useconds / 1000000;
   tv.tv_usec = (unsigned long)useconds % 1000000;
   return select(0, 0, 0, 0, &tv); /* Close enough for now */
}
#endif

/* Copyright (C) 1995,1996 Robert de Bath <rdebath@cix.compulink.co.uk>
 * This file is part of the Linux-8086 C library and is distributed
 * under the GNU Library General Public License.
 *
 * 1998-8-10 ANSIfied and renamed to xstrcasecmp by isani@cfht.hawaii.edu.
 */

#include <string.h>
#include <ctype.h>

#include "cli/cli.h"

int
xstrcasecmp(const char *s, const char *d)
{
   for(;;)
   {
      if( *s != *d )
      {
	 if( tolower(*s) != tolower(*d) )
	    return *s - *d;
      }
      else if( *s == '\0' ) break;
      s++; d++;
   }
   return 0;
}

/* Copyright (C) 1995,1996 Robert de Bath <rdebath@cix.compulink.co.uk>
 * This file is part of the Linux-8086 C library and is distributed
 * under the GNU Library General Public License.
 */

#include <string.h>
#include <ctype.h>

int
xstrncasecmp(const char* s, const char* d, size_t l)
{
   while(l>0)
   {
      if( *s != *d )
      {
	 if( tolower(*s) != tolower(*d) )
	    return *s - *d;
      }
      else
	 if( *s == '\0' ) return 0;
      s++; d++; l--;
   }
   return 0;
}

#include <sys/wait.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>

#include "misc.h"

static child_exit_t
report_exit(pid_t pid, int status)
{
   if (!WIFEXITED(status))
   {
      if (WIFSIGNALED(status))
      {
	 printf("\rwarning: pid %d exited on signal #%d\n",
		(int)pid, WTERMSIG(status));
	 return CHILD_EXITSIG;
      }
      else if (WIFSTOPPED(status))
      {
	 printf("\rwarning: pid %d stopped unexpectedly; killing it\n", (int)pid);
	 return pid_abort(pid);
      }
      else
      {
	 printf("\rerror: Unknown problem with pid %d\n", (int)pid);
	 return CHILD_ERROR;
      }
   }
   else
   {
      if (WEXITSTATUS(status)!=EXIT_SUCCESS)
      {
	 printf("\rlogonly: pid %d exited with exitcode %d\n",
		(int)pid, WEXITSTATUS(status));
	 return CHILD_EXITFAIL;
      }
      else
      {
	 CLI_DEBUG(printf("\rdebug: pid %d completed successfully\n", (int)pid));
	 return CHILD_EXITPASS;
      }
   }
}

child_exit_t
pid_abort(pid_t pid)
{
   int wait_sec = 10*2; /* Wait up to 10 seconds for fits writer(s) to clean up */
   int status, waitrtn;

   if (pid==(pid_t)-1)
      return CHILD_GONE;

   CLI_DEBUG(printf("\rdebug: Sending pid %d the TERM signal.\n", (int)pid));
   kill(pid, SIGTERM);

   do
   {
      xusleep(500000);

      status = -1;
      waitrtn = waitpid(pid, &status, WUNTRACED|WNOHANG);

      if (waitrtn==-1)
      {
	 if (errno==EINTR) wait_sec = 0;
	 else if (errno==ECHILD)
	 {
	    CLI_DEBUG(printf("\rdebug: Child process was already gone; no exit status available.\n"));
	    return CHILD_GONE;
	 }
	 else
	 {
	    perror("error: waitpid failed");
	    return CHILD_ERROR;
	 }
      }
      else if (waitrtn == pid)
      {
	 return report_exit(pid, status);
      }
      else if (waitrtn == 0)
      {
	 printf("\rprogress: Aborting PID %d ... Please wait.\n", (int)pid);
      }
      else
      {
	 printf("\rerror: expect pid %d; got pid %d from waitpid!\n", (int)pid, waitrtn);
	 return CHILD_ERROR;
      }
   } while (wait_sec--);
   
   printf("\rwarning: PID %d won't die; sending it the KILL signal.\n", (int)pid);
   kill(pid, SIGKILL);
   return CHILD_ERROR;
}

child_exit_t
pid_wait(pid_t pid, int poll)
{
   int status, waitrtn;
   BOOLEAN aborted;

   if (pid==(pid_t)-1)
      return CHILD_GONE;

   if (poll == 0)
      printf("\rlogonly: Waiting for pid %d to exit.  Press ^C to abort.\n", (int)pid);

   cli_signal_unblock(SIGINT);
   cli_signal_event(SIGINT);
   waitrtn = waitpid(pid, &status, WUNTRACED|(poll?WNOHANG:0));
   aborted = cli_signal_event(SIGINT);
   cli_signal_block(SIGINT);
   
   if (waitrtn==-1)
   {
      if (errno==EINTR)
      {
	 /*
	  * React to Control-C here by killing the FITS writer thread.
	  */
	 if (aborted)
	    return pid_abort(pid);
      }
      
      if (errno==ECHILD)
      {
	 CLI_DEBUG(printf("\rdebug: Child process was already gone; no exit status available.\n"));
	 return CHILD_GONE;
      }
      perror("error: waitpid failed");
      return CHILD_ERROR;
   }
   else if (waitrtn==0)
   {
      return CHILD_RUNNING;
   }
   else if (waitrtn!=pid)
   {
      printf("\rerror: expect pid %d; got pid %d from waitpid!\n", (int)pid, waitrtn);
      return CHILD_ERROR;
   }

   return report_exit(pid, status);
}

child_exit_t
pid_wait_nointr(pid_t pid)
{
   int waitrtn, status;

   printf("\rlogonly: Waiting for pid %d to exit\n", (int)pid);
   waitrtn = TEMP_FAILURE_RETRY(waitpid(pid, &status, 0));
   if (waitrtn == -1)
   {
      perror("\rerror: waitpid failed");
      return CHILD_ERROR;
   }
   return report_exit(pid, status);
}
