/* EDT_LABEL:  Copyright (c) by Engineering Design Team, Inc.  1994 */

#ifndef INCLUDE_s16d_h
#define INCLUDE_s16d_h

#ifdef __STDC__
#pragma ident "@(#)s16d.h	1.8 03/23/98 EDT"
#else
    static char SccsId_s16d_h[] = "@(#)s16d.h	1.8 03/23/98 EDT" ;
#endif

/*
 * s16d.h
 *      (This file is included on the driver diskette)
 *
 * Copyright (c) 1991, Engineering Design Team, Inc.  All Rights Reserved.
 */

#include <sys/ioccom.h>

/*
 * command register defines
 */
#define S16D_GO         0x0001  /* initiate DMA - clears READY, pulses GO */
#define S16D_DIR        0x0002  /* DMA direction  1 = write		  */
#define S16D_FLUSH      0x0004  /* set to one when aa DMA is aborted	  */
				/* and the register files may have some   */
				/* data remaining waiting for a burst     */
				/* transfer - will cause a final 16 byte  */
				/* transfer to the current DMA buffer     */


#define S16D_FNCT1      0x0010  /* Function control bits - passed to      */
				/* device interface  - FNCT2 is connected */
				/* to the device interrupt input	  */
#define S16D_FNCT2	0x0020
#define S16D_FNCT3	0x0040

#define S16D_INIT       0x0080  /* Initialize Device - not a pulse        */
                                /* INIT signal will be asserted as long   */
                                /* as this is a 1 - see S16D_INIT_PULSE   */
                                /* ioctl                                  */

#define S16D_BCLR       0x0100  /* Board clear - aborts DMA in process    */
                                /* and clears FIFOS - does not send INIT  */

#define S16D_FCLK       0x1000  /* force clock - has same effect as an    */
				/* external cycle request		  */

#define S16D_RDINT      0x2000  /* reset device interrupt bit in          */
                                /* status register - does not affect the  */
                                /* state of DINT signal or DINT status bit */

#define S16D_REOD       0x4000  /* reset End of DMA interrupt - does not  */
                                /* affect state of RDY status bit - if only */
                                /* interrrupt, interrupt also reset       */

#define S16D_DISINT     0x8000  /* disables interrupts to SBus when set    */

/* convienent defines */
#define S16D_RESET (S16D_BCLR | S16D_RDINT | S16D_REOD | S16D_DISINT)
/* bits to save state of for next write to command reg */
#define S16D_SAVE_MASK \
    (S16D_DIR | S16D_INIT | S16D_FNCT1 | S16D_FNCT2 | S16D_FNCT3 | S16D_DISINT)

/* 
 * status register defines
 */
#define S16D_DMAON_S    0x0001  /* 1 - dma in progress 0 - idle */

#define S16D_DIR_S      0x0002  /* inverted read back as written in command */
#define S16D_DINTDIS_S  0x0004  /* inverted read back of config */
#define S16D_EODDIS_S   0x0008  /* inverted read back of config */

#define S16D_FNCT1_S    0x0010  /* read back as written in command */
#define S16D_FNCT2_S    0x0020
#define S16D_FNCT3_S    0x0040


#define S16D_INIT_S   	0x0080  /* read back of command */

#define S16D_STATA_S    0x0100  /* state of external S16D inputs STAT[A-C] */
#define S16D_STATB_S    0x0200
#define S16D_STATC_S    0x0400

#define S16D_DINT_S     0x0800  /* state of external DINT - not latched	   */
#define S16D_INTDEV_S   0x2000  /* latched DINT - device interrupt pending */
                                /* 1 - pending  0 - not pending            */
#define S16D_EODMA_S    0x4000  /* end of dma - interrupt pending          */   
                                /* 1 - pending  0 - not pending            */
#define S16D_INT_S      0x8000  /* attn or eodma interrupt pending/enabled */
                                /* 1 - pending  0 - not pending            */

/* 
 * configuration register defines
 */
#define S16D_DACKP      0x0001  /* sets polarity of DACK signal           */
				/* 0 - negative true, 1 - positive        */
				/* if hardware jumper removed, polarity   */
				/* is forced to positive true and         */
				/* this bit is ignored - see users manual */
					
#define S16D_CLKP       0x0002  /* data clock polarity			  */
                                /* 0 - negative edge			  */
#define S16D_DBMODE	0x0004	/* disable SBus burst DMA transfers	  */
				/* - used in applications where data must */
				/* be immediately transferred to memory   */
#define S16D_DFRST	0x0008	/* disable fifo reset			  */
				/* - when set prvents bd clr from         */
				/* resetting and emptying the fifo        */
#define S16D_SWAP       0x0100  /* 1 - swap bytes                         */
#define S16D_ENDINT	0x2000  /* enable device interrupt		  */
#define S16D_ENEOD	0x4000  /* enable end of dma interrupt		  */

#define S16D_BLK_OUTPUT_MAX	256

/*
 * Ioctl definitions:
 */

#ifdef _SYS_IOCCOM_H
#   define X       'X'
#else
#   ifdef __STDC__
#      define X       'X'
#   else
#      define X        X
#   endif
#endif


/* set command register */
#define  S16S_COMMAND           _IOW(X,1,u_short)
#define  S16G_COMMAND           _IOR(X,1,u_short)

/* get status register */
#define  S16G_STATUS            _IOR(X,2,u_short)

/* set config register */
#define  S16S_CONFIG            _IOW(X,3,u_short)

/* set/get data register */
#define  S16S_DATA              _IOW(X,6,u_short)
#define  S16G_DATA              _IOR(X,7,u_short)

/* send init pulse */
#define  S16S_SEND_INIT_PULSE   _IOW(X,8,u_short)

/* set signal on receipt of DINT interrupt */
/* takes signal to send to process doing this ioctl */
#define  S16S_DINT_SIG           _IOW(X,9,u_short)

/* set read command */
#define  S16S_READ_COMMAND      _IOW(X,10,u_short)

/* set write command */
#define  S16S_WRITE_COMMAND     _IOW(X,11,u_short)

/* set/get dmaaddr register */
#define  S16S_DMAADDR           _IOW(X,15,u_int)
#define  S16G_DMAADDR           _IOR(X,16,u_int)

/* set dmalast register */
#define  S16S_DMALAST           _IOW(X,17,u_int)

/* set/get default read & write commands (s16d_def_readcmd &
 * s16d_def_writecmd) - will remain as defaults until reboot or reload
 * or another S16S_DEF_READ/WRITE ioctl
 */
#define S16G_DEF_READ_COMMAND	_IOR(X,20,u_short)
#define S16S_DEF_READ_COMMAND	_IOW(X,21,u_short)
#define S16G_DEF_WRITE_COMMAND	_IOR(X,22,u_short)
#define S16S_DEF_WRITE_COMMAND	_IOW(X,23,u_short)
#define S16G_READ_COMMAND      _IOR(X,24,u_short)
#define S16G_WRITE_COMMAND     _IOR(X,25,u_short)

/*
 * set error to get when dint occurs
 */
#define S16S_DINT_ERR	_IOW(X,26,u_short)

/* 
 * get config register
 */
#define S16G_CONFIG     _IOR(X,27,u_short)


/*
 * get number of bytes transfered since start of current DMA
 */
#define S16G_XCOUNT	_IOR(X,29,u_int)


/* ioctl's to set what to write to csr when dma done */
#define	S16S_WRITE_DONE		_IOW(X,31,u_short)
#define	S16S_READ_DONE		_IOW(X,32,u_short)

/* cancel read/write done */
#define	S16S_WRITE_DONE_CNCL	_IOW(X,33,u_short)
#define	S16S_READ_DONE_CNCL	_IOW(X,34,u_short)

/* clear IO and AIO counts */
#define	S16S_COUNT_CLR		_IO(X,37)

/* terminate outstanding requests */
#define	S16_TERMINATE		_IO(X,38)

/* set pulse f2 */
#define	S16S_PULSE_F2		_IOW(X,39,u_short)

/* Output a block of data to the s16d  */
#define S16_BLK_OUTPUT		_IOW(X,40,u_short *)

/* clear READ_COMMAND and WRITE_COMMAND */
#define	S16S_READ_COMMAND_CNCL		_IO(X,41)
#define	S16S_WRITE_COMMAND_CNCL		_IO(X,42)

#define	S16G_SAMPLE_TRANSFER_SIZE	_IOR(X,44,u_int)
#define	S16S_DEVINT_ABORTS_DMA		_IOW(X,45,u_int)

#include "drv_ioctl.h"
#endif
