#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fh/fh.h"

   int
   main(int argc, const char* argv[])
   {
      HeaderUnit hu = fh_create();
      int i, exitcode = EXIT_SUCCESS;

      if (argc < 2)
      {
         fprintf(stderr, "usage: fhset <fitsfile> KEYWORD=value [/comment] [KEYWORD=value [/comment] ...]\n");
         exit(2);
      }

      if (fh_file(hu, argv[1], FH_FILE_RDWR) != FH_SUCCESS)
         exit(EXIT_FAILURE);
   
      for (i = 2; i < argc; i++)
      {
         char* arg = strdup(argv[i]);
         char* val = strchr(arg, '=');
         const char* com;

         com = 0;
         if (!val)
         {
            fprintf(stderr, "fhset: error: Specify KEYWORD=value (with no spaces)\n");
            fh_destroy(hu);
            exit(2);
         }
         *val++ = '\0';
         if (i + 1 < argc && *argv[i + 1]=='/')
         { com = argv[++i]; com++; } /* Don't include the '/' itself. */
         fh_set_val(hu, FH_AUTO, arg, val, com);
      }
      /*
       * Write the changes to the file, and unlock it.
       */
      if (fh_rewrite(hu) != FH_SUCCESS) exitcode = EXIT_FAILURE;
      if (fh_destroy(hu) != FH_SUCCESS) exitcode = EXIT_FAILURE;
      exit(exitcode);
   }
