/*                                             -*- c-file-style: "Ellemtel" -*-

`mef8k.c' - Generates multi-extension-fits from separate files.

Adapted from mef.c for CFH12K (just changed number of chips from 12->8.)
Designed to handle separate FITS files as generated by "DetCom".

Author: Sidik Isani <isani@cfht.hawaii.edu>
Date: 2000-6-15

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#define EXTENSIONS 8
#define MEF_BLOCK_SIZE 2880 /* Don't change this */

int
main(int argc, const char* argv[])
{
   char* dirname;
   char* basename;
   char* filename;
   char buf[MEF_BLOCK_SIZE];
   int i, fd, len, nread;

   if (argc!=2 || isatty(STDOUT_FILENO))
   {
      fprintf(stderr,
	      "usage: %s dirname > name-mef.fits\n"
	      "Converts separate fits files into a single MEF (multi-extension FITS) file.\n"
	      "Files must be in a subdirectory called FITSNAME/, which contains FITSNAME.phu\n"
	      "and %d separate FITS files, FITSNAME00.fits to FITSNAME%02d.fits.\n",
	      argv[0], EXTENSIONS, EXTENSIONS-1);
      exit(2);
   }
   dirname = strdup(argv[1]);
   len = strlen(dirname);
   while (len && (dirname[len-1] == '/' || dirname[len-1] <= ' '))
      dirname[--len] = '\0';
   basename = strrchr(dirname, '/');
   if (!basename) basename = dirname;
   else basename++;
   if (chdir(dirname)!=0)
   {
      perror(dirname);
      exit(EXIT_FAILURE);
   }
   filename = (char*)malloc(strlen(basename) + 32);
   if (!filename)
   {
      perror("malloc");
      exit(EXIT_FAILURE);
   }
   sprintf(filename, "%s.phu", basename);
   if ((fd = open(filename, O_RDONLY, 0))==-1)
   {
      perror(filename);
      exit(EXIT_FAILURE);
   }
   while ((nread = read(fd, buf, MEF_BLOCK_SIZE)) != 0)
   {
      if (nread == -1)
      {
	 perror(filename);
	 close(fd);
	 exit(EXIT_FAILURE);
      }
      if (write(STDOUT_FILENO, buf, nread)!=nread)
      {
	 perror("write to stdout failed");
	 close(fd);
	 exit(EXIT_FAILURE);
      }
   }
   close(fd);
   for (i=0; i<EXTENSIONS; i++)
   {
      int first_block;

      sprintf(filename, "%s%02d.fits", basename, i);
      if ((fd = open(filename, O_RDONLY, 0))==-1)
      {
	 perror(filename);
	 exit(EXIT_FAILURE);
      }
      first_block = 1;
      while ((nread = read(fd, buf, MEF_BLOCK_SIZE)) != 0)
      {
	 if (nread == -1)
	 {
	    perror(filename);
	    close(fd);
	    exit(EXIT_FAILURE);
	 }
	 if (first_block)
	 {
	    if (memcmp(buf, "SIMPLE  =                    T", 30))
	    {
	       fprintf(stderr, "error: %s is not a FITS file.\n", filename);
	       close(fd);
	       exit(EXIT_FAILURE);
	    }
	    memcpy(buf,
		   "XTENSION= 'IMAGE  '            / Image extension                                ",
		   80);
	    first_block = 0;
	 }
	 if (write(STDOUT_FILENO, buf, nread)!=nread)
	 {
	    perror("write to stdout failed");
	    close(fd);
	    exit(EXIT_FAILURE);
	 }
      }
      close(fd);
   }
   exit(EXIT_SUCCESS);
}
