/* Copyright (C) 1991, 1994    Canada-France-Hawaii Telescope Corp.      */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_LocalSidTime.c
* $Header: cfht_LocalSidTime.c,v 1.2 96/11/19 22:24:50 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_LocalSidTime()
*    This routine will return a string of the current mean local sidereal time
*    for the location of the CFHT telescope
*    suitable for stamping. 
*
*    INVOKED
*    result_string = cfht_LocalSidTime();
*
*    PARAMETERS IN
*    NONE
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    pointer to string containing date
*
*    GLOBALS
*    NONE, although the string area is static
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* bg           941213          original coding
*
* $Log:	cfht_LocalSidTime.c,v $
 * Revision 1.2  96/11/19  22:24:50  isani
 * cleaned up rcsid
 * 
 * Revision 1.1  1995/03/17  07:32:21  bernt
 * Initial revision
 *
*
****************************************************************************!*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"

RCSID("$Header: cfht_LocalSidTime.c,v 1.2 96/11/19 22:24:50 isani Exp $");

#define     E_TZONE     "TZ"

char *cfht_LocalSidTime(clockp)
long *clockp;        /* pointer to long that has seconds since 1970 */
{

    double jd, tu, st, longitude;
    long t;
    struct tm *tmd;
    static char rtn[12];

    longitude = -155.4716667; /* cfht longitude */
    if(clockp == (long *) NULL) { /* if not passed in - get */
        clockp = &t;
        time(clockp);
    }
    tmd = gmtime(clockp);

/*  cfht_julian(year,month,day,hour,minute,sec) */

    jd = cfht_julian(1900+tmd->tm_year,
                      tmd->tm_mon+1    ,
                      tmd->tm_mday     ,
                      tmd->tm_hour     ,
                      tmd->tm_min      ,
                      (double) tmd->tm_sec );
    tu = (jd-2451545.)/36525.;
    st = 280.46061837 + 360.98564736629*(jd -2451545.0) + 0.000387933*tu*tu
	 - tu*tu*tu/38710000. + longitude;
    st = fmod(st, 360.);
    if (st < 0.0)
       st = st + 360.0;
    st = st*240.;
    cfht_frs (rtn, st, 24, 2);

    return(rtn);
}

