/* Copyright (C) 1991, 1994    Canada-France-Hawaii Telescope Corp.      */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_Mjulian.c
* $Header: cfht_Mjulian.c,v 1.2 96/11/19 22:37:05 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_Mjulian()
*    This routine will return a string of the current modified julian date
*    suitable for stamping. 
*    the modified julian date is defined as:
*    Mod julian date = julian date - 2400000.5
*
*    INVOKED
*    result_string = cfht_Mjulian();
*
*    PARAMETERS IN
*    NONE
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    pointer to string containing date
*
*    GLOBALS
*    NONE, although the string area is static
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* bg           941213          original coding
*
* $Log:	cfht_Mjulian.c,v $
 * Revision 1.2  96/11/19  22:37:05  isani
 * added rcsid
 * changed a "%lf" format to "%f"
 * 
 * Revision 1.1  1995/03/17  07:33:56  bernt
 * Initial revision
 *
*
****************************************************************************!*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"

RCSID("$Header: cfht_Mjulian.c,v 1.2 96/11/19 22:37:05 isani Exp $");

#define     E_TZONE     "TZ"

char *cfht_Mjulian(clockp)
long *clockp;        /* pointer to long that has seconds since 1970 */
{

    double mjd;
    long t;
    struct tm *tmd;
    static char rtn[12];

    if(clockp == (long *) NULL) { /* if not passed in - get */
        clockp = &t;
        time(clockp);
    }
    tmd = gmtime(clockp);

/*  cfht_julian(year,month,day,hour,minute,sec) */

    mjd = cfht_julian(1900+tmd->tm_year,
                      tmd->tm_mon+1    ,
                      tmd->tm_mday     ,
                      tmd->tm_hour     ,
                      tmd->tm_min      ,
                      (double) tmd->tm_sec ) - 2400000.5;

    sprintf(rtn,"%.6f", mjd); 
    return(rtn);
}

