/* Copyright (C) 1991-1999    Canada-France-Hawaii Telescope Corp.       */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING         */
/*!****************************************************************************
 * FILE
 * cfht_PFdate.c
 * $Header: /usr/local/cfht/dev/pegasus/libcfht/RCS/cfht_PFdate.c,v 1.1 1999/07/25 06:32:21 thomas Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_PFdate()
 *    This routine will return a string of the past or future date suitable for
 *    a stamping. 
 *
 *    INVOKED
 *    result_string = cfht_PFdate(delta);
 *
 *    PARAMETERS IN
 *    int delta - # of days from today  neg # past pos # future 
 *
 *    PARAMETERS OUT
 *    NONE
 *
 *    FUNCTION RETURNS
 *    pointer to string containing date
 *
 *    GLOBALS
 *    NONE, although the string area is static
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * bg           941213          original coding
 *
 * $Log: cfht_PFdate.c,v $
 * Revision 1.1  1999/07/25 06:32:21  thomas
 * Initial revision
 *
 * Revision 1.1  95/01/25  11:51:30  11:51:30  john (John Kerr)
 * Initial revision
 * 
 **************************************************************************!*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"

#define     E_TZONE     "TZ"

char *cfht_PFdate(days)
int  days;              /* # of days since today */
{
    long clockp;
    struct tm *tmd;
    static char rtn[CFHT_PFDATE_SIZE];

    time(&clockp);
    clockp = clockp + days * 86400.0;
    tmd = localtime(&clockp); 

    sprintf(rtn,"%04d/%02d/%02d",
	    tmd->tm_year + 1900, /* well, time.h does not provide a symbol */
	    tmd->tm_mon + 1,
	    tmd->tm_mday);
    return(rtn);
}

