/* Copyright (C) 1991-1999    Canada-France-Hawaii Telescope Corp.       */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING         */
/*!****************************************************************************
 * FILE
 * cfht_UTdate.c
 * $Id: cfht_UTdate.c,v 1.3 1999/07/25 06:36:55 thomas Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_UTdate()
 *    This routine will return a string of the current UT date suitable for a
 *    stamping. 
 *
 *    INVOKED
 *    result_string = cfht_UTdate(time);
 *
 *    PARAMETERS IN
 *    long *time - pointer to seconds since 1970 to convert or NULL for now
 *
 *    PARAMETERS OUT
 *    NONE
 *
 *    FUNCTION RETURNS
 *    pointer to string containing date
 *
 *    GLOBALS
 *    NONE, although the string area is static
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * bg           941213          original coding
 *
 * $Log: cfht_UTdate.c,v $
 * Revision 1.3  1999/07/25 06:36:55  thomas
 * Y2K fix
 *
 * Revision 1.2  1996/11/19 22:37:39  isani
 * added rcsid
 *
 * Revision 1.1  1995/01/25  11:51:30  john
 * Initial revision
 *
 **************************************************************************!*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"

RCSID("$Id: cfht_UTdate.c,v 1.3 1999/07/25 06:36:55 thomas Exp $");

#define     E_TZONE     "TZ"

char *cfht_UTdate(clockp)
long *clockp;        /* pointer to long that has seconds since 1970 */
{
    long t;
    struct tm *tmd;
    static char rtn[CFHT_UTDATE_SIZE];

    if (clockp == (long *) NULL) { /* if not passed in - get */
        clockp = &t;
        time(clockp);
    }
    tmd = gmtime(clockp);

    sprintf(rtn,"%4d/%02d/%02d",
	    tmd->tm_year + 1900, /* well, time.h does not provide a symbol */
	    tmd->tm_mon + 1,
	    tmd->tm_mday);
    return(rtn);
}
