/* Copyright (C) 1991-1999    Canada-France-Hawaii Telescope Corp.       */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING         */
/*!****************************************************************************
 * FILE
 * cfht_UTtime.c
 * $Id: cfht_UTtime.c,v 1.4 2001/10/14 20:16:11 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_UTtime()
 *    This routine will return a string suitable for coordinated univerasal
 *    ascii time *    stamping.  If the incoming pointer is NULL, the current
 *    time as returned by time(2) is used.  The return area is static and will
 *    be changed each call.
 *
 *    INVOKED
 *    timestring = cfht_UTtime(clockp);
 *
 *    PARAMETERS IN
 *    long *clockp - pointer to long that has seconds since 1970 or
 *                   NULL to use current time
 *
 *    PARAMETERS OUT
 *    none
 *
 *    FUNCTION RETURNS
 *
 *    GLOBALS
 *    static char rtn[12] - the local area used for constructing the
 *                          return string, reused on each call
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * bg           941214          original coding adapted from cfht_time
 *
 * $Log: cfht_UTtime.c,v $
 * Revision 1.4  2001/10/14 20:16:11  isani
 * Include time.h
 *
 * Revision 1.3  99/07/25  08:27:27  thomas
 * Y2K fix
 * 
 * Revision 1.2  1996/11/19 22:38:38  isani
 * cleaned up rcsid
 * changed hsec from a long to an int
 *
 * Revision 1.1  1995/03/17  07:34:15  bernt
 * Initial revision
 *
 * Revision 1.5  94/12/14  12:07:18  12:07:18  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Fixed type casts for clockp
 * 
 * Revision 1.4  94/09/20  18:58:46  18:58:46  thomas (Jim Thomas)
 * use symbol for time string size
 * make it long enough to hold previously added tenths of seconds
 * 
 * Revision 1.3  94/01/20  02:10:14  02:10:14  jwright (Jim Wright)
 * minor reformat
 * 
 * Revision 1.2  93/06/29  15:47:30  15:47:30  steve (Steven Smith)
 * added fractions of seconds to time value
 * 
 * Revision 1.1  91/09/24  17:03:16  17:03:16  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>

#include "cfht.h"

RCSID("$Id: cfht_UTtime.c,v 1.4 2001/10/14 20:16:11 isani Exp $");

char *cfht_UTtime(clockp)
long *clockp;        /* pointer to long that has seconds since 1970 */
{
    int hsec;
    struct tm *tmd;
    static char rtn[CFHT_UTTIME_SIZE];
    struct timezone tzp;
    static struct timeval tv;

    if (clockp == (long *) NULL) { /* if not passed in - get */
	gettimeofday (&tv, &tzp);
        clockp = (long *) &(tv.tv_sec);
    } else {
	tv.tv_usec = 0;		/* passed in seconds have no fractions */
    }
    hsec = (int)(tv.tv_usec / 10000);
    tmd = gmtime(clockp);
    sprintf (rtn, "%02d:%02d:%02d.%02d",
	     tmd->tm_hour, tmd->tm_min, tmd->tm_sec, hsec);
    
    return(rtn);
}
