/* Copyright (C) 1998   Canada-France-Hawaii Telescope Corp.              */
/* This program is distributed WITHOUT any warranty, and is under the     */
/* terms of the GNU General Public License, see the file COPYING          */
/*!****************************************************************************
 * FILE
 * cfht_argsToString.c
 * $Header: cfht_argsToString.c,v 1.1 98/04/08 15:48:35 thomas Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *  cfht_argsToString()
 *    convert argv back into a string
 *    Given argc and argv , build a string that matches the final version of
 *    the original command (i.e., as much as we can reconstruct from argv -
 *    after aliasing and assuming $PATH was used but including quotes where
 *    needed).  The string is built in an internal static area (but then it's
 *    not likely a program would need this more than once :-) .  If argc is
 *    zero, " " is returned.  Otherwise a string based on the command is.  If
 *    the original command is longer than CFHT_ARGS_SIZE , only the front part
 *    of the command is returned.
 *
 *   INVOKED
 *    String = cfht_argsToString(argc, argv);
 *
 *   PARAMETERS IN
 *    int argc     - copy of argc
 *    char *argv[] - copy of argv
 *
 *   FUNCTION RETURNS
 *    char * - string version of command
 *
 *   GLOBALS
 *    None
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jnt          03-Apr-1998     started
 *
 * $Log:	cfht_argsToString.c,v $
 * Revision 1.1  98/04/08  15:48:35  15:48:35  thomas (Jim Thomas)
 * Original check in
 * 
 ***************************************************************************!*/

#include <string.h>		  /* for string handling defines */

#include "cfht.h"

RCSID("$Header: cfht_argsToString.c,v 1.1 98/04/08 15:48:35 thomas Exp $");

char *cfht_argsToString(argc, argv)
    int argc;
    char *argv[];
{
    unsigned int argIndex;	  /* index into argv */
    static char argsString[CFHT_ARGS_SIZE + 1]; /* resulting command string */
    unsigned int StringIndex;	  /* index into argsString */

    if (argc <= 0) {
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_argsToString:%d) argc (%d) less than 1?",
		  __LINE__, argc);
	(void) strcpy(argsString, " ");
	return(argsString);
    }
    if (argv == (char **) NULL) {
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_argsToString:%d) argv null?",
		  __LINE__);
	(void) strcpy(argsString, " ");
	return(argsString);
    }

    /* first put in command name (less path) - cannot be too big to fit */

    (void) strcpy(argsString,
		  cfht_basename((char *) NULL, argv[0], (char *) NULL));

    /* now copy in each argument, quoting if needed, until no more room */

    argIndex = 1;
    StringIndex = (unsigned int) strlen(argsString);
    while ((argIndex < argc) && (StringIndex < CFHT_ARGS_SIZE)) {
	argsString[StringIndex++] = ' ';
	argsString[StringIndex] = '\0';
	if (strpbrk(argv[argIndex], " \t") == (char *) NULL) {
	    (void) strncpy(&argsString[StringIndex],
			   argv[argIndex],
			   CFHT_ARGS_SIZE - StringIndex);
	    argsString[CFHT_ARGS_SIZE] = '\0';
	    StringIndex += strlen(argv[argIndex]);
	} else {
	    if (StringIndex < CFHT_ARGS_SIZE) {
		argsString[StringIndex++] = '"';
		argsString[StringIndex] = '\0';
		(void) strncpy(&argsString[StringIndex],
			       argv[argIndex],
			       CFHT_ARGS_SIZE - StringIndex);
		argsString[CFHT_ARGS_SIZE] = '\0';
		StringIndex += strlen(argv[argIndex]);
		if (StringIndex < CFHT_ARGS_SIZE) {
		    argsString[StringIndex++] = '"';
		    argsString[StringIndex] = '\0';
		}
	    }
	}
	argIndex++;
    }
    return(argsString);
}
