/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_awake.c
* $Header: cfht_awake.c,v 1.3 96/11/19 22:39:32 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_awake()
*    This routine will cause a SIGALRM "when_from_now" seconds from now
*    and every "how_often" seconds from then on.  Can be cleared by
*    re-issuing with (0.0, 0.0) as arguments.
*
*    INVOKED
*    cfht_awake(when_from_now, how_often);
*
*    PARAMETERS IN
*    double when_from_now - delay before first alarm in seconds
*    double how_often - interval between alarms in seconds
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    NONE
*
*    GLOBALS
*    NONE
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* sss          890322          original coding
* jrw          20 Dec 1990     sun port
*
* $Log:	cfht_awake.c,v $
 * Revision 1.3  96/11/19  22:39:32  isani
 * cleaned up rcsid
 * 
 * Revision 1.2  1994/12/14  10:52:43  thomas
 * Added #ifdef __LINT__ around RCS header
 *
 * Revision 1.1  91/09/24  16:08:25  16:08:25  steve (Steven Smith)
 * Initial revision
 * 
*
****************************************************************************!*/

#include <sys/time.h>

#include "cfht.h"

RCSID("$Header: cfht_awake.c,v 1.3 96/11/19 22:39:32 isani Exp $");

#define     USEC_SEC  1.0e6  /* microseconds per second */

void cfht_awake(when_from_now,how_often)
double when_from_now;        /* seconds from now to cause first sig */
double how_often;            /* interval between 1st,2nd,...  */
{
    struct itimerval val, oval;

    /* set up first time */
    val.it_value.tv_sec = (long)when_from_now; /* integer part */
    val.it_value.tv_usec = (when_from_now - val.it_value.tv_sec) * USEC_SEC;

    /* set up from then on */
    val.it_interval.tv_sec = (long)how_often;
    val.it_interval.tv_usec = (how_often - val.it_interval.tv_sec) * USEC_SEC;

    setitimer(ITIMER_REAL, &val, &oval);
}
