/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_basename.c
* $Header: cfht_basename.c,v 1.5 96/11/19 22:39:59 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_basename()
*    Convert from a file name to the basename ala cfht
*
*    INVOKED
*        result = cfht_basename (dest, filename, suffix);
*
*    PARAMETERS IN
*        char *filename - some filename
*        char *suffix - some suffix
*
*    PARAMETERS OUT
*        char *dest - location for result or NULL
*
*    FUNCTION RETURNS
*        char * - pointer to result; if dest is NULL then points to internal
*                 static area which is reused each time a NULL is passed in
*
*    GLOBALS
*    ???, a static internal string area may be used
*
* HISTORY
* who        when           what
* -------    -----------    ----------------------------------------------
* sss        30 Oct 1990    Original coding
* sss        20 Dec 1990    added <stdio.h> for suns
* jrw        20 Dec 1990    sun port
* jrw        14 Feb 1991    check for null pointers
*
* $Log:	cfht_basename.c,v $
 * Revision 1.5  96/11/19  22:39:59  isani
 * cleaned up rcsid
 * changed a NULL to a `\0`
 * 
 * Revision 1.4  1994/12/14  10:57:44  thomas
 * Added #ifdef __LINT__ around RCS header
 * Took out include cfht.h (was only for strstr on sun)
 *
 * Revision 1.3  93/02/14  16:08:47  16:08:47  john (John Kerr)
 * *** empty log message ***
 * 
 * Revision 1.2  92/11/12  23:13:11  23:13:11  john (John Kerr)
 * correct suffix bug where routine nulled out the filename accidently
 * 
 * Revision 1.1  91/09/24  16:10:44  16:10:44  steve (Steven Smith)
 * Initial revision
 * 
*
****************************************************************************!*/

#include <string.h>       /* for string handling defines */
#include <stdio.h>        /* for NULL on suns */

#include "cfht.h"

RCSID("$Header: cfht_basename.c,v 1.5 96/11/19 22:39:59 isani Exp $");

static char cfht_basename_s[1025];

/*****************************************************************************
 *
 * cfht_basename
 *
 *****************************************************************************/

char *cfht_basename(dest, filename, suffix)
char *dest;        /* place to put the answer, if NULL use our space */
char *filename;
char *suffix;
{
    char *p, *q;
    int i;

    if (filename == NULL)
        return(NULL);

    if (dest == NULL)
        dest = cfht_basename_s;

    if(!(p=strrchr(filename, '/')))
        p = filename;
    else
	p++;

    if(*p == '.' && *(p+1) == ',')
        p += 2;

    if (suffix == NULL) 
	q = p + strlen(p);
    else {
        if(!(q=strstr(p, suffix)))
            q = p + strlen(p);
    }

    for(i=0 ; p<q; p++, i++)
        dest[i] = *p;

    dest[i] = '\0';

    return(dest);
}
