/* Copyright (C) 1999-2000    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the     */
/* terms of the GNU General Public License, see the file COPYING          */
/*!****************************************************************************
 * FILE cfht_cfhthome.c
 * $Id: cfht_cfhthome.c,v 1.2 2000/08/15 06:37:01 thomas Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *  cfht_cfhthome()
 *    Return the path to the CFHT home directory, including the trailing / .
 *    This is "/usr/local/cfht/" or "/cfht" depending on era.
 *
 *   INVOKED
 *    CFHTHome = cfht_cfhthome();
 *
 *   PARAMETERS IN
 *    None
 *
 *   PARAMETERS OUT
 *    None
 *
 *   FUNCTION RETURNS
 *    char * - pointer to path name, in a static buffer (well it does not
 *             change very often :-)
 *
 *   GLOBALS
 *    None
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jnt          17-Feb-1999     original
 *
 * $Log: cfht_cfhthome.c,v $
 * Revision 1.2  2000/08/15 06:37:01  thomas
 * Added string.h
 *
 * Revision 1.1  2000/08/15 06:35:10  thomas
 * Initial revision
 *
 ***************************************************************************!*/

#include <string.h>		/* for strcpy */
#include <stdlib.h>		/* for getenv */

#include "cfht.h"		/* for env variable name and default path */

#define PathLength 255		/* room for path */

RCSID("$Id: cfht_cfhthome.c,v 1.2 2000/08/15 06:37:01 thomas Exp $");

char *cfht_cfhthome(void)
{
    char *EnvironmentValue;	  /* pointer to value in environment */
    static char CFHTHome[PathLength]; /* the result */

    if ((EnvironmentValue = getenv(CFHT_ECFHTPATH)) == (char *) NULL) {
	/* note the hope that strlen(CFHT_CFHTPATH) < 256 :-) */
	(void) strcpy(CFHTHome, CFHT_CFHTPATH);
    } else {
	if (strlen(EnvironmentValue) > ((sizeof CFHTHome) - 1)) {
	    (void) strcpy(CFHTHome, "Bogus CFHT home path value in env");
	} else {
	    (void) strcpy(CFHTHome, EnvironmentValue);
	}
    }
    (void) strcat(CFHTHome, "/");
    return CFHTHome;
}
