/* Copyright (C) 1991, 1994, 1996   Canada-France-Hawaii Telescope Corp. */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
 * FILE
 * cfht_date.c
 * $Header: /cfht/src/pegasus/libcfht/RCS/cfht_date.c,v 1.7 2002/05/06 04:42:02 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_date()
 *    This routine will return a string sutable for local ascii date
 *    stamping.  If the incoming pointer is NULL, the current time as
 *    returned by time(2) is used.  The return area is static and will
 *    be changed each call.
 *
 *    INVOKED
 *    result_string = cfht_date(clock_pointer);
 *
 *    PARAMETERS IN
 *    double *clockp - points to number of seconds since epoch (1970)
 *
 *    PARAMETERS OUT
 *    NONE
 *
 *    FUNCTION RETURNS
 *    pointer to string containing date
 *
 *    GLOBALS
 *    NONE, although the string area is static
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * sss          880412          original coding
 *
 * $Log: cfht_date.c,v $
 * Revision 1.7  2002/05/06 04:42:02  isani
 * Fixed problem with POSIX-style values of TZ like "Europe/Paris" that was
 *   causing cfht_log to crash.  The new way lets the C library handle time
 *   zone and does not rely on the TZ variable.  E_TZONE defines in the other
 *   cfht*date.c files are unused, by the way.  They can be removed too.
 *
 * Revision 1.6  1996/11/19 22:42:17  isani
 * cleaned up rcsid
 * moved sun hack to cfht.h
 *
 * Revision 1.5  1996/03/13  17:04:55  thomas
 * Corrected type in cfht_tok call added in 1.4 :-(
 *
 * Revision 1.4  96/02/12  09:54:39  09:54:39  thomas (Jim Thomas)
 * Added HST as the default time zone if TZ not set
 * 
 * Revision 1.3  94/12/14  11:07:06  11:07:06  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS header
 * Include stdlib.h always
 * 
 * Revision 1.2  94/09/20  18:56:13  18:56:13  thomas (Jim Thomas)
 * use symbol for date string size
 * 
 * Revision 1.1  91/09/24  16:12:30  16:12:30  steve (Steven Smith)
 * Initial revision
 * 
 ***************************************************************************!*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"

RCSID("$Header: /cfht/src/pegasus/libcfht/RCS/cfht_date.c,v 1.7 2002/05/06 04:42:02 isani Exp $");

char *cfht_date(clockp)
    long *clockp;        /* pointer to long that has seconds since 1970 */
{
    long t;
    struct tm *tmd;
    static char fmt[CFHT_DATE_SIZE]; /* intermediate format string */
    static char rtn[CFHT_DATE_SIZE];

    if(clockp == (long *) NULL) { /* if not passed in - get */
	clockp = &t;
	time(clockp);
    }
    tzset(); /* should not be needed, but will not hurt */
    tmd = localtime(clockp);

    /*
     * Instead of %%d and %%2d, %e and %H could have been used, but
     * it would change the spacing of the time strings in the logs
     * slightly.  Doing it this way just in case something relies
     * on the exact formatting of the timestamp to be safe.
     */
    strftime(fmt, CFHT_DATE_SIZE, "%a %b %%d %%2d:%M:%S %Z %Y", tmd);
    sprintf(rtn, fmt, tmd->tm_mday, tmd->tm_hour);
    
    return(rtn);
}
