/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_dtime.c
* $Header: cfht_dtime.c,v 1.3 96/11/19 22:45:47 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_dtime()
*     cfht_dtime uses the gettimeofday() to return number of seconds since
*     the epoch (see gettimeofday(2)).  The resolution of this routine is
*     governed by the granularity of the gettimeofday() implementation on
*     a particular machine.
*
*    INVOKED
*    time_now = cfht_dtime();
*
*    PARAMETERS IN
*    NONE
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    double - number of seconds since epoch
*
*    GLOBALS
*    NONE
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* sss          890505          original
* jrw          20 Dec 1990     sun port
*
* $Log:	cfht_dtime.c,v $
 * Revision 1.3  96/11/19  22:45:47  isani
 * cleaned up rcsid
 * 
 * Revision 1.2  1994/12/14  11:09:16  thomas
 * Added #ifdef __LINT__ around RCS header
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  16:19:49  16:19:49  steve (Steven Smith)
 * Initial revision
 * 
*
****************************************************************************!*/

#include <sys/time.h>

#include "cfht.h"		  /* libcfht defines */

RCSID("$Header: cfht_dtime.c,v 1.3 96/11/19 22:45:47 isani Exp $");

double cfht_dtime()
{
    static struct timeval  now;
    static struct timezone tzp;

    gettimeofday (&now, &tzp);

#ifdef DEBUGDEBUG
    cfht_logv (CFHT_LIBCFHT, CFHT_DEBUG,
               "dtime, sec=%ld, usec=%ld\n", now.tv_sec, now.tv_usec);
#endif

    return(now.tv_sec + (now.tv_usec * 1e-6) );
}
