/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_esc.c
* $Header: cfht_esc.c,v 1.3 96/11/19 22:44:53 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_fre()
*    Takes a source string and replaces all occurences of C-style
*    escape sequences with the designated characters.  For example, the string
*    "hello, world\n" would have a newline substitution performed.  If the
*    destination pointer is NULL, then the substitution will be performed in
*    place on the source string.  Otherwise, the destination should point to
*    space already allocated.
*
*    INVOKED
*    translated_str = cfht_fre(source_str, dest_str, dest_size);
*
*    PARAMETERS IN
*    unsigned char *src - the source string
*
*    PARAMETERS OUT
*    unsigned char *dest - the translated string
*    int *dsize - the size of the translated string
*
*    FUNCTION RETURNS
*    char * - a pointer to the translated string
*
*    GLOBALS
*    NONE
*
* cfht_toe()
*    Takes a string and replaces the non-printing characters with
*    their equivalent octal code.  For example, newlines would be translated
*    to "\012" (four characters).  The argument dest must point to an area
*    already allocated and large enough to hold the resulting expansion.
*    Note that the null character has no special meaning, and will be
*    translated to "\000".
*
*    INVOKED
*    translated_str = cfht_toe(source_str, dest_str, source_size);
*
*    PARAMETERS IN
*    unsigned char *src - the source string
*    int ssize - the number of characters to translate
*
*    PARAMETERS OUT
*    unsigned char *dest - the translated string
*
*    FUNCTION RETURNS
*    char * - a pointer to the translated string
*
*    GLOBALS
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab          26 Aug 1988     Original coding
* jrw          14 Feb 1991     check for null pointers
*
* $Log:	cfht_esc.c,v $
 * Revision 1.3  96/11/19  22:44:53  isani
 * cleaned up rcsid's
 * removed "register" keywords
 * 
 * Revision 1.2  1994/12/14  11:10:21  thomas
 * Added #ifdef __LINT__ around RCS header
 * Added include of stdio.h
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  16:25:55  16:25:55  steve (Steven Smith)
 * Initial revision
 * 
*
****************************************************************************!*/

#include <stdio.h>		  /* for sprintf */
#include <ctype.h>		  /* for isgraph() */
#include "cfht.h"		  /* libcfht defines */

RCSID("$Header: cfht_esc.c,v 1.3 96/11/19 22:44:53 isani Exp $");


/*****************************************************************************
 *
 * cfht_fre
 *
 *****************************************************************************/

    char *                       /* return: destination, or NULL for failure */
cfht_fre(src, dest, dsize)
    unsigned char *src; /* starting string */
    unsigned char *dest;         /* place to put the answer */
    int *dsize;                  /* return: size of dest */
{
    int size;
    unsigned char c;
    unsigned char *curdest;
    int dummy;

    if (src == NULL)
        return(NULL);

    if (dest == NULL)
        dest = src;

    if (dsize == NULL)
        dsize = &dummy;
    
    curdest = dest;
    size = 0;
    while((c = *src++)) {
        size++;
        if (c != '\\') {
            *curdest++ = c;
        } else {
            c = *src++;
            switch (c) {
            case '\0': /* special case, end of string */
                *curdest++ = '\\';
                src--;
                break;
            case 'n':
                *curdest++ = '\n';
                break;
            case 's':
                *curdest++ = ' ';
                break;
            case 't':
                *curdest++ = '\t';
                break;
            case 'b':
                *curdest++ = '\b';
                break;
            case 'r':
                *curdest++ = '\r';
                break;
            case 'f':
                *curdest++ = '\f';
                break;
            case 'e':
                *curdest++ = '\033';
                break;
            default:
                if (c < '0' || c > '7') {
                    *curdest++ = c;
                } else {
                    int count;
                    unsigned char val;

                    val = 0;
                    count = 0;
                    do {
                        val <<= 3;
                        val += c - '0';
                        c = *src++;
                    } while (count++ < 2 && c >= '0' && c <= '7');
                    *curdest++ = val;
                    src--;
                }
                break;
            }
        }
    }

    *curdest = '\0';
    *dsize = size;
    return((char *)dest);
}

/*****************************************************************************
 *
 * cfht_toe
 *
 *****************************************************************************/

    char *        /* return: destination, or NULL for failure */
cfht_toe(src, dest, ssize)
    unsigned char *src;    /* starting string */
    unsigned char *dest;        /* place to put the answer */
    int ssize;                /* size of src */
{
    unsigned char *curdest;
    unsigned char *end;     /* first position outside of src */
    unsigned char c;        /* current char */

    if (!src || !dest)
        return(NULL);
    
    curdest = dest;
    if (ssize > 0)
        end = src + ssize;
    else
        end = src + sizeof(src);
    while (end > src) {
        if (isgraph((c = *src++))) {
            *curdest++ = c;
        } else {
            sprintf((char *)curdest, "\\%03o", c);
            curdest += 4;
        }
    }
    *curdest = '\0';
    return((char *)dest);
}
