/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_etimer.c
* $Header: cfht_etimer.c,v 1.3 96/11/19 22:44:21 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_etimer()
*    Uses the gettimeofday() call to measure elapsed time between resetting
*    of the timer.  It does not clear itself (see cfht_resetetimer(1))
*    automatically after invocation so that it can be used to get splits
*    at various checkpoints.  The resolution of this routine is governed by
*    the granularity of the gettimeofday() implementation on a particular
*    machine.
*
*    INVOKED
*    elapsed_time = cfht_etimer(reset);
*
*    PARAMETERS IN
*    BOOLEAN reset - TRUE return e.t. and reset, FALSE return e.t. only
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    double - number of microseconds since timer was reset
*
*    GLOBALS
*    NONE
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jk           881005          Original version
* jrw          20 Dec 1990     sun port
*
* $Log:	cfht_etimer.c,v $
 * Revision 1.3  96/11/19  22:44:21  isani
 * cleaned up rcsid
 * 
 * Revision 1.2  1994/12/14  11:10:58  thomas
 * Added #ifdef __LINT__ around RCS header
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  16:30:06  16:30:06  steve (Steven Smith)
 * Initial revision
 * 
*
****************************************************************************!*/

#include <sys/time.h>
#include "cfht.h"		  /* libcfht defines */

RCSID("$Header: cfht_etimer.c,v 1.3 96/11/19 22:44:21 isani Exp $");

double cfht_etimer (reset)
BOOLEAN reset;
{
    static struct timeval lapsedtime;
    static struct timeval resettime;
    static struct timezone tzp;
    double elapsedtime, sec, usec;


    /* Get current timer value and adjust it for usec turnover */

    gettimeofday (&lapsedtime, &tzp);

    if (resettime.tv_usec > lapsedtime.tv_usec) {
        lapsedtime.tv_usec += 1000000;
        lapsedtime.tv_sec--;
    }


    /* Calculate elapsed time since last invocation */

    usec = lapsedtime.tv_usec - resettime.tv_usec;
    sec  = lapsedtime.tv_sec  - resettime.tv_sec;


    /* Calculate elapsed time since last reset */

    elapsedtime = sec * 1000000 + usec ;


    if (reset == TRUE) {
        resettime.tv_sec = lapsedtime.tv_sec;
        resettime.tv_usec = lapsedtime.tv_usec;
    }

    return (elapsedtime);
}
