/*!****************************************************************************
* FILE
* $Header: cfht_goDaemon.c,v 1.8 96/11/19 22:48:42 isani Exp $
* $Locker:  $
*
* ROUTINES
*    cfht_goDaemon()
*        Disconnect from the world and become a daemon process.
*
*    INVOKED
*        cfht_goDaemon();
*
*    PARAMETERS IN
*        none
*
*    PARAMETERS OUT
*        none
*
*    FUNCTION RETURNS
*        none
*
*    GLOBALS
*        none
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
*  SSS          ???            Steve did this
*
* $Log:	cfht_goDaemon.c,v $
 * Revision 1.8  96/11/19  22:48:42  isani
 * cleaned up rcsid
 * changed an exit() in forked thread to _exit()
 * 
 * Revision 1.7  1994/12/14  11:31:16  thomas
 * Added #ifdef __LINT__ around RCS header
 * Added includes of string.h and errno.h
 * Changed cfht/cfht.h to cfht.h
 * Added exits.h and fixed exits to EXIT_*
 * Changed CFHT_ERRNO to CFHT_ERROR and CFHT_LOGONLY
 *
 * Revision 1.6  93/03/04  20:50:05  20:50:05  jwright (Jim Wright)
 * *** empty log message ***
 * 
 * Revision 1.5  1993/03/05  02:10:56  jwright
 * add proper error checking on fork()
 *
 * Revision 1.4  1993/02/17  17:02:03  jwright
 * fix to work correctly on both suns and hps.
 *
 * Revision 1.3  93/02/03  20:09:35  20:09:35  steve (Steven Smith)
 * hacked for sun to work, don't think it helped - oh well
 *
 * Revision 1.2  93/02/03  17:30:08  17:30:08  evans (Linda Evans)
 * checked in by SSS, left checkedout, and doesn't work
 *
 * Revision 1.1  92/11/09  13:10:23  13:10:23  john (John Kerr)
 * Initial revision
 *
*
****************************************************************************!*/

#include <fcntl.h>		  /* for open */
#include <unistd.h>		  /* for process control */
#include <string.h>		  /* for string routines */
#include <errno.h>		  /* for errno */

#include "cfht.h"		  /* libcfht defines */
#include "exits.h"		  /* exit codes */

RCSID("$Header: cfht_goDaemon.c,v 1.8 96/11/19 22:48:42 isani Exp $");

void 
cfht_goDaemon ()
{
    int i;
    int f;
    int status;

    /* become a child, make parent exit */
    status = fork();
    if (status < 0) {
	/* fork call failed, no child, we are parent */
        cfht_logv(CFHT_MAIN, CFHT_ERROR,
		  "(cfht_goDaemon:%d) can't fork new process for daemon (%s)",
		  __LINE__, strerror(errno));
	exit(EXIT_FAIL);
    } else if (status > 0) {
	/* we are the parent, bail out and let child continue */
	_exit(EXIT_PASS);
    }
    /* we are the child, keep on going */

    /* how many files to close? */
#if defined(hpux) || defined(__svr4__) || defined(__linux__)
    f = sysconf (_SC_OPEN_MAX);
#else
#ifdef sun
    f = getdtablesize ();
#else
    SHOW ME HOW TO GET MAX NUMBER OF OPEN FILES PER PROCESS
#endif
#endif

    /* disassociate from controlling terminal */
    if (setsid() < 0) {
        cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		  "(cfht_goDaemon:%d) can't disassociate from terminal (%s)",
		  __LINE__, strerror(errno));
        /* continue execution, hope for the best */
    }

    /* close all fd's */
    for (i = 0; i < f; i++)
        close (i);

    /* attach 0,1,2 to /dev/null */
    open ("/dev/null", O_RDONLY);
    dup2 (0, 1);
    dup2 (0, 2);
}
