/* Copyright (C) 1991-2001    Canada-France-Hawaii Telescope Corp.     */
/* This program is distributed WITHOUT any warranty, and is under the  */
/* terms of the GNU General Public License, see the file COPYING       */
/*!****************************************************************************
 * FILE
 * cfht_lock.c
 * $Id: cfht_lock.c,v 1.7 2001/10/14 20:16:34 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_checksem()
 *    Checks on a semaphore and returns TRUE if set, FALSE otherwise.
 *
 *    INVOKED
 *    sem_set = cfht_checksem(semid);
 *
 *    PARAMETERS IN
 *    int semid - semaphore to check
 *
 *    PARAMETERS OUT
 *    NONE
 *
 *    FUNCTION RETURNS
 *    BOOLEAN - TRUE if semaphore set, FALSE otherwise
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_initsem()
 *    Creates/connects to a semaphore for use
 *
 *    INVOKED
 *    semid = cfht_initsem(path, id);
 *
 *    PARAMETERS IN
 *    char *path - path to lock file (?)
 *    char id - identifier for this semaphore
 *
 *    PARAMETERS OUT
 *    None
 *
 *    FUNCTION RETURNS
 *    int - semaphore identifier or FAIL if problem
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_initnewsem()
 *    Creates to a semaphore for use, must not exist yet
 *
 *    INVOKED
 *    semid = cfht_initnewsem(path, id);
 *
 *    PARAMETERS IN
 *    char *path - path to lock file (?)
 *    char id - identifier for this semaphore
 *
 *    PARAMETERS OUT
 *    None
 *
 *    FUNCTION RETURNS
 *    int - semaphore identifier
 *          or FAIL if problem, including semaphore already exists
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_setsem()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_setsemwait()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_relsem()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_waitsemrel()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_waitsemset()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    the semaphore associated with semid
 *
 * cfht_checklock()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_initlock()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_setlock()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_setlockwait()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_rellock()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_waitlockrel()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * cfht_waitlockset()
 *    ???
 *
 *    INVOKED
 *    ???
 *
 *    PARAMETERS IN
 *    ???
 *
 *    PARAMETERS OUT
 *    ???
 *
 *    FUNCTION RETURNS
 *    ???
 *
 *    GLOBALS
 *    ???
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          17 Apr 1989     Original coding
 * jab          25 Apr 1989     Added lock-files as well as semaphores
 * jk           23 May 1991     sun ifdefs commented out
 *
 * $Log: cfht_lock.c,v $
 * Revision 1.7  2001/10/14 20:16:34  isani
 * Create CFHTLOCKS directory automatically so Makefile doesn't have to.
 *
 * Revision 1.6  2001/05/12  06:10:57  thomas
 * Added cfht_initnewsem()
 * 
 * Revision 1.5  2000/11/02 03:15:05  thomas
 * Changed loop in cfht_setlockwait to get around a "bug" (?) in SPARCe gcc or
 * VME interrupts or something
 *
 * Revision 1.4  2000/02/10 13:12:17  isani
 * Added type cast to avoid format string warning.
 *
 * Revision 1.3  1996/11/19 23:07:15  isani
 * cleaned up rcsid's
 * marked argument sigign as UNUSED
 *
 * Revision 1.2  1994/12/14  11:35:43  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Rearranged includes for sun versus hp
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  16:58:29  16:58:29  steve (Steven Smith)
 * Original checkin
 * 
 **************************************************************************!*/

#include <sys/types.h>		  /* for ipc.h */
#include <sys/ipc.h>		  /* for sem.h */
#include <sys/sem.h>		  /* for semaphore stuff */
#include <sys/stat.h>		  /* for umask */
#include <fcntl.h>		  /* for file open stuff */
#include <stdio.h>		  /* for sprintf() */
#include <unistd.h>		  /* file locking stuff */
#include <errno.h>		  /* for error codes and errno define */

#include "cfht.h"		  /* generic defines */

RCSID("$Id: cfht_lock.c,v 1.7 2001/10/14 20:16:34 isani Exp $");

/*****************************************************************************
 *
 * cfht_checksem
 *
 *****************************************************************************/

    BOOLEAN    /* return: true if sem is set */
cfht_checksem(semid)
    int semid;
{
    struct sembuf ops[1];

    ops[0].sem_num = 0;
    ops[0].sem_op = 0;
    ops[0].sem_flg = IPC_NOWAIT;

    if (semop(semid, ops, 1) == -1)
        return TRUE;

    return FALSE;
}

/*****************************************************************************
 *
 * cfht_initsem
 *
 *****************************************************************************/

    int         /* return: semid, or -1 */
cfht_initsem Q((char *path, char id), (path, id), char *path; char id;)
{
    key_t key;
    int semid;

    if ((key = ftok(path, id)) == -1)
        return FAIL;

    if ((semid = semget(key, 1, IPC_CREAT | 0666)) == -1)
        return FAIL;

    return semid;
}

/*****************************************************************************
 *
 * cfht_initnewsem
 *
 *****************************************************************************/

    int         /* return: semid, or -1 */
cfht_initnewsem Q((char *path, char id), (path, id), char *path; char id;)
{
    key_t key;
    int semid;

    if ((key = ftok(path, id)) == -1)
        return FAIL;

    if ((semid = semget(key, 1, IPC_CREAT | IPC_EXCL | 0666)) == -1)
        return FAIL;

    return semid;
}

/*****************************************************************************
 *
 * cfht_setsem
 *
 *****************************************************************************/

    PASSFAIL
cfht_setsem(semid)
    int semid;
{
    struct sembuf ops[2];

    ops[0].sem_num = 0;
    ops[0].sem_op = 0;
    ops[0].sem_flg = IPC_NOWAIT;
    ops[1].sem_num = 0;
    ops[1].sem_op = 1;
    ops[1].sem_flg = SEM_UNDO;

    if (semop(semid, ops, 2) == -1)
        return FAIL;

    return PASS;
}

/*****************************************************************************
 *
 * cfht_setsemwait
 *
 *****************************************************************************/

    PASSFAIL
cfht_setsemwait(semid, sigign)
    int semid;
    BOOLEAN sigign;
{
    int ret;
    struct sembuf ops[2];

    ops[0].sem_num = 0;
    ops[0].sem_op = 0;
    ops[0].sem_flg = 0;
    ops[1].sem_num = 0;
    ops[1].sem_op = 1;
    ops[1].sem_flg = SEM_UNDO;

    while (((ret = semop(semid, ops, 2)) == -1) && errno == EINTR && sigign)
        ;

    if (ret == -1)
        return FAIL;

    return PASS;
}

/*****************************************************************************
 *
 * cfht_relsem
 *
 *****************************************************************************/

    PASSFAIL
cfht_relsem(semid)
    int semid;
{
    struct sembuf ops[1];

    ops[0].sem_num = 0;
    ops[0].sem_op = -1;
    ops[0].sem_flg = SEM_UNDO | IPC_NOWAIT;

    if (semop(semid, ops, 1) == -1)
        return FAIL;

    return PASS;
}

/*****************************************************************************
 *
 * cfht_waitsemrel
 *
 *****************************************************************************/

    PASSFAIL
cfht_waitsemrel(semid, sigign)
    int semid;
    BOOLEAN sigign;
{
    int ret;
    struct sembuf ops[1];

    ops[0].sem_num = 0;
    ops[0].sem_op = 0;
    ops[0].sem_flg = 0;

    while (((ret = semop(semid, ops, 1)) == -1) && errno == EINTR && sigign)
        ;

    if (ret == -1)
        return FAIL;

    return PASS;
}

/*****************************************************************************
 *
 * cfht_waitsemset
 *
 *****************************************************************************/

    PASSFAIL
cfht_waitsemset(semid, sigign)
    int semid;
    BOOLEAN sigign;
{
    int ret;
    struct sembuf ops[2];

    ops[0].sem_num = 0;
    ops[0].sem_op = -1;
    ops[0].sem_flg = SEM_UNDO;
    ops[1].sem_num = 0;
    ops[1].sem_op = 1;
    ops[1].sem_flg = SEM_UNDO;

    while (((ret = semop(semid, ops, 2)) == -1) && errno == EINTR && sigign)
        ;

    if (ret == -1)
        return FAIL;

    return PASS;
}

    BOOLEAN    /* return: true if locked */
cfht_checklock(fd)
    int fd;
{
    if (lockf(fd, F_TEST, 0))
        return TRUE;

    return FALSE;
}

    int         /* return: fd, or -1 */
cfht_initlock Q((char *path, char id), (path, id), char *path; char id;)
{
    key_t key;
    char lfile[32];
    int fd;
    mode_t oldmask;

    if ((key = ftok(path, id)) == -1)
        return FAIL;

    oldmask = umask((mode_t)0);

    if (mkdir(CFHTLOCKS, 0777) == 0)
      chmod(CFHTLOCKS, 0777);
    /*
     * It is normal for CFHTLOCKS to already exist.  Let the open
     * below fail if there is a real problem with the lock directory.
     */

    sprintf(lfile, "%s/lk%d", CFHTLOCKS, (int)key);
    if ((fd = open(lfile, O_RDWR | O_CREAT, 0666)) == -1) {
        umask(oldmask);
        return FAIL;
    }

    umask(oldmask);
    return fd;
}

    PASSFAIL
cfht_setlock(fd)
    int fd;
{
    if (lockf(fd, F_TLOCK, 0))
        return FAIL;

    return PASS;
}

    PASSFAIL
cfht_setlockwait(fd, sigign)
    int fd;
    BOOLEAN sigign;
{
    int ret;

/*
 * this was the original code
 * but it tickled a bug in some interrupt code (I'm guessing) that clobbered
 * a local register on the SPARCe's - so I rewrote it below to change the
 * generated code enough to make it work.
 * (you don't want to know :-)
    while (((ret = lockf(fd, F_LOCK, 0)) == -1) &&
            (errno == EAGAIN || errno == EACCES || (errno == EINTR && sigign)))
        ;
*/
    BOOLEAN KeepGoing = TRUE;
    while (KeepGoing == TRUE) {
	ret = lockf(fd, F_LOCK, 0);
	if (ret >= 0) {
	    KeepGoing = FALSE;
	    continue;
	}
	if (errno == EAGAIN || errno == EACCES || (errno == EINTR && sigign)) {
	    ;
	} else {
	    KeepGoing = FALSE;
	    continue;
	}
    }

    if (ret == -1)
        return FAIL;

    return PASS;
}

    PASSFAIL
cfht_rellock(fd)
    int fd;
{
    if (lockf(fd, F_ULOCK, 0))
        return FAIL;

    return PASS;
}

    PASSFAIL
cfht_waitlockrel(fd, sigign)
    int fd;
    BOOLEAN sigign;
{
    int ret;

    while (((ret = lockf(fd, F_LOCK, 0)) == -1) &&
            (errno == EAGAIN || errno == EACCES || (errno == EINTR && sigign)))
        ;

    if (ret == -1)
        return FAIL;

    lockf(fd, F_ULOCK, 0);
    return PASS;
}

/*ARGSUSED*/
    PASSFAIL
cfht_waitlockset(fd, sigign)
    int fd;
    BOOLEAN sigign UNUSED;
{
    while (1) {
        if (lockf(fd, F_TEST, 0))
            return PASS;
        else
            sleep(1);
    }
}
