/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_prealloc.c
* $Header: cfht_prealloc.c,v 1.3 96/11/19 23:10:53 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* prealloc()
*    Preallocate disk space.  Emulation of system routine on HP.
*
*    INVOKED
*    status = prealloc (fildes, size);
*
*    PARAMETERS IN
*    int fildes - file descriptor obtained from creat, pen, dup, or fcntl
*    unsigned int size - number of bytes to preallocate
*
*    PARAMETERS OUT
*    none
*
*    FUNCTION RETURNS
*    int - 0 for success, otherwise -1 with errno set
*
*    GLOBALS
*    none
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jrw          25 Feb 1991     original, for sun port
*
* $Log:	cfht_prealloc.c,v $
 * Revision 1.3  96/11/19  23:10:53  isani
 * cleaned up rcsid
 * changed ifdef sun to ifndef hpux
 * 
 * Revision 1.2  1994/12/14  12:03:06  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Added dummy variable on HP so ar and purify work
 *
 * Revision 1.1  91/09/24  17:01:37  17:01:37  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

/*** From man page:

          Fildes is a file descriptor obtained from a creat, pen, dup,
          or fcntl system call for an ordinary file of zero length. It
          must be opened writable, since it will be written to by
          prealloc.  Size is the size in bytes to be preallocated for
          the file specified by fildes.  At least size bytes will be
          allocated.  space is be allocated in an implementation-
          dependent fashion for fast sequential reads and writes. The
          EOF in an extended file will be left at the end of the
          preallocated area.  The current file pointer is left at
          zero.  The file is zero-filled.

Note that in my simple-minded implementation, no cleanup is performed
if the full allocation cannot be satisfied.

***/

#include "cfht.h"

RCSID("$Header: cfht_prealloc.c,v 1.3 96/11/19 23:10:53 isani Exp $");

#ifndef hpux

#include <unistd.h>
#include <sys/types.h>
#include <sys/file.h>

int prealloc (fildes, size)
    int fildes;
    unsigned size;
{
    unsigned i;
    char buf[1];

    buf[0] = '\0';
    if (lseek(fildes, 0L, L_SET) != 0L)
        return (-1);
    for (i=0 ; i<size ; i++)
        if (write(fildes, buf, 1) != 1)
            return (-1);
    if (lseek(fildes, 0L, L_SET) != 0L)
        return (-1);
    return (0);
}

#endif /* !hpux */
