/* Copyright (C) 1991-1999    Canada-France-Hawaii Telescope Corp.       */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
 * FILE
 * cfht_sleep.c
 * $Id: cfht_sleep.c,v 1.6 2000/02/10 13:11:37 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 * cfht_sleep()
 *    This routine will suspend execution for a specified number of
 *    milliseconds.  It uses the select() routine to delay in a
 *    multi-tasking friendly manner, and yet still allow interruptions
 *    due to signals.
 *
 *    INVOKED
 *    cfht_sleep(msec, igsig);
 *
 *    PARAMETERS IN
 *    int msec - number of milliseconds to sleep
 *    int igsig - 0 to return early if a signal causes an interrupt,
 *                or 1 to ignore all signals
 *
 *    PARAMETERS OUT
 *    none
 *
 *    FUNCTION RETURNS
 *    int - 0 for success or -1 for early return
 *
 *    GLOBALS
 *    none
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          12 May 1989     Original coding
 * jab          13 Jun 1989     Rewritten to use select instead of SIGALARM
 * jrw          20 Dec 1990     sun port
 *
 * $Log:	cfht_sleep.c,v $
 * Revision 1.6  2000/02/10  13:11:37  isani
 * Changed to use new SELECT() macro.
 * 
 * Revision 1.5  1999/07/14 03:49:41  thomas
 * Fix select for 10.20
 *
 * Revision 1.4  1996/11/22 02:40:41  isani
 * fixed if statement that checked errno after select() call to some more logical
 *   it now says: "if we're not ignoring signals, OR, the errno wasn't even due
 *                 to a signal (some other rare strangeness occurred) then exit"
 *
 * Revision 1.3  96/11/19  23:11:06  isani
 * cleaned up rcsid
 * 
 * Revision 1.2  1994/12/14  12:03:43  thomas
 * Added #ifdef __LINT__ around RCS headers
 *
 * Revision 1.1  91/09/24  17:02:08  17:02:08  steve (Steven Smith)
 * Original checkin
 * 
 **************************************************************************!*/

#include <sys/time.h>
#include <errno.h>

#include "cfht.h"

RCSID("$Id: cfht_sleep.c,v 1.6 2000/02/10 13:11:37 isani Exp $");

    int        /* return: 0 success, -1 early return */
cfht_sleep(msec, igsig)
    int msec;
    int igsig;
{
    struct timeval starttime, curtime;
    struct timezone tz;
    int elapsed;

    gettimeofday(&starttime, &tz);
    elapsed = 0;
    while (elapsed < msec) {
        curtime.tv_sec = (msec - elapsed) / 1000;
        curtime.tv_usec = ((msec - elapsed) % 1000) * 1000;
        if (SELECT(0, 0, 0, 0, &curtime) == -1)
            if (!igsig || errno != EINTR)
                return(-1);
        gettimeofday(&curtime, &tz);
        if (curtime.tv_usec < starttime.tv_usec) {
            curtime.tv_usec += 1000000;
            curtime.tv_sec -= 1;
        }
        elapsed = (curtime.tv_usec - starttime.tv_usec) / 1000 +
                  (curtime.tv_sec - starttime.tv_sec) * 1000;
    }

    return(0);
}
