/* Copyright (C) 1998-1999    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the     */
/* terms of the GNU General Public License, see the file COPYING          */
/*!****************************************************************************
 * FILE cfht_system.c
 * $Header: cfht_system.c,v 1.4 2000/02/10 13:18:05 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *  cfht_system()
 *    Execute a program using a given argument string, using the system(3)
 *    call.
 *
 *   INVOKED
 *    Status = cfht_system(Command);
 *
 *   PARAMETERS IN
 *    char *Command - command string to be executed
 *
 *   PARAMETERS OUT
 *    None
 *
 *   FUNCTION RETURNS
 *    int - exit code returned by program (shell version, i.e., 0-127)
 *       or FAIL if program did not run
 *
 *   GLOBALS
 *    None
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jnt          10-Jul-1998     stolen from multiple copies in AOB code, etc.
 *
 * $Log:	cfht_system.c,v $
 * Revision 1.4  2000/02/10  13:18:05  isani
 * Added missing sterror.h include directive.
 * 
 * Revision 1.3  1999/03/05 17:37:37  thomas
 * Improved messages on error conditions
 *
 * Revision 1.2  98/08/25  18:09:16  18:09:16  thomas (Jim Thomas)
 * Fix documentation
 * 
 * Revision 1.1  98/07/16  15:54:17  15:54:17  thomas (Jim Thomas)
 * Original check in
 * 
 ***************************************************************************!*/

#include <stdlib.h>		  /* for system */
#include <string.h>		  /* for strerror */
#include <errno.h>		  /* for error codes and errno */
#include <sys/wait.h>		  /* for system return ? */

#include "cfht.h"
#include "exits.h"

RCSID("$Header: cfht_system.c,v 1.4 2000/02/10 13:18:05 isani Exp $");

int cfht_system(Command)
    char *Command;
{
    int systemError;		  /* return from system command */
    int ExitCode;

    systemError = system(Command);
    if (systemError < 0) {
	/* fork failed */
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_system:%d) failed to run '%.100s' (%s)",
		  __LINE__, Command, strerror(errno));
	return FAIL;
    }
    if (WIFEXITED(systemError) == 0) {
	/* something else broke */
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_system:%d) failed running '%s' status %#0.8x",
		  __LINE__, Command, systemError);
	if (WIFSIGNALED(systemError) != 0) {
	    cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		      "           because of signal %d, %s core",
		      __LINE__, WTERMSIG(systemError),
		      (WCOREDUMP(systemError) == 0) ? "no" : "with");
	}
	if (WIFSTOPPED(systemError) != 0) {
	    cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		      "           stopped on signal %d",
		      __LINE__, WSTOPSIG(systemError));
	}
	return FAIL;
    }
    if ((ExitCode = WEXITSTATUS(systemError)) == EXIT_PASS) {
	/* OK */
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_system:%d) sent '%s'",
		  __LINE__, Command);
    } else {
	/* failed :-( */
	cfht_logv(CFHT_LIBCFHT, CFHT_LOGONLY,
		  "(cfht_system:%d) command '%s' failed, status %#0.8x exit code %d",
		  __LINE__, Command, systemError, ExitCode);
    }

    return ExitCode;
}
