/* Copyright (C) 1991, 1944    Canada-France-Hawaii Telescope Corp.      */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_time.c
* $Header: /cfht/src/pegasus/libcfht/RCS/cfht_time.c,v 1.7 2001/10/14 20:18:54 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_time()
*    This routine will return a string suitable for local ascii time
*    stamping.  If the incoming pointer is NULL, the current time as
*    returned by time(2) is used.  The return area is static and will
*    be changed each call.
*
*    INVOKED
*    timestring = cfht_time(clockp);
*
*    PARAMETERS IN
*    long *clockp - pointer to long that has seconds since 1970 or
*                   NULL to use current time
*
*    PARAMETERS OUT
*    none
*
*    FUNCTION RETURNS
*
*    GLOBALS
*    static char rtn[12] - the local area used for constructing the
*                          return string, reused on each call
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* sss          880412          original coding
*
* $Log: cfht_time.c,v $
* Revision 1.7  2001/10/14 20:18:54  isani
* Include time.h
*
 * Revision 1.6  96/11/19  23:12:04  isani
 * cleaned up rcsid
 * changed a "%d" format string to "%ld" to match argument
 * 
 * Revision 1.5  1994/12/14  12:07:18  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Fixed type casts for clockp
 *
 * Revision 1.4  94/09/20  18:58:46  18:58:46  thomas (Jim Thomas)
 * use symbol for time string size
 * make it long enough to hold previously added tenths of seconds
 * 
 * Revision 1.3  94/01/20  02:10:14  02:10:14  jwright (Jim Wright)
 * minor reformat
 * 
 * Revision 1.2  93/06/29  15:47:30  15:47:30  steve (Steven Smith)
 * added fractions of seconds to time value
 * 
 * Revision 1.1  91/09/24  17:03:16  17:03:16  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>

#include "cfht.h"

RCSID("$Header: /cfht/src/pegasus/libcfht/RCS/cfht_time.c,v 1.7 2001/10/14 20:18:54 isani Exp $");

char *cfht_time(clockp)
long *clockp;        /* pointer to long that has seconds since 1970 */
{
    long hsec;
    struct tm *tmd;
/* JT mod 940919 - field size was too short with added .00 sec */
    static char rtn[CFHT_TIME_SIZE];
/* end mod 940919 - field size was too short with added .00 sec */
    struct timezone tzp;
    static struct timeval tv;

    if (clockp == (long *) NULL) { /* if not passed in - get */
	gettimeofday (&tv, &tzp);
        clockp = (long *) &(tv.tv_sec);
    } else {
	tv.tv_usec = 0;		/* passed in seconds have no fractions */
    }
    hsec = tv.tv_usec / 10000;
    tmd = localtime(clockp);
    sprintf (rtn, "%2d:%02d:%02d.%02ld",
	     tmd->tm_hour, tmd->tm_min, tmd->tm_sec, hsec);
    
    return(rtn);
}
