/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_tob.c
* $Header: cfht_tob.c,v 1.4 99/09/29 08:40:15 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_tob()
*    This routine takes a string, and returns the string's boolean value.
*    In addition to the usual "TRUE"/"FALSE" and "YES"/"NO" pairs, many
*    other possibilities are handled.
*    The following strings return TRUE:
*        "True"        "Yes"        "Up"        "Automatic"
*        "Enable"      "In"         "1"         "ON"
*        "OPen"        "COlumn"
*    Case is not significant in the tests.  In the above strings, uppercase
*    is used to denote the characters used in determining if a string is
*    TRUE.  Any string whose first characters do not match one of the above
*    is considered FALSE.
*
*    INVOKED
*    bool = cfht_tob(string);
*
*    PARAMETERS IN
*    char *string - the string to test for boolean value
*
*    PARAMETERS OUT
*    none
*
*    FUNCTION RETURNS
*    BOOLEAN - the sense of the string
*
*    GLOBALS
*    none
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab           7 Mar 1988     Original coding
* jab          21 Mar 1988     Added IN/OUT
*
* $Log:	cfht_tob.c,v $
 * Revision 1.4  99/09/29  08:40:15  isani
 * Fixed typo in "Up" case.
 * Added break's to O and C cases to fix logic.
 * 
 * Revision 1.3  1996/11/19  23:12:15  isani
 * cleaned up rcsid
 *
 * Revision 1.2  1994/12/14  12:08:18  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  17:03:41  17:03:41  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <string.h>		  /* for string handling defines */
#include <errno.h>		  /* for error codes and errno define */

#include "cfht.h"		  /* libcfht defines */

RCSID("$Header: cfht_tob.c,v 1.4 99/09/29 08:40:15 isani Exp $");

/*****************************************************************************
 *
 * cfht_tob
 *
 *****************************************************************************/

BOOLEAN          /* return: TRUE, or FALSE */
cfht_tob(string)
    char *string;    /* string to convert */
{
    int len;

    if ((!string) || ((len = strlen(string)) < 1))
        return FALSE;

    switch (string[0]) {
    case 'U': case 'u':        /* Up */
    case 'A': case 'a':        /* Automatic */
    case 'E': case 'e':        /* Enable */
    case 'I': case 'i':        /* In */
    case 'T': case 't':        /* True */
    case 'Y': case 'y':        /* Yes */
    case '1':            /* 'C' boolean */
        return TRUE;
	/*NOTREACHED*/
        break;
    case 'O': case 'o':        /* On, or In/Out, Open */
        if (len < 2)
            return FALSE;
        switch(string[1]) {
        case 'P': case 'p':    /* Open */
        case 'N': case 'n':    /* On */
            return TRUE;
        }
        return FALSE;
	break;
    case 'C': case 'c':        /* Column or Close */
        if (len < 2)
            return FALSE;
        switch(string[1]) {
        case 'O': case 'o':    /* Column */
            return TRUE;
        }
	return FALSE;
	break;
    }
    return FALSE;
}
