/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_tok.c
* $Header: cfht_tok.c,v 1.3 96/11/19 23:12:24 isani Exp $
* $Locker:  $
*
* ROUTINES
*
* cfht_tok()
*    This routine is used to break a string into its component tokens.
*
*    INVOKED
*    token = cfht_tok(string, sep, tail);
*
*    PARAMETERS IN
*    char *string - source string
*    char *sep - string of separator characters
*
*    PARAMETERS OUT
*    char *string - source string, contents of string will be modified
*    char **tail - string to use in next call
*
*    FUNCTION RETURNS
*    char * - next token found, or NULL
*
*    GLOBALS
*    ???
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab           5 Aug 1988     Original coding
* jrw          15 Feb 1991     check for null pointers
*
* $Log:	cfht_tok.c,v $
 * Revision 1.3  96/11/19  23:12:24  isani
 * cleaned up rcsid
 * 
 * Revision 1.2  1994/12/14  12:08:54  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h
 *
 * Revision 1.1  91/09/24  17:04:58  17:04:58  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <string.h>		  /* for string handling defines */
#include <errno.h>		  /* for error codes and errno define */

#include "cfht.h"		  /* libcfht defines */

RCSID("$Header: cfht_tok.c,v 1.3 96/11/19 23:12:24 isani Exp $");

/*****************************************************************************
 *
 * cfht_tok
 *
 *****************************************************************************/

    char *           /* return: next token, or NULL */
cfht_tok(string, sep, tail)
    char *string;    /* source string, will be modified */
    char *sep;       /* separators */
    char **tail;     /* return: next string to call with */
{
    char *dummy;
    int size;

    /* initialize tail */
    if (!tail)
        tail = &dummy;
    *tail = NULL;

    /* check validity of input data */
    if (!string)
        return(NULL);

    if (!sep)
        return(string);

    /* find start of current token */
    string += strspn(string, sep);

    /* find end of current token */
    if (*string == '\'') {       /* quoted strings */
        string++;
        size = strcspn(string, "'");
    } else {                     /* normal strings */
        size = strcspn(string, sep);
    }

    if (size <= 0)
        return(NULL);

    /* chop off and pass back tail if appropriate */
    if (string[size]) {
        string[size] = '\0';
        *tail = string + size + 1;
    }

    return(string);
}
