/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfht_util.c
* $Header: cfht_util.c,v 1.3 96/11/19 23:12:57 isani Exp $
* $Locker:  $
*
* Various support routines for the generic library.
*
* ROUTINES
*
* usage()
*    Prints a usage message on file descriptor 2 (stderr).  After printing
*    the message, exits from program with status -1.  Control does not return to
*    calling routine.
*
*    INVOKED
*    usage(msg);
*
*    PARAMETERS IN
*    char **msg - pointer to array of strings, last string is empty
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    (function does not return)
*
*    GLOBALS
*    NONE
*
* getcurdeath()
*    This routine will print a message on the log, stating that getcur()
*    failed.  It then exits with status -1.  Control does not return to
*    calling routine.
*
*    INVOKED
*    getcurdeath(reason_str);
*
*    PARAMETERS IN
*    char *what - string on which getcur() failed
*
*    PARAMETERS OUT
*    NONE
*
*    FUNCTION RETURNS
*    (function does not return)
*
*    GLOBALS
*    NONE
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jrw          01 May 1991     took usage and getcurdeath out of cfht_genh,
*                              made usage function take an argument
*
* $Log:	cfht_util.c,v $
 * Revision 1.3  96/11/19  23:12:57  isani
 * cleaned up rcsid
 * added parens around assignment used as truth statement in for loop
 * 
 * Revision 1.2  1994/12/14  12:12:31  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/exits.h to exits.h
 * Changed CFHT_FATAL to CFHT_ERROR
 * Added error message for what null in getcurdeath
 *
 * Revision 1.1  91/09/24  17:05:48  17:05:48  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <stdio.h>
#include <string.h>		  /* for string routines */
#include <unistd.h>		  /* write() */
#include "cfht.h"		  /* libcfht defines */
#include "exits.h"		  /* standard exit codes */

RCSID("$Header: cfht_util.c,v 1.3 96/11/19 23:12:57 isani Exp $");

/***************************************************************************
 *
 * usage
 *
 * This routine will print a usage message to standard error.
 * The message is extracted from an array of strings
 * terminated with "" or NULL.
 *
 ***************************************************************************/

void usage(msg)
char **msg;
{
    int line;          /* line number and index into msg */
    unsigned int n;    /* number of characters in current line */

    if (msg && *msg)
        for (line = 0; (n = strlen(msg[line])); line++)
            write(2, msg[line], n);
    exit(EXIT_FAIL);
}

/***************************************************************************
 *
 * getcurdeath
 *
 ***************************************************************************/

void getcurdeath(what)
char *what;
{
    if (what)
        cfht_logv(CFHT_MAIN, CFHT_ERROR,
		  "(cfht_util(getcurdeath):%d) getcur failed on '%s', exiting",
		  __LINE__, what);
    else
        cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		  "(cfht_util(getcurdeath):%d) getcur failed on ?, exiting",
		__LINE__);
    
    exit(EXIT_FAIL);
}
