/* Copyright (C) 1991-2001    Canada-France-Hawaii Telescope Corp.       */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING         */
/*****************************************************************************
 * FILE cfp_file.c
 * $Id: cfp_file.c,v 2.14 2001/08/23 03:50:59 thomas Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *   cfp_file - search for and return the full path to a par file
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab           8 Mar 1988     Original coding
 * sss           8 Apr 1988     added copy of default file to home dir
 * sss          11 Apr 1988     if .home older than default copy anyway
 * sss          14 Apr 1988     cp only if mode is "w", also ., prefix
 * jab           7 Jul 1988     add environment overide and global conf dir
 * jrw          15 Feb 1991     check for null pointers
 *
 * $Log: cfp_file.c,v $
 * Revision 2.14  2001/08/23 03:50:59  thomas
 * Removed dev/ from all dev/conf references
 *
 * Revision 2.13  1999/08/23 22:09:58  thomas
 * Moved unlink's from DEBUG to DEBUGCPP
 *
 * Revision 2.12  1999/08/05 01:06:28  thomas
 * Fix strcat -> strcpy in one path of interactive copy
 *
 * Revision 2.11  1999/04/29 17:23:46  thomas
 * Changed cpp execution to find it and use absolute path
 *
 * Revision 2.10  99/01/11  10:56:31  10:56:31  thomas (Jim Thomas)
 * Added try for $USER if $LOGNAME fails, don't do $Session if ReadFixed
 * 
 * Revision 2.9  98/12/17  18:41:55  18:41:55  thomas (Jim Thomas)
 * Don't ask user for session on CFP_READFIXED files
 * 
 * Revision 2.8  98/12/17  03:46:55  03:46:55  thomas (Jim Thomas)
 * Added CFP_READFIXED processing for .lod .state ...
 * 
 * Revision 2.7  98/12/16  11:07:07  11:07:07  thomas (Jim Thomas)
 * Changed $USER to $LOGNAME
 * 
 * Revision 2.6  98/12/15  13:35:04  13:35:04  thomas (Jim Thomas)
 * Keep going even if no $Session variable
 * 
 * Revision 2.5  98/12/15  10:06:15  10:06:15  thomas (Jim Thomas)
 * Changed search logic to use . $HOME SessionHome dev/conf
 * Added run through cpp for preprocessing
 * 
 * Revision 2.4  96/11/19  23:14:53  23:14:53  isani (Sidik Isani)
 * cleaned up rcsid
 * changed filemode from int to unsigned
 * added parens around assignment used as truth statement
 *
 * Revision 2.3  1996/03/14  09:24:36  thomas
 * Print errno on failure in current directory
 *
 * Revision 2.2  94/12/22  17:02:57  17:02:57  thomas (Jim Thomas)
 * Added more DEBUG messages while looking for file
 *
 * Revision 2.1  94/12/14  12:44:35  12:44:35  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Removed include of sys/types.h
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h
 * Changed CFHT_ERROR to CFHT_LOGONLY, made debug messages give more info
 *
 * Revision 1.1  91/09/24  17:07:43  17:07:43  steve (Steven Smith)
 * Original checkin
 *
 ***************************************************************************/

#include <stdio.h>		  /* for BUFSIZ */
#include <fcntl.h>		  /* for O_RDWR */
#include <string.h>		  /* for string handling defines */
#include <unistd.h>		  /* close(), read(), write(), unlink() */
#include <sys/stat.h>		  /* for stat() */
#include <errno.h>		  /* for error codes and errno define */
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* file parser defines */
#include "cfp_internal.h"	  /*  including internals */
#include "exits.h"		  /* shouldn't be here, but we exit on input */

#define HomePrefix ".,"		  /* par file prefix when in $HOME */
#define HomeDirectoryPrefix "/h/" /* where we have home directories */
#define cppName "cpp"		  /* well, duh :-) */
#define cppInputExtension ".c"	  /* tags for cpp input and output files */
#define cppOutputExtension ".cppOut" /*  (lie for input so c does it) */
#define FileMode 0664		  /* mode for output files */
#define cppCharacter '@'	  /* lead character for hidden cpp commands */

/*
 * We want to use cpp so we don't have to write our own processor.  m4 has too
 * many reserved words, e.g. "shift", "index", to deal with hiding.  So the
 * following hacks are done to the defining par file before passing it to cpp
 * .  First cpp outputs a blank line in place of any line it's left out.  We
 * want to kill those but keep any that were in the original file.  These are
 * marked by %b .  Second cpp barfs on unmatched apostrophes.  Any apostrophes
 * in the original file are changed to %a .  Third cpp does not do
 * substitutions within quoted strings (this is true for apostrophed
 * characters too).  Any quotes in the original file are changed to %q .  So
 * of course any % in the original file is changed to %% .
 * These changes are made to preprocessor lines as well as par file lines.
 * After running the file through cpp, we undo these changes.
 */
#define quoteProblem '"'
#define quoteSolution 'q'
#define apostropheProblem '\''
#define apostropheSolution 'a'
#define newlineSolution 'b'
#define SolutionQuote '%'

#define cfpHOME "cfpHOME="	  /* defines we generate to cpp */
#define cfpSession "cfpSession="
#define cfpSessionHOME "cfpSessionHOME="
#define cfpCFHTDir "cfpCFHTDir="
#define cfpWhichSession1 "cfp"
#define cfpWhichSession2 "Session"

RCSID("$Id: cfp_file.c,v 2.14 2001/08/23 03:50:59 thomas Exp $");

/*
 * cpp lives in strange places, often not in people's PATH's.
 * This table defines where to look.
 *
 * I tried using gcc's version, but it adds a gratuitous space after every
 * substitution it does, e.g. "cfpSession" becomes "mos " not "mos" :-(
 *
 * Note for debugging purposes that HP's cpp removes trailing blanks so use
 * -bwt when running diff .
 *
 * Note, I started doing this with "PATH=/lib:/usr/ccs/lib cpp ...", but 10.20
 * broke it.  Their posix shell uses the PATH value when typed on the command
 * line but not as the arg to "sh -c 'PATH=...'", which is what system does.
 * Bah :-(
 * I could have done a putenv(PATH=...); but I did not think of that until
 * after this was written :-(
 */

static char *cppPathTable[] = {
    "/lib/cpp",
    "/usr/ccs/lbin/cpp",
    "/usr/ccs/lib/cpp",
    (char *) NULL
};

/* return values from CheckFile */
typedef enum {
    statAcceptable,		  /* file there and mode OK */
    statReadOnly,		  /* file there but cannot write and need to */
    statNoFile,			  /* file not there */
    statError,			  /* file there but some error processing it */
    statIllegal			  /* bad call */
} statReturnType;

/* returned value if user does not provide a spot for it */
static char cfp_file_holder[CFP_PATHSIZE];

/* the following are filled in by cfp_file and referenced by CopyOnWrite */
char User[CFP_PATHSIZE];	  /* $LOGNAME */
char UserHOME[CFP_PATHSIZE];	  /* user's $HOME */
char Session[CFP_PATHSIZE];	  /* session name */
char SessionHOME[CFP_PATHSIZE];	  /* session's $HOME */
char CFHTPath[CFP_PATHSIZE];	  /* value of $cfhtdir */

/***********************************************************************
 *
 *   CheckFile()
 * DESCRIPTION
 *    This routine checks on the existence of a file, for internal use only.
 *
 * INVOKED
 *    result = CheckFile(Name, Mode);
 *
 * PARAMETERS IN
 *    char *Name - name of file to check
 *    char *Mode - the file access desired, CFP_READWRITE or CFP_READONLY
 *
 * PARAMETERS OUT
 *    none
 *
 * FUNCTION RETURNS
 *    statReturnType - statNoFile      for file not found
 *                     statReadOnly    for access denied
 *                     statAcceptable  for all's OK
 *                     statIllegal     for bogus call
 *
 * GLOBALS
 *    none
 *
 * NOTE
 *    There should be no error messages on "expected" failures, e.g. file not
 *    found.
 *
 ***********************************************************************/

static statReturnType CheckFile Q((char *Name, char *Mode),
				 (Name, Mode), char *Name; char *Mode;)
{
    int File;
    statReturnType Return;
    int OpenMode;

    if ((Name == (char *) NULL) || (Mode == (char *) NULL)) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(CheckFile:%d) Name or Mode address null",
		  __LINE__);
	return statIllegal;
    }

    if (strcmp(Mode, CFP_READWRITE) == 0)
        OpenMode = O_RDWR;
    else if (strcmp(Mode, CFP_READONLY) == 0)
        OpenMode = O_RDONLY;
    else {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(CheckFile:%d) Mode string (%s) not r or rw",
		  __LINE__, Mode);
	return statIllegal;
    }

    if ((File = open(Name, OpenMode)) < 0)
      switch (errno) {
        case EROFS:
        case EACCES:
        case EAGAIN:
	  Return = statReadOnly;  /* it's there, but you can't use it */
	  break;
        default:
	  Return = statNoFile;	  /* not there */
	  break;
      }
    else
      Return = statAcceptable;	  /* usable */

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(CheckFile:%d) for '%s' got %d",
	      __LINE__, Name, Return);
    close(File);
    return Return;
}

/***********************************************************************
 *
 *   ReadFile
 *
 * DESCRIPTION
 *    malloc a block for a currently open file and read it in
 *    Find out how big the file is, allocate memory for it, read the file in,
 *    and close the file.  Tack a newline and a null on the end of the copy.
 *
 * INVOKED
 *    Status = ReadFile(SourceFile, &FileBuffer, &BytesRead);
 *
 * PARAMETERS IN
 *    int SourceFile - file id of open file
 *
 * PARAMETERS OUT
 *    char **FileBuffer - pointer to malloc'ed memory area
 *    size_t *BytesRead - returned count of bytes read
 *
 * FUNCTION RETURNS
 *    PASSFAIL - PASS if read OK, FAIL if problem
 *
 * GLOBALS
 *    None
 *
 ***********************************************************************/

static PASSFAIL ReadFile Q((int SourceFile,
			    char **FileBuffer, size_t *BytesRead),
			   (SourceFile, FileBuffer, BytesRead),
			   int SourceFile;
			   char **FileBuffer; size_t *BytesRead;)
{
    struct stat FileStatus;	  /* fstat results on file */

    if (fstat(SourceFile, &FileStatus) < 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(ReadFile:%d) failed getting status (%s)",
		  __LINE__, strerror(errno));
	if (close(SourceFile) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(ReadFile:%d) then failed to close it? (%s)",
		      __LINE__, strerror(errno));
	    return FAIL;
	}
	return FAIL;
    }
    if ((*FileBuffer = (char *) malloc(FileStatus.st_size + 1 + 1))
	== (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(ReadFile:%d) failed getting %d bytes for file copy",
		  __LINE__, FileStatus.st_size + 1 + 1);
	if (close(SourceFile) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(ReadFile:%d) then failed to close input (%s)",
		      __LINE__, strerror(errno));
	    return FAIL;
	}
	return FAIL;
    }
    if ((*BytesRead
	 = read(SourceFile, (void *) *FileBuffer, FileStatus.st_size))
	!= FileStatus.st_size) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(ReadFile:%d) read only %d bytes (%s)",
		  __LINE__, *BytesRead,
		  (*BytesRead == -1) ? strerror(errno) : "incomplete read");
	free((void *) *FileBuffer);
	if (close(SourceFile) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(ReadFile:%d) then failed to close input (%s)",
		      __LINE__, strerror(errno));
	    return FAIL;
	}
	return FAIL;
    }
    if (close(SourceFile) < 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(ReadFile:%d) source close failed? (%s)",
		  __LINE__, strerror(errno));
	free((void *) *FileBuffer);
	errno = EACCES;
	return FAIL;
    }

    /* make sure there's a newline at the end plus a nul */
    if ((*FileBuffer)[*BytesRead - 1] != '\n') {
	(*FileBuffer)[*BytesRead] = '\n';
	(*BytesRead)++;
    }
    (*FileBuffer)[*BytesRead] = '\0';
    return PASS;
}

/***********************************************************************
 *
 *   WriteFile
 *
 * DESCRIPTION
 *    write out a buffer to a given file
 *    Create a file and write the buffer out to it.  Close the file.  Free the
 *    buffer.
 *
 * INVOKED
 *    Status = WriteFile(FileBuffer, FileSize, FileName);
 *
 * PARAMETERS IN
 *    char *FileBuffer - pointer to data to be written
 *    size_t FileSize  - number of bytes to be written
 *    char *FileName   - name to put file under
 *
 * PARAMETERS OUT
 *    file via *FileName - file on disk
 *
 * FUNCTION RETURNS
 *    PASSFAIL - PASS if OK, FAIL if any problem creating file
 *
 * GLOBALS
 *    None
 *
 ***********************************************************************/

static PASSFAIL WriteFile Q((char *FileBuffer, size_t FileSize,
			     char *FileName),
			    (FileBuffer, FileSize, FileName),
			    char *FileBuffer; size_t FileSize;
			    char *FileName;)
{
    int CopyFile;		  /* file id of output file */
    size_t BytesWritten;	  /* return from write(2) */

/*
    for a while the creat was failing under 10.20 ????
    errno = 0;
    printf("name='%s' size=%d mode=%o\n", FileName, strlen(FileName), FileMode);
*/
    if ((CopyFile = creat(FileName, FileMode)) < 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(WriteFile:%d) failed creating '%s' (%s)",
		  __LINE__, FileName, strerror(errno));
	errno = EACCES;
	return FAIL;
    }
    if ((BytesWritten = write(CopyFile, (void *) FileBuffer, FileSize))
	!= FileSize) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(WriteFile:%d) write to '%s' wrote only %d bytes instead of %s (%s)",
		  __LINE__, FileName, BytesWritten, FileSize,
		  (BytesWritten == -1) ? strerror(errno) : "incomplete write");
	free((void *) FileBuffer);
	if (close(CopyFile) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(WriteFile:%d) then could not close it to unlink (%s)?",
		      __LINE__, strerror(errno));
	    errno = EACCES;
	    return FAIL;
	}
	if (unlink(FileName) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(WriteFile:%d) then could not unlink '%s' (%s)",
		      __LINE__, FileName, strerror(errno));
	    errno = EACCES;
	    return FAIL;
	}
	errno = EACCES;
	return FAIL;
    }
    free((void *) FileBuffer);	  /* cpp output could be larger so kill it */
    if (close(CopyFile) < 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(WriteFile:%d) failed closing '%s' (%s)",
		  __LINE__, FileName, strerror(errno));
	return FAIL;
    }
    return PASS;
}

/***********************************************************************
 *
 *   BeforePreprocess
 *
 * DESCRIPTION
 *    munge a par file before running it past cpp for substitution
 *    Do all the needed substitutions (see the #define section for a
 *    description) to a buffer containing a raw par file.  This replaces (and
 *    frees) the original buffer with the new one.
 *
 * INVOKED
 *    Status = BeforePreprocess(&FileBuffer, &BufferSize);
 *
 * PARAMETERS IN
 *    char **FileBuffer  - the original par file
 *    size_t *BufferSize - original size of the par file
 *
 * PARAMETERS OUT
 *    char **FileBuffer  - the reprocessed par file
 *    size_t *BufferSize - the size of the new buffer
 *
 * FUNCTION RETURNS
 *    PASSFAIL - PASS if OK, FAIL if malloc problem
 *
 * GLOBALS
 *    None
 *
 ***********************************************************************/

static PASSFAIL BeforePreprocess Q((char **FileBuffer, size_t *FileSize),
				   (FileBuffer, FileSize),
				   char **FileBuffer; size_t *FileSize;)
{
    char *Scan;			  /* pointer into FileBuffer */
    char *NewBuffer;		  /* munged par file */
    char *NewScan;		  /* pointer into NewBuffer */
    size_t NewSize;		  /* count of bytes we've put into NewBuffer */
    size_t BlankLength;		  /* count of leading white space */

    /*
     * The worst possible case for the munged file being larger is if it's all
     * newlines.  Each newline is then replaced by %b\n , and the new buffer
     * needs to be three times the size of the input file, plus one for our
     * nul.  Instead of cluttering the code below with checks for needing a
     * realloc , just allocate enough to be sure.
     */
    if ((NewBuffer = (char *) malloc((*FileSize) + (*FileSize) + (*FileSize)
				     + 1))
	== (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(BeforePreprocess:%d) failed allocating new buffer of %d",
		  __LINE__, (*FileSize) + (*FileSize) + (*FileSize) + 1);
	return FAIL;
    }
    NewSize = 0;

    Scan = *FileBuffer;
    NewScan = NewBuffer;
    while (*Scan != '\0') {
	BlankLength = strspn(Scan, " \t");
	if (BlankLength > 0) {
	    (void) strncpy(NewScan, Scan, BlankLength);
	    Scan += BlankLength;
	    NewScan += BlankLength;
	    NewSize += BlankLength;
	}
	if (*Scan == CommentCharacter) {
	    *NewScan++ = cppCharacter;
	    Scan++;
	    NewSize++;
	} else if (*Scan == cppCharacter) {
	    *NewScan++ = CommentCharacter;
	    Scan++;
	    NewSize++;
	} else if (*Scan == '\n') {
	    *NewScan++ = SolutionQuote;
	    NewSize++;
	    *NewScan++ = newlineSolution;
	    NewSize++;
	}
	while (*Scan != '\n') {
	    if (*Scan == quoteProblem) {
		Scan++;
		*NewScan++ = SolutionQuote;
		NewSize++;
		*NewScan++ = quoteSolution;
		NewSize++;
	    } else if (*Scan == apostropheProblem) {
		Scan++;
		*NewScan++ = SolutionQuote;
		NewSize++;
		*NewScan++ = apostropheSolution;
		NewSize++;
	    } else if (*Scan == SolutionQuote) {
		Scan++;
		*NewScan++ = SolutionQuote;
		NewSize++;
		*NewScan++ = SolutionQuote;
		NewSize++;
	    } else {
		*NewScan++ = *Scan++;
		NewSize++;
	    }
	}
	*NewScan++ = *Scan++;
	NewSize++;
    }
    free((void *) (*FileBuffer));
    *FileBuffer = NewBuffer;
    *FileSize = NewSize;
    return PASS;
}

/***********************************************************************
 *
 *   AfterPreprocess
 *
 * DESCRIPTION
 *    munge the cpp'ed file back into a par file
 *    Undo the hacks done in BeforePreprocess so that we have a par file
 *    again.  In this case the result in guaranteed to be smaller than the
 *    input so it is done in place.
 *
 * INVOKED
 *    AfterPreprocess(FileBuffer, &FileSize);
 *
 * PARAMETERS IN
 *    char *FileBuffer - processed munged file
 *
 * PARAMETERS OUT
 *    char *FileBuffer - unmunged par file
 *    size_t *FileSize - length of result
 *
 * FUNCTION RETURNS
 *    void - "can't fail :-)"
 *
 * GLOBALS
 *    None
 *
 ***********************************************************************/

static void AfterPreprocess Q((char *FileBuffer, size_t *FileSize),
			      (FileBuffer, FileSize),
			      char *FileBuffer; size_t *FileSize;)
{
    size_t FromIndex;		  /* into file buffer for @-># and compress */
    size_t ToIndex;
    size_t nonblankIndex;

    FromIndex = 0;
    ToIndex = 0;
    /*
     * want to convert all line initial "@" back into "#"
     * also want to eliminate blank lines - cpp doesn't eat any lines it
     * removes all text from
     * plus we need to put back quotes, apostrophes, and real blank lines
     */
    while (FileBuffer[FromIndex] != '\0') {
	nonblankIndex = strspn(&FileBuffer[FromIndex], " \t");
	if (FileBuffer[FromIndex + nonblankIndex] == '\n') {
	    FromIndex += nonblankIndex + 1;
	} else {
	    if (FileBuffer[FromIndex + nonblankIndex] == cppCharacter) {
		FileBuffer[FromIndex + nonblankIndex] = CommentCharacter;
	    }
	    while (FileBuffer[FromIndex] != '\n') {
		if (FileBuffer[FromIndex] == SolutionQuote) {
		    FromIndex++;
		    switch (FileBuffer[FromIndex++]) {
		      case quoteSolution:
			FileBuffer[ToIndex++] = quoteProblem;
			break;
		      case apostropheSolution:
			FileBuffer[ToIndex++] = apostropheProblem;
			break;
		      case newlineSolution:
			break;
		      case SolutionQuote:
			FileBuffer[ToIndex++] = SolutionQuote;
			break;
		      default:
			cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
				  "(cfp_AfterPreprocess:%d) bogus after %",
				  __LINE__);
			break;
		    }
		} else {
		    FileBuffer[ToIndex++] = FileBuffer[FromIndex++];
		}
	    }
	    FileBuffer[ToIndex++] = FileBuffer[FromIndex++];
	}
    }
    FileBuffer[ToIndex] = '\0';
    /*
     * ToIndex points to the nul, so as a
     * count it points to the last newline in the file.
     */
    *FileSize = ToIndex;
    return;
}

/***********************************************************************
 *
 *   CopyOnWrite
 *
 * DESCRIPTION
 *    If the par file exists as given, copy it to $HOME if being opened
 *    read/write otherwise accept path as given.
 *
 *    First check to see if the given path matches a file.  If not, return
 *    failure.  If so and the open is read only, accept the path.  If the open
 *    is read/write copy the file to the home directory as specified.  If the
 *    Preprocess option is TRUE, run the file through cpp on the way.
 *
 * INVOKED
 *    Status = CopyOnWrite(ParPath, Mode, Preprocess, HomePath);
 *
 * PARAMETERS IN
 *    char *ParPath          - path to the potential source for the par file
 *    char *Mode             - CFP_READONLY or CFP_READWRITE
 *    BooleanType Preprocess - TRUE for cpp preprocessing
 *                             FALSE for straight copy
 *    char *HomePath         - path for file if it is copied to $HOME
 *
 * PARAMETERS OUT
 *    char *ParPath          - updated to HomePath if copied
 *
 * FUNCTION RETURNS
 *    statReturnType         - statAcceptable if found and read only or copied
 *                           - statNoFile if not found
 *                           - statError if found but error copying
 *                           - statIllegal if bug in call
 *
 * GLOBALS
 *    UserHOME, Session, SessionHOME, CFHTPath, cppPathTable are read
 *      if cpp is done
 *
 ***********************************************************************/

static statReturnType CopyOnWrite Q((char *ParPath, char *Mode,
				     BooleanType Preprocess, char *HomePath),
				    (ParPath, Mode, Preprocess, HomePath),
				    char *ParPath; char *Mode;
				    BooleanType Preprocess; char *HomePath;)
{
    int SourceFile;		  /* master file */
    char cppNameIn[CFP_PATHSIZE]; /* intermediate file names if doing cpp */
    char cppNameOut[CFP_PATHSIZE];
    char *FileBuffer;		  /* copy of file */
    size_t BytesRead;		  /* returned from read(2) */

    if ((ParPath == (char *) NULL) || (Mode == (char *) NULL)
	|| (HomePath == (char *) NULL)) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(CopyOnWrite:%d) null arg",
		  __LINE__);
	return statIllegal;
    }

    if ((SourceFile = open(ParPath, O_RDONLY)) < 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(CopyOnWrite:%d) '%s' not found (%s)",
		  __LINE__, ParPath, strerror(errno));
	return statNoFile;
    }
    if (strcmp(Mode, CFP_READONLY) == 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(CopyOnWrite:%d) '%s' found and access read only",
		  __LINE__, ParPath);
	if (close(SourceFile) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) found '%s' but error on close (%s)",
		      __LINE__, ParPath, strerror(errno));
	    return statError;
	}
	return statAcceptable;
    }

    /* file exists, need to copy it, possibly via cpp */

    if (ReadFile(SourceFile, &FileBuffer, &BytesRead) != PASS) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(CopyOnWrite:%d) failed reading in original file '%s'",
		  __LINE__, ParPath);
	return statError;
    }

    if (Preprocess == TRUE) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(CopyOnWrite:%d) found '%s', running through cpp",
		  __LINE__, ParPath);
	(void) strcpy(cppNameIn, HomePath);
	(void) strcat(cppNameIn, cppInputExtension);
	(void) strcpy(cppNameOut, HomePath);
	(void) strcat(cppNameOut, cppOutputExtension);
	if (BeforePreprocess(&FileBuffer, &BytesRead) != PASS) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) failed converting for cpp",
		      __LINE__);
	    return statError;
	}
    } else {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(CopyOnWrite:%d) found '%s', copying",
		  __LINE__, ParPath);
	(void) strcpy(cppNameIn, HomePath);
    }
    if (WriteFile(FileBuffer, BytesRead, cppNameIn) != PASS) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(CopyOnWrite:%d) failed writing out '%s'",
		  __LINE__, cppNameIn);
#ifndef DEBUGCPP
	if (unlink(cppNameIn) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) could not unlink '%s' (%s)",
		      __LINE__, cppNameIn, strerror(errno));
	}
#endif
	return statError;
    }

    /* if we need to run cpp on it, do so then convert the result back */

    if (Preprocess == TRUE) {
	unsigned int cppIndex;	  /* into cppPathTable */
	int cppID;		  /* temporary file handle of cpp */
	char cppCommand[BUFSIZ];  /* build cpp command string here */

	cppIndex = 0;
	while (cppPathTable[cppIndex] != (char *) NULL) {
	    if ((cppID = open(cppPathTable[cppIndex], O_RDONLY)) >= 0) {
		(void) close(cppID);
		break;
	    }
	    cppIndex++;
	}
	if (cppPathTable[cppIndex] == (char *) NULL) {
#ifndef DEBUGCPP
	    if (unlink(cppNameIn) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameIn, strerror(errno));
	    }
#endif
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) could not find a copy of cpp",
		      __LINE__);
	    return statError;
	}
	
	(void) strcpy(cppCommand, cppPathTable[cppIndex]);
	(void) strcat(cppCommand, " -E -C -P -D");
	(void) strcat(cppCommand, cfpHOME);
	(void) strcat(cppCommand, UserHOME);
	(void) strcat(cppCommand, " -D");
	(void) strcat(cppCommand, cfpSession);
	(void) strcat(cppCommand, Session);
	(void) strcat(cppCommand, " -D");
	(void) strcat(cppCommand, cfpSessionHOME);
	(void) strcat(cppCommand, SessionHOME);
	(void) strcat(cppCommand, " -D");
	(void) strcat(cppCommand, cfpCFHTDir);
	(void) strcat(cppCommand, CFHTPath);
	(void) strcat(cppCommand, " -D");
	(void) strcat(cppCommand, cfpWhichSession1);
	(void) strcat(cppCommand, Session);
	(void) strcat(cppCommand, cfpWhichSession2);
	(void) strcat(cppCommand, " ");
	(void) strcat(cppCommand, cppNameIn);
	(void) strcat(cppCommand, " ");
	(void) strcat(cppCommand, cppNameOut);
	if (cfht_system(cppCommand) != PASS) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) cpp exec failed",
		      __LINE__);
#ifndef DEBUGCPP
	    if (unlink(cppNameIn) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameIn, strerror(errno));
	    }
	    if (unlink(cppNameOut) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameOut, strerror(errno));
	    }
#endif
	    return statError;
	}

	/* redo it */

	if ((SourceFile = open(cppNameOut, O_RDONLY)) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		      "(CopyOnWrite:%d) '%s' not found (%s)",
		      __LINE__, cppNameOut, strerror(errno));
#ifndef DEBUGCPP
	    if (unlink(cppNameIn) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameIn, strerror(errno));
	    }
	    if (unlink(cppNameOut) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameOut, strerror(errno));
	    }
#endif
	    return statNoFile;
	}
	if (ReadFile(SourceFile, &FileBuffer, &BytesRead) != PASS) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) failed reading in cpp'ed file '%s'",
		      __LINE__, cppNameOut);
#ifndef DEBUGCPP
	    if (unlink(cppNameIn) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameIn, strerror(errno));
	    }
	    if (unlink(cppNameOut) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameOut, strerror(errno));
	    }
#endif
	    return statError;
	}
	AfterPreprocess(FileBuffer, &BytesRead);
	if (WriteFile(FileBuffer, BytesRead, HomePath) != PASS) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) failed writing cpp'ed file '%s'",
		      __LINE__, HomePath);
#ifndef DEBUGCPP
	    if (unlink(cppNameIn) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameIn, strerror(errno));
	    }
	    if (unlink(cppNameOut) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameOut, strerror(errno));
	    }
#endif
	    return statError;
	}
#ifndef DEBUGCPP
	if (unlink(cppNameIn) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) could not unlink '%s' (%s)",
		      __LINE__, cppNameIn, strerror(errno));
	    if (unlink(cppNameOut) < 0) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(CopyOnWrite:%d) could not unlink '%s' (%s)",
			  __LINE__, cppNameOut, strerror(errno));
	    }
	    return statError;
	}
	if (unlink(cppNameOut) < 0) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(CopyOnWrite:%d) could not unlink '%s' (%s)",
		      __LINE__, cppNameOut, strerror(errno));
	    return statError;
	}
#endif
    }

    /* OK, update where file really is */

    (void) strcpy(ParPath, HomePath);
    return statAcceptable;
}

/*!**********************************************************************
 *
 *  cfp_file()
 *
 * DESCRIPTION
 *    
 *    This routine locates a file, typically a par file.  It will first look in
 *    the current directory, then look in the "home" directory, then in the
 *    "session home" directory and finally a "default" directory.  If the file
 *    is to be written to, and is only found in the session home or default
 *    directory, then the file is copied to the home directory and this new
 *    file is returned.  For read-only access, the file in the session home or
 *    default directory may be used.
 *
 *    The following search algorithm uses sh variable definitions and
 *    references.  First, using the Name and Suff argument values,
 *      File=$Name:  $Suff
 *    .  Next, if $File is the name of an environment variable,
 *      File=$File	# note that this $File is a reference to the env
 *    .  Next, look for ./$File .  If mode is "rw", the file is there, but it
 *    cannot be written, the call fails and returns NULL.  Otherwise if the
 *    file is there, "./$File" is returned in Addr.  Next, look for ~/.,$File
 *    (note the added ".,").  If mode is "rw", the file is there, but it
 *    cannot be written, the call fails and returns NULL.  Otherwise if the
 *    file is there, "~/.,$File" is returned in Addr.  Next, if the environment
 *    variable Session is defined and not null, look for $Session/.,$File (note
 *    the added ".,").  If the file is there and mode is "rw", copy the file to
 *    ~/.,$File and return "~/.,$File" in Addr.  Otherwise if the file is there
 *    return "$Session/.,$File" in Addr.  Next, if the environment variable
 *    CFHTCONFPATH is defined and not null, set
 *      Default=$CFHTCONFPATH
 *    , otherwise if the environment variable cfhtdir is defined and not null,
 *    set
 *      Default=$cfhtdir/conf
 *    , otherwise the call fails and returns NULL.  If
 *    the file is there and mode is "rw", copy the file to ~/.,$File and return
 *    "~/.,$File" in Addr.  Otherwise if the file is there, return
 *    "$Default/$File" in Addr.  Otherwise the call fails and returns NULL.
 *
 *    In the above, the last reference to "copy" actually means switch all line
 *    initial "#" to "@" and "@" to "#", run the result through cpp, and switch
 *    all the "@" back to "#".  During the cpp run, the following definitions
 *    are provided
 *
 *      #define cfpHOME $HOME
 *      #define cfpSession $Session
 *      #define cfpSessionHOME /h/$Session
 *      #define cfpCFHTDir $cfhtdir
 *      #define cfp${Session}Session 1
 *
 *    where the "$" implies reference to environment variables.
 *    For example starting with
 *
 *      # sample
 *      @ifdef cfpmosSession
 *      @define FormSize 16
 *      @else
 *      @define FormSize 12
 *      @endif
 *      Session = cfpSession
 *      +form = 20 20 -1 -1 +f FormSize
 *
 *    , for, say MOS, cpp is fed
 *
 *      #define cfpHOME /h/mos
 *      #define cfpSession mos
 *      #define cfpSessionHOME /h/mos
 *      #define cfpCFHTDir /usr/local/cfht
 *      #define cfpmosSession 1
 *      @ sample
 *      #ifdef cfpmosSession
 *      #define FormSize 16
 *      #else
 *      #define FormSize 12
 *      #endif
 *      Session = cfpSession
 *      +form = 20 20 -1 -1 +f FormSize
 *
 *    , which results in the following par file
 *      # sample
 *      Session = mos
 *      +form = 20 20 -1 -1 +f 16
 *    .
 *
 *    Plus, if the file name is ioconfig , we only look in $cfhtdir/conf .
 *
 *    Plus, if the Mode string is CFP_READFIXED no copy happens.
 *
 * INVOKED
 *    full_filename = cfp_file(Addr, Name, Suff, Mode);
 *
 * PARAMETERS IN
 *    char *Name - the base name of the file we want
 *    char *Suff - the suffix to append to the base name
 *    char *Mode - the file access desired, CFP_READWRITE or CFP_READONLY
 *                     or CFP_READFIXED
 *
 * PARAMETERS OUT
 *    char *Addr - the resulting file name; if a NULL pointer is passed in,
 *                 then an internal static area is used
 *
 * FUNCTION RETURNS
 *    char * - the resulting file name, or NULL if an error occured
 *
 * GLOBALS
 *    if the internal string area is used, it will be destroyed by
 *    subsequent calls using it
 *
 **********************************************************************!*/

char *cfp_file(Addr, Name, Suff, Mode)
    char *Addr;
    char *Name;
    char *Suff;
    char *Mode;
{
    char ParName[CFP_PATHSIZE];	  /* Name plus suffix */
    char *EnvironmentName;	  /* value of par name from environment */
    char ParPath[CFP_PATHSIZE];	  /* current path being tried */
    char *Home;			  /* HOME or Session from environment */
    char *OurMode;		  /* Mode less "rf" -> "ro" */
    BOOLEAN ReadFixed;		  /* TRUE if MODE=="rf", FALSE if not */
    char HomePath[CFP_PATHSIZE];  /* full file path if in home directory */
    char *ConfPath;		  /* conf path from environment */
    int FieldCount;		  /* return from fscanf for session name */
    statReturnType Return;	  /* return from CheckFile or CopyOnWrite */

    if (Addr == (char *) NULL)
      Addr = cfp_file_holder;	  /* use internal space if required */
    *Addr = '\0';		  /* make sure it starts empty */

    if ((Name == (char *) NULL) && (Suff == (char *) NULL)) {
	/* no hope, give up */
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) neither file name nor suffix provided",
		  __LINE__);
	errno = ENOENT;
	return (char *) NULL;
    }
    if (Mode == (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) no access mode for file provided", __LINE__);
	errno = EPERM;
	return((char *) NULL);
    }
    if (strcmp(Mode, CFP_READFIXED) == 0) {
	OurMode = CFP_READONLY;
	ReadFixed = TRUE;
    } else {
	OurMode = Mode;
	ReadFixed = FALSE;
    }

    /* if we are given a null pointer, substitute a pointer to a null string */
    if (Name == (char *) NULL)
      Name = "";
    if (Suff == (char *) NULL)
      Suff = "";

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(cfp_file) start, Name = '%s' Suff = '%s'", Name, Suff);

    if ((Home = getenv("LOGNAME")) == (char *) NULL) {
	if ((Home = getenv("USER")) == (char *) NULL) {
	    (void) strcpy(User, "foo");
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(cfp_file:%d) no $LOGNAME nor $USER definition for '%s' ?",
		      __LINE__, ParName);
	} else {
	    (void) strcpy(User, Home);
	}
    } else {
	(void) strcpy(User, Home);
    }
    if ((Home = getenv(CFHT_ECFHTPATH)) == (char *) NULL) {
	(void) strcpy(CFHTPath, CFHT_CFHTPATH); /* sigh :-( */
    } else {
	(void) strcpy(CFHTPath, Home);
    }

    (void) strcpy(ParName, Name);
    (void) strcat(ParName, Suff);

    /* ioconfig is special - just look for it directly :-( */

    if (strcmp(ParName, CFHT_IOCONFIG) == 0) {
	(void) strcpy(ParName, CFHTPath);
	(void) strcat(ParName, "/");
	(void) strcat(ParName, CFHT_IOCONFDIR);
	(void) strcat(ParName, "/");
	(void) strcat(ParName, CFHT_IOCONFIG);
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file:%d) forcing ioconfig to '%s'",
		  __LINE__, ParName);
    }

    /* if the given name is in the environment, override it */

    if ((EnvironmentName = getenv(ParName)) != (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file:%d) using '%s' from environment variable '%s'",
		  __LINE__, EnvironmentName, ParName);
	(void) strcpy(ParName, EnvironmentName);
    }

    /* check current directory or actual path if given */

    if ((Return = CheckFile(ParName, OurMode)) == statAcceptable) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file) end, chose '%s' in current dir", ParName);
	(void) strcpy(Addr, ParName);
	return Addr;
    }
    if (Return == statReadOnly) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) access to '%s' in current dir denied (%s)",
		  __LINE__, ParName, strerror(errno));
	errno = EACCES;
	return (char *) NULL;
    }
    if (Return == statIllegal) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) code broken for '%s'",
		  __LINE__, ParName);
	errno = ENOTTY;
	return (char *) NULL;
    }
    /* don't even try to look any farther if they gave more than a file name */
    if (strchr(ParName, '/') != (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) '%s' not found",
		  __LINE__);
	errno = ENOENT;
	return (char *) NULL;
    }
    /* also don't go any farther if they put the ., in */
    if (strncmp(ParName, HomePrefix, strlen(HomePrefix)) == 0) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) '%s' not found in . - not looking farther",
		  __LINE__, ParName);
	errno = ENOENT;
	return (char *) NULL;
    }

    /* try $HOME */

    if ((Home = getenv("HOME")) == (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) no HOME environment definition?",
		  __LINE__);
	errno = ENOTTY;
	return (char *) NULL;
    }
    (void) strcpy(UserHOME, Home);
    (void) strcpy(ParPath, Home);
    (void) strcat(ParPath, "/");
    (void) strcat(ParPath, HomePrefix);
    (void) strcat(ParPath, ParName);
    (void) strcpy(HomePath, ParPath);
    if ((Return = CheckFile(ParPath, OurMode)) == statAcceptable) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file) end, chose '%s' in $HOME", ParPath);
	(void) strcpy(Addr, ParPath);
	return Addr;
    }
    if (Return == statReadOnly) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) access to '%s' denied (%s)",
		  __LINE__, ParPath, strerror(errno));
	errno = EACCES;
	return (char *) NULL;
    }

    /* try $Session unless ReadFixed */
    /*
     * If this is not a variable file, just read the conf copy
     * (ReadFixed).  Note that this means that CFP_READFIXED should really be
     * used on only unchanging files.  That makes sense, and, anyway, for this
     * if there's a problem the user is right there anyway :-)
     */
    if (ReadFixed == FALSE) {
	if ((Home = getenv(CFHT_ESESSION)) == (char *) NULL) {
	    /*
	     * if a user is running this outside session, ask for master file
	     * location.
	     */
	    if ((isatty(0) == 1) && (ReadFixed == FALSE)) {
		(void) printf("Which session should be used for %s definition?\n",
			      ParName);
		(void) printf(" start with '-' to get from conf : ");
		(void) fflush(stdout);
		FieldCount = fscanf(stdin, "%s", Session);
		if (FieldCount == 1) {
		    Home = Session;
		} else {
		    printf("try a session name next time, not '%s' :-)", Session);
		    exit(EXIT_FAIL);
		}
		if (Session[0] == '-') {
		    char *SessionCopy;
		    Home = User;
		    if ((SessionCopy = strdup(Session)) == (char *) NULL) {
			printf("malloc copy of Session failed\n");
			exit(EXIT_FAIL);
		    }
		    (void) strcpy(Session, &SessionCopy[1]);
		    (void) free(SessionCopy);
		}
	    } else {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(cfp_file:%d) no $Session definition and not a tty for '%s'",
			  __LINE__, ParName);
		Home = User;
	    }
	} else {
	    (void) strncpy(Session, Home, sizeof Session);
	    Session[(sizeof Session) - 1] = '\0';
	}
	(void) strcpy(ParPath, HomeDirectoryPrefix);
	(void) strcat(ParPath, Home);
	(void) strcpy(SessionHOME, ParPath);
	/* don't need to try this if it's just the user's home directory */
	if (strcmp(UserHOME, SessionHOME) != 0) {
	    (void) strcat(ParPath, "/");
	    (void) strcat(ParPath, HomePrefix);
	    (void) strcat(ParPath, ParName);
	    if ((Return = CopyOnWrite(ParPath, OurMode, FALSE, HomePath))
		== statAcceptable) {
		cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
			  "(cfp_file) end, chose '%s' from $Session", ParPath);
		(void) strcpy(Addr, ParPath);
		return Addr;
	    }
	    if (Return == statError) {
		cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
			  "(cfp_file:%d) problem processing file '%s'",
			  __LINE__, ParPath);
		return (char *) NULL;
	    }
	}
    }

    /* try conf */

    if ((ConfPath = getenv(CFHT_ECONFPATH)) == (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file:%d) no conf environment definition?",
		  __LINE__);
	(void) strcpy(ParPath, CFHTPath);
	(void) strcat(ParPath, "/");
	(void) strcat(ParPath, CFHT_CONFDIR);
    } else {
	(void) strcpy(ParPath, ConfPath);
    }
    (void) strcat(ParPath, "/");
    (void) strcat(ParPath, ParName);
    /* note - if from conf, have to run it through cpp now :-( */
    /* but not if Mode was CFP_READFIXED ("rf") */
    if ((Return = CopyOnWrite(ParPath,
			      ((ReadFixed == TRUE)
			       ? CFP_READONLY : CFP_READWRITE),
			      (ReadFixed == TRUE) ? FALSE : TRUE,
			      HomePath))
	== statAcceptable) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_file) end, chose '%s' from conf", ParPath);
	(void) strcpy(Addr, ParPath);
    } else {
	if (Return == statError) {
	    cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(cfp_file:%d) problem processing conf file '%s'",
		      __LINE__, ParPath);
	    return (char *) NULL;
	}
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_file:%d) '%s' not found anywhere",
		  __LINE__, ParName);
	errno = ENOENT;
	return (char *) NULL;
    }

    return Addr;
}
