/* Copyright (C) 1991, 94, 95    Canada-France-Hawaii Telescope Corp.    */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
 * FILE
 * cfp_finish.c
 * $Header: cfp_finish.c,v 2.3 96/11/19 23:15:03 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *   cfp_finish()
 *
 * DESCRIPTION
 *    This routine frees any allocated memory that is associated with id.
 *
 * INVOKED
 *    return = cfp_finish(id);
 *
 * PARAMETERS IN
 *    FILE_ID id - the id of the file we are (done) working with
 *
 * PARAMETERS OUT
 *    NONE
 *
 * FUNCTION RETURNS
 *    PASSFAIL - indicates whether the id appeared to have been used
 *
 * GLOBALS
 *    int errno - the global error value
 *    struct cfp_inst cfp_inst_table[] - the structure used for file access
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          27 Jan 1988     Original coding
 *
 * $Log:	cfp_finish.c,v $
 * Revision 2.3  96/11/19  23:15:03  isani
 * cleaned up rcsid
 * 
 * Revision 2.2  1996/01/25  17:23:59  thomas
 * Free new hash table
 *
 * Revision 2.1  94/12/14  12:47:20  12:47:20  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h, added cfp_internal.h
 * Changed CFHT_ERROR to CFHT_LOGONLY
 * Used new internal utility routines to free line space
 * 
 * Revision 1.1  91/09/24  17:08:22  17:08:22  steve (Steven Smith)
 * Original checkin
 * 
 ***************************************************************************!*/

#include <stdlib.h>		  /* free() */
#include <errno.h>		  /* for error codes and errno defines */

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* configuration file parsing defines */
#include "cfp_internal.h"	  /*   and internals */

RCSID("$Header: cfp_finish.c,v 2.3 96/11/19 23:15:03 isani Exp $");

PASSFAIL			  /* return: PASS, or FAIL */
cfp_finish(id)
    FILE_ID id;			  /* instance handle */
{
    FileEntryType *FileEntry;	  /* pointer to this file's data */

    cfht_log(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_finish) start");

    /**********************
     * check for valid id *
     **********************/

    if ((FileEntry = cfp_CheckID(id)) == (FileEntryType *) NULL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
                  "(cfp_finish:%d) CheckID failed on %d", __LINE__, id);
        errno = ENOTTY;
        return(FAIL);
    }

    /*********************
     * free up resources *
     *********************/

    cfp_FreeLineSpace(FileEntry);
    FileEntry->InUseCode = NotInUse;
    if (FileEntry->HashTable != (HashTableType) NULL) {
	free(FileEntry->HashTable);
	FileEntry->HashTable = (HashTableType) NULL;
    }

    cfht_log(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_finish) end");

    return(PASS);
}
