/* Copyright (C) 1994    Canada-France-Hawaii Telescope Corp.            */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
 * FILE
 * cfp_getf.c
 * $Header: cfp_getf.c,v 2.3 96/11/19 22:57:52 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *   cfp_getf()
 *
 * DESCRIPTION
 *    This routine releases the passed in file id, and reopens the same
 *    file.  The new id is then returned.  This is used when some other
 *    process may have changed the file, and you now want to get the
 *    new values.
 *
 * INVOKED
 *    new_id = cfp_getf(old_id);
 *
 * PARAMETERS IN
 *    FILE_ID id - the old file id, to be released
 *
 * PARAMETERS OUT
 *    none
 *
 * FUNCTION RETURNS
 *    FILE_ID - the new file id or FAIL
 *
 * GLOBALS
 *    none
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          25 Apr 1989     Original coding
 *
 * $Log:	cfp_getf.c,v $
 * Revision 2.3  96/11/19  22:57:52  isani
 * cleaned up rcsid
 * 
 * Revision 2.2  1995/06/28  16:22:22  thomas
 * Fix cfp_init return check, can't use != PASS dummy :-(
 *
 * Revision 2.1  94/12/14  12:53:27  12:53:27  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h, added cfp_internal.h
 * Changed CFHT_ERROR to CFHT_LOGONLY
 * 
 * Revision 1.2  91/09/25  09:22:19  09:22:19  jwright (Jim Wright)
 * updated comments
 * 
 * Revision 1.1  91/09/24  17:13:28  17:13:28  steve (Steven Smith)
 * Original checkin
 * 
 ***************************************************************************!*/

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* configuration file parsing defines */
#include "cfp_internal.h"	  /*   and internals */

RCSID("$Header: cfp_getf.c,v 2.3 96/11/19 22:57:52 isani Exp $");

FILE_ID				  /* return: file id or FAIL */
cfp_getf(id)
    FILE_ID id;			  /* instance handle */
{
    FileEntryType *FileEntry;	  /* file descriptor to update */
    char path[CFP_PATHSIZE];

    if ((FileEntry = cfp_CheckID(id)) == (FileEntryType *) NULL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getf:%d) cfp_CheckID failed", __LINE__);
        return(FAIL);
    }

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(cfp_getf:%d) start on '%s'", __LINE__, FileEntry->FilePath);

    (void) strcpy(path, FileEntry->FilePath);
    if (cfp_finish(id) != PASS) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getf:%d) cfp_finish failed", __LINE__);
        return(FAIL);
    }
    if ((id = cfp_init(path)) == FAIL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getf:%d) cfp_init failed", __LINE__);
        return(FAIL);
    }

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(cfp_getf:%d) end on '%s'", __LINE__, path);

    return(id);
}
