/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfp_getln.c
* $Header: cfp_getln.c,v 2.2 96/11/19 22:58:38 isani Exp $
* $Locker:  $
*
* ROUTINES
*
*   cfp_getln()
*
* DESCRIPTION
*    This routine is misnamed.  It should be called cfp_getvalue().
*    This routine will retrieve the value from a line matching a given key
*    from the named file.  It does a combination of cfp_file, cfp_init, and
*    cfp_getcur to return the value.
*
* INVOKED
*    string = cfp_getln(dest, key, name, suff);
*
* PARAMETERS IN
*    char *key - the token to match in the file
*    char *name - the name of the file
*    char *suff - the suffix to append to name
*
* PARAMETERS OUT
*    char *dest - this is filled in with the matching value; it should
*                 be able to hold at least CFHT_VALUE_SIZE characters;
*                 if it is NULL, a local static buffer will be used
*
* FUNCTION RETURNS
*    char * - pointer to the matching string, or NULL for failure
*
* GLOBALS
*    none
*    the internal buffer will be overwritten each time it is used
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab          15 Mar 1988     Original coding
* jab          06 Jul 1988     Change in definition
*
* $Log:	cfp_getln.c,v $
 * Revision 2.2  96/11/19  22:58:38  isani
 * cleaned up rcsid
 * 
 * Revision 2.1  1994/12/14  13:02:17  thomas
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h
 * Changed CFHT_ERROR to CFHT_LOGONLY, added more info to diagnostic messages
 *
 * Revision 1.1  91/09/24  17:13:55  17:13:55  steve (Steven Smith)
 * Original checkin
 * 
****************************************************************************!*/

#include <string.h>		  /* for string defines */
#include <errno.h>		  /* for error codes and errno defines */

#include "cfht.h"		  /* generic defines */
#include "cfp.h"		  /* parsing defines */

RCSID("$Header: cfp_getln.c,v 2.2 96/11/19 22:58:38 isani Exp $");

static char line[CFHT_VALUE_SIZE];

char *				  /* return: pointer to line or NULL */
cfp_getln(dest, key, name, suff)
    char *dest;
    char *key;
    char *name;
    char *suff;
{
    char path[CFP_PATHSIZE + 1];
    FILE_ID id;

    if (!dest)
        dest = line;    /* use temporary space */

    if (!key || (!name && !suff)) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getln:%d) invalid arguments", __LINE__);
        errno = EINVAL;
        return(NULL);
    }

    if (name == (char *) NULL)
      path[0] = '\0';
    else {
	strncpy(path, name, CFP_PATHSIZE);
	path[CFP_PATHSIZE] = '\0';
    }
    if (suff != (char *) NULL) {
	strncpy(path, suff, CFP_PATHSIZE - strlen(path));
	path[CFP_PATHSIZE] = '\0';
    }
    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
              "(cfp_getln) start, key '%s', file '%s'", key, path);

    if (cfp_file(path, name, suff, "r")) {
        id = cfp_init(path);
        if (cfp_getcur(id, key, dest, CFP_STRING, CFHT_VALUE_SIZE) == FAIL) {
            cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(cfp_getln:%d) cfp_getcur failed for '%s' in '%s'",
		      __LINE__, key, path);
            dest = (char *) NULL;
        }
        (void)cfp_finish(id);
    } else {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getln:%d) cfp_file failed on '%s'", __LINE__, name);
        return((char *) NULL);
    }

    cfht_log(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_getln) end");

    return(dest);
}
