/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfp_getsiz.c
* $Header: cfp_getsiz.c,v 2.3 96/11/19 22:58:50 isani Exp $
* $Locker:  $
*
* ROUTINES
*
*   cfp_getsiz()
*
* DESCRIPTION
*    This routine returns the number of characters required to hold the
*    value for the given identifier (name) in the given file (id).  This
*    call is typically followed by a malloc and a call to cfp_getcur or
*    cfp_getval.
*
* INVOKED
*    size = cfp_getsiz(id, name);
*
* PARAMETERS IN
*    FILE_ID id - the file to examine
*    char *name - the name of the identifier under consideration
*
* PARAMETERS OUT
*    NONE
*
* FUNCTION RETURNS
*    int - the size in characters available, or FAIL
*
* GLOBALS
*    none
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab          27 Jan 1988     Original coding
*
* $Log:	cfp_getsiz.c,v $
 * Revision 2.3  96/11/19  22:58:50  isani
 * cleaned up rcsid
 * 
 * Revision 2.2  1994/12/21  12:43:02  thomas
 * Added "not just a lookup" flag to cfp_FindName call
 *
 * Revision 2.1  94/12/14  13:04:26  13:04:26  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h, added cfp_internal.h
 * Changed CFHT_ERROR to CFHT_LOGONLY
 * Rewrote to use version 2 routines
 * Corrected documentation to match reality
 * 
 * Revision 1.1  91/09/24  17:14:19  17:14:19  steve (Steven Smith)
 * Original checkin
 * 
*
****************************************************************************!*/

#include <string.h>		  /* for string handling defines */
#include <errno.h>		  /* for error codes and errno define */

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* configuration file parsing defines */
#include "cfp_internal.h"	  /*   and internals */

RCSID("$Header: cfp_getsiz.c,v 2.3 96/11/19 22:58:50 isani Exp $");

int				  /* return: size, or FAIL */
cfp_getsiz(id, name)
    FILE_ID id;			  /* instance handle */
    char *name;			  /* identifier to look for */
{
    FileEntryType *FileEntry;	  /* pointer to file data for errors */
    LineType *Line;		  /* pointer to line of interest */

    if (name == (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getsiz:%d) given null name pointer",
		  __LINE__);
	return(FAIL);
    }

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
              "(cfp_getsiz) start (name = '%s')", name);

    if ((FileEntry = cfp_CheckID(id)) == (FileEntryType *) NULL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getsiz:%d) cfp_CheckID failed on %d for '%s'",
		  __LINE__, id, name);
        return(FAIL);
    }

    if ((Line = cfp_FindName(FileEntry, name, FALSE)) == (LineType *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_getsiz:%d) name '%s' not found for '%s'",
		  __LINE__, name, FileEntry->FilePath);
	return(FAIL);
    }

    if (Line->TypeCode == ConstantLine) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_getsiz:%d) end, returning original length = %d",
		  __LINE__, Line->OriginalValueLength);
	return((int) Line->OriginalValueLength);
    } else {
	if (Line->TypeCode == VariableLine) {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_getsiz:%d) end, returning current length = %d",
		  __LINE__, Line->CurrentValueLength);
	    return((int) Line->CurrentValueLength);
	}
    }

    cfht_logv(CFHT_LIBCFP, CFHT_ERROR,
	      "(cfp_getsiz:%d) Jim made a bad assumption about how this",
	      __LINE__);
    cfht_logv(CFHT_LIBCFP, CFHT_ERROR,
	      "(...)  is used, name=%s, file=%s",
	      Line->NameString, FileEntry->FilePath);
    return(FAIL);
    /*NOTREACHED*/
}
