/* Copyright (C) 1994    Canada-France-Hawaii Telescope Corp.            */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING         */
/*!****************************************************************************
* FILE
* cfp_lookup.c
* $Header: cfp_lookup.c,v 1.2 96/11/19 23:02:48 isani Exp $
* $Locker:  $
*
* ROUTINES
*
*   cfp_lookup()
*
* DESCRIPTION
*    This routine looks up the given identifier (name) in the given file (id)
*    to see if it is defined.  It returns PASS if name is defined or FAIL if
*    name is not defined or is ambiguous or there is some other problem.  If
*    name is not defined, no message is output to the roll file.
*
* INVOKED
*    status = cfp_lookup(id, name);
*
* PARAMETERS IN
*    FILE_ID id - the file to examine
*    char *name - the name of the identifier under consideration
*
* PARAMETERS OUT
*    NONE
*
* FUNCTION RETURNS
*    PASSFAIL - PASS if name is defined
*               FAIL if name is not defined or there is some other problem
*
* GLOBALS
*    none
*
* HISTORY
*
* $Log:	cfp_lookup.c,v $
 * Revision 1.2  96/11/19  23:02:48  isani
 * cleaned up rcsid
 * 
 * Revision 1.1  1994/12/21  12:47:55  thomas
 * Original check in
 *
****************************************************************************!*/

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* configuration file parsing defines */
#include "cfp_internal.h"	  /*   and internals */

RCSID("$Header: cfp_lookup.c,v 1.2 96/11/19 23:02:48 isani Exp $");

PASSFAIL
cfp_lookup(id, name)
    FILE_ID id;			  /* instance handle */
    char *name;			  /* identifier to look for */
{
    FileEntryType *FileEntry;	  /* pointer to file data for errors */
    LineType *Line;		  /* pointer to line of interest */

    if (name == (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_lookup:%d) given null name pointer",
		  __LINE__);
	return(FAIL);
    }

#ifdef DEBUG
    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
              "(cfp_lookup) start (name = '%s')", name);
#endif

    if ((FileEntry = cfp_CheckID(id)) == (FileEntryType *) NULL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_lookup:%d) cfp_CheckID failed on %d for '%s'",
		  __LINE__, id, name);
        return(FAIL);
    }

    if ((Line = cfp_FindName(FileEntry, name, TRUE)) == (LineType *) NULL) {
#ifdef DEBUG
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
		  "(cfp_lookup:%d) name '%s' not found",
		  __LINE__, name);
#endif
	return(FAIL);
    }
#ifdef DEBUG
    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(cfp_lookup:%d) name '%s' found",
	      __LINE__, name);
#endif
    return(PASS);
}
