/* Copyright (C) 1991, 94, 1996    Canada-France-Hawaii Telescope Corp.  */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
 * FILE cfp_putln.c
 * $Header: cfp_putln.c,v 2.3 96/11/19 22:50:10 isani Exp $
 * $Locker:  $
 *
 * ROUTINES
 *
 *   cfp_putln()
 *
 * DESCRIPTION
 *    This routine is misnamed.  It should be cfp_putvalue().
 *    This routine updates the value for the line matching a given key in
 *    the given file.  It does a combination of cfp_file, cfp_init,
 *    cfp_putcur, cfp_putf, and cfp_finish.
 *
 * INVOKED
 *    status = cfp_putln(src, key, name, suff);
 *
 * PARAMETERS IN
 *    char *src - new value for key in name
 *    char *key - line name to update value for (result is key=src)
 *    char *name - base file name to operate on
 *    char *suff - file name suffix (e.g. ".par")
 *
 * PARAMETERS OUT
 *    none
 *
 * FUNCTION RETURNS
 *    PASSFAIL - PASS if update happened OK, FAIL if not
 *
 * GLOBALS
 *    specified file is updated
 *
 * HISTORY
 * who          when            what
 * ---------    ------------    ----------------------------------------------
 * jab          19 Mar 1988     Original coding
 * jab           6 Jul 1988     Change in definition
 *
 * $Log:	cfp_putln.c,v $
 * Revision 2.3  96/11/19  22:50:10  isani
 * cleaned up rcsid
 * 
 * Revision 2.2  1996/03/14  16:49:30  thomas
 * Made path array 1 larger so [PATHSIZE] would work :-(
 *
 * Revision 2.1  94/12/14  16:56:48  16:56:48  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h
 * Changed CFHT_ERROR to CFHT_LOGONLY
 * Rewrote to use version 2 routines
 * 
 * Revision 1.1  91/09/24  17:31:46  17:31:46  steve (Steven Smith)
 * Original checkin
 * 
 ***************************************************************************!*/

#include <string.h>		  /* for string defines */
#include <errno.h>		  /* for error codes and errno defines */

#include "cfht.h"		  /* generic defines */
#include "cfp.h"		  /* parsing defines */

RCSID("$Header: cfp_putln.c,v 2.3 96/11/19 22:50:10 isani Exp $");

PASSFAIL			  /* return: Pass, or FAIL */
cfp_putln(src, key, name, suff)
    char *src;
    char *key;
    char *name;
    char *suff;
{
    char path[CFP_PATHSIZE + 1];
    FILE_ID id;

    if (!src || !key || (!name && !suff)) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putln:%d) invalid args", __LINE__);
        errno = EINVAL;
        return(FAIL);
    }

    if (name == (char *) NULL)
      path[0] = '\0';
    else {
	strncpy(path, name, CFP_PATHSIZE);
	path[CFP_PATHSIZE] = '\0';
    }
    if (suff != (char *) NULL) {
	strncpy(path, suff, CFP_PATHSIZE - strlen(path));
	path[CFP_PATHSIZE] = '\0';
    }
    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
              "(cfp_putln) start, key '%s', file '%s'", key, path);

    if (cfp_file(path, name, suff, "rw")) {
        id = cfp_init(path);
        if (cfp_putcur(id, key, src, CFP_STRING) != PASS) {
            cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		      "(cfp_putln:%d) cfp_putcur failed on '%s=%s' for '%s'",
		      __LINE__, key, src, path);
            (void) cfp_finish(id);
            return(FAIL);
        }
    } else {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putln:%d) cfp_file failed for '%s'", __LINE__, name);
        return(FAIL);
    }

    if (cfp_putf(id) != PASS) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putln:%d) cfp_putf failed for '%s'", __LINE__, name);
        (void) cfp_finish(id);
        return(FAIL);
    }

    (void) cfp_finish(id);

    cfht_log(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_putln) end");

    return(PASS);
}
