/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
* cfp_putstr.c
* $Header: cfp_putstr.c,v 2.3 96/11/19 22:50:29 isani Exp $
* $Locker:  $
*
* ROUTINES
*
*   cfp_putstr()
*
* DESCRIPTION
*    This routine will find the entry for the given identifier in the
*    in-memory copy of the par file, and then update the string for that
*    identifier.
*
* INVOKED
*    status = cfp_putstr(id, name, string);
*
* PARAMETERS IN
*    FILE_ID id - par file id
*    char *name - identifier to look for
*    char *string - string to insert
*
* PARAMETERS OUT
*    none
*
* FUNCTION RETURNS
*    PASSFAIL - PASS if string updated successfully, FAIL if not
*
* GLOBALS
*    none
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* jab          27 Jan 1988     Original coding
* jrw          16 Feb 1991     check for null pointers
*
* $Log:	cfp_putstr.c,v $
 * Revision 2.3  96/11/19  22:50:29  isani
 * cleaned up rcsid
 * 
 * Revision 2.2  1994/12/21  12:44:18  thomas
 * Added "not just a lookup" flag to cfp_FindName call
 *
 * Revision 2.1  94/12/14  16:59:28  16:59:28  thomas (Jim Thomas)
 * Added #ifdef __LINT__ around RCS headers
 * Removed sys/types.h
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h, added cfp_internal.h
 * Changed CFHT_ERROR to CFHT_LOGONLY
 * Rewrote to use version 2 routines
 * 
 * Revision 1.1  91/09/24  17:32:12  17:32:12  steve (Steven Smith)
 * Original checkin
 * 
****************************************************************************!*/

#include <string.h>		  /* for string handling defines */
#include <errno.h>		  /* for error codes and errno define */
#include <stdlib.h>		  /* malloc(), free() */
#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* configuration file parsing defines */
#include "cfp_internal.h"	  /*   and internals */

RCSID("$Header: cfp_putstr.c,v 2.3 96/11/19 22:50:29 isani Exp $");

PASSFAIL			  /* return: PASS, or FAIL */
cfp_putstr(id, name, string)
    FILE_ID id;			  /* instance handle */
    char *name;			  /* identifier to look for */
    char *string;		  /* string to insert */
{
    FileEntryType *FileEntry;	  /* pointer to data on this file */
    LineType *Line;		  /* pointer to line for this name */
    char ValueCopy[CFP_STRSIZE + 1]; /* copy for results printout */

    if ((name == (char *) NULL) || (string == (char *) NULL)) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) given null pointer", __LINE__);
        return (FAIL);
    }

    cfht_logv(CFHT_LIBCFP, CFHT_DEBUG,
	      "(cfp_putstr) start, name = '%s', string = '%s'", name, string);

    if ((FileEntry = cfp_CheckID(id)) == (FileEntryType *) NULL) {
        cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) cfp_CheckID failed on %d for '%s'",
		  __LINE__, id, name);
        return(FAIL);
    }

    if ((Line = cfp_FindName(FileEntry, name, FALSE)) == (LineType *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) name '%s' not found for '%s'",
		  __LINE__, name, FileEntry->FilePath);
	return(FAIL);
    }

    if (Line->TypeCode == VariableLine) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) cannot change variable with putstr ('%s' for '%s')",
		  __LINE__, Line->NameString, FileEntry->FilePath);
	return(FAIL);
    } else if (Line->TypeCode != ConstantLine) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) line type (%d) not constant or variable",
		  __LINE__, Line->TypeCode);
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(...)           for '%s' for '%s'",
		  Line->NameString, FileEntry->FilePath);
	return(FAIL);
    }

    if (*string == VariableDefStartCharacter) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) constant starting with '%c' '%s=%s %s'",
		  __LINE__, VariableDefStartCharacter, Line->NameString,
		  string, FileEntry->FilePath);
	return(FAIL);
    }

    if (Line->OriginalValue != (char *) NULL)
      free((void *) Line->OriginalValue);
    if ((Line->OriginalValue
	 = (char *) malloc(strlen(string) + 1))
	== (char *) NULL) {
	cfht_logv(CFHT_LIBCFP, CFHT_LOGONLY,
		  "(cfp_putstr:%d) malloc %d failed for '%s' for '%s'",
		  __LINE__, strlen(string) + 1, Line->NameString,
		  FileEntry->FilePath);
	return(FAIL);
    }
    strcpy(Line->OriginalValue, string);
    Line->OriginalValueLength = strlen(string);

    if (Line->OriginalValueLength > CFP_STRSIZE) {
	strncpy(ValueCopy, Line->OriginalValue, CFP_STRSIZE);
	ValueCopy[CFP_STRSIZE] = '\0';
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_putstr) end (value = '%s')",
		  ValueCopy);
    } else {
	cfht_logv(CFHT_LIBCFP, CFHT_DEBUG, "(cfp_putstr) end (value = '%s')",
		  Line->OriginalValue);
    }

    return(PASS);
}
