/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */

/*!****************************************************************************
* FILE
*
* $Header: exp_detector.c,v 1.3 96/11/19 22:52:49 isani Exp $
* $Locker:  $
*
* ROUTINES
*
*
*    INVOKED
*    detector = exp_detector();
*
*    PARAMETERS IN
*    none
*
*    PARAMETERS OUT
*    none
*
*    FUNCTION RETURNS
*    pointer to string (local to this routine) containing detector name or
*    on error returns NULL and sets errno
*
*    GLOBALS
*    local space used to store return string
*
* HISTORY
* $Log:	exp_detector.c,v $
 * Revision 1.3  96/11/19  22:52:49  isani
 * cleaned up rcsid
 * added parens around truth statement
 * 
 * Revision 1.2  1994/12/14  17:35:24  thomas
 * Added RCS header
 * Changed cfht/cfht.h to cfht.h, cfht/cfp.h to cfp.h, cfht/states.h to states.h
 *
 * Revision 1.1  94/01/20  04:52:19  04:52:19  jwright (Jim Wright)
 * Initial revision
 * 
****************************************************************************!*/

#include <string.h>		  /* for string routines */
#include <errno.h>		  /* for errno */
#include <stdlib.h>		  /* for getenv() */

#include "cfht.h"		  /* libcfht defines */
#include "cfp.h"		  /* parser defines */
#include "states.h"		  /* exposure states */

RCSID("$Header: exp_detector.c,v 1.3 96/11/19 22:52:49 isani Exp $");

#define DETNAMELEN 80
#define CFHT_EDETECTOR "CFHTDETECTOR"

static char exp_detname[DETNAMELEN];

char *exp_detector()
{
    char *name;
    char *confpath;
    char *confname;
    char path[256];
    FILE_ID id;

    if (!(name = (char *)getenv(CFHT_EDETECTOR))) {
        confname = (char *)getenv(CFHT_ECONFNAME);
        if (!confname)
            confname = CFHT_CONFNAME;
        if ((confpath = cfp_file(path, confname, NULL, "r"))) {
            id = cfp_init(confpath);
            if (cfp_getstr(id, CFHT_DETECTOR, exp_detname, DETNAMELEN) != FAIL)
                name = exp_detname;
            (void)cfp_finish(id);
        }
    }

    if (!name) {
        errno = ENODEV;
        return(NULL);
    }

    if (strlen(name) > DETNAMELEN) {
        errno = E2BIG;
        return(NULL);
    }

    return(name);
}
