/* Copyright (C) 1991, 94    Canada-France-Hawaii Telescope Corp.        */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */
/*!****************************************************************************
* FILE
*
* $Header: exp_sigchld.c,v 1.2 94/12/14 17:39:59 thomas Exp $
* $Locker:  $
*
* ROUTINES
*
*    INVOKED
*
*    PARAMETERS IN
*
*    PARAMETERS OUT
*
*    FUNCTION RETURNS
*
*    GLOBALS
*
* HISTORY
* who          when            what
* ---------    ------------    ----------------------------------------------
* sss		20Mar1992	orignal, cloned from exp_alarm.c
*
* $Log:	exp_sigchld.c,v $
 * Revision 1.2  94/12/14  17:39:59  17:39:59  thomas (Jim Thomas)
 * Added RCS header
 * Changed cfht/cfht.h to cfht.h, cfht/states.h to states.h
 * Changed function prototype to use P macro
 * 
 * Revision 1.1  92/03/20  10:23:23  10:23:23  steve (Steven Smith)
 * Initial revision
 * 
****************************************************************************!*/

#include <stdio.h>
#include <signal.h>

#include "cfht.h"		  /* libcfht defines */
#include "states.h"		  /* exposure states */

#define MASK(s) (1L<<((s) - 1))
#define UNMASK(s) (0L<<((s) - 1))

#ifdef sun
#define sigvector(sig,vec,ovec) sigvec((sig),(vec),(ovec))
#endif

/* global to these functions */

static int exp_global_sigchld_event=0;    
static void exp_sigchld_trapper P((int sig));

/*!***************************************************************************
*  This function sets up the signal catching for SIGCHLD.
*
*  INVOKED: exp_sigchld (block)
*
*  PARAMETERS IN:
*      BOOLEAN block -> TRUE is block SIGCHLD
*                       FALSE is pass SIGCHLD
*
*  PARAMETERS OUT:
*      none
*
*  FUNCTION RETURNS:
*      void
*
*  GLOBALS: exp_global_sigchld_event
*
*  -Date- -by- - D e s c r i p t i o n -
*
*
***************************************************************************!*/

void exp_sigchld (block)
BOOLEAN block;
{
#ifndef __LINT__
    static char rcs_id[] =
        "$Header: exp_sigchld.c,v 1.2 94/12/14 17:39:59 thomas Exp $";
#endif
    static int init=0;
#ifdef DEBUGGER
    long oldmask;
#endif
    struct sigvec vec_ignore ;
    struct sigvec vec_trap ; 

    /* Init section */
    if (!init)  {

        /*  Ignore SIGINT */
        vec_ignore.sv_handler = SIG_IGN;
        vec_ignore.sv_mask = 0;
        vec_ignore.sv_onstack = 0;
        sigvector (SIGINT, &vec_ignore, (struct sigvec *)NULL);


        /* In setting up the event, SIGCHLD, block the signal initially */

        vec_trap.sv_handler = exp_sigchld_trapper;
        vec_trap.sv_mask = MASK (SIGCHLD);
        vec_trap.sv_onstack = 0;
        sigvector (SIGCHLD, &vec_trap, (struct sigvec *)NULL);

        init = 1;
    }

    /***********************************
    *  Handle hold/passing the signal  
    ***********************************/

    if (block == TRUE) {
#ifdef DEBUGGER
        oldmask = sigsetmask(~0L);        /* block all to get old value */
        fprintf(stderr,"block, was 0x%lx\n", oldmask);
        oldmask |= sigmask(SIGCHLD);        /* make SIGCHLD bit 1 */
        fprintf(stderr,"block, all block=0x%lx\n", sigsetmask(oldmask));  
        fprintf(stderr,"block, final=0x%lx\n", sigsetmask(oldmask));  
#else
        (void)sigsetmask(sigsetmask(~0L)|sigmask(SIGCHLD));
#endif
    } else {
#ifdef DEBUGGER
        oldmask = sigsetmask(~0L);        /* block all to get old value */
        fprintf(stderr,"unblock, was 0x%lx\n", oldmask);
        oldmask &= (~sigmask(SIGCHLD));     /* make SIGCHLD bit 0 */
        fprintf(stderr,"unblock, all block=0x%lx\n", sigsetmask(oldmask));  
        fprintf(stderr,"unblock, final=0x%lx\n", sigsetmask(oldmask));  
#else
        (void)sigsetmask(sigsetmask(~0L)&(~sigmask(SIGCHLD)));
#endif
    }
  
  return;
}

/*!***************************************************************************
*
* exp_sigchld_trapper -- internal to this routine, not to be called from elsewhere
*
***************************************************************************!*/

static void exp_sigchld_trapper (sig)
int sig;
{
#ifndef __LINT__
    static char rcs_id[] =
        "$Header: exp_sigchld.c,v 1.2 94/12/14 17:39:59 thomas Exp $";
#endif

    if (sig == SIGCHLD) 
        exp_global_sigchld_event = 1;

}

/*!***************************************************************************
*
*  NAME:  BOOLEAN exp_sigchld_event
*
*  This function tests and clears the global flag "exp_global_sigchld_event"
*  
*  The idea here is that if a SIGCHLD was set up and subsequently trapped in
*  exp_sigchld(), the global flag would have been set. When a process that is
*  pausing (see pause(2)) on this event is awaken, it should first check 
*  that the wakeup call was really for SIGCHLD.  
*
*  That is this function's job.  It will test that the flag is set, and if
*  so, clear it and return true.  Else return false.
*
*  INVOKED: while not (exp_sigchld_event ())
*               sleep
*           end while
*           proceed with state checking
*
*  PARAMETERS IN:
*      none
*
*  PARAMETERS OUT:
*      none
*
*  FUNCTION RETURNS:
*      TRUE if global flag set
*      FALSE  if spurrious event
*
*  GLOBALS: exp_global_event
*
*  -Date- -by- - D e s c r i p t i o n -
*
*  880930  jk  - Original version - exp_sig1event
*
***************************************************************************!*/

BOOLEAN exp_sigchld_event ()
{
#ifndef __LINT__
    static char rcs_id[] =
        "$Header: exp_sigchld.c,v 1.2 94/12/14 17:39:59 thomas Exp $";
#endif
    BOOLEAN rtn;
    long oldmask;

    oldmask = sigsetmask(~0L);    /* block all sigs */

    if (exp_global_sigchld_event) {
        exp_global_sigchld_event = 0;
        rtn=TRUE;
    } else {
        rtn=FALSE;
    }

    (void)sigsetmask(oldmask);        /* restore all sigs */
  
    return(rtn);
}
