/* Copyright (C) 1991, 94, 97    Canada-France-Hawaii Telescope Corp.    */
/* This program is distributed WITHOUT any warranty, and is under the    */
/* terms of the GNU General Public License, see the file COPYING    */

/*****************************************************************************
 *
 * file: states.h
 * $Header: states.h,v 1.10 2000/05/24 20:12:37 isani Exp $
 * $Locker:  $
 *
 * This file defines the various states & related goodies available to 
 * equipment, detectors, etc.
 *
 *
 * HISTORY
 *
 * who       when           what
 * ------    -----------    ----------------------------------------------
 * jk        23 Jan 1989    Original
 * sss       20 Mar 1989    added sigalarm stuffs
 * jrw       09 May 1991    added function definitions (ANSI and crufty flavors)
 *
 * $Log:	states.h,v $
 * Revision 1.10  2000/05/24  20:12:37  isani
 * Changed <cfht/cfht.h> to "cfht.h"
 * 
 * Revision 1.9  97/03/06  18:11:57  thomas
 * Reordered signal function definitions
 * Removed BLOCK/UNBLOCK, exp_sigTerm*, exp_sigInt*, exp_sigBlocker
 * 
 * Revision 1.8  96/11/19  22:55:43  22:55:43  isani (Sidik Isani)
 * changed void(*func)()'s to PFV's
 * 
 * Revision 1.7  95/06/15  12:20:20  12:20:20  thomas (Jim Thomas)
 * Added MaxStateSize for consistent array sizes
 * 
 * Revision 1.6  94/12/14  17:53:06  17:53:06  thomas (Jim Thomas)
 * Used P macro for function prototype definitions
 * 
 * Revision 1.5  94/01/20  04:54:44  04:54:44  jwright (Jim Wright)
 * add function declaration for determining controller type
 * 
 * Revision 1.4  93/03/05  16:24:10  16:24:10  steve (Steven Smith)
 * added exp_sig1event_fastcheck()
 * 
 * Revision 1.3  93/02/16  15:48:43  15:48:43  john (John Kerr)
 * added exp_instrument declarations
 * 
 * Revision 1.2  92/03/20  10:25:33  10:25:33  steve (Steven Smith)
 * added exp_sigchld, exp_sigbus stuff, ala sigalarm
 * 
 * Revision 1.1  91/09/24  17:54:00  17:54:00  steve (Steven Smith)
 * Original checkin
 * 
 ****************************************************************************/

#ifndef STATESDOTH
#define STATESDOTH

#include "cfht.h"		  /* for ANSI params macro */

/* exposure states */

#define MaxStateSize 30		  /* maximum length of a state name string */

#define EXP_EXPOSE          "expose" 
#define EXP_HOLD            "hold"  
#define EXP_EXTEND          "extend"  
#define EXP_RESTART         "restart"
#define EXP_RESUME          "resume"
#define EXP_STOP            "stop" 
#define EXP_ABORT           "abort"
#define EXP_IDLE            "idle"
#define EXP_GO              "go"
#define EXP_BREAK           "break"
#define EXP_NULL            "nada"

#define AUTO                "automatic"
#define MANUAL              "manual"
#define ENABLED             "enabled" 
#define DISABLED            "disabled"

#define LF_PATH             "/tmp/lock"    /* lock file pathname */

enum exp_controller_t exp_controller P((void));
char *exp_detector P((void));
char *exp_instrument P((void));
BOOLEAN exp_isfake P((char *det));
char *exp_curstate P((char *detector, int *pid));
int exp_newstate P((char *detector, char *newstate));
int exp_lockf P((char *detector));
PASSFAIL exp_kill P((int pid));
int stale P((int fd));
PASSFAIL exp_stop P((void));
PASSFAIL exp_abort P((void));
PASSFAIL exp_hold P((void));
PASSFAIL exp_restart P((void));

/* signal routines - do not use for any new code, use cfht_signal(3) */
void exp_sigalarm P((BOOLEAN block));
void exp_sigbus P((BOOLEAN block));
void exp_sigchld P((BOOLEAN block));
void exp_sigusr1 P((BOOLEAN block));
BOOLEAN exp_sigalarm_event P((void));
BOOLEAN exp_sigbus_event P((void));
BOOLEAN exp_sigchld_event P((void));
BOOLEAN exp_sig1event P((void));
int exp_sig1event_fastcheck P((void));

#endif /* STATESDOTH */
