#!/bin/sh
#
# Copyright (C) 1994 Canada-France-Hawaii Telescope Corporation
#
# Test script to drive testing of cfht_exec
#
# History
#
# JT	20-Sep-94	created for cfht_log.c
# JT	17-Oct-94	started adapted for cfp*
# JT	10-Jan-95	started adapting for cfht_exec
#

#
# function to clear out all the CFHT environment variables
#
# call:
#  ClearEnviroment
#
ClearEnvironment() {
  unset CFHTLOGS CFHTLOGU
  CFHTCOLORLOG=false;export CFHTCOLORLOG # display options
  CFHTERROR=true;export CFHTERROR
  CFHTWARN=true;export CFHTWARN
  CFHTNODISP=false;export CFHTNODISP
  for Lib in $Libraries
    do
      eval CFHT$Lib=false;export CFHT$Lib
    done;
  CFHTLIBCFHT=true;export CFHTLIBCFHT	# turn our guy back on
  CFHTLIBCFP=true;export CFHTLIBCFP	#  and for more help
  CFHTDEBUG=true;export CFHTDEBUG	# to see our messages
}

#
# function to run cfht_exec
#   no args
#
# Call:
#  TestIt
#

TestIt () {
#
# make a par file for exec to substitute from
#
  echo "GoodName = { , \"./execee\" }"				 >$ParFile
  echo "BadName = { , \"./NoSuchFoo\" }"			>>$ParFile
  echo "EXIT_PASS = 0"						>>$ParFile
  echo "EXIT_FAIL = 1"						>>$ParFile
  echo "EXIT_TWO = 2"						>>$ParFile
#
# run the program
#
  ./execor >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -ne 0 ]
    then
      echo "=======================================================" >>$LogFile
      echo "FAIL - cfht_exec did not work"			     >>$LogFile
      echo "---------- stderr (messages from cfht_exec()) ---------" >>$LogFile
      cat $stderrFile						     >>$LogFile
      echo "---------- stdout (messages from test routine) --------" >>$LogFile
      cat $stdoutFile						     >>$LogFile
  fi

#
# clean up
#
  /bin/rm -f $stdoutFile $stderrFile $ParFile
}

#
# set up known environment
#
stdoutFile=/usr/tmp/test$$.stdout
stderrFile=/usr/tmp/test$$.stderr
LogFile=test.Log
ParFile=execor.par

ClearEnvironment
cp /dev/null $LogFile

#
# run test
#

make all >$stderrFile 2>&1
Status=$?
if [ $Status -ne 0 ]
  then
    echo "******* Error making all, giving up"
    echo "====================================================" >>$LogFile
    echo "FAIL - error making all"				>>$LogFile
    echo "----------------------------------------------------" >>$LogFile
    cat $stderrFile						>>$LogFile
    rm -f $stderrFile
  else
    echo "testing cfht_exec()"
    TestIt
fi

#
# let user know if there were any problems found
#
if [ -s $LogFile ]
  then
    echo "******** there were errors, look at test.Log"
    grep "^FAIL" $LogFile
  else
    echo "no errors found"
    rm -f $LogFile
fi

#
# now clean up
#
make clean >/dev/null 2>&1

exit 0
