/*
 * Check_cursub.c - test cfp_cursub()
 *
 * This program is used to make sure that cfp_cursub() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * cursub errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * TrueBool = { , True }
 * FalseBool = { , False }
 * var = { , "some value" }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    char TestString[CFHT_VALUE_SIZE + 1]; /* cursub input/output values */
    char *cursubReturn;		  /* cursub return value */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    /* try legit substitution */
    strcpy(TestString, "a %var b $TrueBool c $FalseBool d");
    cursubReturn = cfp_cursub(FileId, TestString, sizeof TestString);
    if (cursubReturn == (char *) NULL) {
	printf("FAIL - legit cursub failed\n");
	ReturnValue++;
    } else {
	if (strcmp(TestString, "a some value b -TrueBool c  d") != 0) {
	    printf("FAIL - cursub of const returned '%s' not '%s'\n",
		   TestString, "a some value b -TrueBool c  d");
	    ReturnValue++;
	}
	if (strcmp(TestString, cursubReturn) != 0) {
	    printf("FAIL - cursub returned '%s' but put '%s' in dest\n",
		   cursubReturn, TestString);
	    ReturnValue++;
	}
    }

    /* try non-existing variables */
    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    strcpy(TestString, "a %foo b $bar c");
    cursubReturn = cfp_cursub(FileId, TestString, sizeof TestString);
    if (cursubReturn != (char *) NULL) {
	printf("FAIL - non-existing cursub did not fail, returned '%s'\n",
	       TestString);
	ReturnValue++;
    }

    /* try short answer space */
    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with too small failure",
	      __LINE__);
    strcpy(TestString, "a %var b $TrueBool c");
    cursubReturn = cfp_cursub(FileId, TestString, 1);
    if (cursubReturn != (char *) NULL) {
	printf("FAIL - too small cursub did not fail, returned '%s'\n",
	       TestString);
	ReturnValue++;
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
