/*
 * Check_getf.c - test cfp_getf()
 *
 * This program is used to make sure that cfp_getf() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * getf errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * var = { r, value }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    HistoryType History;	  /* return value from getcur */
    char TestString[CFP_STRSIZE]; /* output value from getcur */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    /* var = { r, value } */
    /* first change var (to make sure new value is not written out) */
    if (cfp_putcur(FileId, "var", (void *) "new value", CFP_STRING) != PASS) {
	    printf("FAIL - putcur of var failed\n");
	    ReturnValue++;
    }

    if ((FileId = cfp_getf(FileId)) == FAIL) {
	printf("FAIL - getf failed\n");
	ReturnValue++;
    } else {
	if ((History = cfp_getcur(FileId, "var", (void *) TestString,
				  CFP_STRING, sizeof TestString)) == FAIL) {
	    printf("FAIL - getcur after getf failed\n");
	    ReturnValue++;
	} else {
	    if (strcmp(TestString, "value") != 0) {
		printf("FAIL - getcur after getf returned '%s' not '%s'\n",
		       TestString, "value");
		ReturnValue++;
	    }
	}
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
