/*
 * Check_getinvokes.c - test cfp_getinvokes()
 *
 * This program is used to make sure that cfp_getinvokes() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * getinvokes errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * invokes = invoke1 invoke2 invoke3
 * invoke1 = { , value1 }                   # comment
 * invoke2 = { , value2 }                   # comment
 * invoke3 = { , value3 }                   # comment
 * invoke4 = { , value4 }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

char Answer[] = "\
invoke1 = { , value1 }                   # comment\n\
invoke2 = { , value2 }                   # comment\n\
invoke3 = { , value3 }                   # comment\n";

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    char *Value;		  /* output value from getinvokes */
    int Count;			  /* output value from getinvokes */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    if (cfp_getinvokes(FileId, &Value, &Count) == FAIL) {
	printf("FAIL - getinvokes failed\n");
	ReturnValue++;
    } else {
	if (Count != 3) {
	    printf("FAIL - getinvokes count %d != actual 3\n", Count);
	    ReturnValue++;
	}
	if (strcmp(Value, Answer) != 0) {
	    printf("FAIL - getinvokes return wrong\nwas:\n%s\nnot:\n%s\n",
		   Value, Answer);
	    ReturnValue++;
	}
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
