/*
 * Check_getln.c - test cfp_getln()
 *
 * This program is used to make sure that cfp_getln() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * getln errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * const = some value  # comment
 * var = { nds, value, default, supplied, range }   # c
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    char TestString[CFHT_VALUE_SIZE + 1]; /* getln output values */
    char *getlnReturn;		  /* getln return value */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    cfht_logv(CFHT_MAIN, CFHT_LOG_ID, argv[0]);

    /* const = some value  # comment */
    getlnReturn = cfp_getln(TestString, "const", argv[0], ".par");
    if (getlnReturn == (char *) NULL) {
	printf("FAIL - getln of const failed\n");
	ReturnValue++;
    } else {
	if (strcmp(TestString, "some value") != 0) {
	    printf("FAIL - getln of const returned '%s' not '%s'\n",
		   TestString, "some value");
	    ReturnValue++;
	}
	if (strcmp(TestString, getlnReturn) != 0) {
	    printf("FAIL - getln returned '%s' but put '%s' in dest\n",
		   getlnReturn, TestString);
	    ReturnValue++;
	}
    }

    /* var = { nds, value, default, supplied, range }   # c */
    getlnReturn = cfp_getln((char *) NULL, "var", argv[0], ".par");
    if (getlnReturn == (char *) NULL) {
	printf("FAIL - getln var failed\n");
	ReturnValue++;
    } else {
	if (strcmp(getlnReturn, "value")
	    != 0) {
	    printf("FAIL - getln of var returned '%s' != '%s'\n", getlnReturn,
		   "value");
	    ReturnValue++;
	}
    }

    /* non-existent name */
    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    getlnReturn = cfp_getln(TestString, "nosuchvar", argv[0], ".par");
    if (getlnReturn != (char *) NULL) {
	printf("FAIL - getln of nosuchvar did not fail, returned '%s'\n",
	       TestString);
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
