/*
 * Check_getsiz.c - test cfp_getsiz()
 *
 * This program is used to make sure that cfp_getsiz() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * getsiz errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * var1 = {  }
 * var2 = { , abcd }
 * var3 = { , "abc def" }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    int getsizReturn;		  /* return value from getsiz */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    /* var1 = {  } */
    if ((getsizReturn = cfp_getsiz(FileId, "var1")) == FAIL) {
	printf("FAIL - getsiz of var1 failed\n");
	ReturnValue++;
    } else {
	if (getsizReturn != 0) {
	    printf("FAIL - getsiz of var1 returned %d not 0\n", getsizReturn);
	    ReturnValue++;
	}
    }

    /* var2 = { , abcd } */
    if ((getsizReturn = cfp_getsiz(FileId, "var2")) == FAIL) {
	printf("FAIL - getsiz of var2 failed\n");
	ReturnValue++;
    } else {
	if (getsizReturn != (int) strlen("abcd")) {
	    printf("FAIL - getsiz of var2 returned %d not %d\n",
		   getsizReturn, strlen("abcd"));
	    ReturnValue++;
	}
    }

    /* var3 = { , "abc def" } */
    if ((getsizReturn = cfp_getsiz(FileId, "var3")) == FAIL) {
	printf("FAIL - getsiz of var3 failed\n");
	ReturnValue++;
    } else {
	if (getsizReturn != (int) strlen("abc def")) {
	    printf("FAIL - getsiz of var3 returned %d not %d\n",
		   getsizReturn, strlen("abc def"));
	    ReturnValue++;
	}
    }

    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    if ((getsizReturn = cfp_getsiz(FileId, "nosuchvar")) != FAIL) {
	printf("FAIL - getsiz nosuchvar did not fail, returned %d\n",
	       getsizReturn);
	ReturnValue++;
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
