/*
 * Check_getvars.c - test cfp_getvars()
 *
 * This program is used to make sure that cfp_getvars() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * getvars errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * variables = var1 var2 var3
 * var1 = { , value1 }                         # comment
 * var2 = { , value2 }                         # comment
 * var3 = { , value3 }                         # comment
 * var4 = { , value4 }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

char Answer[] = "\
var1 = { , value1 }                         # comment\n\
var2 = { , value2 }                         # comment\n\
var3 = { , value3 }                         # comment\n";


main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    char *Value;		  /* output value from getvars */
    int Count;			  /* output value from getvars */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    if (cfp_getvars(FileId, &Value, &Count) == FAIL) {
	printf("FAIL - getvars failed\n");
	ReturnValue++;
    } else {
	if (Count != 3) {
	    printf("FAIL - getvars count %d != actual 3\n", Count);
	    ReturnValue++;
	}
	if (strcmp(Value, Answer) != 0) {
	    printf("FAIL - getvars return wrong\nwas:\n%s\nnot:\n%s\n",
		   Value, Answer);
	    ReturnValue++;
	}
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
