/*
 * Check_putln.c - test cfp_putln()
 *
 * This program is used to make sure that cfp_putln() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * putln errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * const = some value  # comment
 * var = { ndsr, value, default, supplied, range }   # c
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    char TestString[CFHT_VALUE_SIZE + 1]; /* getstr output values */
    char *getlnReturn;		  /* getln return value */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    cfht_logv(CFHT_MAIN, CFHT_LOG_ID, argv[0]);

    /* const = some value  # comment */
    if (cfp_putln("new value", "const", argv[0], ".par") != PASS) {
	printf("FAIL - putln to const failed\n");
	ReturnValue++;
    } else {
	getlnReturn = cfp_getln(TestString, "const", argv[0], ".par");
	if (getlnReturn == (char *) NULL) {
	    printf("FAIL - getln of const failed\n");
	    ReturnValue++;
	} else {
	    if (strcmp(TestString, "new value") != 0) {
		printf("FAIL - getln of const returned '%s' not '%s'\n",
		       TestString, "new value");
		ReturnValue++;
	    }
	}
    }

    /* var = { ndsr, value, default, supplied, range }   # c */
    if (cfp_putln("{ o, \"a b\" }", "var", argv[0], ".par") != PASS) {
	printf("FAIL - putln to var failed\n");
	ReturnValue++;
    } else {
	getlnReturn = cfp_getln(TestString, "var", argv[0], ".par");
	if (getlnReturn == (char *) NULL) {
	    printf("FAIL - getln var failed\n");
	    ReturnValue++;
	} else {
	    if (strcmp(TestString, "{ o, \"a b\" }") != 0) {
		printf("FAIL - getln var returned '%s' != '%s'\n", TestString,
		       "{ o, \"a b\" }");
		ReturnValue++;
	    }
	}
    }

    /* non-existent name */
    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    if (cfp_putln("foo", "nosuchvar", argv[0], ".par") != FAIL) {
	printf("FAIL - putln of nosuchvar did not fail\n");
	ReturnValue++;
	getlnReturn = cfp_getln(TestString, "nosuchvar", argv[0], ".par");
	if (getlnReturn != (char *) NULL) {
	    printf("FAIL - nor did getln, returned '%s'\n", TestString);
	    ReturnValue++;
	}
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
