/*
 * Check_putstr.c - test cfp_putstr()
 *
 * This program is used to make sure that cfp_putstr() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * putstr errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file:
 *
 * const = some value
 * var = { nds, value, default, supplied, range }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    char TestString[CFP_STRSIZE + 1]; /* getstr returned values */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    /* const = some value */
    if (cfp_putstr(FileId, "const", (void *) "new value") != PASS) {
	printf("FAIL - putstr to const failed\n");
	ReturnValue++;
    } else {
	if (cfp_getstr(FileId, "const", (void *) TestString, sizeof TestString)
	    != PASS) {
	    printf("FAIL - getstr of const failed\n");
	    ReturnValue++;
	} else {
	    if (strcmp(TestString, "new value") != 0) {
		printf("FAIL - getstr of const returned '%s' not '%s'\n",
		       TestString, "new value");
		ReturnValue++;
	    }
	}
    }

    /* var = { nds, value, default, supplied, range } */
    if (cfp_putstr(FileId, "var", (void *) "{ o, \"a b\" }") != FAIL) {
	printf("FAIL - putstr to var did not fail\n");
	ReturnValue++;
    } else {
	if (cfp_getstr(FileId, "var", (void *) TestString, sizeof TestString)
	    != PASS) {
	    printf("FAIL - getstr var failed\n");
	    ReturnValue++;
	} else {
	    printf("FAIL - getstr var returned '%s'\n", TestString);
	}
    }

    /* non-existent name */
    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    if (cfp_putstr(FileId, "nosuchvar", (void *) "foo") != FAIL) {
	printf("FAIL - putstr of nosuchvar did not fail\n");
	ReturnValue++;
	if (cfp_getstr(FileId, "nosuchvar", (void *) TestString,
		       sizeof TestString) != FAIL) {
	    printf("FAIL - nor did getstr, returned '%s'\n", TestString);
	    ReturnValue++;
	}
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
