/*
 * Check_rmline.c - test cfp_rmline()
 *
 * This program is used to make sure that cfp_rmline() works correctly.  Its
 * expected environment is that the calling shell script has created a .par
 * file as defined below and called this program with no arguments.  This runs
 * through a set of tests outputting any unexpected results to stdout.  Each
 * output starts with "FAIL" for the calling shell script to find.  If any
 * rmline errors occurred, this exits with a positive error count.  If any
 * parse errors occurred, this exits with -1.
 *
 * The expected .par file is different orders of the following three after a
 * comment line:
 *
 * const = abc
 * var1 = { nds, , False, True }  # some comment
 * var2 = { , "some value" }
 *
 *
 * CFHTLOGS and CFHTLOGU are expected to be undefined (though this doesn't
 * care)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cfht.h"
#include "cfp.h"
#include "exits.h"

main(argc, argv)
    int argc;
    char *argv[];
{
    FILE_ID FileId;		  /* file index from parse */
    HistoryType History;	  /* return value from getcur */
    char TestString[CFP_STRSIZE]; /* output value from getcur */
    int ReturnValue;		  /* starts at zero, +1 for each error */

    ReturnValue = 0;

    if ((FileId = cfp_parse(CFP_PAR, argc, argv)) == FAIL) {
	printf("FAIL - cfp_parse died on %s\n", argv[0]);
	return(EXIT_FAIL);
    }

    /* const = abc */
    if (cfp_rmline(FileId, "const") == FAIL) {
	printf("FAIL - rmline of const failed\n");
	ReturnValue++;
    } else {
	cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		  "(main:%d) following is supposed to fail with lookup failure",
		  __LINE__);
	if ((History = cfp_getcur(FileId, "const", TestString, CFP_STRING,
				  sizeof TestString)) != FAIL) {
	    printf("FAIL - rmline did not, const = '%s'\n", TestString);
	    ReturnValue++;
	}
    }

    /* var1 = { nds, , False, True }  # some comment */
    if (cfp_rmline(FileId, "var1") == FAIL) {
	printf("FAIL - rmline of var1 failed\n");
	ReturnValue++;
    } else {
	cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		  "(main:%d) following is supposed to fail with lookup failure",
		  __LINE__);
	if ((History = cfp_getcur(FileId, "var1", TestString, CFP_STRING,
				  sizeof TestString)) != FAIL) {
	    printf("FAIL - rmline did not, var1 = '%s'\n", TestString);
	    ReturnValue++;
	}
    }

    /* var2 = { , "some value" } */
    if (cfp_rmline(FileId, "var2") == FAIL) {
	printf("FAIL - rmline of var2 failed\n");
	ReturnValue++;
    } else {
	cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
		  "(main:%d) following is supposed to fail with lookup failure",
		  __LINE__);
	if ((History = cfp_getcur(FileId, "var2", TestString, CFP_STRING,
				  sizeof TestString)) != FAIL) {
	    printf("FAIL - rmline did not, var2 = '%s'\n", TestString);
	    ReturnValue++;
	}
    }

    cfht_logv(CFHT_MAIN, CFHT_LOGONLY,
	      "(main:%d) following is supposed to fail with lookup failure",
	      __LINE__);
    if (cfp_rmline(FileId, "nosuchvar") != FAIL) {
	printf("FAIL - rmline nosuchvar did not fail\n");
	ReturnValue++;
    }

    if (cfp_putf(FileId) != PASS) {
	printf("FAIL - cfp_putf failed\n");
	ReturnValue++;
    }

    if (cfp_finish(FileId) != PASS) {
	printf("FAIL - cfp_finish failed\n");
	ReturnValue++;
    }

    return(ReturnValue);
    /*NOTREACHED*/
}
