#!/usr/bin/sh
#
# Copyright (C) 1994-2001 Canada-France-Hawaii Telescope Corporation
#
# Test script to drive testing of cfp*.c
#
# Please don't tabify this, several parse checks are sensitive to that
#
# History
#
# JT	20-Sep-94	created for cfht_log.c
# JT	17-Oct-94	started adapted for cfp*
# JT	18-Oct-94	cfp_file()
# JT	19-Oct-94	cfp_parse()
# JT	20-Oct-94	cfp_getcur(), cfp_putcur()
# JT	24-Oct-94	cfp_getval()
# JT	25-Oct-94	cfp_putval(), cfp_getstr(), cfp_putstr(), cfp_getln(),
#			cfp_putln(), cfp_cursub(), cfp_getsiz(), cfp_rmline(),
#			cfp_getf(), cfp_getvars()
# JT	26-Oct-94	cfp_getinvokes()
# JT	31-Oct-94	added test to cfp_getcur() for bug Mark found
# JT	02-Nov-94	more detail on errors found message
# JT	09-Nov-94	put mode strings into alpha order, added null constant
# JT	14-Nov-94	added cfp_range() and cfp_rangemess()
# JT	02-Feb-95	updated tests for range, added cfp_editvalue()
# JT	09-Feb-95	added tests for indirect ranges
# JT    02-Jul-98	added tests for removing empty comments
# JT    22-Aug-01       added test for [] in names, updated to /cfht/conf
#

#
# function to clear out all the CFHT environment variables
#
# call:
#  ClearEnviroment
#
ClearEnvironment() {
  unset CFHTLOGS CFHTLOGU
  CFHTCOLORLOG=false;export CFHTCOLORLOG # display options
  CFHTERROR=true;export CFHTERROR
  CFHTWARN=true;export CFHTWARN
  CFHTNODISP=false;export CFHTNODISP
  CFHTDEBUG=false;export CFHTDEBUG
  for Lib in $Libraries
    do
      eval CFHT$Lib=false;export CFHT$Lib
    done;
  CFHTLIBCFP=true;export CFHTLIBCFP	# turn our guy back on
}

#
# function to convert simple . and ~ paths to full paths
#
# call:
#  CleanPath <path>
# e.g.:
#  CleanPath .
#
CleanPath () {
  Path=$1
  case $Path in
    ~/*)  Path=$HOME`echo $Path | cut -c 2-200`;;
    ~)	  Path=$HOME;;
    \./*) Path=`pwd``echo $Path | cut -c 2-200`;;
    \.)	  Path=`pwd`;;
  esac
  echo $Path
}

#
# function to run the cfp_file() routine against a file
#   args:
#	source directory .par file is to be created in
#	mode (e.g., 644) for created .par file (if null, 644)
#	access (i.e., rf, r, or rw) for .par file access (if null, rw)
#	target directory .par file should end up in (if null, same as source)
#	value for CFHTCONFPATH environment variable (if null, unset)
#   understands simple ~ and . at front of path
#
# call:
#  TestFile <source-dir> <mode> <access> <target-dir> <CFHTCONFPATH>
# e.g.:
#  TestFile .
#
TestFile () {
  Source=$1			# save arguments
  Mode=$2
  Access=$3
  Target=$4
  CONF=$5

  if [ x$CONF = "x" ]
    then
      unset CFHTCONFPATH
    else
      CFHTCONFPATH=$CONF;export CFHTCONFPATH
  fi

  if [ x$Target = "x" ]
    then
      Target=$Source
  fi

  if [ x$Access = "x" ]
    then
      Access=rw
  fi

  if [ x$Mode = "x" ]
    then
      Mode=644
  fi

  FullSource=`CleanPath $Source`
  FullTarget=`CleanPath $Target`
  if [ $FullTarget = `pwd` ]
    then
      Expected=
    else
      Expected=$FullTarget
  fi

#
# create the .par file (just the name actually - doesn't need anything inside)
#
  if [ \( $FullSource = $HOME \) -o \( $FullSource = /h/$Session \) ]
    then
      RealParFile=.,$ParFile
    else
      RealParFile=$ParFile
  fi
  touch $FullSource/$RealParFile
  chmod $Mode $FullSource/$RealParFile

  if [ \( $FullTarget = $HOME \) -o \( $FullTarget = /h/$Session \) ]
    then
      RealTarget=.,$ParFile
    else
      RealTarget=$ParFile
  fi

#
# run the program
#
# echo "Testing cfp_file() on $Source/$ParFile($Mode) to $Target($Access)"
  ./Check_file $ParFile "$Expected" $Access >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -ne 0 ]
    then
      echo "=======================================================" >>$LogFile
      echo "FAIL - cfp_file() from $Source($Mode) to $Target($Access) ($Status)" >>$LogFile
      echo "	   ex=$Expected	 fs=$FullSource	 ft=$FullTarget"     >>$LogFile
      echo "	   rp=$RealParFile  rt=$RealTarget"		     >>$LogFile
      echo "---------- stderr (messages from cfp_file()) ----------" >>$LogFile
      cat $stderrFile						     >>$LogFile
      echo "---------- stdout (messages from Check_file) ----------" >>$LogFile
      cat $stdoutFile						     >>$LogFile
  fi

#
# make sure the target file exists and is writable if access is rw
#
  if [ $Access = "rw" -a ! -w $FullTarget/$RealTarget ]
    then
      echo "=======================================================" >>$LogFile
      echo "FAIL - cfp_file() did not create target for rw access"   >>$LogFile
      echo "	   from $Source($Mode) to $Target($Access)"	     >>$LogFile
      echo "	   ex=$Expected	 fs=$FullSource	 ft=$FullTarget"     >>$LogFile
      echo "	   rp=$RealParFile  rt=$RealTarget"		     >>$LogFile
      if [ -f $FullTarget/$RealTarget ]
	then
	  echo "       ls $Target/$ParFile = `ls -ld $FullTarget/$RealTarget`" >>$LogFile
	else
	  echo "       $Target/$ParFile not there at all"	     >>$LogFile
      fi
      echo "---------- stderr (messages from cfp_file()) ----------" >>$LogFile
      cat $stderrFile						     >>$LogFile
      echo "---------- stdout (messages from Check_file) ----------" >>$LogFile
      cat $stdoutFile						     >>$LogFile
  fi

#
# clean up
#
  /bin/rm -f $FullSource/$ParFile $FullTarget/$ParFile $stdoutFile $stderrFile
  /bin/rm -f $HOME/$ParFile $HOME/.,$ParFile ./$ParFile
}

#
# function to run Check_parse on Check_parse.par and diff the results
#
# call it with
#    the options for diff (either -t or -bwt)
#    the arguments to be used in the actual program call, e.g.
#	-num 1 -boolean -string "foo and bar"
#  for example:
#	TestParseDiff -t -num 1 -boolean -string "foo and bar"
#
# expects the .par file to be created already, but it saves a copy for diff'ing
# and deletes both when finished
#
TestParseDiff () {
  diffOptions=$1
  shift 1
  cp Check_parse.par Check_parse.par.ORIG
  ./Check_parse $* >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -ne 0 ]
    then
      echo "=====================================================" >>$LogFile
      echo "FAIL - diff cfp_parse() routine failure"		   >>$LogFile
      echo "----------command------------------------------------" >>$LogFile
      echo ./Check_parse $*					   >>$LogFile
      echo "----------stderr (messages from cfp routines) -------" >>$LogFile
      cat $stderrFile						   >>$LogFile
      echo "----------stdout (messages from Check_parse) --------" >>$LogFile
      cat $stdoutFile						   >>$LogFile
      echo "----------orginal Check_parse.par--------------------" >>$LogFile
      cat Check_parse.par.ORIG					   >>$LogFile
      echo "----------new Check_parse.par------------------------" >>$LogFile
      cat Check_parse.par					   >>$LogFile
    else
      if diff $diffOptions Check_parse.par.ORIG Check_parse.par >/dev/null 2>&1
	then
	  :
	else
	  echo "===================================================" >>$LogFile
	  echo "FAIL - cfp_parse() input and output are different"   >>$LogFile
	  echo "----------commands---------------------------------" >>$LogFile
	  echo ./Check_parse $*					     >>$LogFile
	  echo diff $diffOptions Check_parse.par.ORIG Check_parse.par>>$LogFile
	  echo "----------stderr (messages from cfp routines) -----" >>$LogFile
	  cat $stderrFile					     >>$LogFile
	  echo "----------stdout (messages from Check_parse) ------" >>$LogFile
	  cat $stdoutFile					     >>$LogFile
	  echo "----------orginal Check_parse.par------------------" >>$LogFile
	  cat Check_parse.par.ORIG				     >>$LogFile
	  echo "----------new Check_parse.par----------------------" >>$LogFile
	  cat Check_parse.par					     >>$LogFile
	  echo "----------diff output------------------------------" >>$LogFile
	  diff $diffOptions Check_parse.par.ORIG Check_parse.par>>$LogFile 2>&1
      fi	  
  fi

#
# clean up
#
  /bin/rm -f Check_parse.par.ORIG Check_parse.par $stdoutFile $stderrFile
}

#
# function to run Check_parse on Check_parse.par and grep the results
#
# call it with
#    a pattern for 'grep pattern Check_parse.par'
#	i.e., the pattern will be bare in the command used
#	the pattern should be such that if it is not found parse has failed
#    the arguments to be used in the actual program call, e.g.
#	-num 1 -boolean -string "foo and bar"
#  for example:
#	TestParseGrep "'string.*foo and'" -num 1 -boolean -string "foo and bar"
#
# expects the .par file to be created already and deletes it when finished
#
TestParseGrep () {
  grepPattern=$1
  shift 1
  cp Check_parse.par Check_parse.par.ORIG
  eval ./Check_parse $* >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -ne 0 ]
    then
      echo "=====================================================" >>$LogFile
      echo "FAIL - grep cfp_parse() routine failure"		   >>$LogFile
      echo "----------command------------------------------------" >>$LogFile
      echo ./Check_parse $*					   >>$LogFile
      echo "----------stderr (messages from cfp routines) -------" >>$LogFile
      cat $stderrFile						   >>$LogFile
      echo "----------stdout (messages from Check_parse) --------" >>$LogFile
      cat $stdoutFile						   >>$LogFile
      echo "----------orginal Check_parse.par--------------------" >>$LogFile
      cat Check_parse.par.ORIG					   >>$LogFile
      echo "----------new Check_parse.par------------------------" >>$LogFile
      cat Check_parse.par					   >>$LogFile
    else
      if eval grep -q "$grepPattern" Check_parse.par
	then
	  :
	else
	  echo "===================================================" >>$LogFile
	  echo "FAIL - cfp_parse() grep on output did not find it"   >>$LogFile
	  echo "----------commands---------------------------------" >>$LogFile
	  echo ./Check_parse $*					     >>$LogFile
	  echo "grep $grepPattern Check_parse.par"		     >>$LogFile
	  echo "----------stderr (messages from cfp routines) -----" >>$LogFile
	  cat $stderrFile					     >>$LogFile
	  echo "----------stdout (messages from Check_parse) ------" >>$LogFile
	  cat $stdoutFile					     >>$LogFile
	  echo "----------orginal Check_parse.par------------------" >>$LogFile
	  cat Check_parse.par.ORIG				     >>$LogFile
	  echo "----------new Check_parse.par----------------------" >>$LogFile
	  cat Check_parse.par					     >>$LogFile
      fi	  
  fi

#
# clean up
#
  /bin/rm -f Check_parse.par.ORIG Check_parse.par $stdoutFile $stderrFile
}

#
# function to run Check_parse on Check_parse.par when a FAIL is expected
#
# call it with
#    the arguments to be used in the actual program call, e.g.
#	-num 1 -boolean -string "foo and bar"
#  for example:
#	TestParseFail -num 1 -boolean -string "foo and bar"
#
# expects the .par file to be created already and deletes it when finished
#
TestParseFail () {
  cp Check_parse.par Check_parse.par.ORIG
  eval ./Check_parse $* >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -eq 0 ]
    then
      echo "=====================================================" >>$LogFile
      echo "FAIL -  cfp_parse() routine did not fail"		   >>$LogFile
      echo "----------command------------------------------------" >>$LogFile
      echo ./Check_parse $*					   >>$LogFile
      echo "----------stderr (messages from cfp routines) -------" >>$LogFile
      cat $stderrFile						   >>$LogFile
      echo "----------stdout (messages from Check_parse) --------" >>$LogFile
      cat $stdoutFile						   >>$LogFile
      echo "----------orginal Check_parse.par--------------------" >>$LogFile
      cat Check_parse.par.ORIG					   >>$LogFile
      echo "----------new Check_parse.par------------------------" >>$LogFile
      cat Check_parse.par					   >>$LogFile
    else
      if [ $Status -gt 127 ]
	then
	  echo "===================================================" >>$LogFile
	  echo "FAIL -	cfp_parse() routine bombed ($Status)"	     >>$LogFile
	  echo "----------command----------------------------------" >>$LogFile
	  echo ./Check_parse $*					     >>$LogFile
	  echo "----------stderr (messages from cfp routines) -----" >>$LogFile
	  cat $stderrFile					     >>$LogFile
	  echo "----------stdout (messages from Check_parse) ------" >>$LogFile
	  cat $stdoutFile					     >>$LogFile
	  echo "----------orginal Check_parse.par------------------" >>$LogFile
	  cat Check_parse.par.ORIG				     >>$LogFile
#	else
# there really should be a provision here for grep'ing stderrFile to make
# sure the expected error message actually happened ....
      fi
  fi

#
# clean up
#
  /bin/rm -f Check_parse.par.ORIG Check_parse.par $stdoutFile $stderrFile
}

#
# function to run a routine check program on its par file
#
# call it with
#    the program name
#    any arguments the program wants
#  for example:
#	TestOne Check_getcur -int1 13
#
# expects the .par file to be created already and deletes it when finished
# expects to make the executable
#
TestOne () {
  Routine=$1
  shift 1
  make $Routine >$stderrFile 2>&1
  Status=$?
  if [ $Status -ne 0 ]
    then
      echo "******* Error making $Routine, skipping"
      echo "====================================================" >>$LogFile
      echo "FAIL - error making $Routine"			  >>$LogFile
      echo "----------------------------------------------------" >>$LogFile
      cat $stderrFile						  >>$LogFile
    else
      echo "testing cfp_`echo $Routine | sed 's/Check_//'`()"
      cp $Routine.par $Routine.par.ORIG
      ./$Routine $* >$stdoutFile 2>$stderrFile
      Status=$?
      if [ $Status -ne 0 ]
	then
	  echo "===================================================" >>$LogFile
	  echo "FAIL - $Routine saw problems (count = $Status) "     >>$LogFile
	  echo "----------command----------------------------------" >>$LogFile
	  echo ./$Routine $*					     >>$LogFile
	  echo "----------stderr (messages from cfp routines) -----" >>$LogFile
	  cat $stderrFile					     >>$LogFile
	  echo "----------stdout (messages from $Routine) ---------" >>$LogFile
	  cat $stdoutFile					     >>$LogFile
	  echo "----------orginal $Routine.par---------------------" >>$LogFile
	  cat $Routine.par.ORIG					     >>$LogFile
	  echo "----------new $Routine.par-------------------------" >>$LogFile
	  cat $Routine.par					     >>$LogFile
      fi
  fi
#
# clean up
#
  /bin/rm -f $Routine.par.ORIG $Routine.par $stdoutFile $stderrFile
}

#
# function to run rmline check program on its par file
#
# call it with
#	Testrmline
#
# expects the .par file to be created already and deletes it when finished
#
Testrmline () {
  ./Check_rmline $* >$stdoutFile 2>$stderrFile
  Status=$?
  if [ $Status -eq 0 ]
    then
      if cmp -s Check_rmline.min Check_rmline.par
	then
	  : # it worked
	else
	  echo "===================================================" >>$LogFile
	  echo "FAIL - Check_rmline output wrong "		     >>$LogFile
	  echo "----------commands---------------------------------" >>$LogFile
	  echo ./Check_rmline $*				     >>$LogFile
	  echo cmp -s Check_rmline.min Check_rmline.par		     >>$LogFile
	  echo "----------stderr (messages from cfp routines) -----" >>$LogFile
	  cat $stderrFile					     >>$LogFile
	  echo "----------stdout (messages from Check_rmline) -----" >>$LogFile
	  cat $stdoutFile					     >>$LogFile
	  echo "----------orginal Check_rmline.par-----------------" >>$LogFile
	  cat Check_rmline.par.ORIG				     >>$LogFile
	  echo "----------new Check_rmline.par---------------------" >>$LogFile
	  cat Check_rmline.par					     >>$LogFile
	  echo "----------should be--------------------------------" >>$LogFile
	  cat Check_rmline.min					     >>$LogFile
      fi
    else
      echo "==================================================="     >>$LogFile
      echo "FAIL - Check_rmline saw problems (count = $Status) "     >>$LogFile
      echo "----------command----------------------------------"     >>$LogFile
      echo ./Check_rmline $*					     >>$LogFile
      echo "----------stderr (messages from cfp routines) -----"     >>$LogFile
      cat $stderrFile						     >>$LogFile
      echo "----------stdout (messages from Check_rmline) ---------" >>$LogFile
      cat $stdoutFile						     >>$LogFile
      echo "----------orginal Check_rmline.par---------------------" >>$LogFile
      cat Check_rmline.par.ORIG					     >>$LogFile
      echo "----------new Check_rmline.par-------------------------" >>$LogFile
      cat Check_rmline.par					     >>$LogFile
  fi
#
# clean up
#
  /bin/rm -f Check_rmline.par $stdoutFile $stderrFile
}

#
# set up known environment
#
ParFile=test$$.par
stdoutFile=/usr/tmp/test$$.stdout
stderrFile=/usr/tmp/test$$.stderr
LogFile=test.Log

ClearEnvironment
cp /dev/null $LogFile
Session=pegasus;export Session

#
# first test cfp_file()
#

make Check_file >$stderrFile 2>&1
Status=$?
if [ $Status -ne 0 ]
  then
    echo "******* Error making Check_file, skipping"
    echo "====================================================" >>$LogFile
    echo "FAIL - error making Check_file"			>>$LogFile
    echo "----------------------------------------------------" >>$LogFile
    cat $stderrFile						>>$LogFile
  else
    echo "testing cfp_file()"
#  TestFile <source-dir> <mode> <access> <target-dir> <CFHTCONFPATH>
    TestFile .
    TestFile . 444 r
    TestFile ~
    TestFile /usr/tmp 444 rf /usr/tmp /usr/tmp
    TestFile /usr/tmp 444 r ~ /usr/tmp
    TestFile /usr/tmp 644 rw ~ /usr/tmp
    TestFile $cfhtdir/conf 444 rf
    TestFile $cfhtdir/conf 444 r ~
    TestFile $cfhtdir/conf 644 rw ~
    TestFile /h/pegasus 444 r
    TestFile /h/pegasus 444 rw ~

# make sure it fails well too
    ./Check_file foo$$bar$$.par /erehwon rw >$stdoutFile 2>$stderrFile
    Status=$?
    if [ $Status -eq 0 ]
      then
	echo "=====================================================" >>$LogFile
	echo "FAIL - found non-existent file"			     >>$LogFile
	echo "---------- stderr (messages from cfp_file()) --------" >>$LogFile
	cat $stderrFile						     >>$LogFile
	echo "---------- stdout (messages from Check_file) --------" >>$LogFile
	cat $stdoutFile						     >>$LogFile
      else
	if [ $Status -gt 127 ]
	  then
	    echo "=================================================" >>$LogFile
	    echo "FAIL - look for non-existent file bombed"	     >>$LogFile
	    echo "---------- stderr (messages from cfp_file()) ----" >>$LogFile
	    cat $stderrFile					     >>$LogFile
	    echo "---------- stdout (messages from Check_file) ----" >>$LogFile
	    cat $stdoutFile					     >>$LogFile
	fi
    fi
fi

#
# now check cfp_parse()
#

make Check_parse >$stderrFile 2>&1
Status=$?
if [ $Status -ne 0 ]
  then
    echo "******* Error making Check_parse, skipping"
    echo "====================================================" >>$LogFile
    echo "FAIL - error making Check_parse"			>>$LogFile
    echo "----------------------------------------------------" >>$LogFile
    cat $stderrFile						>>$LogFile
  else
    echo "testing cfp_parse()"

# should fail on empty file :-)
    cp /dev/null Check_parse.par
    TestParseFail

# just blank lines
    echo ""						       >Check_parse.par
    echo "		  "				      >>Check_parse.par
    echo "			"			      >>Check_parse.par
    echo "" >>Check_parse.par
    TestParseDiff -wt

# just comments
    echo "# comment 1"					       >Check_parse.par
    echo "	  # comment 2"				      >>Check_parse.par
    echo "#"						      >>Check_parse.par
    echo "    #"					      >>Check_parse.par
    TestParseDiff -wt

# things that do not change with parsing
    echo "# null constant"				       >Check_parse.par
    echo "cnull = "					      >>Check_parse.par
    echo "# null constant with comment"			      >>Check_parse.par
    echo "cnullc =                                  # comment">>Check_parse.par
    echo "# simple unquoted string"			      >>Check_parse.par
    echo "simple = abc  def 123 \$%^&"			      >>Check_parse.par
    echo "#   plus comment"				      >>Check_parse.par
    echo "simplec = abc  def 123 \$%^&     # comment"	      >>Check_parse.par
    echo "#   plus quotes"				      >>Check_parse.par
    echo "simpleq = \"abc  def 123 \$%^&\""		      >>Check_parse.par
    echo "# null variable"				      >>Check_parse.par
    echo "null = {  }"					      >>Check_parse.par
    echo "#   plus comment"				      >>Check_parse.par
    echo "nullc = {  }            # comment"		      >>Check_parse.par
    echo "# just a value"				      >>Check_parse.par
    echo "value = { , 123 }"				      >>Check_parse.par
    echo "#   plus quotes"				      >>Check_parse.par
    echo "valueq = { , \"abc def  ghi\" }"		      >>Check_parse.par
    echo "#   plus nasty quotes"			      >>Check_parse.par
#note that sh is broken relative to the documentation (on both HP and Sun)
#     hence the following strange number of backslashes
    echo "valuen = { , \" \\\\\\\\ \\\" # ' } \" }"	      >>Check_parse.par
    echo "# default"					      >>Check_parse.par
    echo "defaultv = { , , abcd }"			      >>Check_parse.par
    echo "# supplied"					      >>Check_parse.par
    echo "suppliedv = { , , , abcd }"			      >>Check_parse.par
    echo "# range possibilities"			      >>Check_parse.par
    echo "intrange = { , , , , int }"			      >>Check_parse.par
    echo "doublerange = { , , , , double }"		      >>Check_parse.par
    echo "realintrange = { , , , , \" [ -145, +201 ) \" }"    >>Check_parse.par
    echo "realdblrange = { , , , , \" [ -14.5, +2.1 ) \" }"   >>Check_parse.par
    echo "realtimerange = { , , , , \" [ 0, 24:: ) \" }"      >>Check_parse.par
    echo "expintrange = { , , , , \"int [ -145, +201 ) \" }"  >>Check_parse.par
    echo "expdblrange = { , , , , \"d [ -145, +201 ) \" }"    >>Check_parse.par
    echo "exptimerange = { , , , , \"ti [ -145, +201 ) \" }"  >>Check_parse.par
    echo "# all legit modes"				      >>Check_parse.par
    echo "dodefault = { d, , foo }"			      >>Check_parse.par
    echo "dosupplied = { s, , , bar }"			      >>Check_parse.par
    echo "valoptional = { o }"				      >>Check_parse.par
    echo "valnever = { n }"				      >>Check_parse.par
    echo "# alias"					      >>Check_parse.par
    echo "dalias -> null"				      >>Check_parse.par
    echo "#   plus comment"				      >>Check_parse.par
    echo "daliasc -> null        # comment"		      >>Check_parse.par
    echo "# indirect alias"				      >>Check_parse.par
    echo "ialias -> %ThisWontActuallyWork"		      >>Check_parse.par
    echo "# check all label characters"			      >>Check_parse.par
    echo "abcdefghijklmnopqrstuvwxyz = 1"		      >>Check_parse.par
    echo "ABCDEFGHIJKLMNOPQRSTUVWXYZ = 1"		      >>Check_parse.par
    echo "a0123456789%&._\$+[a] = 1"			      >>Check_parse.par
    TestParseDiff -t

# make sure empty comments get eaten properly
    echo "const = a           #"			       >Check_parse.par
    TestParseGrep "'^const = a\$'"

# now see how argument processing does
# should be able to set the value
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool1.*r, false'" -bool1 false
#    to anything, actually, there's no type at this level :-)
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool1.*r, jello'" -bool1 jello

# check a couple of invalid modes
    echo "modea = { a, True }"				       >Check_parse.par
    TestParseFail -modea False
    echo "modem = { m, True }"				       >Check_parse.par
    TestParseFail -modem False
    echo "modet = { t, True }"				       >Check_parse.par
    TestParseFail -modet False

# check [] in names
    echo "filter[A] = { r, 1 }"				       >Check_parse.par
    echo "filter[B] = { r, 2 }"				      >>Check_parse.par
    echo "filter[C] = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'filter\[B\].*r, 4'" "-filter[B]" 4

# check name abbreviation
    echo "bool = { r, False }"				       >Check_parse.par
    echo "int = { , 1 }"				      >>Check_parse.par
    TestParseGrep "'bool.*r, True'" -boo True
    echo "bool = { r, False }"				       >Check_parse.par
    echo "int = { , 1 }"				      >>Check_parse.par
    TestParseGrep "'bool.*r, True'" -bo True
    echo "bool = { r, False }"				       >Check_parse.par
    echo "int = { , 1 }"				      >>Check_parse.par
    TestParseGrep "'bool.*r, True'" -b True

# should FAIL if ambiguous
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    echo "bool3 = { dor, , False }"			      >>Check_parse.par
    echo "bool4 = { dor, , False }"			      >>Check_parse.par
    TestParseFail -bool

# should FAIL if ambiguous, even if starting in middle
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    echo "int = { dor, , 0 }"				      >>Check_parse.par
    echo "bool3 = { dor, , False }"			      >>Check_parse.par
    TestParseFail -int 1 -bool

# should find later match
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    echo "bool3 = { dor, , False }"			      >>Check_parse.par
    echo "bool = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool .*r, here'" -bool here

# should FAIL if not there
    echo "bool = { r, True }"				       >Check_parse.par
    TestParseFail -int
# even if no variables there at all
    echo "# nothing here"				       >Check_parse.par
    TestParseFail -int
# or nothing at all :-)
    cp /dev/null Check_parse.par
    TestParseFail -int

# should ignore second if duplicate
    echo "bool = { or, True }"				       >Check_parse.par
    echo "bool = { r, True }"				      >>Check_parse.par
    TestParseGrep "'bool.*or, on '" -bool on
    echo "bool = { or, True }"				       >Check_parse.par
    echo "int = { r, 1 }"				      >>Check_parse.par
    echo "bool = { r, True }"				      >>Check_parse.par
    TestParseGrep "'bool.* or, off '" -bool on -int 2 -bool off
    echo "bool = { or, True }"				       >Check_parse.par
    echo "int = { r, 1 }"				      >>Check_parse.par
    echo "bool = { r, True }"				      >>Check_parse.par
    TestParseGrep "'bool.* r, True '" -bool on -int 2 -bool off

# a straight alias should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool -> bool"				      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refbool false
# an abbreviated reference should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool -> bool"				      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -r false
# an abbreviated indirect should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool -> b"					      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refbool false
# a non-existent indirect should fail
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool -> loob"				      >>Check_parse.par
    TestParseFail -refbool false

# an indirect alias should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , bool }"				      >>Check_parse.par
    echo "refrefbool -> %refbool"			      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refrefbool false
# an abbreviated reference should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , bool }"				      >>Check_parse.par
    echo "refrefbool -> %refbool"			      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refr false
# an abbreviated indirect should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , bool }"				      >>Check_parse.par
    echo "refrefbool -> %refb"				      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refrefbool false
# an abbreviated double indirect should work
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , b }"				      >>Check_parse.par
    echo "refrefbool -> %refbool"			      >>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refrefbool false
# a non-existent indirect reference should fail
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , bool }"				      >>Check_parse.par
    echo "refrefbool -> %loob"				      >>Check_parse.par
    TestParseFail "'bool.*r, false '" -refrefbool false
# a non-existent double indirect should fail
    echo "bool = { r, True }"				       >Check_parse.par
    echo "refbool = { , loob }"				      >>Check_parse.par
    echo "refrefbool -> %refbool"			      >>Check_parse.par
    TestParseFail "'bool.*r, false '" -refrefbool false

# several levels should work (sorry about the name inconsistency :-)
    echo "bool = { r, True }"				       >Check_parse.par
    echo "ibool = bool"					      >>Check_parse.par
    echo "refbool = { , ibool }"			      >>Check_parse.par
    echo "refrefbool -> %refbool"			      >>Check_parse.par
    echo "refrefrefbool = { , refrefbool }"		      >>Check_parse.par
    echo "refrefrefrefbool -> %refrefrefbool"		      >>Check_parse.par
    echo "refrefrefrefrefbool = { , refrefrefrefbool }"	      >>Check_parse.par
    echo "refrefrefrefrefrefbool -> %refrefrefrefrefbool"     >>Check_parse.par
    echo "refrefrefrefrefrefrefbool -> %refrefrefrefrefrefbool">>Check_parse.par
    TestParseGrep "'bool.*r, false '" -refrefrefrefrefrefrefbool false

# and a loop should fail
    echo "bool = { r, True }"				       >Check_parse.par
    echo "ref1 -> ref2"					      >>Check_parse.par
    echo "ref2 -> ref3"					      >>Check_parse.par
    echo "ref3 -> ref1"					      >>Check_parse.par
    TestParseFail -ref1 down
# and a harder one
    echo "bool = { r, True }"				       >Check_parse.par
    echo "ref1 -> %ref2"				      >>Check_parse.par
    echo "ref2 = { , ref3 }"				      >>Check_parse.par
    echo "ref3 -> ref1"					      >>Check_parse.par
    TestParseFail -ref1 down

# make sure we cannot change a constant
    echo "const = boo and far"				       >Check_parse.par
    TestParseFail -const "'bar and foo'"

# try floaters
    echo "&floater0 = { r, True }"			       >Check_parse.par
    TestParseGrep "'&floater0.*r, off '" off
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    TestParseGrep "'&floater1.*r, in '" off in
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    TestParseGrep "'&floater2.*r, out '" off in out
# with variables too
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'&floater0.*r, off '" -int1 11 off -int2 22 in out -int3 33
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'&floater1.*r, in '" -int1 11 off -int2 22 in out -int3 33
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'&floater2.*r, out '" -int1 11 off -int2 22 in out -int3 33
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'int1.*r, 11 '" -int1 11 off -int2 22 in out -int3 33
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'int2.*r, 22 '" -int1 11 off -int2 22 in out -int3 33
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    echo "&floater2 = { r, True }"			      >>Check_parse.par
    echo "int1 = { r, 1 }"				      >>Check_parse.par
    echo "int2 = { r, 2 }"				      >>Check_parse.par
    echo "int3 = { r, 3 }"				      >>Check_parse.par
    TestParseGrep "'int3.*r, 33 '" -int1 11 off -int2 22 in out -int3 33
# and too many should complain
    echo "&floater0 = { r, True }"			       >Check_parse.par
    echo "&floater1 = { r, True }"			      >>Check_parse.par
    TestParseFail off in out

# from here on, believe that the name lookup works?

# od not mentioned on command line should leave it alone
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool2.*r, ,'"
# should allow specific value
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool2.*r, true'" -bool2 true
# just the variable should use the default
    echo "bool1 = { r, True }"				       >Check_parse.par
    echo "bool2 = { dor, , False }"			      >>Check_parse.par
    TestParseGrep "'bool2.*r, False'" -bool2

# nd should set the value from default
    echo "bool = { dnr, False, True }"			       >Check_parse.par
    TestParseGrep "'bool.*r, True'" -bool
#   and should leave it alone if not mentioned
    echo "bool = { dnr, False, True }"			       >Check_parse.par
    TestParseGrep "'bool.*r, False'"
#   and should FAIL if value given
    echo "bool = { dnr, False, True }"			       >Check_parse.par
    TestParseFail -bool true

# os should leave it alone if not mentioned
    echo "bool = { ors, , , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, ,'"
#   and use supplied if mentioned without value
    echo "bool = { ors, , , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, True'" -bool
#   and allow setting value
    echo "bool = { ors, , , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, off'" -bool off

# ns should leave it alone if not mentioned
    echo "bool = { nrs, , , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, ,'"
#   and use supplied if mentioned without value
    echo "bool = { nrs, , , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, True'" -bool
#   and FAIL if value given
    echo "bool = { nrs, , , True }"			       >Check_parse.par
    TestParseFail -bool true

# ods should leave it alone if not mentioned
    echo "bool = { dors, , False, True }"		       >Check_parse.par
    TestParseGrep "'bool.*s, ,'"
#   and use supplied if mentioned without value
    echo "bool = { dors, , False, True }"		       >Check_parse.par
    TestParseGrep "'bool.*s, True'" -bool
#   and allow setting value
    echo "bool = { dors, , False, True }"		       >Check_parse.par
    TestParseGrep "'bool.*s, off'" -bool off

# this section tests the "don't update the .par file" possibilities
#  to make sure the file doesn't change
    echo "bool = { , True }"				       >Check_parse.par
    TestParseGrep "'bool.*{ , True'" -bool off
    echo "bool = { do, False, True }"			       >Check_parse.par
    TestParseGrep "'bool.*o, False'" -bool
    echo "bool = { dn, False, True }"			       >Check_parse.par
    TestParseGrep "'bool.*n, False'" -bool
    echo "bool = { os, False, , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, False'" -bool
    echo "bool = { ns, False, , True }"			       >Check_parse.par
    TestParseGrep "'bool.*s, False'" -bool
    echo "bool = { dos, , False, True }"		       >Check_parse.par
    TestParseGrep "'bool.*s, ,'" -bool
    echo "bool = { dns, , False, True }"		       >Check_parse.par
    TestParseGrep "'bool.*s, ,'" -bool

# test the integer range checking
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseGrep "'int.*r, 40,'" -int 40
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseGrep "'int.*r, \" -40\",'" -int "' -40'"
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseGrep "'int.*r, 0xff,'" -int 0xff
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseFail -int abc
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseFail -int 0123abc
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseFail -int abc123abc
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseFail -int 40.1
    echo "int = { r, 1, , , int }"			       >Check_parse.par
    TestParseFail -int 12:0:0

    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 0,'" -int 0
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 100,'" -int 100
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 200,'" -int 200
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseFail -int "' -1'"
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseFail -int 201
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseFail -int foobar
    echo "int = { r, 1, , , \"[0,200]\" }"		       >Check_parse.par
    TestParseFail -int 123foobar

    echo "int = { r, 1, , , \"i[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 100,'" -int 100
    echo "int = { r, 1, , , \"in[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 100,'" -int 100
    echo "int = { r, 1, , , \"int[0,200]\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 100,'" -int 100

    echo "int = { r, 1, , , \"(0,200)\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 1,'" -int 1
    echo "int = { r, 1, , , \"(0,200)\" }"		       >Check_parse.par
    TestParseGrep "'int.*r, 199,'" -int 199
    echo "int = { r, 1, , , \"(0,200)\" }"		       >Check_parse.par
    TestParseFail -int 0
    echo "int = { r, 1, , , \"(0,200)\" }"		       >Check_parse.par
    TestParseFail -int 200

# test the double range checking
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseGrep "'double.*r, 18.1,'" -double 18.1
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseGrep "'double.*r, \" -10.0\",'" -double "' -10.0'"
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseGrep "'double.*r, 14,'" -double 14
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseFail -double abc
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseFail -double 1.0abc
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseFail -double abc1.0
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseGrep "'double.*r, 10e-6,'" -double 10e-6
    echo "double = { r, 1.0, , , double }"		       >Check_parse.par
    TestParseGrep "'double.*r, \"1:1:1\",'" -double 1:1:1

    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseGrep "'double.*r, 0,'" -double 0
    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseGrep "'double.*r, 100,'" -double 100
    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseGrep "'double.*r, 200,'" -double 200
    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseFail -double "' -0.000000001'"
    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseFail -double 200.00000001
    echo "double = { r, 1.0, , , \"[0.0,200.0]\" }"	       >Check_parse.par
    TestParseFail -double 20foo

    echo "double = { r, 1.0, , , \"double [0.0,200.0]\" }"     >Check_parse.par
    TestParseGrep "'double.*r, 0,'" -double 0
    echo "double = { r, 1.0, , , \"d [0.0,200.0]\" }"          >Check_parse.par
    TestParseGrep "'double.*r, 0,'" -double 0
    echo "double = { r, 1.0, , , \"do[0.0,200.0]\" }"          >Check_parse.par
    TestParseGrep "'double.*r, 0,'" -double 0

    echo "double = { r, 1.0, , , \"(0.0,200.0)\" }"	       >Check_parse.par
    TestParseGrep "'double.*r, 0.00000001,'" -double 0.00000001
    echo "double = { r, 1.0, , , \"(0.0,200.0)\" }"	       >Check_parse.par
    TestParseGrep "'double.*r, 199.9999999'" -double 199.9999999
    echo "double = { r, 1.0, , , \"(0.0,200.0)\" }"	       >Check_parse.par
    TestParseFail -double 0
    echo "double = { r, 1.0, , , \"(0.0,200.0)\" }"	       >Check_parse.par
    TestParseFail -double 200

    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseGrep "'time.*r, \"0:0:00\",'" -time 0:0:00
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseGrep "'time.*r, \"23:59:59.9\",'" -time 23:59:59.9
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseFail -time "' -5:0:0'"
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseFail -time 24:00:00
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseFail -time 1:300:20
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseFail -time 1:1:300
    echo "time = { r, 0, , , \"[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseFail -time 1:10:10abc

    echo "time = { r, 0, , , \"d[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseGrep "'time.*r, \"0:0:00\",'" -time 0:0:00
    echo "time = { r, 0, , , \"t[0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseGrep "'time.*r, \"0:0:00\",'" -time 0:0:00
    echo "time = { r, 0, , , \"time [0:0:0,24:0:0)\" }"	       >Check_parse.par
    TestParseGrep "'time.*r, \"0:0:00\",'" -time 0:0:00

# this section checks string quoting
    echo "string = { r, foo }"				       >Check_parse.par
    TestParseGrep "'string.*r, bar '" -string bar
    echo "string = { r, foo }"				       >Check_parse.par
    TestParseGrep "'string.*r, \"foo and bar \" '" -string "'foo and bar '"
    echo "string = { r, foo }"				       >Check_parse.par
# I suggest you don't mess with the following :-)
    TestParseGrep "'string.*r, \" #\\\\\\\\ \\\\\" '" -string "' #\\ \" '"

    echo "string = { r, \"foo and bar\" }"		       >Check_parse.par
    TestParseGrep "'string.*r, \"bar and foo \" '" -string "'bar and foo '"
    echo "string = { r, \"foo and bar\" }"		       >Check_parse.par
    TestParseGrep "'string.*r, mumble '" -string mumble

fi

#
# now check all of the small routines
# where there's a get/put pair, check the get first so we can use it in put
#
# These are different from parse in that one call to a program for each
# routine is done.  The .par file is set up here, and what is written here must
# agree with what the program expects :-)
#

# cfp_getcur()
echo "const = some value"				      >Check_getcur.par
echo "bool1 = { , True }"				     >>Check_getcur.par
echo "bool2 = { dn, , True }"				     >>Check_getcur.par
echo "bool3 = { ns, , , True }"				     >>Check_getcur.par
echo "bool4 = { dns, , True, On }"			     >>Check_getcur.par
echo "bool5 = { dns, , , On }"				     >>Check_getcur.par
echo "int1 = { , 1 }"					     >>Check_getcur.par
echo "int2 = { , 2, , , int }"				     >>Check_getcur.par
echo "int3 = { , 3, , , \"[0,10]\" }"			     >>Check_getcur.par
echo "int4 = { , -1, , , \"[0,10]\" }"			     >>Check_getcur.par
echo "int5 = { , 11, , , \"[0,10]\" }"			     >>Check_getcur.par
echo "int6 = { , abc, , , int }"			     >>Check_getcur.par
echo "double1 = { , 1 }"				     >>Check_getcur.par
echo "double2 = { , 2, , , double }"			     >>Check_getcur.par
echo "double3 = { , 3, , , \"[0.0,10.0]\" }"		     >>Check_getcur.par
echo "double4 = { , -0.001, , , \"[0.0,10.0]\" }"	     >>Check_getcur.par
echo "double5 = { , 10.0001, , , \"[0.0,10.0]\" }"	     >>Check_getcur.par
echo "double6 = { , abc, , , double }"			     >>Check_getcur.par
echo "time1 = { , \"0:1:0\" }"				     >>Check_getcur.par
echo "time2 = { , \"1:0:0\", , , double }"		     >>Check_getcur.par
echo "time3 = { , \"0:0:0\", , , \"(0:0:0,1:0:0)\" }"	     >>Check_getcur.par
echo "time4 = { , \"5:5:5\", , , \"[0:0:0,1:0:0]\" }"	     >>Check_getcur.par
echo "time5 = { , \"0:5:0abc\", , , double }"		     >>Check_getcur.par
echo "string = { , foo }"				     >>Check_getcur.par
TestOne Check_getcur

# cfp_putcur()
echo "const = some value"				      >Check_putcur.par
echo "bool = { , True }"				     >>Check_putcur.par
echo "int1 = { , 1 }"					     >>Check_putcur.par
echo "int2 = { , 2, , , int }"				     >>Check_putcur.par
echo "int3 = { , 3, , , \"[0,10]\" }"			     >>Check_putcur.par
echo "double1 = { , 1 }"				     >>Check_putcur.par
echo "double2 = { , 2, , , double }"			     >>Check_putcur.par
echo "double3 = { , 3, , , \"[0.0,10.0]\" }"		     >>Check_putcur.par
echo "time1 = { , \"1:0:0\", , , double }"		     >>Check_putcur.par
echo "time2 = { , \"0:0:0\", , , \"(0:0:0,1:0:0)\" }"	     >>Check_putcur.par
echo "string = { , foo }"				     >>Check_putcur.par
TestOne Check_putcur

# cfp_getval() including environment fetch
echo "const = this should fail"				      >Check_getval.par
echo "var = { dns, value, default, supplied, range }"	     >>Check_getval.par
echo "bool = { dns, original, false, true }"		     >>Check_getval.par
echo "int1 = { , 1 }"					     >>Check_getval.par
echo "int2 = { , 2, , , \"[0,10]\" }"			     >>Check_getval.par
echo "int3 = { , 30, , , \"[0,10]\" }"			     >>Check_getval.par
echo "double = { , 5.0 }"				     >>Check_getval.par
echo "TEST = {  }"					     >>Check_getval.par
echo "INT = {  }"					     >>Check_getval.par
echo "NOT_there = {  }"					     >>Check_getval.par
Check_getvalTEST="abc 123";export Check_getvalTEST
Check_getvalINT=123;export Check_getvalINT
TestOne Check_getval

# cfp_putval() including environment store
echo "const = this should fail"				      >Check_putval.par
echo "var = { dns, 100, default, supplied, int }"	     >>Check_putval.par
echo "bool = { dns, original, false, true }"		     >>Check_putval.par
echo "int1 = { , 1 }"					     >>Check_putval.par
echo "int2 = { , 2, , , \"[0,10]\" }"			     >>Check_putval.par
echo "double = { , 5.0 }"				     >>Check_putval.par
echo "TEST = {  }"					     >>Check_putval.par
echo "INT = {  }"					     >>Check_putval.par
echo "NOT_there = {  }"					     >>Check_putval.par
Check_putvalTEST="abc 123";export Check_putvalTEST
Check_putvalINT=123;export Check_putvalINT
TestOne Check_putval

# cfp_getstr()
echo "const = some value"				      >Check_getstr.par
echo "var = { dns, value, default, supplied, range }"	     >>Check_getstr.par
TestOne Check_getstr

# cfp_putstr()
echo "const = some value"				      >Check_putstr.par
echo "var = { dns, value, default, supplied, range }"	     >>Check_putstr.par
TestOne Check_putstr

# cfp_getln()
echo "const = some value  # comment"			      >Check_getln.par
echo "var = { dns, value, default, supplied, range }   # c"  >>Check_getln.par
TestOne Check_getln

# cfp_putln()
echo "const = some value  # comment"			      >Check_putln.par
echo "var = { dnrs, value, default, supplied, range }   # c" >>Check_putln.par
TestOne Check_putln

# cfp_cursub()
echo "TrueBool = { , True }"				      >Check_cursub.par
echo "FalseBool = { , False }"				     >>Check_cursub.par
echo "var = { , \"some value\" }"			     >>Check_cursub.par
TestOne Check_cursub

# cfp_getsiz()
echo "var1 = {  }"					      >Check_getsiz.par
echo "var2 = { , abcd }"				     >>Check_getsiz.par
echo "var3 = { , \"abc def\" }"				     >>Check_getsiz.par
TestOne Check_getsiz

# cfp_range() and cfp_rangemess()
echo "int = { , , , , int }"				      >Check_range.par
echo "intsp = { , , , , \"  int  \" }"			     >>Check_range.par
echo "double = { , , , , double }"			     >>Check_range.par
echo "linf = { , , , , \" ( , 0 ] \" }"			     >>Check_range.par
echo "rinf = { , , , , \" [ 0, ) \" }"			     >>Check_range.par
echo "cc = { , , , , \" [ 0 , 10 ] \" }"		     >>Check_range.par
echo "oc = { , , , , \" ( 0 , 10 ] \" }"		     >>Check_range.par
echo "oo = { , , , , \" ( 0 , 10 ) \" }"		     >>Check_range.par
echo "co = { , , , , \" [ 0 , 10 ) \" }"		     >>Check_range.par
echo "indintmin = { , , , , \" [ %indint, 15 ] \" }"	     >>Check_range.par
echo "indintmax = { , , , , \" [ 1, %indint ] \" }"	     >>Check_range.par
echo "indint = { , 5 }"					     >>Check_range.par
echo "inddoublemin = { , , , , \" [ %inddouble, 15. ] \" }"  >>Check_range.par
echo "inddoublemax = { , , , , \" [ 1., %inddouble ] \" }"   >>Check_range.par
echo "inddouble = { , 5.0 }"				     >>Check_range.par
echo "float = { , , , , \" [ -0.1e-5 , +0.1E-5 ] \" }"	     >>Check_range.par
echo "time = { , , , , \" [ 0:0:0 , 10:0:0 ) \" }"	     >>Check_range.par
echo "norange = { , }"					     >>Check_range.par
echo "const = foobar"					     >>Check_range.par
# cfp_editvalue()
echo "editint = { , , , , int }"			     >>Check_range.par
echo "editdouble = { , , , , double }"			     >>Check_range.par
echo "edittime = { , , , , time }"			     >>Check_range.par
TestOne Check_range

# cfp_rmline()
make Check_rmline >$stderrFile 2>&1
Status=$?
if [ $Status -ne 0 ]
  then
    echo "******* Error making Check_rmline, skipping"
    echo "===================================================="   >>$LogFile
    echo "FAIL - error making Check_rmline"			  >>$LogFile
    echo "----------------------------------------------------"   >>$LogFile
    cat $stderrFile						  >>$LogFile
  else
    echo "testing cfp_rmline()"
    echo "# some comment"				      >Check_rmline.min
    cp Check_rmline.min Check_rmline.par
    echo "const = abc"					     >>Check_rmline.par
    echo "var1 = { dns, , False, True }  # some comment"     >>Check_rmline.par
    echo "var2 = { , \"some value\" }"			     >>Check_rmline.par
    Testrmline
    cp Check_rmline.min Check_rmline.par
    echo "var1 = { dns, , False, True }  # some comment"     >>Check_rmline.par
    echo "var2 = { , \"some value\" }"			     >>Check_rmline.par
    echo "const = abc"					     >>Check_rmline.par
    Testrmline
    cp Check_rmline.min Check_rmline.par
    echo "var2 = { , \"some value\" }"			     >>Check_rmline.par
    echo "const = abc"					     >>Check_rmline.par
    echo "var1 = { dns, , False, True }  # some comment"     >>Check_rmline.par
    Testrmline
    /bin/rm -f Check_rmline.min
fi

# cfp_getf()
echo "var = { r, value }"				      >Check_getf.par
TestOne Check_getf

# cfp_getvars() (though not tested very hard - but then no one uses it :-(
echo "variables = var1 var2 var3"			     >Check_getvars.par
echo "var1 = { , value1 }                         # comment">>Check_getvars.par
echo "var2 = { , value2 }                         # comment">>Check_getvars.par
echo "var3 = { , value3 }                         # comment">>Check_getvars.par
echo "var4 = { , value4 }"				    >>Check_getvars.par
TestOne Check_getvars

# cfp_getinvokes() (though not tested very hard - but then no one uses it :-(
echo "invokes = invoke1 invoke2 invoke3"		  >Check_getinvokes.par
echo "invoke1 = { , value1 }                   # comment">>Check_getinvokes.par
echo "invoke2 = { , value2 }                   # comment">>Check_getinvokes.par
echo "invoke3 = { , value3 }                   # comment">>Check_getinvokes.par
echo "invoke4 = { , value4 }"				 >>Check_getinvokes.par
TestOne Check_getinvokes

#
# let user know if there were any problems found
#
if [ -s $LogFile ]
  then
    echo "******** there were errors, look at test.Log"
    grep FAIL $LogFile
  else
    echo "no errors found"
    rm -f $LogFile
fi

#
# now clean up
#
make clean >/dev/null 2>&1

exit 0
