/****************************************************************************
    Copyright (C) 1991	Canada-France-Hawaii Telescope Corp.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact information:
    CFHT, PO Box 1597, Kamuela, HI 96743, USA
    daprog@cfht.hawaii.edu

****************************************************************************/
/* string.c
 *
 * test the cfht_string() routine and demonstrate how to use
 */


#include <cfht/cfht.h>
#include <stdio.h>
#include <string.h>
#include <cfht/exits.h>

void usage();
void main();


void main(argc,argv)
int argc;
char *argv[];
{
    char in[1024];
    char *out;
    struct cfht_rng_string str_rng;
    struct cfht_rng_char   chr_rng;

    if (argc != 2) {
        usage();
    }

    switch (argv[1][0]) {
    case 'm':    /* check for match */
    case 'M':
        str_rng.string_array = (char **)calloc(5, sizeof(char *));
        str_rng.string_array[0] = "foo";
        str_rng.string_array[1] = "bar";
        str_rng.string_array[2] = "baz";
        str_rng.string_array[3] = "cfht";
        str_rng.string_array[4] = NULL;
        while (! feof(stdin)) {
            gets(in);
            out = cfht_string(in, CFHT_STR_MATCH, (cfht_range_t)(&str_rng));
            if (out == NULL)
                fprintf(stderr, "bomb - cfht_errno = %d\n", cfht_errno);
            else
                if (*out == NULL)
                    printf("in = '%s' did not match any\n", in);
                else
                    printf("in = '%s' matched out = '%s'\n", in, out);
            free(out);
        }
        break;

    case 'i':    /* include specified chars */
    case 'I':
    chr_rng.list_of_chars = strdup("aeiouyAEIOUY");
        while (! feof(stdin)) {
            gets(in);
            out = cfht_string(in, CFHT_STR_INCLUDE, (cfht_range_t)&chr_rng);
            if (out == NULL)
                fprintf(stderr, "bomb - cfht_errno = %d\n", cfht_errno);
            else
                if (*out == NULL)
                    printf("in = '%s' reduced to empty string\n", in);
                else
                    printf("in = '%s'\nout = '%s'\n", in, out);
            free(out);
        }
        break;

    case 'e':    /* exclude specified chars */
    case 'E':
    chr_rng.list_of_chars = strdup("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        while (! feof(stdin)) {
            gets(in);
            out = cfht_string(in, CFHT_STR_EXCLUDE, (cfht_range_t)&chr_rng);
            if (out == NULL)
                fprintf(stderr, "bomb - cfht_errno = %d\n", cfht_errno);
            else
                if (*out == NULL)
                    printf("in = '%s' reduced to empty string\n", in);
                else
                    printf("in = '%s'\nout = '%s'\n", in, out);
            free(out);
        }
        break;

    case 'w':    /* strip whitespace */
    case 'W':
        while (! feof(stdin)) {
            gets(in);
            out = cfht_string(in, CFHT_STR_NOWHITE, NULL);
            if (out == NULL)
                fprintf(stderr, "bomb - cfht_errno = %d\n", cfht_errno);
            else
                if (*out == NULL)
                    printf("in = '%s' reduced to empty string\n", in);
                else
                    printf("in = '%s'\nout = '%s'\n", in, out);
            free(out);
        }
        break;

    case 't':    /* strip trailing whitespace */
    case 'T':
        while (! feof(stdin)) {
            gets(in);
            out = cfht_string(in, CFHT_STR_NOTRAIL, NULL);
            if (out == NULL)
                fprintf(stderr, "bomb - cfht_errno = %d\n", cfht_errno);
            else
                if (*out == NULL)
                    printf("in = '%s' reduced to empty string\n", in);
                else
                    printf("in = '%s'\nout = '%s'\n", in, out);
            free(out);
        }
        break;

    default:
        fprintf(stderr, "unrecognized option: '%c'\n", argv[1][0]);
        usage();
        break;
    }
}


void usage()
{
    fprintf(stderr, "usage: string option\n");
    fprintf(stderr, "    where option is one of\n");
    fprintf(stderr, "    M - check for match\n");
    fprintf(stderr, "    I - only allow characters in a set\n");
    fprintf(stderr, "    E - only allow characters not in a set\n");
    fprintf(stderr, "    W - strip out all whitespace characters\n");
    fprintf(stderr, "    T - strip out all trailing whitespace chars\n");
/*  fprintf(stderr, "    S - convert non-printing chars to codes\n"); */
/*  fprintf(stderr, "    U - convert codes to non-printing chars\n"); */
    exit (EXIT_FAIL);
}

