/****************************************************************************
    Copyright (C) 1991	Canada-France-Hawaii Telescope Corp.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact information:
    CFHT, PO Box 1597, Kamuela, HI 96743, USA
    daprog@cfht.hawaii.edu

****************************************************************************/
#include <errno.h>
#include <cfht/cfht.h>
#include <cfht/cfp.h>
#include <cfht/ui.h>
#include <cfht/exits.h>

struct uifieldtb tb =
    {
    UI_FIELDTB,
    0.0, 0.0,
    -1, -1,
    "Test Form"
    };

struct uifieldst ststring =
    {
    UI_FIELDST,
    1.0, 2.0,
    -1, 0,
    "string",
    -1
    };

struct uifieldte string =
    {
    UI_FIELDTE,
    1.0, 0.5,
    -1, 1,
    "string",
    50
    };

char *rblabs[] =
    { "up", "down", "left", "right", "backward", "forward" };

char *rbvals[] =
    { "up", "down", "left", "right", "backward", "forward" };

struct uifieldrb rb =
    {
    UI_FIELDRB,
    1.0, 2.0,
    -1, 2,
    "direction",
    rblabs, rbvals,
    1,
    6, 2
    };

char *cbvars[] =
    { "salad", "soup", "vegies", "steak", "potato", "cake" };

char *cblabs[] =
    { "salad", "soup", "vegies", "steak", "potato", "cake" };

struct uifieldcb cb =
    {
    UI_FIELDCB,
    4.0, 2.5,
    3, 2,
    cbvars, cblabs,
    6, 3
    };

int actionmap[] =
    {
    UI_DEFAULT_BUTTON, UI_ORIGINAL_BUTTON,
    UI_CANCEL_BUTTON, UI_ACCEPT_BUTTON
    };

char *pblabels[] =
    {
    "Defaults", "Original\nvalues",
    "Cancel", "Accept"
    };

struct uifieldpb pb =
    {
    UI_FIELDPB,
    1.0, 1.0,
    4, 0,
    actionmap,
    pblabels,
    4, 1
    };

struct uifield *fieldarray[] =
    {
    UI_FP tb, UI_FP ststring, UI_FP string, UI_FP rb, UI_FP cb, UI_FP pb
    };

struct uiform form =
    {
    20, 20,
    -1, -1,
    fieldarray,
    6
    };

main(argc, argv, envp)
   int argc;
   char **argv;
   char **envp;
{
    FILE_ID id;
    char string[100];
    char message[100];
    int integer;
    double doub;
    int retval;

    if ((id = cfp_parse(CFP_PAR, argc, argv)) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "parse failed");
	exit(EXIT_FAIL);
	}

    retval = ui_forminput(id, &form);
    printf("forminput returned %d\n", retval);
    if (retval == UI_CANCEL_BUTTON)
	exit(EXIT_PASS);

    if (cfp_getcur(id, "direction", string, CFP_STRING, 100) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%s' for direction", string);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    cfht_log(CFHT_MAIN, CFHT_STATUS, "a status message");
    cfht_log(CFHT_MAIN, CFHT_ERROR, "an error message");
    open("asdfasdfasdf", 0);
    cfht_log(CFHT_MAIN, CFHT_ERRNO, "an errno message");

    if (cfp_getcur(id, "floater", string, CFP_STRING, 100) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%s' as a floater string", string);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_getcur(id, "string", string, CFP_STRING, 100) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%s' as a string", string);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_getcur(id, "boolean", &integer, CFP_BOOLEAN, sizeof(int)) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%d' as a boolean", integer);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_getcur(id, "num", &integer, CFP_INTEGER, sizeof(int)) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%d' as a integer", integer);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_getcur(id, "num", &doub, CFP_DOUBLE, sizeof(double)) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%lg' as a double", doub);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_getstr(id, "simplestring", string, 100) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "get failed");
	exit(EXIT_FAIL);
	}
    sprintf(message, "got '%s'", string);
    cfht_log(CFHT_MAIN, CFHT_STATUS, message);

    if (cfp_putf(id) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "putf failed");
	exit(EXIT_FAIL);
	}
    if (cfp_finish(id) == FAIL)
	{
	cfht_log(CFHT_MAIN, CFHT_FATAL, "finish failed");
	exit(EXIT_FAIL);
	}

    cfht_log(CFHT_MAIN, CFHT_DONE, "All done");
}




