#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>

int
main(int argc, const char* argv[])
{
   const char* root[] = {
      /*
       * Programs which are allowed to run setuid root from /usr/local/cfht/bin
       * must be hardcoded into the following array.  If you add a program, be
       * sure to manually make setuidinst itself setuid-root again after
       * make install.  Please remember to add a breif comment indicating
       * why the program requires setuid root priveleges.  Also, it is highly
       * recommended that all setuid programs in this list check and complain
       * loudly if their setuid-ness was not successful.  Currently the makefiles
       * do not treat failure of this program as a fatal error (I.e., if this
       * program fails, it just installs the program with normal priveleges,
       * so have the setuid-wannabe do its own sanity checks!)
       *
       * NOTE: After making your program setuid-root, good luck getting
       * a core dump on Solaris 2.5!
       */
      DIR_BIN"/deticli",
      DIR_BIN"/deticli2",
      DIR_BIN"/detio",
      DIR_BIN"/chmem",
      DIR_BIN"/cdma",
      DIR_BIN"/uh8kcom",  /* mlock and nice */
      DIR_BIN"/12kcom",   /* mlock and nice */
      DIR_BIN"/detcom",   /* mlock and nice */
      DIR_BIN"/runon",     /* rcmd */
      DIR_BIN"/tvr",       /* shared memory */
      DIR_BIN"/setuidinst", /* So makefile rule can test if its installed or not */
      0
   };
   int i;

   if (argc!=2)
   {
      fprintf(stderr,
"`%s' must be passed exactly one argument.\n"
"The argument should be the name of one of the program names,\n"
"entered exactly as shown in the following list:\n",
argv[0]);
      for (i=0; root[i]; i++)
      {
	 fprintf(stderr, "\t%s\n", root[i]);
      }
      fprintf(stderr,
"If the argument is not in the following list, nothing happens (no error).\n"
"If the argument is found, and it was already setuid root, nothing happens.\n"
"If the argument is found and it needed changing, a message is printed and\n"
"the program's owner is set to root and the setuid bit is set.\n");
      exit(EXIT_FAILURE);
   }

   for (i=0; root[i]; i++)
   {
      struct stat stbuf;

      if (strcmp(root[i], argv[1])==0 || strcmp(argv[1], "all")==0)
      {
	 if (stat(root[i], &stbuf)==0)
	 {
	    if (stbuf.st_uid!=0)
	    {
	       fprintf(stderr, "Changing %s owner to root\n",
		       root[i]);
	       if (chown(root[i], 0, -1)!=0)
	       {
		  perror("chown");
		  exit(EXIT_FAILURE);
	       }
	    }
	    if ((stbuf.st_mode&S_ISUID)==0)
	    {
	       fprintf(stderr, "Setting %s's setuid bit\n",
		       root[i]);
	       if (chmod(root[i], stbuf.st_mode|S_ISUID)!=0)
	       {
		  perror("chmod");
		  exit(EXIT_FAILURE);
	       }
	    }
	 }
	 else
	 {
	    perror(root[i]);
	    exit(EXIT_FAILURE);
	 }
      }
   }
   exit(EXIT_SUCCESS);
}
