

/*
 * This is a utility which scans a directory for the most recent
 * .c, .cc, .h, or .hh file and converts its timestamp into a
 * string with the same format as date +'%b %d %Y'.  If Make.Common
 * finds this utility, it will add -DSOURCE_DATE="string" so
 * that C/C++ programs can display the true version date on startup.
 * -DBUILD_DATE="string" is always defined as the current time, and
 * could also be displayed by C programs.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <time.h>

int
main(int argc, const char* argv[])
{
   DIR* dir;
   char s[80];
   struct dirent* entry;
   time_t dir_time = 0; /* Timestamp on the directory itself */
   time_t src_time = 0; /* Timestamp on most recently modified source file */

   if (argc != 2 || !(dir = opendir((char*)argv[1])))
   {
      fprintf(stderr, "usage: sourcedate DIRNAME\n"
	      "Shows date of most recently modified source file in DIRNAME\n");
      exit(2); /* incorrect usage */
   }

   if (chdir((char*)argv[1]) != 0)
   {
      perror(argv[1]);
      exit(EXIT_FAILURE);
   }

   while ((entry = readdir(dir)) != 0)
   {
      struct stat stbuf;

      if ((entry->d_name[0] == '.' && entry->d_name[1] != '\0') ||
	  !strchr(entry->d_name, '.') ||
	  strchr(entry->d_name, '~'))
	 continue;

      if (lstat(entry->d_name, &stbuf) != 0)
	 continue;

      if (!strcmp(entry->d_name, "."))
	 dir_time = stbuf.st_mtime;
      else if (S_ISREG(stbuf.st_mode) &&
	       stbuf.st_mtime > src_time)
	 src_time = stbuf.st_mtime;
   }
   closedir(dir);
   /*
    * If no source files were found (strange!) use timestamp
    * of the directory instead. 
    */
   if (src_time == 0) src_time = dir_time;
   tzset();
   strftime(s, sizeof(s), "%b %d %Y", localtime(&src_time));
   printf("%.80s\n", s);
   exit(EXIT_SUCCESS);
}
